package com.yizhi.statistics.application.entity;

import com.baomidou.mybatisplus.activerecord.Model;
import com.baomidou.mybatisplus.annotations.TableField;
import com.baomidou.mybatisplus.annotations.TableId;
import com.baomidou.mybatisplus.annotations.TableName;
import com.baomidou.mybatisplus.enums.FieldFill;
import com.yizhi.statistics.application.refactor.TrResearchAnswerVO;
import com.yizhi.statistics.application.refactor.TrResearchAuthorizeVO;
import com.yizhi.statistics.application.refactor.TrResearchQuestionVO;
import com.yizhi.statistics.application.refactor.TrResearchRemindVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 调研
 * </p>
 *
 * @author shengchenglong
 * @since 2018-03-12
 */
@Data
@Api (tags = "Research", description = "调研")
@TableName ("statistics_research")
public class StatisticsResearch extends Model<StatisticsResearch> {

    private static final long serialVersionUID = 1L;


    @ApiModelProperty (value = "调研主键ID，主键")
    @TableId ("id")
    private Long id;

    @ApiModelProperty (value = "培训项目ID")
    @TableField ("training_project_id")
    private Long trainingProjectId;

    @ApiModelProperty (value = "调研编码，系统将自动生成调研编码；编码规则，按创建顺序累加，例如DY000001，DY000002")
    @TableField ("research_no")
    private String researchNo;

    @ApiModelProperty (value = "积分，默认0")
    @TableField ("point")
    private Integer point;

    @ApiModelProperty (value = "调研名称")
    @TableField ("name")
    private String name;

    @ApiModelProperty (value = "开始时间")
    @TableField ("start_time")
    private Date startTime;

    @ApiModelProperty (value = "结束时间")
    @TableField ("end_time")
    private Date endTime;

    @ApiModelProperty (value = "可见范围，1平台用户可见（企业下所有人员） 2指定学员可见")
    @TableField ("visible_range")
    private Integer visibleRange;

    @ApiModelProperty (value = "状态，0草稿 1上架 2下架")
    @TableField ("state")
    private Integer state;

    @ApiModelProperty (value = "删除状态：0未删，1已删（默认0）")
    @TableField ("deleted")
    private Integer deleted;

    @ApiModelProperty (value = "创建时间")
    @TableField (value = "create_time", fill = FieldFill.INSERT)
    private Date createTime;

    @ApiModelProperty (value = "创建时间")
    @TableField (value = "record_create_time", fill = FieldFill.INSERT)
    private Date recordCreateTime;

    @ApiModelProperty (value = "创建人ID")
    @TableField (value = "create_by_id", fill = FieldFill.INSERT)
    private Long createById;

    @ApiModelProperty (value = "创建人姓名")
    @TableField (value = "create_by_name", fill = FieldFill.INSERT)
    private String createByName;


    @ApiModelProperty (value = "企业_ID")
    @TableField ("company_id")
    private Long companyId;

    @ApiModelProperty (value = "部门_ID")
    @TableField ("org_id")
    private Long orgId;

    @ApiModelProperty (value = "站点_ID")
    @TableField ("site_id")
    private Long siteId;

    /**
     * 该调研可参加人数
     */
    @ApiModelProperty (value = "可参加人数")
    @TableField (exist = false)
    private Integer count;

    /**
     * 实际参加人数
     */
    @ApiModelProperty (value = "实际参加人数")
    @TableField (exist = false)
    private Integer realCount;

    /**
     * 提醒集合，不持久化
     */
    @ApiModelProperty (value = "消息提醒")
    @TableField (exist = false)
    private List<TrResearchRemindVO> reminds;

    /**
     * 关联人员集合
     */
    @ApiModelProperty (value = "关联人员")
    @TableField (exist = false)
    private List<TrResearchAuthorizeVO> authorizes;

    @ApiModelProperty (value = "关联人员id")
    @TableField (exist = false)
    private List<Long> relationIds;


    /**
     * 问题集合，不持久化
     */
    @ApiModelProperty (value = "消息提醒")
    @TableField (exist = false)
    private List<TrResearchQuestionVO> questions;

    @ApiModelProperty (value = "改调研的答卷")
    @TableField (exist = false)
    private List<TrResearchAnswerVO> trResearchAnswers;

    @ApiModelProperty (value = "搜索列表显示状态：1已完成，2进行中，3已过期")
    @TableField (exist = false)
    private Integer finishState;

    @ApiModelProperty (value = "完成时间")
    @TableField (exist = false)
    private Date finishTime;


    @Override
    protected Serializable pkVal() {
        return this.id;
    }

}
