package com.yizhi.statistics.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.statistics.application.exam.domain.po.ExamGroupPO;
import com.yizhi.statistics.application.exam.domain.po.UserExamGroupPO;
import com.yizhi.statistics.application.exam.domain.vo.ExamGroupOrgVO;
import com.yizhi.statistics.application.exam.domain.vo.UserExamByOrgVO;
import com.yizhi.statistics.application.refactor.exam.*;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;


/**
 * 报表服务考试
 *
 * @param
 * @author ding
 * @date 2019/4/18
 */
@FeignClient(name = "statistics",contextId = "StatisticsExamClient")
public interface StatisticsExamClient {

    /**
     * 部门用户考试统计
     * @param po
     * @return
     */
    @GetMapping("statisticsExam/group/org/users")
    Page<UserExamByOrgVO> getExamByGroupOrgUser(@RequestBody UserExamGroupPO po);

    /**
     * 部门用户考试统计
     * @param po
     * @return
     */
    @GetMapping("statisticsExam/group/org/users/download")
    String getExamByGroupOrgUserDownload(@RequestBody UserExamGroupPO po);

    @GetMapping("statisticsExam/group/org/users/detail/download")
    String getExamByGroupOrgUserDetailDownload(@RequestBody UserExamGroupPO po);

    /**
     * 部门考试统计
     * @param po
     * @return
     */
    @GetMapping("statisticsExam/group/org")
    Page<ExamGroupOrgVO> getExamGroupByOrg(@RequestBody ExamGroupPO po);

    /**
     * 部门统计下载
     * @param po
     * @return
     */
    @GetMapping("statisticsExam/group/org/download")
    String getExamGroupByOrgDownload(@RequestBody ExamGroupPO po);

    /**
     * 报表服务考试定时服务
     *
     * @param startDate
     * @param endDate
     * @return
     */
    @GetMapping("/statisticsExam/asynchronous/load/date/old")
    public void AsynchronousExam(@RequestParam(value = "startDate", required = false) String startDate,
                                 @RequestParam(value = "endDate", required = false) String endDate);


    /**
     * 按考试统计
     *
     * @param reportExamParam
     * @return
     */
    @GetMapping("statisticsExam/group")
    public Page<ReportExamVO> getExamGroup(@RequestBody ReportExamParam reportExamParam);


    /**
     * 按考试统计下载s
     *
     * @param reportExamParam
     * @return
     */
    @GetMapping("statisticsExam/group/download")
    public String getExamGroupDownload(@RequestBody ReportExamParam reportExamParam);

    /**
     * 按考试查看按用户
     *
     * @param vo
     * @return
     */
    @GetMapping("statisticsExam/group/account")
    public Page<ReportExamAccountsViewVo> getExamAccountView(@RequestBody ExamCountByUserVO vo);

    /**
     * 按考试查看按用户下载
     *
     * @param vo
     * @return
     */
    @GetMapping("statisticsExam/group/account/trExams/download")
    public String getExamAccountTrExamsDownload(@RequestBody ExamCountByUserVO vo);

    /**
     * 按用户统计
     * @param vo
     * @return
     */
    @GetMapping("statisticsExam/account/group")
    public Page<ReportExamAccountVO> getAccountGroup(@RequestBody ExamCountByUserVO vo);

    /**
     * 按用户统计下载
     * @param vo
     * @return
     */
    @GetMapping("statisticsExam/account/group/download")
    public String getAccountGroupDownload(@RequestBody ExamCountByUserVO vo);


    @GetMapping("statisticsExam/account/group/view")
    public Page<ReportExamAccountViewVo> getAccountGroupView(
           @RequestParam(name = "startTime", required = false) String startTime,
           @RequestParam(name = "endTime", required = false) String endTime,
           @RequestParam(name = "accountId", required = true) Long accountId,
           @RequestParam(name = "examNameLike", required = false) String examNameLike,
           @RequestParam(name = "pageSize", required = false) Integer pageSize,
           @RequestParam(name = "pageNo", required = false) Integer pageNo);

    @GetMapping("statisticsExam/account/group/view/download")
    public String getAccountGroupViewDownload(
            @RequestParam(name = "startTime", required = false) String startTime,
            @RequestParam(name = "endTime", required = false) String endTime,
            @RequestParam(name = "examNameLike", required = false) String examNameLike,
            @RequestParam(name = "accountId", required = true) Long accountId);

    @GetMapping("statisticsExam/examView")
    public StatisticsExamVO getExam(
            @RequestParam(name = "examId", required = true) Long examId);
}
