package com.yizhi.statistics.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.statistics.application.refactor.sign.StatisticsSignVO;
import com.yizhi.statistics.application.vo.sign.SignAccountGroupVo;
import com.yizhi.statistics.application.vo.sign.StatisticsSignVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;


/**
 * 报表服务
 *
 * @param
 * @author ding
 * @date 2019/4/22
 */
@FeignClient(name = "statistics", contextId = "StatisticsSignClient")
public interface StatisticsSignClient {


    /**
     * 报表服务考试定时服务
     *
     * @param startDate
     * @param endDate
     * @return
     */
    @GetMapping("/statisticsSign/asynchronous/load/date")
    public void AsynchronousSign(@RequestParam(value = "startDate", required = false) String startDate,
                                 @RequestParam(value = "endDate", required = false) String endDate);

    /**
     * 签到统计
     *
     * @param projectId
     * @param kwd
     * @param startDate
     * @param endDate
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/statisticsSign/sign/group")
    public Page<StatisticsSignVO> signGroup(
            @RequestParam(name = "projectId", required = false) Long projectId,
            @RequestParam(name = "kwd", required = false) String kwd,
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate,
            @RequestParam(name = "pageNo", required = false) Integer pageNo,
            @RequestParam(name = "pageSize", required = false) Integer pageSize);



    /**
     * 签到统计下载
     *
     * @param projectId
     * @param kwd
     * @param startDate
     * @param endDate
     * @return
     */
    @GetMapping("/statisticsSign/sign/group/export")
    public String signGroupDownload(
            @RequestParam(name = "projectId", required = false) Long projectId,
            @RequestParam(name = "kwd", required = false) String kwd,
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate);

    /**
     * 签到统计查看
     *
     * @param signId
     * @param signTimeId
     * @param startDate
     * @param endDate
     * @param orgNameOrCode
     * @param accountName
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/statisticsSign/sign/group/view")
    public Page<StatisticsSignVO> signView(
            @RequestParam(name = "signId", required = true) Long signId,
            @RequestParam(name = "signTimeId", required = true) Long signTimeId,
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate,
            @RequestParam(name = "orgNameOrCode", required = false) String orgNameOrCode,
            @RequestParam(name = "accountName", required = false) String accountName,
            @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo,
            @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize);


    /**
     * 签到统计查看下载
     *
     * @param signId
     * @param signTimeId
     * @param startDate
     * @param endDate
     * @param orgNameOrCode
     * @param accountName
     * @return
     */
    @GetMapping("/statisticsSign/sign/group/view/export")
    public String signViewDownload(
            @RequestParam(name = "signId", required = true) Long signId,
            @RequestParam(name = "signTimeId", required = true) Long signTimeId,
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate,
            @RequestParam(name = "orgNameOrCode", required = false) String orgNameOrCode,
            @RequestParam(name = "accountName", required = false) String accountName);


    /**
     * 签到-按用户统计
     * @param accountKwd
     * @param orgKwd
     * @param startDate
     * @param endDate
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/statisticsSign/sign/account/group")
    public Page<SignAccountGroupVo> signAccountGroup(
            @RequestParam(name = "accountKwd", required = false) String accountKwd,
            @RequestParam(name = "orgKwd", required = false) String orgKwd,
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate,
            @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo,
            @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize);


    /**
     * 签到-按用户统计-导出签到统计数据
     * @param accountKwd
     * @param orgKwd
     * @param startDate
     * @param endDate
     * @return
     */
    @GetMapping("/statisticsSign/sign/account/group/export")
    public String signAccountGroupExport(@RequestParam(name = "accountKwd", required = false) String accountKwd,
                                         @RequestParam(name = "orgKwd", required = false) String orgKwd,
                                         @RequestParam(name = "startDate", required = false) String startDate,
                                         @RequestParam(name = "endDate", required = false) String endDate);


    /**
     * 签到-按用户统计-导出全部签到统计数据和明细
     * @param accountKwd
     * @param orgKwd
     * @param startDate
     * @param endDate
     * @return
     */
    @GetMapping("/statisticsSign/sign/account/group/exportAll")
    public String signAccountGroupExportAll(@RequestParam(name = "accountKwd", required = false) String accountKwd,
                                            @RequestParam(name = "orgKwd", required = false) String orgKwd,
                                            @RequestParam(name = "startDate", required = false) String startDate,
                                            @RequestParam(name = "endDate", required = false) String endDate);


    /**
     * 签到-按用户统计-查看
     * @param accountId
     * @param startDate
     * @param endDate
     * @param projectKwd
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/statisticsSign/sign/account/group/view")
    public Page<StatisticsSignVo> signAccountView(
            @RequestParam(name = "accountId", required = true) Long accountId,
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate,
            @RequestParam(name = "projectKwd", required = false) String projectKwd,
            @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo,
            @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize);


    /**
     * 签到-按用户统计-查看-下载全部数据
     * @param accountId
     * @param startDate
     * @param endDate
     * @param projectKwd
     * @return
     */
    @GetMapping("/statisticsSign/sign/account/group/view/export")
    public String signAccountViewExport(@RequestParam(name = "accountId", required = true) Long accountId,
                                        @RequestParam(name = "accountName", required = true) String accountName,
                                        @RequestParam(name = "startDate", required = false) String startDate,
                                        @RequestParam(name = "endDate", required = false) String endDate,
                                        @RequestParam(name = "projectKwd", required = false) String projectKwd);


}
