package com.yizhi.statistics.application.controller.album;

import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.album.application.feign.AlThemeActivityClient;
import com.yizhi.album.application.feign.AlbumClient;
import com.yizhi.album.application.feign.StatisticsAlbumClient;
import com.yizhi.album.application.vo.AlbumVo;
import com.yizhi.album.application.vo.domain.AlbumVisibleRange;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.course.application.vo.CourseAccountReport;
import com.yizhi.statistics.application.commands.IStatisticsMetadataTask;
import com.yizhi.statistics.application.entity.vo.CourseCalculateTimeVo;
import com.yizhi.statistics.application.mapper.StatisticsAlbumActivityRecordMapper;
import com.yizhi.statistics.application.scope.Scope;
import com.yizhi.statistics.application.scope.Type;
import com.yizhi.statistics.application.service.IStatisticsAlbumActivityRecordService;
import com.yizhi.statistics.application.service.IStatisticsAlbumActivityService;
import com.yizhi.statistics.application.service.IStatisticsAlbumService;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.statistics.application.util.TrainingUtil;
import com.yizhi.statistics.application.work.CourseService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;

@Component
public class AlbumControllerAbs extends AbstractDefaultTask<String, Map<String, Object>> implements IStatisticsMetadataTask<AlbumVo> {

	private Logger logger = LoggerFactory.getLogger(AlbumControllerAbs.class);

	final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");
	
	final ThreadLocal<SimpleDateFormat> localdf = new ThreadLocal<SimpleDateFormat>();
	
	@Autowired
	private JedisUtil jedisUtil;
	
	@Autowired
	private TrainingUtil trainingUtil;
	
	@Autowired
	private IdGenerator idGenerator;
	
	@Autowired
	private StatisticsAlbumClient statisticsAlbumClient;
	
	@Autowired
	private AlbumClient albumClient;
	
	@Autowired
	private CourseService courseService;
	
	@Autowired
	private AlThemeActivityClient alThemeActivityClient;
	
	@Autowired
	private IStatisticsAlbumService statisticsAlbumService;
	
	@Autowired
	private IStatisticsAlbumActivityService statisticsAlbumActivityService;
	
	@Autowired
	private IStatisticsAlbumActivityRecordService statisticsAlbumActivityRecordService;
	
	@Autowired
	private StatisticsAlbumActivityRecordMapper StatisticsAlbumActivityRecordMapper;
	
	@Override
	protected String execute(Map<String, Object> map) {
		// TODO Auto-generated method stub
		Long taskId=idGenerator.generate();
		Long t1=System.currentTimeMillis();
		localdf.set(new SimpleDateFormat("yyyy-MM-dd"));
		SimpleDateFormat df = localdf.get();
		/**
		 * 走异步任务
		 */
		Date curDateWork=new Date();
		// 如果是28号凌晨0：10开始跑 则curDate是27号的日期
		TaskContext taskContext = new TaskContext(taskId, sim.format(curDateWork) + " 号0:10专辑报表分析前一天的数据", 999L, curDateWork);
		working(taskContext);

		Long companyIdParm=null;
		Long siteIdParm=null;;
		Object companyIdObject=map.get("companyId");
		Object siteIdObject=map.get("siteId");
		try {
			if(companyIdObject!=null) {
				companyIdParm=Long.valueOf(companyIdObject.toString());
			}
			if(siteIdObject!=null) {
				siteIdParm=Long.valueOf(siteIdObject.toString());
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			logger.info("companyIdParm:{},转换错误",companyIdParm);
			logger.info("siteIdParm:{},转换错误",siteIdParm);
			e.printStackTrace();
		}
		
		// 得到所有的专辑
		List<AlbumVo> allAlbums = statisticsAlbumClient.getAllAlbums(companyIdParm, siteIdParm);
		Integer countNum=allAlbums.size();
		
	
		
		if(CollectionUtils.isNotEmpty(allAlbums)){
			taskDetail(taskId, "***** 专辑统计开始 ,专辑数量:" + allAlbums.size() +" *****");
			logger.info("***** 专辑统计开始 ,专辑数量:" + allAlbums.size() +" *****");
		} else {
			taskDetail(taskId, "***** 专辑统计开始 ,专辑数量获取异常:0或者为空 *****");
			logger.info("***** 专辑统计开始 ,专辑数量获取异常:0或者为空 *****");
		}
		
		Date middleMaxTime = StatisticsAlbumActivityRecordMapper.selectMaxDate();
		if(middleMaxTime != null){
			//判断获取记录时间是否大于当前时间，是的话重置该时间（当前时间减一天）
			if(middleMaxTime.compareTo(curDateWork) == 1 || df.format(curDateWork).equals(df.format(middleMaxTime))){
				middleMaxTime = addDay(curDateWork, -1);
			} else {
				middleMaxTime = addDay(middleMaxTime, 1);
			}
		}
		CourseCalculateTimeVo cctv=trainingUtil.calculateTime(map,middleMaxTime);
		Date startDate = cctv.getStartDate();
		Date endDate = cctv.getEndDate();
		if(startDate.compareTo(endDate) == 1){
			startDate = cctv.getEndDate();
			endDate = cctv.getStartDate();
		}
		String startDateString=df.format(startDate);
		String endDateString=df.format(endDate);
		
		int dealSuccess=0;

		//转换scope对象
		Scope scope=new Scope();
		
		int i=1;

		if (CollectionUtils.isNotEmpty(allAlbums)) {

			for (AlbumVo album : allAlbums) {
				
				Long temp1=System.currentTimeMillis();
				Long siteId = album.getSiteId();
				Long albumId = album.getId();
				Integer workScope = album.getScope(); // 原始数据可见范围 1 平台用户 2 指定范围//只能传1和2
				//每个业务都需要将本身的scope进行转换，下边所有的代码全部是switchScope
				Integer switchScope=scope.album(workScope);
				Long companyId = album.getCompanyId();
				
				
				List<AlbumVisibleRange> authorizationRanges = null;
				CourseAccountReport courseAccountReport = new CourseAccountReport();
				
				// 对这个专辑的学习记录开始操作
				if (siteId == null || companyId == null || albumId == null || (switchScope==-1)) {
					taskDetail(taskId, "专辑ID:"+album.getId()+"处理失败,原因:参数不正确");
					continue;
				}
				if (switchScope == 2) {
					authorizationRanges = albumClient.VisibleRange(albumId);
                    // set集合转换为arrayList集合
                    if (CollectionUtils.isEmpty(authorizationRanges)) {
                        logger.info("专辑:" + albumId + ",是指定人可见，但是原始数据没有查询到可见范围权限");
                        continue;
                    } else {
                        List<Long> accountIds = new ArrayList<>();
                        List<Long> orgIds = new ArrayList<>();
                        for (AlbumVisibleRange authorizationRange : authorizationRanges) {
                            //1部门 2用户
                            if (authorizationRange.getType() == 1) {
                                orgIds.add(authorizationRange.getRelationId());
                            } else if (authorizationRange.getType() == 2) {
                                accountIds.add(authorizationRange.getRelationId());
                            }
                        }
                        courseAccountReport.setCourseId(albumId);
                        courseAccountReport.setAccountIds(accountIds);
                        courseAccountReport.setOrgIds(orgIds);
                    }
				}
				courseService.executeTask(taskId,album, albumId, startDate, endDate, Type.ALBUM.getType(), companyId, siteId,courseAccountReport,this, switchScope);
				dealSuccess++;
				Long temp2=System.currentTimeMillis();
				String me="统计完成"+(i++)+"专辑,总数:"+countNum+"。专辑id："+albumId+"。时间(ms):"+(temp2-temp1);
				logger.info(me);
			}
			//所有专辑进行循环完毕，把活动,实质上一个活动和专辑是多对一的关系记录导入
			statisticsAlbumActivityRecordService.dealGetAllRecordeByTimeLimit(startDateString, endDateString);
		}
		
		
		StringBuffer updateOrgIdResult=new StringBuffer("  ");
		taskDetail(taskId, updateOrgIdResult.toString());
		Long t2=System.currentTimeMillis();
		String result="总数量:"+allAlbums.size()+"。处理时间ms:"+(t2-t1)+",处理成功:"+dealSuccess+",处理失败:"+(allAlbums.size()-dealSuccess);
		taskDetail(taskId, result);
		success(taskContext, result);
		
		try {
			//设置可参加人数缓存
			jedisUtil.setWorkIsSuccess(Type.ALBUM.getType(), curDateWork);
			updateOrgIdResult.append(curDateWork+"专辑和account跑成功。处理orgId成功。");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			updateOrgIdResult.append(curDateWork+"专辑和account跑成功。处理orgId失败。原因:"+e.getMessage());
		}
		
		return "success";
	}
	
	@Override
	public void executeMetadata(Long taskId, AlbumVo album, Date startDate, Date endDate, Long companId,
			Long siteId) {
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
		//处理专辑本身
		statisticsAlbumService.dealStatisticsAlbum(album);
		//处理专辑活动
		statisticsAlbumActivityService.dealStatisticsAlbumActivity(album);
		taskDetail(taskId, "专辑ID:"+album.getId()+"处理成功");
	}



	@Override
	public void exceptionLog(Long taskId, AlbumVo metadataObject, Exception e) {
		// TODO Auto-generated method stub
		taskDetail(taskId, "专辑ID:"+metadataObject.getId()+"处理失败"+e.getMessage());
	}

	protected Date addDay(Date date, int num) {
		Calendar calendar = new GregorianCalendar();
		calendar.setTime(date);
		calendar.add(calendar.DATE, num);
		date = calendar.getTime();
		return date;
	}

}
