package com.yizhi.statistics.application.controller.album.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.album.vo.AlbumGroupAccountParam;
import com.yizhi.statistics.application.album.vo.ReportStudyAlbumAccountViewVo;
import com.yizhi.statistics.application.service.IStatisticsAlbumActivityRecordService;
import com.yizhi.statistics.application.service.IStatisticsAlbumService;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Component
public class AlbumAccountGroupViewExport extends AbstractDefaultTask<String, AlbumGroupAccountParam> {

	private static final Logger logger = LoggerFactory.getLogger(AlbumAccountGroupViewExport.class);

	@Autowired
	private IStatisticsAlbumActivityRecordService statisticsAlbumActivityRecordService;
	
	@Autowired
	private IStatisticsAlbumService statisticsAlbumService;
	
	@Override
	protected String execute(AlbumGroupAccountParam param) {
		String sheetName="专辑-按用户统计-查看-下载专辑学习结果";
		String upLoadUrl = null;
		Long adminId = param.getAdminId();
		Boolean isAdmin=param.getIsAdmin();
		Long companyId = param.getCompanyId();
		Long siteId = param.getSiteId();
		String startDate = param.getStartDate();
		String endDate = param.getEndDate();
		String kwd = param.getAlbumKwd();
		Long taskId=param.getTaskId();
		Date submitTime=param.getSubmitTime();
		String serialNo=param.getSerialNo();
		String taskName=param.getTaskName();
		Long accountId=param.getAdminId();
		String orgKwd = param.getOrgKwd();
		String accountKwd = param.getAccountKwd();
		Integer pageNo = 1;
		Integer pageSize = 10;
		Integer maxNumOneSheet=50000;   //一个sheet最多有多少数据量
		Integer numSheet=0;             //有多少个sheet
		Integer curSheet=1;
		TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,adminId, submitTime, siteId, companyId);// 任务id  任务名字      操作人id   操作时间
		working(taskContext);
		Page<ReportStudyAlbumAccountViewVo> page = statisticsAlbumActivityRecordService.albumAccountGroupView(companyId, accountId, siteId, kwd, startDate, endDate, pageNo-1, pageSize, adminId, isAdmin);
		List<ReportStudyAlbumAccountViewVo> list = new ArrayList<ReportStudyAlbumAccountViewVo>();
		if (page != null) {
			logger.info("专辑-按照用户统计-查看-下载专辑学习结果-总记录数：{}", page.getTotal());
			list.addAll(page.getRecords());
			int num=page.getTotal()/pageSize+1;
			for (int i = 2; i <= num; i++) {
				page= statisticsAlbumActivityRecordService.albumAccountGroupView(companyId, accountId, siteId, kwd, startDate, endDate, i, pageSize, adminId, isAdmin);
				if(page == null || page.getRecords().size() == 0) {
					break;
				}
				list.addAll(page.getRecords());
			}
		}
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
		SimpleDateFormat df2 = new SimpleDateFormat("yyyyMMddHHmmss");
		try {
			String headerName=null;
			if(!CollectionUtils.isEmpty(list)) {
				headerName=list.get(0).getAccountName();
			}
			XSSFWorkbook wb = new XSSFWorkbook();
			if(CollectionUtils.isNotEmpty(list)) {
				numSheet = list.size() / maxNumOneSheet + 1;
				XSSFSheet sheet=null;
				XSSFRow row =null;
				XSSFCell cell=null;
				XSSFRow row2 =null;
				XSSFCell cell2 =null;
				XSSFRow row3 =null;
				for (int i = 1; i <= numSheet; i++) {
					sheet = wb.createSheet(sheetName+i);
					row = sheet.createRow((int) 0);
					XSSFCellStyle style = wb.createCellStyle();
					style.setAlignment(HorizontalAlignment.CENTER); 
					CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 8);
					sheet.addMergedRegion(cra);
					cell = row.createCell(0);
					cell.setCellValue(headerName + "专辑学习明细统计");
					cell.setCellStyle(style);
					row2 = sheet.createRow(1);
					CellRangeAddress cra2 = new CellRangeAddress(1, 1, 0, 3);
					sheet.addMergedRegion(cra2);
					cell2 = row2.createCell(0);
					cell2.setCellValue("统计周期： " + startDate + "~" + endDate);
					row3 = sheet.createRow((int) 2);
					row3.createCell(0).setCellValue("用户名");
					row3.createCell(1).setCellValue("姓名");
					row3.createCell(2).setCellValue("专辑名称");
					row3.createCell(3).setCellValue("连载状态");
					row3.createCell(4).setCellValue("开始时间");
					row3.createCell(5).setCellValue("最早完成时间");
					row3.createCell(6).setCellValue("学习状态");
					row3.createCell(7).setCellValue("用户当前状态");
					row3.createCell(8).setCellValue("所在部门");
					if (list != null && list.size() > 0) {
						ReportStudyAlbumAccountViewVo rsav = null;
						for (int j = 0; j < maxNumOneSheet; j++) {
							row = sheet.createRow(j + 3);
							int index = j + (i - 1) * maxNumOneSheet;
							if (index >= list.size()) {
								break;
							}
							rsav = list.get(index);
							row.createCell(0).setCellValue(rsav.getAccountName());
							row.createCell(1).setCellValue(rsav.getAccountFullName());
							row.createCell(2).setCellValue(rsav.getAlbumName());
							row.createCell(3).setCellValue(rsav.getOvered().equals(0)?"连载中":"已完结");
							if (rsav.getStartTime() != null) {
								row.createCell(4).setCellValue(df.format(rsav.getStartTime()));
							}
							//完成时间就是首次完成时间
							if (rsav.getEndTime() != null&&rsav.getCompleteness()==100) {
								row.createCell(5).setCellValue(df.format(rsav.getEndTime()));
							}
							row.createCell(6).setCellValue(rsav.getCompleteness());
							row.createCell(7).setCellValue(rsav.getAccountState() == 0 ? "禁用" : "启用");
							row.createCell(8).setCellValue(rsav.getOrgName());
						}
					}
				}
			}
			String fileName = sheetName + df2.format(submitTime) + ".xlsx";
			String requestPath = FileConstant.SAVE_PATH;
			File fileDir = new File(requestPath);
			if (!fileDir.exists()) {
				fileDir.mkdir();
			}
			String path = requestPath+File.separator + fileName;
			FileOutputStream os = null;
			File file = null;
			try {
				os = new FileOutputStream(path);
				wb.write(os);
				// 阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file = new File(path);
				success(taskContext, "成功", upLoadUrl);
			} catch (Exception e1) {
				e1.printStackTrace();
				fail(taskContext, "写入过程中发生错误");
				logger.error("上传过程中发生错误，请查看日志");
			} finally {
				if (os != null) {
					os.close();
				}
				if (wb != null) {
					wb.close();
				}
				if (file != null) {
					file.delete();
				}
			}

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, sheetName+"导出过程中发生错误，请查看日志"+e.getMessage());
			logger.error(sheetName+"导出过程中发生错误，请查看日志");
		}
		return upLoadUrl;
	}

}
