package com.yizhi.statistics.application.controller.album.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.album.vo.AlbumGroupParam;
import com.yizhi.statistics.application.album.vo.ReportStudyAlbumVo;
import com.yizhi.statistics.application.service.IStatisticsAlbumActivityRecordService;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Component
public class AlbumGroupExport  extends AbstractDefaultTask<String, AlbumGroupParam> {

	private static final Logger logger = LoggerFactory.getLogger(AlbumGroupExport.class);

	@Autowired
	private IStatisticsAlbumActivityRecordService statisticsAlbumActivityRecordService;
	
	@Override
	protected String execute(AlbumGroupParam param) {
		String sheetName="专辑完成情况统计";
		String upLoadUrl = null;
		Long adminId = param.getAdminId();
		Boolean isAdmin=param.getIsAdmin();
		Long companyId = param.getCompanyId();
		Long siteId = param.getSiteId();
		String startDate = param.getStartDate();
		String endDate = param.getEndDate();
		String kwd = param.getKwd();
		Long taskId=param.getTaskId();
		Date submitTime=param.getSubmitTime();
		String serialNo=param.getSerialNo();
		String taskName=param.getTaskName();
		Integer pageNo = 2;
		Integer pageSize = 10;
		
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,adminId, submitTime, siteId, companyId);// 任务id  任务名字      操作人id   操作时间
		working(taskContext);
		Page<ReportStudyAlbumVo> page = statisticsAlbumActivityRecordService.albumGroup(kwd, startDate, endDate, companyId, siteId, pageNo-1, pageSize, adminId, isAdmin);
		List<ReportStudyAlbumVo> list = new ArrayList<ReportStudyAlbumVo>();
		
		if (page != null) {
			logger.info("专辑-专辑完成情况统计-下载-总记录数：{}", page.getTotal());
			list.addAll(page.getRecords());
			int num=page.getTotal()/pageSize+1;
			for (int i = 2; i <= num; i++) {
				page=statisticsAlbumActivityRecordService.albumGroup(kwd, startDate, endDate, companyId, siteId, i, pageSize, adminId, isAdmin);
				if(page == null || page.getRecords().size() == 0) {
					break;
				}
				list.addAll(page.getRecords());
			}
		}
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
		SimpleDateFormat df2 = new SimpleDateFormat("yyyyMMddHHmmss");
		
		
	try {
				   XSSFWorkbook wb = new XSSFWorkbook();
					XSSFSheet sheet = wb.createSheet(sheetName);
					XSSFRow row = sheet.createRow((int) 0);
					XSSFCellStyle style = wb.createCellStyle();
					style.setAlignment(HorizontalAlignment.CENTER); // 创建一个居中格式
					CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 9); // 起始行, 终止行, 起始列, 终止列
					sheet.addMergedRegion(cra);
					XSSFCell cell = row.createCell(0);
					cell.setCellValue(sheetName);
					cell.setCellStyle(style);
					XSSFRow row2 = sheet.createRow(1);
					CellRangeAddress cra2 = new CellRangeAddress(1, 1, 0, 3);
					sheet.addMergedRegion(cra2);
					XSSFCell cell2 = row2.createCell(0);
					cell2.setCellValue("统计周期： " + startDate + "~" + endDate);
					XSSFRow row3 = sheet.createRow((int) 2);
					row3.createCell(0).setCellValue("专辑名称");
					row3.createCell(1).setCellValue("连载状态");
					row3.createCell(2).setCellValue("已学习人数");
					row3.createCell(3).setCellValue("学习次数");
					row3.createCell(4).setCellValue("专辑时长");
					row3.createCell(5).setCellValue("总学习时长(HH:mm:ss)");
					row3.createCell(6).setCellValue("平均学习时长(s)");
					row3.createCell(7).setCellValue("评论数");
					row3.createCell(8).setCellValue("订阅数");
					row3.createCell(9).setCellValue("完成人数");
					// 第五步，写入实体数据 实际应用中这些数据从数据库得到，
					if (CollectionUtils.isNotEmpty(list)) {
						ReportStudyAlbumVo rsav = null;
						for (int i = 0; i < list.size(); i++) {
							row = sheet.createRow(i+ 3);
							rsav = list.get(i);
							// 第四步，创建单元格，并设置值
							row.createCell(0).setCellValue(rsav.getAlbumName());
							row.createCell(1).setCellValue(rsav.getOvered().equals(0)?"连载中":"已完结");
							row.createCell(2).setCellValue(rsav.getTotalInAccount());
							row.createCell(3).setCellValue(rsav.getTotalCount());
							row.createCell(4).setCellValue(rsav.getDuration());
							row.createCell(5).setCellValue(rsav.getTotalDuration());
							row.createCell(6).setCellValue(rsav.getAvgDuration());
							row.createCell(7).setCellValue(rsav.getCommentNum());
							row.createCell(8).setCellValue(rsav.getSubscribeNum());
							row.createCell(9).setCellValue(rsav.getFinish());
						}
					}
			String fileName = sheetName + df2.format(submitTime) + ".xlsx";
			String requestPath = FileConstant.SAVE_PATH;
			File fileDir = new File(requestPath);
			if (!fileDir.exists()) {
				fileDir.mkdir();
			}
			String path = requestPath +File.separator+fileName;
			FileOutputStream os = null;
			File file = null;
			try {
				os = new FileOutputStream(path);
				wb.write(os);
				// 阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file = new File(path);
				success(taskContext, "成功", upLoadUrl);
			} catch (Exception e1) {
				e1.printStackTrace();
				fail(taskContext, "写入过程中发生错误");
				logger.error("上传过程中发生错误，请查看日志");
			} finally {
				if (os != null) {
					os.close();
				}
				if (wb != null) {
					wb.close();
				}
				if (file != null) {
					file.delete();
				}
			}

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, sheetName+"导出过程中发生错误，请查看日志"+e.getMessage());
			logger.error(sheetName+"导出过程中发生错误，请查看日志");
		}
		return upLoadUrl;
	}

}
