package com.yizhi.statistics.application.controller.album.export;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.album.vo.AlbumGroupParam;
import com.yizhi.statistics.application.album.vo.ReportStudyAlbumViewVo3;
import com.yizhi.statistics.application.entity.StatisticsAlbum;
import com.yizhi.statistics.application.service.IStatisticsAlbumActivityRecordService;
import com.yizhi.statistics.application.service.IStatisticsAlbumActivityService;
import com.yizhi.statistics.application.service.IStatisticsAlbumService;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

@Component
public class AlbumGroupViewExport3   extends AbstractDefaultTask<String, AlbumGroupParam> {


	private static final Logger logger = LoggerFactory.getLogger(AlbumGroupViewExport3.class);

	@Autowired
	private IStatisticsAlbumActivityRecordService statisticsAlbumActivityRecordService;
	
	@Autowired
	private IStatisticsAlbumService statisticsAlbumService;
	
	@Autowired
	private IStatisticsAlbumActivityService statisticsAlbumActivityService;
	
	@Override
	protected String execute(AlbumGroupParam param) {
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		SimpleDateFormat df2 = new SimpleDateFormat("yyyyMMddHHmmss");
		String sheetName="按专辑统计-查看-下载学员学习明细";
		String upLoadUrl = null;
		Long adminId = param.getAdminId();
		Boolean isAdmin=param.getIsAdmin();
		Long companyId = param.getCompanyId();
		Long siteId = param.getSiteId();
		Long taskId=param.getTaskId();
		Date submitTime=param.getSubmitTime();
		String serialNo=param.getSerialNo();
		String taskName=param.getTaskName();
		Long albumId=param.getAlbumId();
		
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,adminId, submitTime, siteId, companyId);// 任务id  任务名字      操作人id   操作时间
		working(taskContext);

		/**
		 * 专辑主体信息-表头
		 */
	
		StatisticsAlbum album = new StatisticsAlbum();
		album.setAlbumId(albumId);
		EntityWrapper<StatisticsAlbum> wrapper = new EntityWrapper<StatisticsAlbum>(album);
		StatisticsAlbum one = statisticsAlbumService.selectOne(wrapper);
		String startDate = param.getStartDate();
		String endDate = param.getEndDate();
		List<ReportStudyAlbumViewVo3> list = statisticsAlbumActivityRecordService.albumGroupView3(albumId, startDate, endDate, companyId, siteId, adminId, isAdmin);
		Integer maxNumOneSheet=1000000;   //一个sheet最多有多少数据量
		Integer numSheet=0;             //有多少个sheet

		try {
			XSSFWorkbook wb = new XSSFWorkbook();
			if(CollectionUtils.isNotEmpty(list)) {
				numSheet = list.size() / maxNumOneSheet + 1;
				String headerName=null;
				XSSFSheet sheet =null;
				XSSFCell cell2 =null;
				XSSFRow row3=null;
				for (int i = 1; i <= numSheet; i++) {
					headerName = one.getAlbumName() + "-学员学习明细统计";
					sheet = wb.createSheet(sheetName + i);
					XSSFRow row = sheet.createRow((int) 0);
					XSSFCellStyle style = wb.createCellStyle();
					style.setAlignment(HorizontalAlignment.CENTER); 
					CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 10); 
					sheet.addMergedRegion(cra);
					XSSFCell cell = row.createCell(0);
					cell.setCellValue(headerName);
					cell.setCellStyle(style);
					XSSFRow row2 = sheet.createRow((int) 1);
					CellRangeAddress cra2 = new CellRangeAddress(1, 1, 0, 3);
					sheet.addMergedRegion(cra2);
					cell2 = row2.createCell(0);
					cell2.setCellValue("统计周期： " + startDate + "~" + endDate);
					row3 = sheet.createRow((int) 2);
					row3.createCell(0).setCellValue("用户名");
					row3.createCell(1).setCellValue("姓名");
					row3.createCell(2).setCellValue("主题名称");
					row3.createCell(3).setCellValue("课程名称");
					row3.createCell(4).setCellValue("开始时间");
					row3.createCell(5).setCellValue("最早完成时间");
					row3.createCell(6).setCellValue("课程学习时长");
					row3.createCell(7).setCellValue("解锁状态");
					row3.createCell(8).setCellValue("完成状态");
					row3.createCell(9).setCellValue("用户当前状态");
					row3.createCell(10).setCellValue("所在部门");
					if (list != null && list.size() > 0) {
						ReportStudyAlbumViewVo3 rsav = null;
						for (int j = 0; j < maxNumOneSheet; j++) {
							row = sheet.createRow(j + 3);
							int index = j + (i - 1) * maxNumOneSheet;
							if (index >= list.size()) {
								break;
							}
							rsav = list.get(index);
							// 第四步，创建单元格，并设置值
							row.createCell(0).setCellValue(rsav.getAccountName());
							row.createCell(1).setCellValue(rsav.getAccountFullName());
							row.createCell(2).setCellValue(rsav.getThemeName());
							row.createCell(3).setCellValue(rsav.getCourseName());
							if (rsav.getStartTime() != null) {
								row.createCell(4).setCellValue(df.format(rsav.getStartTime()));
							}
							if (rsav.getEndTime() != null) {
								row.createCell(5).setCellValue(df.format(rsav.getEndTime()));
							}
							row.createCell(6).setCellValue(rsav.getTotalDuration());
							row.createCell(7).setCellValue(rsav.getUnlock().equals(0)?"已解锁":"未解锁");
							row.createCell(8).setCellValue(rsav.getState().equals(0)?"未完成":"已完成");
							row.createCell(9).setCellValue(rsav.getAccountState().equals(0)?"禁用":"启用");
							row.createCell(10).setCellValue(rsav.getOrgName());
						}
					}
				}
			}
			String fileName = sheetName + df2.format(submitTime) + ".xlsx";
			String requestPath = FileConstant.SAVE_PATH;
			File fileDir = new File(requestPath);
			if (!fileDir.exists()) {
				fileDir.mkdir();
			}
			String path = requestPath +File.separator+ fileName;
			FileOutputStream os = null;
			File file = null;
			try {
				os = new FileOutputStream(path);
				wb.write(os);
				// 阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file = new File(path);
				success(taskContext, "成功", upLoadUrl);
			} catch (Exception e1) {
				e1.printStackTrace();
				fail(taskContext, "写入过程中发生错误");
				logger.error("上传过程中发生错误，请查看日志");
			} finally {
				if (os != null) {
					os.close();
				}
				if (wb != null) {
					wb.close();
				}
				if (file != null) {
					file.delete();
				}
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, sheetName+"导出过程中发生错误，请查看日志"+e.getMessage());
			logger.error(sheetName+"导出过程中发生错误，请查看日志");
		}
		return upLoadUrl;
	}

}
