package com.yizhi.statistics.application.controller.course;

import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.certificate.application.feign.CertificateClient;
import com.yizhi.certificate.application.vo.CertificateNumParam;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.course.application.feign.CourseReportClient;
import com.yizhi.course.application.vo.*;
import com.yizhi.course.application.vo.domain.CourseEntityVo;
import com.yizhi.statistics.application.commands.IStatisticsMetadataTask;
import com.yizhi.statistics.application.controller.report.ReportControllerAbs;
import com.yizhi.statistics.application.controller.report.StudentReportControllerAbs;
import com.yizhi.statistics.application.entity.vo.CourseCalculateTimeVo;
import com.yizhi.statistics.application.mapper.StatisticsCourseRecordeMapper;
import com.yizhi.statistics.application.scope.Scope;
import com.yizhi.statistics.application.scope.Type;
import com.yizhi.statistics.application.service.StatisticsCourseMessageService;
import com.yizhi.statistics.application.service.StatisticsCourseRecordeService;
import com.yizhi.statistics.application.service.StatisticsCourseService;
import com.yizhi.statistics.application.util.CourseUtil;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.statistics.application.work.CourseService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.datasource.DataSourceUtils;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 课程异步
 * 
 * @author wangfeida
 *
 */
@Component
public class CourseControllerAbs extends AbstractDefaultTask<String, Map<String, Object>> implements IStatisticsMetadataTask<CourseEntityVo> {

	
	private Logger logger = LoggerFactory.getLogger(CourseControllerAbs.class);

	final ThreadLocal<SimpleDateFormat> localdf = new ThreadLocal<SimpleDateFormat>();
	
	@Autowired
	private CourseUtil courseUtil;
	
	@Autowired
	private JedisUtil jedisUtil;
	
	@Autowired
	private IdGenerator idGenerator;
	

	@Autowired
	private CourseService courseService;

	@Autowired
	private CourseReportClient courseReportClient;

	@Autowired
	private StatisticsCourseService statisticsCourseService;

	@Autowired
	private StatisticsCourseMessageService statisticsCourseMessageService;

	@Autowired
	private StatisticsCourseRecordeService statisticsCourseRecordeService;

	@Autowired
	private StatisticsCourseRecordeMapper statisticsCourseRecordeMapper;

	@Autowired
	private RedisCache redisCache;
	
	@Autowired
	private ReportControllerAbs reportControllerAbs;
	
    @Autowired
    private StudentReportControllerAbs studentReportControllerAbs;

	@Autowired
    private CertificateClient certificateClient;
	
	//字段分割符
	private final static String FIELD_STERMINATED = "','";		
	//行分割符
	private final static String LINE_STERMINATED = "'\r\n'";
	
	private final static String FENGEFU = ",";  //tab键
	private final static String HUANHANGFU = "\r\n";  //换行
		
	private StringBuilder strBuilder = new StringBuilder();
	private StringBuilder strCourseBuilder = new StringBuilder();
	
	private final static String STATISTICS_COURSE_MESSAGE_COLUMNS = "(id,site_id,course_id,account_id,level,comment,finish_time,type,"
			+ "recorde_time,collect_num)";
	private final static String STATISTICS_COURSE_MESSAGE_EXECUTE_SQL = "load data local infile ''  into table statistics_course_message fields terminated by "
			+ FIELD_STERMINATED + "enclosed by '' escaped by 'N'" + " lines terminated by " + LINE_STERMINATED + " "
			+ STATISTICS_COURSE_MESSAGE_COLUMNS;
	
	private final static String STATISTICS_COURSE_RECORDE_COLUMNS = "(id,course_id,account_id,site_id,learn_count,certificate_count,start_time,end_time,"
			+ "learn_time,recorde_time)";
	private final static String STATISTICS_COURSE_RECORDE_EXECUTE_SQL = "load data local infile ''  into table statistics_course_recorde fields terminated by "
			+ FIELD_STERMINATED + "enclosed by '' escaped by 'N'" + " lines terminated by " + LINE_STERMINATED + " "
			+ STATISTICS_COURSE_RECORDE_COLUMNS;
	
	private final static String STATISTICS_COURSE_COLUMNS = "(id,course_id,course_name,course_code,course_state,course_duration,"
			+ "course_create_time,company_id,site_id,org_id,grant_certificate)";
	private final static String STATISTICS_COURSE_EXECUTE_SQL = "load data local infile ''  into table statistics_course fields terminated by "
			+ FIELD_STERMINATED + "enclosed by '' escaped by 'N'" + " lines terminated by " + LINE_STERMINATED + " "
			+ STATISTICS_COURSE_COLUMNS;
	
	@Autowired
	private DataSource dataSource;
	
	@Override
	public String execute(Map<String, Object> map) {
		// TODO Auto-generated method stub
		Long taskId=idGenerator.generate();
		Long t1=System.currentTimeMillis();
		
		localdf.set(new SimpleDateFormat("yyyy-MM-dd"));
		SimpleDateFormat df = localdf.get();
		/**
		 * 走异步任务
		 */
		Date curDateWork = new Date();
		
		TaskContext taskContext = new TaskContext(taskId, df.format(curDateWork) + " 号0:10在线课程报表分析前一天的数据", 999L, curDateWork);
		working(taskContext);
		Long companyIdParam=null;
		Long siteIdParam=null;;
		Object companyIdObject=map.get("companyId");
		Object siteIdObject=map.get("siteId");
		try {
			if(companyIdObject!=null) {
				companyIdParam=Long.valueOf(companyIdObject.toString());
			}
			if(siteIdObject!=null) {
				siteIdParam=Long.valueOf(siteIdObject.toString());
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			logger.info("companyIdParam:{},转换错误",companyIdParam);
			logger.info("siteIdParam:{},转换错误",siteIdParam);
			e.printStackTrace();
		}
		// 得到所有的课程
		StatisCourse sc = courseReportClient.getStatisCourseMessage(companyIdParam,siteIdParam);
		List<CourseEntityVo> getAllCourse=null;
		Integer courseCount=0;
		if(sc!=null) {
			getAllCourse = sc.getListCourse();
			if(CollectionUtils.isNotEmpty(getAllCourse)) {
				courseCount=getAllCourse.size();
			}
		}
		if(CollectionUtils.isNotEmpty(getAllCourse)){
			taskDetail(taskId, "***** 在线课程统计开始 ,课程数量:" + courseCount +" *****");
			logger.info("***** 在线课程统计开始 ,课程数量:" +courseCount+" *****");
		} else {
			taskDetail(taskId, "***** 在线课程统计开始 ,课程数量获取异常:0或者为空 *****");
			logger.info("***** 在线课程统计开始 ,课程数量获取异常:0或者为空 *****");
		}
		

		Date middleMaxTime = statisticsCourseRecordeMapper.selectMaxDate();
		if(middleMaxTime != null){
			//判断获取记录时间是否大于当前时间，是的话重置该时间（当前时间减一天）
			if(middleMaxTime.compareTo(curDateWork) == 1 || df.format(curDateWork).equals(df.format(middleMaxTime))){
				middleMaxTime = addDay(curDateWork, -1);
			} else {
				middleMaxTime = addDay(middleMaxTime, 1);
			}
		}
		CourseCalculateTimeVo cctv=courseUtil.calculateTime(map,middleMaxTime);
		Date startDate = cctv.getStartDate();
		Date endDate = cctv.getEndDate();
		if(startDate.compareTo(endDate) == 1){
			startDate = cctv.getEndDate();
			endDate = cctv.getStartDate();
		}
		String startDateString=df.format(startDate);
		String endDateString=df.format(endDate);
		
		//删除学习记录
		statisticsCourseRecordeService.deleateStatisticsCourseRecorde(startDate,endDate, null);
		
		//删除完成、收藏、首次完成时间记录
		statisticsCourseMessageService.deleateStatisticsCourseMessage(startDate,endDate, null);
		
		int dealSuccess=0;

		// 如果是28号凌晨0：10开始跑 则curDate是27号的日期
		
		//转换scope对象
		Scope scope=new Scope();
		
		int i=1;
		if (CollectionUtils.isNotEmpty(getAllCourse)) {
			CourseAccountReport orgIdsAndAccountIds = null;
			for (CourseEntityVo course : getAllCourse) {
				Long temp1=System.currentTimeMillis();
				Long siteId = course.getSiteId();
				Long courseId = course.getId();
				Long workId = course.getId();

				Integer workScope = course.getScope(); // 原始数据可见范围 0 游客 1 平台用户 2 指定范围//只能传1和2
				//每个业务都需要将本身的scope进行转换，下边所有的代码全部是switchScope
				Integer switchScope=scope.course(workScope);
				Long companyId = course.getCompanyId();

				// 对这个课程的学习记录开始操作
				if (siteId == null || companyId == null || workId == null || (switchScope==-1)) {
					String dealFaultMesage="课程ID:"+course.getId()+"处理失败,原因:参数不正确";
					taskDetail(taskId, dealFaultMesage);
					logger.info(dealFaultMesage);
					continue;
				}
				if (switchScope == 2) {
					orgIdsAndAccountIds = courseReportClient.listCourseAccountByCourseId(courseId);
					if (orgIdsAndAccountIds==null) {
						String dealSuccessMessage= "课程ID:"+course.getId()+"处理失败,原因:课程是指定范围的，但是可见范围没有找到人的信息";
						taskDetail(taskId,dealSuccessMessage);
						continue;
					}
					if(orgIdsAndAccountIds!=null) {
						//两个并列都为空也执行代码
						if(CollectionUtils.isEmpty(orgIdsAndAccountIds.getOrgIds())
					 &&CollectionUtils.isEmpty(orgIdsAndAccountIds.getAccountIds())) {
							String dealSuccessMessage= "课程ID:"+course.getId()+"处理失败,原因:课程是指定范围的，但是可见范围没有找到人的信息";
							taskDetail(taskId,dealSuccessMessage);
							continue;
						}
					}
				}
				courseService.executeTask(taskId,course, workId, startDate, endDate, Type.COURSE.getType(), companyId, siteId,orgIdsAndAccountIds,this, switchScope);
				dealSuccess++;
				Long temp2=System.currentTimeMillis();
				String me="统计完成第"+(i++)+"门课程,课程总数:"+courseCount+"。课程id："+workId+"。花费时间(ms):"+(temp2-temp1);
				logger.info(me);
			}
		}
		
		//批量添加课程信息
		insertBatchStatisticsCourse();
		
		Long t2=System.currentTimeMillis();
		String result="总数量:"+getAllCourse.size()+"。处理时间ms:"+(t2-t1)+",处理成功:"+dealSuccess+",处理失败:"+(getAllCourse.size()-dealSuccess);
		taskDetail(taskId, result);
		//课程得分
		logger.info("开始查询课程得分");
		List<ScormRecordVO> listTestCase=courseReportClient.getCourseGrade();
		statisticsCourseRecordeService.insertIntoCourseGrade(listTestCase);
		logger.info("课程得分记录入库成功");
		//因为有些课程是当天完成了，但是recorde的duration为空，比如测验 因此插入空记录证明此人当天来学习过
		//mybatiesbu支持这种插入，因此用jdbc操作
		//statisticsCourseRecordeMapper.selectIntoRecorde(startDateString, endDateString);
		Long time1=System.currentTimeMillis();
		String insertTempRecordeResult=statisticsCourseRecordeService.selectIntoRecorde(startDateString, endDateString);
		logger.info(insertTempRecordeResult);
		taskDetail(taskId, insertTempRecordeResult);
		Long time2=System.currentTimeMillis();
		StringBuffer updateOrgIdResult=new StringBuffer("  ");
		try {
			updateOrgIdResult.append(curDateWork+"在线课程和account跑成功。处理orgId成功。");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			updateOrgIdResult.append(curDateWork+"在线课程和account跑成功。处理orgId失败。原因:"+e.getMessage());
		}
		Long time3=System.currentTimeMillis();
		String message="处理插入空记录消耗时间(ms)："+(time2-time1)+"。课程记录处理orgId消耗时间(ms)："+(time3-time2);
		logger.info(message);
		taskDetail(taskId, updateOrgIdResult.toString()+"。。。"+message);
		
		//设置可参加人数缓存
		jedisUtil.setWorkIsSuccess(0, curDateWork);
		
		success(taskContext, result);

        //课程统计结束后执行平台报告统计
	        /**
	         * 每年3月1日、6月1日、9月1日、12月1日生成平台报告可编辑记录
	         */
	    String[] strNow = df.format(curDateWork).split("-");
	    Integer year = Integer.parseInt(strNow[0]);
	    Integer month = Integer.parseInt(strNow[1]);
	    Integer day = Integer.parseInt(strNow[2]);
	        if (month.equals(3)&&day.equals(1)) {
				map.put("type", 0);
	            reportControllerAbs.execute(map, true);
			}else if (month.equals(6)&&day.equals(1)) {
				map.put("type", 1);
	            reportControllerAbs.execute(map, true);
			}else if (month.equals(9)&&day.equals(1)) {
				map.put("type", 2);
	            reportControllerAbs.execute(map, true);
			}else if (month.equals(12)&&day.equals(1)) {
				map.put("type", 3);
	            reportControllerAbs.execute(map, true);
			}
	        /**
	         * 每年1月1日、4月1日、7月1日、10月1日生成平台报告学员数据
	         */
	        if ((month.equals(1)||month.equals(4)||month.equals(7)||month.equals(10))&&day.equals(1)) {
		        studentReportControllerAbs.execute(map, true);
			}
		return "success";
	}

	@Override
	public void executeMetadata(Long taskId,CourseEntityVo metadataObject, Date startDate, Date endDate, Long companId, Long siteId) {
		// TODO Auto-generated method stub
	//	System.out.println(Thread.currentThread().getName() + "   开始执行学习统计" + metadataObject.getName());
		SimpleDateFormat df = localdf.get();
		String startDateString=df.format(startDate);
		String endDateString=df.format(endDate);
		CourseEntityVo course =  metadataObject;
		// 对这个课程的学习记录开始操作
		Long courseId = course.getId();
		Integer type=course.getCourseType();
		if(type==null) {//因为原始数据没有类型，特别坑
			type=0;
		}
		// 处理课程主信息
		String courseDuration = courseReportClient.getCourseDuration(courseId);
		
		//批量添加课程信息，单个更新课程信息
		com.yizhi.statistics.application.entity.StatisticsCourse statisticsCourse = statisticsCourseService.dealCourseMessage(course, courseDuration);
		//判断是否是新课程
		if(statisticsCourse != null){
			switchStreamStatisticsCourse(statisticsCourse);
		}
		
		// 插入这天的学习记录
		List<StatisticsCourseRecorde> getCourseRecordeByCourseId = courseReportClient
				.getCourseRecordeByCourseId(courseId, startDateString,endDateString,type);

		// 进行插入
		Integer stuRecoreCount = 0;
		if (CollectionUtils.isNotEmpty(getCourseRecordeByCourseId)) {
			stuRecoreCount = getCourseRecordeByCourseId.size();
			StatisticsCourseRecorde statisticsCourseRecorde = getCourseRecordeByCourseId.get(0);
			// 从证书服务获取该课程的证书数量
			CertificateNumParam certificateNumParam = new CertificateNumParam();
			certificateNumParam.setAccountId(statisticsCourseRecorde.getAccountId());
			certificateNumParam.setBizdId(statisticsCourseRecorde.getCourseId());
			certificateNumParam.setStartTime(startDateString);
			certificateNumParam.setEndTime(endDateString);
			certificateNumParam.setType(1);
			Integer certificateNum = certificateClient.getCertificateNum(certificateNumParam);
			statisticsCourseRecorde.setCertificateCount(certificateNum);
			//statisticsCourseRecordeService.insertBatch(getCourseRecordeByCourseId);
			
			insertBatchStatisticsCourseRecorde(getCourseRecordeByCourseId);
		}

		// 插入评论/评价/完成时间的信息
		// 调用feign
		List<StatisticsCourseMessage> getCourseMessageByCourseId = courseReportClient
				.getCourseMessageByCourseId(courseId, df.format(startDate),df.format(endDate),siteId);

		if (CollectionUtils.isNotEmpty(getCourseMessageByCourseId)) {
			//statisticsCourseMessageService.insertBatch(getCourseMessageByCourseId);
			insertBatchStatisticsCourseMessage(getCourseMessageByCourseId);
		}
		
		//System.out.println(Thread.currentThread().getName() + "   完成执行学习统计" + metadataObject.getName());
		Integer recoreCount = 0;
		if(CollectionUtils.isNotEmpty(getCourseMessageByCourseId)) {
			recoreCount = getCourseMessageByCourseId.size();
		}
		
		taskDetail(taskId, "课程ID:"+metadataObject.getId()+"处理成功。查询日期:" + startDateString + "-" + endDateString + 
				" course_recorde添加数量:"+ stuRecoreCount+ " course_message添加数量:"+ recoreCount); 
		
		
	}

	
	@Override
	public void exceptionLog(Long taskId,CourseEntityVo metadataObject, Exception e) {
		// TODO Auto-generated method stub
		taskDetail(taskId, "课程ID:"+metadataObject.getId()+"处理失败"+e.getMessage());
	}

	

	protected Date addDay(Date date, int num) {
		Calendar calendar = new GregorianCalendar();
		calendar.setTime(date);
		calendar.add(calendar.DATE, num);
		date = calendar.getTime();
		return date;
	}



	public boolean insertBatchStatisticsCourseRecorde(List<StatisticsCourseRecorde> listStatisticsCourse) {

		Connection conn = null;
		InputStream in = null;
		PreparedStatement preStatement = null;
		com.mysql.jdbc.PreparedStatement mysqlPreStatement = null;
		try{
			// sql编译
			conn = DataSourceUtils.getConnection(dataSource);
			preStatement = conn.prepareStatement(STATISTICS_COURSE_RECORDE_EXECUTE_SQL);
			mysqlPreStatement = preStatement.unwrap(com.mysql.jdbc.PreparedStatement.class);
			in = switchStreamStatisticsCourseRecorde(listStatisticsCourse);
			mysqlPreStatement.setLocalInfileInputStream(in);
			// 执行
			preStatement.executeQuery();
		} catch(Exception e){
			e.printStackTrace();
			return false;
		} finally {
			//大对象手动显示的设置为空保证GC时被释放
			if(in != null){
				in = null;
			}
			
			try{
				//关闭数据库连接
				if (mysqlPreStatement != null) {
					mysqlPreStatement.close();
				}
				if (preStatement != null) {
					preStatement.close();
				}
				if (conn != null) {
					DataSourceUtils.releaseConnection(conn, dataSource);
				}
			} catch(Exception e){
				e.printStackTrace();
				return false;
			}
		}
		 
		return true;
	}
	
	public InputStream switchStreamStatisticsCourseRecorde(List<StatisticsCourseRecorde> list) {
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		for (StatisticsCourseRecorde sc : list) {
			strBuilder.append(idGenerator.generate());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getCourseId());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getAccountId());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getSiteId());		
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getLearnCount());		
			strBuilder.append(FENGEFU);

			Integer certificateCount = sc.getCertificateCount();
			if(certificateCount != null){
				strBuilder.append(certificateCount);
			}else{
				strBuilder.append(0);
			}
			strBuilder.append(FENGEFU);

						
			if (sc.getStartTime() != null) {
				strBuilder.append(simpleDateFormat.format(sc.getStartTime()));
			}
			strBuilder.append(FENGEFU);
			
			if (sc.getEndTime() != null) {
				strBuilder.append(simpleDateFormat.format(sc.getEndTime()));
			}
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getLearnTime());		
			strBuilder.append(FENGEFU);
			
			if (sc.getRecordeTime() != null) {
				strBuilder.append(simpleDateFormat.format(sc.getRecordeTime()));
			}
			strBuilder.append(FENGEFU);
			
			strBuilder.append(HUANHANGFU);

		}
		byte[] bytes = strBuilder.toString().getBytes();
		InputStream in = new ByteArrayInputStream(bytes);

		//大对象手动显示的设置为空保证GC时被释放
		bytes = null;
		
		//清空之前的内容
		strBuilder.delete(0, strBuilder.length());
		
		return in;
	}
	
	public boolean insertBatchStatisticsCourseMessage(List<StatisticsCourseMessage> listStatisticsCourse) {

		Connection conn = null;
		InputStream in = null;
		PreparedStatement preStatement = null;
		com.mysql.jdbc.PreparedStatement mysqlPreStatement = null;
		try{
			// sql编译
			conn = DataSourceUtils.getConnection(dataSource);
			preStatement = conn.prepareStatement(STATISTICS_COURSE_MESSAGE_EXECUTE_SQL);
			mysqlPreStatement = preStatement.unwrap(com.mysql.jdbc.PreparedStatement.class);
			in = switchStreamStatisticsCourseMessage(listStatisticsCourse);
			mysqlPreStatement.setLocalInfileInputStream(in);
			// 执行
			preStatement.executeQuery();
		} catch(Exception e){
			e.printStackTrace();
			return false;
		} finally {
			//大对象手动显示的设置为空保证GC时被释放
			if(in != null){
				in = null;
			}
			
			try{
				//关闭数据库连接
				if (mysqlPreStatement != null) {
					mysqlPreStatement.close();
				}
				if (preStatement != null) {
					preStatement.close();
				}
				if (conn != null) {
					DataSourceUtils.releaseConnection(conn, dataSource);
				}
			} catch(Exception e){
				e.printStackTrace();
				return false;
			}
		}
		 
		return true;
	}
	
	public InputStream switchStreamStatisticsCourseMessage(List<StatisticsCourseMessage> list) {
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		for (StatisticsCourseMessage sc : list) {
			strBuilder.append(idGenerator.generate());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getSiteId());		
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getCourseId());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getAccountId());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getLevel());
			strBuilder.append(FENGEFU);
			 						
			strBuilder.append(sc.getComment());
			strBuilder.append(FENGEFU);
			
			if (sc.getFinishTime() != null) {
				strBuilder.append(simpleDateFormat.format(sc.getFinishTime()));
			}
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getType());
			strBuilder.append(FENGEFU);
			
			if (sc.getRecordeTime() != null) {
				strBuilder.append(simpleDateFormat.format(sc.getRecordeTime()));
			}
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getCollectNum());		
			strBuilder.append(FENGEFU);
			
			strBuilder.append(HUANHANGFU);

		}
		byte[] bytes = strBuilder.toString().getBytes();
		InputStream in = new ByteArrayInputStream(bytes);

		//大对象手动显示的设置为空保证GC时被释放
		bytes = null;
		
		//清空之前的内容
		strBuilder.delete(0, strBuilder.length());
		
		return in;
	}
	
	
	public void switchStreamStatisticsCourse(com.yizhi.statistics.application.entity.StatisticsCourse statisticsCourse) {
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		strCourseBuilder.append(statisticsCourse.getId());
		strCourseBuilder.append(FENGEFU);
		
		strCourseBuilder.append(statisticsCourse.getCourseId());
		strCourseBuilder.append(FENGEFU);
		
		strCourseBuilder.append(statisticsCourse.getCourseName());
		strCourseBuilder.append(FENGEFU);
		
		strCourseBuilder.append(statisticsCourse.getCourseCode());
		strCourseBuilder.append(FENGEFU);
		
		strCourseBuilder.append(statisticsCourse.getCourseState());
		strCourseBuilder.append(FENGEFU);
		
		strCourseBuilder.append(statisticsCourse.getCourseDuration());
		strCourseBuilder.append(FENGEFU);
		
		if (statisticsCourse.getCourseCreateTime() != null) {
			strCourseBuilder.append(simpleDateFormat.format(statisticsCourse.getCourseCreateTime()));
		}
		strCourseBuilder.append(FENGEFU);
		
		strCourseBuilder.append(statisticsCourse.getCompanyId());
		strCourseBuilder.append(FENGEFU);
		
		strCourseBuilder.append(statisticsCourse.getSiteId());
		strCourseBuilder.append(FENGEFU);
		
		strCourseBuilder.append(statisticsCourse.getOrgId());
		strCourseBuilder.append(FENGEFU);

		Boolean grantCertificate = statisticsCourse.getGrantCertificate();
		if(grantCertificate == null || !grantCertificate.booleanValue()){
			strCourseBuilder.append(0);
		}else {
			strCourseBuilder.append(1);
		}
		strCourseBuilder.append(FENGEFU);
		
		strCourseBuilder.append(HUANHANGFU);
	}
	
	
	public boolean insertBatchStatisticsCourse() {

		Connection conn = null;
		InputStream in = null;
		PreparedStatement preStatement = null;
		com.mysql.jdbc.PreparedStatement mysqlPreStatement = null;
		try{
			// sql编译
			conn = DataSourceUtils.getConnection(dataSource);
			preStatement = conn.prepareStatement(STATISTICS_COURSE_EXECUTE_SQL);
			mysqlPreStatement = preStatement.unwrap(com.mysql.jdbc.PreparedStatement.class);
			byte[] bytes = strCourseBuilder.toString().getBytes();
			in = new ByteArrayInputStream(bytes);

			//大对象手动显示的设置为空保证GC时被释放
			bytes = null;
			
			//清空之前的内容
			strCourseBuilder.delete(0, strCourseBuilder.length());
			
			mysqlPreStatement.setLocalInfileInputStream(in);
			// 执行
			preStatement.executeQuery();
		} catch(Exception e){
			e.printStackTrace();
			return false;
		} finally {
			//大对象手动显示的设置为空保证GC时被释放
			if(in != null){
				in = null;
			}
			
			try{
				//关闭数据库连接
				if (mysqlPreStatement != null) {
					mysqlPreStatement.close();
				}
				if (preStatement != null) {
					preStatement.close();
				}
				if (conn != null) {
					DataSourceUtils.releaseConnection(conn, dataSource);
				}
			} catch(Exception e){
				e.printStackTrace();
				return false;
			}
		}
		 
		return true;
	}
}
