package com.yizhi.statistics.application.controller.course;

import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.entity.StatisticsCourseVisibleRange;
import com.yizhi.statistics.application.mapper.StatisticsAccountRoleMapper;
import com.yizhi.statistics.application.scope.JoinNum;
import com.yizhi.statistics.application.util.JedisUtil;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class ThreadRole extends AbstractDefaultTask<String, Map<String, Object>> {
	
	private Logger logger = LoggerFactory.getLogger(ThreadRole.class);

	@Autowired
	private StatisticsAccountRoleMapper statisticsAccountRoleMapper;

	@Autowired
	private JedisUtil jedisUtil;
	
	@Autowired
	private RedisCache redisCache;
	
	@Override
	protected String execute(Map<String, Object> map) {
		//按企业统计各业务的参加人数
		List<StatisticsCourseVisibleRange> visibleRanges = statisticsAccountRoleMapper.getVisibleRangeAllCompanyIdAndSiteId();
		if(CollectionUtils.isEmpty(visibleRanges)){
			return "error";
		}
		logger.info("------开始统计业务参加人数缓存站点数量:{}------", visibleRanges.size());
		List<JoinNum> listJoinNums = null;
		//获取企业管理员和普通管理员的管辖区人数
		Long sTime = null;
		int count = 0;
		for(StatisticsCourseVisibleRange visibleRange : visibleRanges){
			sTime = System.currentTimeMillis();
			//获取企业下的管理员
			List<Long> adminIds = statisticsAccountRoleMapper.getAdminAccountIdBySiteId(visibleRange.getSiteId());
			if(CollectionUtils.isNotEmpty(adminIds)){
				//获取企业管理员的管辖区人数
				listJoinNums = statisticsAccountRoleMapper.isAdminAllPlatformNew(visibleRange.getCompanyId(),visibleRange.getSiteId());
				if(CollectionUtils.isNotEmpty(listJoinNums)){
					for(Long adminId : adminIds){
						for (JoinNum joinNum : listJoinNums) {
							Long workId=joinNum.getWorkId();
							Long askNum=joinNum.getAskNum();
							Long siteId=joinNum.getAdminSiteId();
							jedisUtil.setAskNumByAdminIdAndWorkId(adminId, siteId, workId, askNum);
						}
					}
				}
			}
			
			//获取企业普通管理员的管辖区人数
			listJoinNums = statisticsAccountRoleMapper.notAdminLimitPlatformNew(visibleRange.getCompanyId(),visibleRange.getSiteId());
			if(CollectionUtils.isNotEmpty(listJoinNums)) {
				for (JoinNum joinNum : listJoinNums) {
					Long adminId=joinNum.getAdminId();
					Long workId=joinNum.getWorkId();
					Long askNum=joinNum.getAskNum();
					Long siteId=joinNum.getAdminSiteId();
					jedisUtil.setAskNumByAdminIdAndWorkId(adminId, siteId, workId, askNum);
				}
			}
			
			logger.info("已完成第{}个站点:{},(总数:{}),管理员管辖人数统计耗时(ms):{}", ++count, 
					visibleRange.getSiteId(), visibleRanges.size(), System.currentTimeMillis() - sTime);
		}
		
		logger.info("------结束统计业务参加人数缓存耗时(ms):{}------", System.currentTimeMillis() - sTime);
		return "success:";
	}

	@Deprecated
	protected String execute_bak(Map<String, Object> map) {
		// TODO Auto-generated method stub
		Integer currentThread=(Integer) map.get("threadCurrent");//当前是几号线程
		List<Long> workIds=(List<Long>) map.get("threadListDateSublist");//当前线程需要跑的数据
		Integer pageSize=(Integer) map.get("threadByOneWorkNum");//当前线程每次跑多少个;
		Integer threadNum=(Integer) map.get("threadNum");
		Long t1=System.currentTimeMillis();
		if(CollectionUtils.isNotEmpty(workIds)) {
			Integer dateNum=workIds.size();
			Integer workIdsNum=dateNum/pageSize;
			Integer remainder=dateNum%pageSize;
			//如果求余>0  循环次数+1
			if(remainder>0) {
				workIdsNum=workIdsNum+1;
			}
			for (int i = 1; i <= workIdsNum; i++) {
				Integer start=(i-1)*pageSize;
				Integer end=i*pageSize;
				if(i==workIdsNum) {
					//走的是下标，但是不包含end，因此end不用减1，包含前下标，不包含后下标
					end=workIds.size();
				}
				List<Long> workIdsLimit=new ArrayList<Long>(workIds.subList(start, end));
				System.out.println("正在运行线程"+currentThread);
				dealRoleRedisMethod(workIdsLimit);
			}
		}
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		Long t3=System.currentTimeMillis();
		System.out.println(currentThread+"本线程运行时间"+(t3-t1));
		String key="statisticsTree:role:"+sdf.format(new Date())+":"+currentThread+":号线程。"+threadNum;
		boolean roleThreadSuccess=redisCache.set(key, "success");
		if(roleThreadSuccess) {
			redisCache.expire(key, 60*60*26);
		}
		return "success:"+currentThread;
	}
	
	
	@Deprecated
	void dealRoleRedisMethod(List<Long> workIdsLimit) {
		List<JoinNum> listJoinNum=new ArrayList<JoinNum>();
		//循环业务
		List<JoinNum> isAdminAllPlatform=statisticsAccountRoleMapper.isAdminAllPlatform(workIdsLimit);
		if(CollectionUtils.isNotEmpty(isAdminAllPlatform)) {
			listJoinNum.addAll(isAdminAllPlatform);
		}
		
		List<JoinNum> notAdminLimitPlatform=statisticsAccountRoleMapper.notAdminLimitPlatform(workIdsLimit);
		if(CollectionUtils.isNotEmpty(notAdminLimitPlatform)) {
			listJoinNum.addAll(notAdminLimitPlatform);
		}
		if(CollectionUtils.isNotEmpty(listJoinNum)) {
			for (JoinNum joinNum : listJoinNum) {
				Long adminId=joinNum.getAdminId();
				Long workId=joinNum.getWorkId();
				Long askNum=joinNum.getAskNum();
				Long siteId=joinNum.getAdminSiteId();
				jedisUtil.setAskNumByAdminIdAndWorkId(adminId, siteId, workId, askNum);
			}
		}
	}
}
