package com.yizhi.statistics.application.controller.dashboard;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.course.application.feign.CourseDashboardClient;
import com.yizhi.course.application.feign.CourseReportClient;
import com.yizhi.course.application.vo.CourseDashboardParam;
import com.yizhi.course.application.vo.CourseDashboardVO;
import com.yizhi.exam.application.feign.ExamApiClient;
import com.yizhi.exam.application.feign.StatisticsExamMetadataClient;
import com.yizhi.exam.application.vo.dashboard.ExamDashboardQueryVO;
import com.yizhi.statistics.application.entity.AccountDashboard;
import com.yizhi.statistics.application.service.IAccountDashboardService;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.system.remote.SiteClient;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;

import cn.hutool.core.date.DateUtil;

/**
 * 跑批任务
 */
@Component
public class AccountDashboardAbs extends AbstractDefaultTask<String, Map<String, Object>> {

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IAccountDashboardService accountDashboardService;
    @Autowired
    private SiteClient siteClient;
    @Autowired
    private ReportClient reportClient;
    @Autowired
    private CourseDashboardClient courseDashboardClient;
    @Autowired
    private CourseReportClient courseReportClient;
    @Autowired
    private StatisticsExamMetadataClient examClient;
    @Autowired
    private ExamApiClient examApiClient;

    private final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");
    private final SimpleDateFormat sim1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    private static final Logger LOGGER = LoggerFactory.getLogger(AccountDashboardAbs.class);


    @Override
    protected String execute(Map<String, Object> map) {
        Long taskId = idGenerator.generate();
        Long taskStartTime = System.currentTimeMillis();
        /**
         * 走异步任务
         */
        Date curDateWork = new Date();
        String taskTitle = "dashboard按用户统计";
        TaskContext taskContext = new TaskContext(taskId, sim.format(curDateWork) + " 号0:10 " + taskTitle + "报表分析前一天的数据", 999L, curDateWork);
        working(taskContext);

        String result = "";
        String yesterdayString = (String) map.get("yesterday");
        Date yesterday = null;
        try {
            yesterday = sim1.parse(yesterdayString);
        } catch (ParseException e) {
            e.printStackTrace();
            result = "***** " + taskTitle + "时间转换错误*****";
            taskDetail(taskId, result);
            LOGGER.info(result);
        }
        Date startDate = DateUtil.beginOfDay(yesterday);
        Date endDate = DateUtil.endOfDay(yesterday);


        //先得清空accountDashboard表数据
        try {
            accountDashboardService.deleteAllData(startDate, endDate);
            result = "***** " + taskTitle + "报表数据开始前先删除数据*****";
            taskDetail(taskId, result);
            LOGGER.info(result);
        } catch (Exception e) {
            result = "***** " + taskTitle + "报表前删除数据，出现异常*****";
            taskDetail(taskId, result);
            LOGGER.info(result);
            e.printStackTrace();
        }

        List<Long> allSiteId = getAllSiteId();
        
        if (CollectionUtils.isEmpty(allSiteId)){
            result = "*****" + taskTitle + "查询站点为空*****";
            taskDetail(taskId, result);
            LOGGER.info(result);
            return null;
        }
        ReportRangeAccountReqVO tem = null;
        List<ReportAccountRespVO> accounts = null;
        tem = new ReportRangeAccountReqVO();
        
        CourseDashboardParam param = new CourseDashboardParam();
        param.setStartDate(startDate);
        param.setEndDate(endDate);
        
        ExamDashboardQueryVO examParam = new ExamDashboardQueryVO();
        examParam.setStartDate(startDate);
        examParam.setEndDate(endDate);
        
        List<AccountDashboard> accountDashboards = null;
        Integer num = 0;
        for (Long siteId : allSiteId) {
            param.setSiteId(siteId);
            examParam.setSiteId(siteId);
            
            //根据站点获取课程的完成数量、课程素材总时长
            Map<Long, CourseDashboardVO> courseDashboardDataMap = courseDashboardClient.getCourseDashboardData(param);
            //根据站点获取课程累计学习时长
            Map<Long, Long> courseStudyTimeMap = courseDashboardClient.getCourseStudyTime(param);
            // 根据站点获取考试累计时长 by userId
            Map<Long, Integer> examDashboardDataMap = examApiClient.getExamDashboardData(examParam);

            tem.setSiteId(siteId);
            // 根据站点id获得用户
            accounts = reportClient.getRangeAccounts(tem);

            if (CollectionUtils.isNotEmpty(accounts)) {
                accountDashboards = new ArrayList<>(accounts.size());
                result = "***** " + taskTitle + " 开始插入：" + siteId + " 数据*****";
                taskDetail(taskId, result);
                LOGGER.info(result);
                for (ReportAccountRespVO a : accounts) {
                    AccountDashboard accountDashboard = buildRecords(a, siteId, courseDashboardDataMap
                    		, courseStudyTimeMap, examDashboardDataMap, yesterday);
                    if (accountDashboard != null) {
                        num++;
                        accountDashboards.add(accountDashboard);
                    }
                }
                try {
                    if (CollectionUtils.isNotEmpty(accountDashboards)) {
                        accountDashboardService.insertBatch(accountDashboards);
                    }
                } catch (Exception e) {
                    result = "***** \" + taskTitle + \" 插入：" + siteId + " 数据出现异常*****";
                    taskDetail(taskId, result);
                    LOGGER.info(result);
                    e.printStackTrace();
                    continue;
                }
            }
        }
        Long taskEndTime = System.currentTimeMillis();
        Long t = (taskEndTime - taskStartTime)/1000;
        success(taskContext, "执行成功,耗时:" + t + "s", "");
        taskDetail(taskId, "总数量:" + num + "。处理时间s:" + t);
        LOGGER.info("***** 数据"+yesterdayString+"入库结束 ,数量:" + num + "。处理时间s:" + t + " *****");
        return "success";

    }
    
    /**
     * 获取要处理的业务的所有siteId
     * @return
     */
    private List<Long> getAllSiteId(){
    	Set<Long> allSiteId = new HashSet<Long>();
        //先获取课程所有的站点id
        List<Long> allCourseSiteId = courseReportClient.getAllSiteId();
        
        // 获取考试所有的站点id
        List<Long> allExamSiteId = examClient.getAllExamSiteIds();
        
        allSiteId.addAll(allCourseSiteId);
        allSiteId.addAll(allExamSiteId);
        return new ArrayList<Long>(allSiteId);
    }

    /**
     * 组建主体信息
     *
     * @param account
     * @param siteId
     * @param courseDashboardDataMap 课程的完成数量、课程素材总时长
     * @param courseStudyTimeMap     课程累计学习时长
     * @return
     */
    public AccountDashboard buildRecords(ReportAccountRespVO account, Long siteId,
                                         Map<Long, CourseDashboardVO> courseDashboardDataMap,
                                         Map<Long, Long> courseStudyTimeMap
                                         , Map<Long, Integer> examDurationMap
                                         , Date date) {
        Boolean canSave = true;
        AccountDashboard a = new AccountDashboard();
        a.setId(idGenerator.generate());
        a.setAccountId(account.getUserId());
        a.setSiteId(siteId);
        a.setCompanyId(account.getCompanyId());
        a.setStudyRecordTime(date);
        a.setCourseStudyTime(0L);
        a.setCourseFinishNum(0);
        a.setCourseTime(0L);
        a.setExamDurationSecond(0);
        if (courseDashboardDataMap != null) {
            CourseDashboardVO vo = courseDashboardDataMap.get(account.getUserId());
            if (vo != null) {
                a.setCourseFinishNum(vo.getCourseFinishNum() == null ? 0 : vo.getCourseFinishNum());
                a.setCourseTime(vo.getCourseTime() == null ? 0L : vo.getCourseTime());
            }
        }
        if (courseStudyTimeMap != null) {
            Long studyTime = courseStudyTimeMap.get(account.getUserId());
            a.setCourseStudyTime(studyTime == null ? 0L : studyTime);
        }
        if(null != examDurationMap) {
        	Integer examDuration = examDurationMap.get(account.getUserId());
        	a.setExamDurationSecond(examDuration == null ? 0 : examDuration);
        }
        //如果课程的完成数量、课程素材总时长、课程累计学习时长只要有一个不为0就可以插入
        if (a.getCourseTime().equals(0L) && a.getCourseFinishNum().equals(0) 
        		&& a.getCourseStudyTime().equals(0L) && a.getExamDurationSecond().equals(0)) {
            canSave = false;
        } else {
            canSave = true;
        }
        if (canSave) {
            return a;
        }
        return null;
    }
}
