package com.yizhi.statistics.application.controller.exam;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.statistics.application.controller.exam.export.*;
import com.yizhi.statistics.application.exam.domain.po.ExamGroupPO;
import com.yizhi.statistics.application.exam.domain.po.UserExamGroupPO;
import com.yizhi.statistics.application.exam.domain.vo.ExamGroupOrgVO;
import com.yizhi.statistics.application.exam.domain.vo.UserExamByOrgVO;
import com.yizhi.statistics.application.feign.StatisticsExamClient;
import com.yizhi.statistics.application.refactor.exam.*;
import com.yizhi.statistics.application.service.StatisticsAccountService;
import com.yizhi.statistics.application.service.StatisticsExamService;
import com.yizhi.statistics.application.vo.report.AccountVO;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@RestController()
@RequestMapping("/statisticsExam")
public class ExamController implements StatisticsExamClient {


    @Autowired
    private ExamControllerAbs examControllerAbs;
    @Autowired
    StatisticsAccountService statisticsAccountService;

    @Autowired
    private StatisticsExamService statisticsExamService;

    @Autowired
    private ExamGroupExport examGroupExport;

    @Autowired
    private ExamAccountTrExamsExport examAccountTrExamsExport;

    @Autowired
    private AccountGroupExport accountGroupExport;

    @Autowired
    private AccountGroupViewExport accountGroupViewExport;
    @Autowired
    private ExamGroupOrgExport examGroupOrgExport;
    @Autowired
    private ExamByGroupOrgUserExport examByGroupOrgUserExport;
    @Autowired
    private ExamByOrgUserDetailExport examByOrgUserDetailExport;

    /**
     * 考试报表定时异步任务
     *
     * @param startDate
     * @param endDate
     */
    @Override
    @GetMapping("/asynchronous/load/date/old")
    public void AsynchronousExam(String startDate, String endDate) {
        Map<String, Object> map = new HashMap<String, Object>();
        if (StringUtils.isNotBlank(startDate)) {
            map.put("startDate", startDate);
        }
        if (StringUtils.isNotBlank(endDate)) {
            map.put("endDate", endDate);
        }
        examControllerAbs.execute(map, true);
    }


    @Override
    @GetMapping("/group")
    public Page<ReportExamVO> getExamGroup(
            @RequestBody ReportExamParam reportExamParam) {

        RequestContext res = ContextHolder.get();
        reportExamParam.setAdmin(res.isAdmin());
        reportExamParam.setAccountId(res.getAccountId());
        return statisticsExamService.getExamGroup(reportExamParam);

    }

    @Override
    @GetMapping("/group/org")
    @ApiOperation(value = "考试按部门统计", response = ExamGroupOrgVO.class)
    public Page<ExamGroupOrgVO> getExamGroupByOrg(@RequestBody ExamGroupPO po) {
        RequestContext requestContext = ContextHolder.get();
        po.setAdmin(requestContext.isAdmin());
        po.setCompanyId(requestContext.getCompanyId());
        po.setOrgId(requestContext.getOrgId());
        po.setSiteId(requestContext.getSiteId());
        po.setAccountId(requestContext.getAccountId());
        return statisticsExamService.getExamGroupByOrg(po);
    }

    @Override
    @GetMapping("/group/org/download")
    @ApiOperation(value = "考试按部门统计下载")
    public String getExamGroupByOrgDownload(@RequestBody ExamGroupPO po) {

        RequestContext requestContext = ContextHolder.get();
        po.setAdmin(requestContext.isAdmin());
        po.setCompanyId(requestContext.getCompanyId());
        po.setOrgId(requestContext.getOrgId());
        po.setSiteId(requestContext.getSiteId());
        po.setAccountId(requestContext.getAccountId());
        AccountVO vo = statisticsAccountService.getOrgInfoByOrgId(
                po.getCompanyId(),
                po.getSiteId(),
                po.getOrgId()
        );
        po.setOrgName("");
        if (vo != null) {
            po.setOrgName(vo.getOrgName());
        }

        Date date = new Date();
        Long taskId = date.getTime();
        String serialNo = "EXAM-ORG-EXPORT-" + taskId.toString();
        String taskName = "按部门统计-报表管理-考试报表" + taskId.toString();

        Map<String, Object> map = new HashMap<>();
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("reportExamParam", po);
        map.put("accountId", po.getAccountId());
        map.put("isAdmin", po.isAdmin());
        examGroupOrgExport.execute(map,true);
        return serialNo;
    }

    @Override
    @GetMapping("/group/org/users")
    public Page<UserExamByOrgVO> getExamByGroupOrgUser(@RequestBody UserExamGroupPO po) {
        RequestContext requestContext = ContextHolder.get();
        po.setAdmin(requestContext.isAdmin());
        po.setCompanyId(requestContext.getCompanyId());
        po.setSiteId(requestContext.getSiteId());
        po.setAccountId(requestContext.getAccountId());
        return statisticsExamService.getUserExam(po);
    }

    @Override
    @GetMapping("/group/org/users/download")
    public String getExamByGroupOrgUserDownload(@RequestBody UserExamGroupPO po) {
        RequestContext requestContext = ContextHolder.get();
        po.setAdmin(requestContext.isAdmin());
        po.setCompanyId(requestContext.getCompanyId());
        po.setOrgId(po.getOrgId());
        po.setSiteId(requestContext.getSiteId());
        po.setAccountId(requestContext.getAccountId());

        AccountVO vo = statisticsAccountService.getOrgInfoByOrgId(
                po.getCompanyId(),
                po.getSiteId(),
                po.getOrgId()
        );
        po.setOrgName("");
        if (vo != null) {
            po.setOrgName(vo.getOrgName());
        }
        Date date = new Date();
        Long taskId = date.getTime();
        String serialNo = "EXAM-ORG-USER-EXPORT-" + taskId.toString();
        String taskName = "部门名称-报表系统-考试统计-按部门统计-下载结果" + taskId.toString();

        Map<String, Object> map = new HashMap<>();
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("reportExamParam", po);
        map.put("accountId", po.getAccountId());
        map.put("isAdmin", po.isAdmin());

        examByGroupOrgUserExport.execute(map,true);
        return serialNo;
    }

    @Override
    @GetMapping("/group/org/users/detail/download")
    public String getExamByGroupOrgUserDetailDownload(@RequestBody UserExamGroupPO po) {
        RequestContext requestContext = ContextHolder.get();
        po.setAdmin(requestContext.isAdmin());
        po.setCompanyId(requestContext.getCompanyId());
        po.setOrgId(po.getOrgId());
        po.setSiteId(requestContext.getSiteId());
        po.setAccountId(requestContext.getAccountId());
        AccountVO vo = statisticsAccountService.getOrgInfoByOrgId(
                po.getCompanyId(),
                po.getSiteId(),
                po.getOrgId()
        );
        po.setOrgName("");
        if (vo != null) {
            po.setOrgName(vo.getOrgName());
        }

        Date date = new Date();
        Long taskId = date.getTime();
        String serialNo = "EXAM-ORG-USER-DETAIL-EXPORT-" + taskId.toString();
        String taskName = "部门名称用户考试明细-报表系统-考试统计-按部门统计" + taskId.toString();

        Map<String, Object> map = new HashMap<>();
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("reportExamParam", po);
        map.put("accountId", po.getAccountId());
        map.put("isAdmin", po.isAdmin());
        examByOrgUserDetailExport.execute(map,true);
        return serialNo;
    }

    @Override
    @GetMapping("/group/download")
    public String getExamGroupDownload(
            @RequestBody ReportExamParam reportExamParam) {
        Date date = new Date();
        Long taskId = date.getTime();
        String serialNo = "EXAM-GROUP-EXPORT-" + taskId.toString();
        String taskName = "按考试统计-报表管理-考试报表" + taskId.toString();
        RequestContext requestContext = ContextHolder.get();
        reportExamParam.setCompanyId(requestContext.getCompanyId());
        reportExamParam.setSiteId(requestContext.getSiteId());
        reportExamParam.setOrgIds(requestContext.getOrgIds());
        Map<String, Object> map = new HashMap<>();
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("reportExamParam", reportExamParam);
        map.put("accountId", requestContext.getAccountId());
        map.put("isAdmin", requestContext.isAdmin());
        examGroupExport.execute(map, true);
        return serialNo;
    }

    @Override
    @GetMapping("/group/account")
    public Page<ReportExamAccountsViewVo> getExamAccountView(@RequestBody ExamCountByUserVO vo) {

        return statisticsExamService.getExamAndAccountGroup(vo);

    }

    @Override
    @GetMapping("/group/account/trExams/download")
    public String getExamAccountTrExamsDownload(@RequestBody ExamCountByUserVO vo) {
        Date date = new Date();
        Long taskId = date.getTime();
        String serialNo = "EXAMGROUP-ACCOUNT-EXPORT-" + taskId.toString();
        String taskName = "按考试统计-查看-按用户统计-考试结果-" + taskId.toString();

        Map<String, Object> map = new HashMap<>();
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("vo", vo);
        map.put("accountId", ContextHolder.get().getAccountId());
        map.put("siteId", ContextHolder.get().getSiteId());
        map.put("companyId", ContextHolder.get().getCompanyId());
        map.put("isAdmin", ContextHolder.get().isAdmin());
        examAccountTrExamsExport.execute(map, true);
        return serialNo;
    }

    @Override
    @GetMapping("/account/group")
    public Page<ReportExamAccountVO> getAccountGroup(@RequestBody ExamCountByUserVO vo) {

        RequestContext res = ContextHolder.get();
        vo.setAdmin(res.isAdmin());
        vo.setAccountId(res.getAccountId());
        return statisticsExamService.getAccountGroup(vo);
    }

    @Override
    @GetMapping("/account/group/download")
    public String getAccountGroupDownload(@RequestBody ExamCountByUserVO vo) {
        Date date = new Date();
        Long taskId = date.getTime();
        String serialNo = "EXAM-ACCOUNTGROUP-EXPORT-" + taskId.toString();
        String taskName = "按用户统计下载-" + taskId.toString();
        vo.setOrgIds(ContextHolder.get().getOrgIds());
        Map<String, Object> map = new HashMap<>();
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("vo", vo);
        map.put("accountId", ContextHolder.get().getAccountId());
        map.put("siteId", ContextHolder.get().getSiteId());
        map.put("companyId", ContextHolder.get().getCompanyId());
        map.put("isAdmin", ContextHolder.get().isAdmin());
        accountGroupExport.execute(map, true);
        return serialNo;
    }

    @Override
    @GetMapping("/account/group/view")
    public Page<ReportExamAccountViewVo> getAccountGroupView(
            @RequestParam(name = "startTime", required = false) String startTime,
            @RequestParam(name = "endTime", required = false) String endTime,
            @RequestParam(name = "accountId", required = true) Long accountId,
            @RequestParam(name = "examNameLike", required = false) String examNameLike,
            @RequestParam(name = "pageSize", required = false) Integer pageSize,
            @RequestParam(name = "pageNo", required = false) Integer pageNo) {
        if (StringUtils.isNotBlank(examNameLike)) {
            examNameLike = examNameLike.trim();
        }

        return statisticsExamService.getAccountGroupView(null, null, accountId, startTime, endTime, examNameLike, pageNo, pageSize);

    }

    @Override
    @GetMapping("/account/group/view/download")
    public String getAccountGroupViewDownload
            (
                    @ApiParam(value = "开始时间", required = false) @RequestParam(name = "startTime", required = false) String startTime,
                    @ApiParam(value = "结束时间", required = false) @RequestParam(name = "endTime", required = false) String endTime,
                    @ApiParam(value = "考试关键字", required = false) @RequestParam(name = "examNameLike", required = false) String examNameLike,
                    @ApiParam(value = "学员id", required = true) @RequestParam(name = "accountId", required = true) Long accountId) {
        Date date = new Date();
        Long taskId = date.getTime();
        String serialNo = "EXAM-ACCOUNTGROUP-VIEW-EXPORT-" + taskId.toString();
        String taskName = "按用户统计-查看下载报表-" + taskId.toString();
        Map<String, Object> map = new HashMap<>();
        map.put("taskId", taskId);
        map.put("serialNo", serialNo);
        map.put("taskName", taskName);
        map.put("startTime", startTime);
        map.put("endTime", endTime);
        map.put("accountId", accountId);
        map.put("ACCOUNTID", ContextHolder.get().getAccountId());
        map.put("siteId", ContextHolder.get().getSiteId());
        map.put("companyId", ContextHolder.get().getCompanyId());
        if (StringUtils.isNotBlank(examNameLike)) {
            map.put("examNameLike", examNameLike.trim());
        }

        accountGroupViewExport.execute(map, true);
        return serialNo;
    }

    @Override
    @GetMapping("/examView")
    public StatisticsExamVO getExam(
            @RequestParam(name = "examId", required = true) Long examId){
        return statisticsExamService.getExam(examId);
    }


}
