package com.yizhi.statistics.application.controller.exam;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.course.application.vo.CourseAccountReport;
import com.yizhi.exam.application.feign.ExamAuthorizeClient;
import com.yizhi.exam.application.feign.StatisticsExamMetadataClient;
import com.yizhi.exam.application.vo.domain.Exam;
import com.yizhi.exam.application.vo.domain.StatisticsExamLearnMetadata;
import com.yizhi.exam.application.vo.domain.TrExamAuthorize;
import com.yizhi.statistics.application.commands.IStatisticsMetadataTask;
import com.yizhi.statistics.application.domain.StatisticsExam;
import com.yizhi.statistics.application.domain.StatisticsExamRecorde;
import com.yizhi.statistics.application.entity.vo.ErrorMsg;
import com.yizhi.statistics.application.mapper.StatisticsExamRecordeMapper;
import com.yizhi.statistics.application.scope.Scope;
import com.yizhi.statistics.application.scope.Type;
import com.yizhi.statistics.application.service.StatisticsExamRecordeService;
import com.yizhi.statistics.application.service.StatisticsExamService;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.statistics.application.work.CourseService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;

/**
 * @author dingxiaowei
 * @date 2019/4/16
 * @auther ding
 */
@Component
public class ExamControllerAbs extends AbstractDefaultTask<String, Map<String, Object>> implements IStatisticsMetadataTask<Exam> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExamControllerAbs.class);

    final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");


    @Autowired
    private CourseService courseService;

    @Autowired
    private StatisticsExamMetadataClient statisticsExamMetadataClient;

    @Autowired
    private StatisticsExamRecordeMapper statisticsExamRecordeMapper;

    @Autowired
    private StatisticsExamService statisticsExamService;

    @Autowired
    private StatisticsExamRecordeService statisticsExamRecordeService;

    @Autowired
    private ExamAuthorizeClient examAuthorizeClient;

    @Autowired
    private JedisUtil jedisUtil;

    @Autowired
    private IdGenerator idGenerator;


    @Override
    public String execute(Map<String, Object> map) {
        Long taskId = idGenerator.generate();
        Long t1 = System.currentTimeMillis();
        /**
         * 走异步任务
         */
        Date curDateWork = new Date();

        TaskContext taskContext = new TaskContext(taskId, sim.format(curDateWork) + " 考试报表分析前一天的数据", 999L, curDateWork);
        working(taskContext);

        String startDate = (String) map.get("startDate");
        String endDate = (String) map.get("endDate");

        SimpleDateFormat formatDate = new SimpleDateFormat("yyyy-MM-dd");

        Date dCurDate;
        boolean isRunDel = true;

        AtomicReference<Integer> dealSuccess = new AtomicReference<>(0);

        if (startDate == null || "".equals(startDate)) {
            //获取最近记录时间
            Date middleMaxTime = statisticsExamRecordeMapper.selectMaxDate();
            if (middleMaxTime != null) {
                dCurDate = addDay(middleMaxTime, 1);
                if (dCurDate.getTime() > System.currentTimeMillis()) {
                    dCurDate = new Date();
                }
                startDate = formatDate.format(dCurDate);
                //有记录,删当天记录,防止重复插入数据
                isRunDel = true;
            } else {
                //没有记录,获取所有考试的最早开始时间
                Date recordMinTime = statisticsExamMetadataClient.selectRecordMinTime();
                startDate = formatDate.format(recordMinTime);
                //没有记录,不用删除当天数据
                isRunDel = false;
            }
        }

        if (endDate == null || "".equals(endDate)) {
            dCurDate = addDay(new Date(), -1);
            endDate = formatDate.format(dCurDate);
        }

        Date start = null;
        Date end = null;

        try {
            start = formatDate.parse(startDate);
            end = formatDate.parse(endDate);
        } catch (ParseException e1) {
            return null;
        }

        // 得到所有的考试列表
        List<Exam> examList = statisticsExamMetadataClient.getAllExams("1970-01-01", endDate);
        if (CollectionUtils.isEmpty(examList)) {
            taskDetail(taskId, "***** 考试统计开始 ,考试数量获取异常:0或者为空 *****");
            LOGGER.warn("***** 考试数量获取异常:0或者为空 *****");
            return "success";
        }

        taskDetail(taskId, "***** 考试统计开始 ,考试数量:" + examList.size() + " *****");
        LOGGER.info("***** 考试统计开始 ,考试数量:" + examList.size() + " *****");

        List<ErrorMsg> listError = new ArrayList<ErrorMsg>();
        //转换scope对象
        Scope scope = new Scope();
        boolean finalIsRunDel = isRunDel;
        Date finalStart = start;
        Date finalEnd = end;
        examList.parallelStream().forEach(exam -> {
            LOGGER.info("-----------"+Thread.currentThread().getName());
            Long siteId = exam.getSiteId();
            Long examId = exam.getId();
            //  1 平台用户 2 指定范围
            Integer workScope = exam.getVisibleRange();
            if (null == workScope) {
                workScope = 1;
            }
            //每个业务都需要将本身的scope进行转换，下边所有的代码全部是switchScope
            Integer switchScope = scope.Exam(workScope);
            Long companyId = exam.getCompanyId();
            CourseAccountReport courseAccountReport = new CourseAccountReport();

            // 对该考试考试记录进行插入
            if (siteId == null || companyId == null || examId == null || (switchScope == -1)) {
                LOGGER.info("考试:" + examId + "异常,跳过操作");
                ErrorMsg  error = new ErrorMsg(examId, "异常,跳过操作");
                listError.add(error);
                return;
            }
            if (switchScope == 2) {
                scopHandler(listError, examId, courseAccountReport);
            }

            //需要删除当天记录,避免重复删除
            if (finalIsRunDel) {
                statisticsExamRecordeService.deleteRcordesByDate(examId, formatDate.format(finalStart), formatDate.format(finalEnd));
            }

            try {
                courseService.executeTask(taskId, exam, examId, finalStart, finalEnd, Type.EXAM.getType(), companyId, siteId, courseAccountReport, this, workScope);
                dealSuccess.getAndSet(dealSuccess.get() + 1);
            } catch (Exception e) {
                LOGGER.error("***** 考试报表新增异常 examId:" + examId, e);
                taskDetail(taskId, "可见范围处理过程中出现异常" + examId);
            }
        });

        Long t2 = System.currentTimeMillis();
        Long t = (t2 - t1) / 1000;
        System.out.println("耗时(s)：" + t);
        for (int i = 0; i < listError.size(); i++) {
            LOGGER.warn("执行异常数量为", listError.get(i).toString());
        }
        //设置可参加人数缓存
        jedisUtil.setWorkIsSuccess(1, new Date());
        taskDetail(taskId, "总数量:" + examList.size() + "。处理时间ms:" + (t2 - t1) + ",处理成功:" + dealSuccess + ",处理失败:" + (examList.size() - dealSuccess.get()));
        LOGGER.info("***** 考试统计结束 ,考试数量:" + examList.size() + "。处理时间ms:" + (t2 - t1) + ",处理成功:" + dealSuccess + ",处理失败:" + (examList.size() - dealSuccess.get()) + " *****");

        return "success";
    }

    private boolean scopHandler(List<ErrorMsg> listError, Long examId, CourseAccountReport courseAccountReport) {
        ErrorMsg error;
        List<TrExamAuthorize> trExamAuthorizes = examAuthorizeClient.examCount(examId);
        List<Long> accountIds = new ArrayList<>();
        List<Long> orgIds = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(trExamAuthorizes)) {
            trExamAuthorizes.stream().forEach(trExamAuthorize -> {
                //1组织 2用户
                if (trExamAuthorize.getType() == 1) {
                    orgIds.add(trExamAuthorize.getRelationId());
                } else if (trExamAuthorize.getType() == 2) {
                    accountIds.add(trExamAuthorize.getRelationId());
                }
            });
        }
        courseAccountReport.setCourseId(examId);
        courseAccountReport.setAccountIds(accountIds);
        courseAccountReport.setOrgIds(orgIds);
        // set集合转换为arrayList集合
        if (CollectionUtils.isEmpty(trExamAuthorizes)) {
            LOGGER.info("考试:" + examId + ",是指定人可见，但是原始数据没有查询到可见范围权限");
            error = new ErrorMsg(examId, ",是指定人可见，但是原始数据没有查询到可见范围权限");
            listError.add(error);
            return true;
        }
        return false;
    }

    @Override
    public void executeMetadata(Long taskId, Exam metadataObject, Date startDate, Date endDate, Long companId, Long siteId) {

        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        Exam exam = metadataObject;
        // 对这个考试的考试记录开始操作
        Long examId = exam.getId();

        StatisticsExam statisticsExam = new StatisticsExam();
        statisticsExam.setExamId(exam.getId());
        statisticsExam.setExamNo(exam.getExamNo());
        statisticsExam.setExamName(exam.getName());
        statisticsExam.setExamStartTime(exam.getStartTime());
        statisticsExam.setExamEndTime(exam.getEndTime());
        statisticsExam.setExamCompanyId(exam.getCompanyId());
        statisticsExam.setExamSiteId(exam.getSiteId());
        statisticsExam.setExamOrgId(exam.getOrgId());
        statisticsExam.setExamCreateTime(exam.getCreateTime());
        //或插入或更新
        System.out.println(statisticsExam.toString());
        try {
            StatisticsExam statisticsExam1 = new StatisticsExam();
            statisticsExam1.setExamId(examId);
            List<StatisticsExam> statisticsExamList = null;
            statisticsExamList = statisticsExamService.selectList(new EntityWrapper<>(statisticsExam1));
            if (CollectionUtils.isNotEmpty(statisticsExamList)) {
                statisticsExam1 = statisticsExamList.get(0);
            }
            if (null != statisticsExam1 && statisticsExam1.getId() != null) {
                statisticsExam.setId(statisticsExam1.getId());
                statisticsExamService.updateById(statisticsExam);
            } else {
                statisticsExamService.insert(statisticsExam);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        try {
            //插入考试记录
            List<StatisticsExamLearnMetadata> statisticsExamRecordeList = null;
            // 获取考试记录传时间
            statisticsExamRecordeList = statisticsExamMetadataClient
                    .getRecords(examId, df.format(startDate), df.format(endDate));

            // 插入考试记录
            if (CollectionUtils.isNotEmpty(statisticsExamRecordeList)) {
                List<StatisticsExamRecorde> list = new ArrayList<>();
                list = JSON.parseArray(JSON.toJSONString(statisticsExamRecordeList), StatisticsExamRecorde.class);
                for (StatisticsExamRecorde examRecorde : list) {
                    examRecorde.setId(idGenerator.generate());
                }
                statisticsExamRecordeService.insertBatch(list);
//                taskDetail(taskId, "考试:" + examId + "处理成功");
                LOGGER.info("完成执行考试记录插入--" + metadataObject.getName() + "  " + list.size() + "条记录");
            } else {
                LOGGER.warn("未找到考试记录--" + metadataObject.getName());
            }
        } catch (BeansException e) {
            e.printStackTrace();
            taskDetail(taskId, "考试记录插入过程中出现错误" + examId);
        }
    }

    @Override
    public void exceptionLog(Long taskId, Exam metadataObject, Exception e) {
        taskDetail(taskId, "" + e.getMessage());
    }


    protected Date addDay(Date date, int num) {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        calendar.add(calendar.DATE, num);
        date = calendar.getTime();
        return date;
    }


}
