package com.yizhi.statistics.application.controller.exam.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.toolkit.StringUtils;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.refactor.exam.ReportExamAccountVO;
import com.yizhi.statistics.application.service.StatisticsExamService;
import com.yizhi.statistics.application.util.WmyDateUtils;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Auther: dingxiaowei
 * @Date: 2018/8/10 15:56
 * @Description:按用户统计报表导出excel
 */
@Component
public class AccountGroupExport extends AbstractDefaultTask<String, Map<String, Object>> {
    private static final Logger LOGGER = LoggerFactory.getLogger(AccountGroupExport.class);

    @Autowired
    private StatisticsExamService statisticsExamService;


    @Override
    protected String execute(Map<String, Object> map) {
        // TODO 友邦数据修改 下载结果数据 3
        String uploadUrl = null;
        Long accountId = (Long) map.get("accountId");
        Long siteId = (Long) map.get("siteId");
        Long companyId = (Long) map.get("companyId");
        boolean isAdmin = (boolean) map.get("isAdmin");
        Date comitTime = new Date();
        Long taskId = (Long) map.get("taskId");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMddHHmmss");
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, comitTime, siteId, companyId);
        working(taskContext);

//        Integer pageSize = 10;
        com.yizhi.statistics.application.refactor.exam.ExamCountByUserVO vo = (com.yizhi.statistics.application.refactor.exam.ExamCountByUserVO) map.get("vo");
        vo.setAccountId(accountId);
        vo.setAdmin(isAdmin);
        vo.setCompanyId(companyId);
        vo.setSiteId(siteId);
        vo.setPageNo(1);
        vo.setPageSize(Integer.MAX_VALUE);
        Page<com.yizhi.statistics.application.refactor.exam.ReportExamAccountVO> page = null;
        List<ReportExamAccountVO> list = new ArrayList<>();

        try {
            page = statisticsExamService.getAccountGroup(vo);
            if (null != page) {
                list = page.getRecords();
                if (CollectionUtils.isEmpty(list)) {
                    success(taskContext, "未找到数据", "");
                }
            } else {
                success(taskContext, "未找到数据", "");
            }
        } catch (Exception e) {
            e.printStackTrace();
            LOGGER.info("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }
        try {
            XSSFWorkbook wb = new XSSFWorkbook();
            XSSFSheet sheet = wb.createSheet("按用户统计报表");
            XSSFRow row = sheet.createRow(0);

            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 9);
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell(0);
            cell.setCellValue("按用户统计报表结果");
//            cell.setCellStyle(style);

            row = sheet.createRow(1);
            cra = new CellRangeAddress(1, 1, 0, 9);
            sheet.addMergedRegion(cra);
            cell = row.createCell(0);
            cell.setCellValue("统计周期:" + vo.getStartTime()
                    + "~" + vo.getEndTime());

            row = sheet.createRow(2);
            row.createCell(0).setCellValue("用户名");
            row.createCell(1).setCellValue("姓名");
            row.createCell(2).setCellValue("应参加考试数");
            row.createCell(3).setCellValue("实际参加考试数");
            row.createCell(4).setCellValue("已通过考试数");
            row.createCell(5).setCellValue("未通过考试数");
            row.createCell(6).setCellValue("用户当前状态");
            row.createCell(7).setCellValue("通过率");
            row.createCell(8).setCellValue("考试累计时长");
            row.createCell(9).setCellValue("所在部门");
            row.createCell(10).setCellValue("所在组织架构");

            if (list != null && list.size() > 0) {
                for (int i = 0; i < list.size(); i++) {
                    row = sheet.createRow(i + 3);
                    row.createCell(0).setCellValue(list.get(i).getAccountName());
                    row.createCell(1).setCellValue(list.get(i).getAccountFullName());
                    row.createCell(2).setCellValue(list.get(i).getTotalAsk());
                    row.createCell(3).setCellValue(list.get(i).getTotalIn());
                    row.createCell(4).setCellValue(list.get(i).getTotalPass());
                    row.createCell(5).setCellValue(list.get(i).getTotalFail());
                    if (list.get(i).getAccountState() != null) {
                        if (list.get(i).getAccountState() == 1) {
                            row.createCell(6).setCellValue("启用");
                        } else {
                            row.createCell(6).setCellValue("停用");
                        }
                    }
                    row.createCell(7).setCellValue(list.get(i).getPassRate() + "%");
                    row.createCell(8).setCellValue(WmyDateUtils.durationFormart(String.valueOf(list.get(i).getDuration())));
                    row.createCell(9).setCellValue(list.get(i).getOrgName());
                    if (StringUtils.isNotEmpty(list.get(i).getOrgNames())) {
                        row.createCell(10).setCellValue(list.get(i).getOrgNames());
                    } else {
                        row.createCell(10).setCellValue("--");
                    }
                }
            }


            String fileName = "按用户统计报表" + sdf2.format(comitTime) + ".xls";
//            String requestPath = "/Users/dingxiaowei/aaa";
            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }
            String path = requestPath + "/" + fileName;
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                uploadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功", uploadUrl);
            } catch (Exception e1) {
                e1.printStackTrace();
                fail(taskContext, "按用户统计报表下载写入过程发生错误");
                LOGGER.error("按用户统计报表下载写入过程发生错误");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "按用户统计报表下载出现错误");
            LOGGER.error("按用户统计报表下载出现错误");
        }

        return uploadUrl;
    }
}
