package com.yizhi.statistics.application.controller.exam.export;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.toolkit.StringUtils;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.point.application.feign.PointUserFeignClients;
import com.yizhi.point.application.vo.domain.PointDetailsVo;
import com.yizhi.statistics.application.domain.StatisticsExam;
import com.yizhi.statistics.application.refactor.exam.ExamCountByUserVO;

import com.yizhi.statistics.application.refactor.exam.ReportExamAccountsViewVo;
import com.yizhi.statistics.application.service.StatisticsExamService;
import com.yizhi.statistics.application.util.WmyDateUtils;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Auther: ding
 * @Date: 2019/04/18
 * @Description:按考试统计查看结果下载
 */
@Component
public class ExamAccountTrExamsExport extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExamAccountTrExamsExport.class);

    @Autowired
    private StatisticsExamService statisticsExamService;

    @Autowired
    private PointUserFeignClients pointUserFeignClients;

    @Override
    protected String execute(Map<String, Object> map) {

        // TODO 友邦数据修改 下载考试结果数据 1
        String uploadUrl = null;
        com.yizhi.statistics.application.refactor.exam.ExamCountByUserVO vo = (ExamCountByUserVO) map.get("vo");
        Long examId = vo.getExamId();

        Long accountId = (Long) map.get("accountId");
        Long siteId = (Long) map.get("siteId");
        Long companyId = (Long) map.get("companyId");
        boolean isAdmin = (boolean) map.get("isAdmin");

        Date comitTime = new Date();
        Long taskId = (Long) map.get("taskId");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMddHHmmss");
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, comitTime, siteId, companyId);
        working(taskContext);

        Page<com.yizhi.statistics.application.refactor.exam.ReportExamAccountsViewVo> page = null;
        List<com.yizhi.statistics.application.refactor.exam.ReportExamAccountsViewVo> list = null;

        try {
            vo.setPageNo(1);
            vo.setPageSize(Integer.MAX_VALUE);
            vo.setAccountId(accountId);
            vo.setAdmin(isAdmin);
            vo.setCompanyId(companyId);
            vo.setSiteId(siteId);
            page = statisticsExamService.getExamAndAccountGroup(vo);
        } catch (Exception e) {
            LOGGER.info("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }


        try {
            if (null != page) {
                list = page.getRecords();
                if (CollectionUtils.isEmpty(list)) {
                    success(taskContext, "未找到数据", "");
                }
            } else {
                success(taskContext, "未找到数据", "");
            }
            StatisticsExam statisticsExam = new StatisticsExam();
            statisticsExam.setExamId(examId);
            StatisticsExam exam = statisticsExamService.selectList(new EntityWrapper<>(statisticsExam)).get(0);

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

            XSSFWorkbook wb = new XSSFWorkbook();
            XSSFSheet sheet = wb.createSheet("下载考试结果数据");
            XSSFRow row = sheet.createRow(0);

            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 10);
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell(0);

            cell.setCellValue(exam.getExamName() + "下载考试结果数据");

            row = sheet.createRow(1);
            cra = new CellRangeAddress(1, 1, 0, 10);
            sheet.addMergedRegion(cra);
            cell = row.createCell(0);
            cell.setCellValue("统计周期:");

            XSSFRow row2 = sheet.createRow(2);
            row2.createCell(0).setCellValue("用户名");
            row2.createCell(1).setCellValue("姓名");
            row2.createCell(2).setCellValue("最早交卷时间");
            row2.createCell(3).setCellValue("最迟交卷时间");
            row2.createCell(4).setCellValue("考试次数");
            row2.createCell(5).setCellValue("考试累计时长");
            row2.createCell(6).setCellValue("最高分");
            row2.createCell(7).setCellValue("考试结果");
            row2.createCell(8).setCellValue("获得积分");
            row2.createCell(9).setCellValue("用户状态");
            row2.createCell(10).setCellValue("所在部门");
            row2.createCell(11).setCellValue("所在组织架构");
            if (list != null && list.size() > 0) {

                List<PointDetailsVo> pointDetails = pointUserFeignClients.getReportPoints(examId, null, companyId, siteId, false);

                Map<Long, Integer> pointMap = new HashMap<>();
                if (CollectionUtils.isNotEmpty(pointDetails)) {
                    for (PointDetailsVo pointDetailVo : pointDetails) {
                        pointMap.put(pointDetailVo.getAccountId(), pointDetailVo.getPoint());
                    }
                }

                for (int i = 0; i < list.size(); i++) {
                    ReportExamAccountsViewVo reportExamAccountsViewVo = list.get(i);
                    row = sheet.createRow(i + 3);
                    row.createCell(0).setCellValue(reportExamAccountsViewVo.getAccountName());
                    row.createCell(1).setCellValue(reportExamAccountsViewVo.getAccountFullName());

                    if (reportExamAccountsViewVo.getFinishTime() != null) {
                        row.createCell(2).setCellValue(sdf.format(reportExamAccountsViewVo.getFinishTime()));
                    }

                    if (reportExamAccountsViewVo.getLastFinishTime() != null) {
                        row.createCell(3).setCellValue(sdf.format(reportExamAccountsViewVo.getLastFinishTime()));
                    }

                    row.createCell(4).setCellValue(reportExamAccountsViewVo.getExamCount());
                    row.createCell(5).setCellValue(WmyDateUtils.durationFormart(String.valueOf(reportExamAccountsViewVo.getDuration())));

                    if (reportExamAccountsViewVo.getScore() != null) {
                        row.createCell(6).setCellValue(reportExamAccountsViewVo.getScore().doubleValue());
                    }
                    if (reportExamAccountsViewVo.getExamState() == 1) {
                        row.createCell(7).setCellValue("通过");
                    } else if (reportExamAccountsViewVo.getExamState() == 0) {
                        row.createCell(7).setCellValue("不通过");
                    } else {
                        row.createCell(7).setCellValue("--");
                    }
                    Integer point = pointMap.get(reportExamAccountsViewVo.getAccountId());
                    if (null != point) {
                        row.createCell(8).setCellValue(point);
                    } else {
                        row.createCell(8).setCellValue(reportExamAccountsViewVo.getPoint());
                    }
                    if (reportExamAccountsViewVo.getAccountState() != null) {
                        if (reportExamAccountsViewVo.getAccountState() == 1) {
                            row.createCell(9).setCellValue("启用");
                        } else {
                            row.createCell(9).setCellValue("停用");
                        }
                    }
                    row.createCell(10).setCellValue(reportExamAccountsViewVo.getOrgName());
                    if (StringUtils.isNotEmpty(reportExamAccountsViewVo.getOrgParentNames())) {
                        row.createCell(11).setCellValue(reportExamAccountsViewVo.getOrgParentNames());
                    } else {
                        row.createCell(11).setCellValue("--");
                    }
                }
            }


            String fileName = exam.getExamName() + "考试结果" + sdf2.format(comitTime) + ".xls";
//            String requestPath = "/Users/dingxiaowei/aaa";
            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }
            String path = requestPath + "/" + fileName;
            FileOutputStream os = null;
            File file = null;

            try {
                os = new FileOutputStream(path);
                wb.write(os);
                uploadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功", uploadUrl);
            } catch (Exception e2) {
                e2.printStackTrace();
                fail(taskContext, "按考试统计-查看 (按用户统计)下载考试结果写入过程出现错误");
                LOGGER.error("按考试统计-查看 (按用户统计)下载考试结果写入过程出现错误");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "按考试统计-查看 (按用户统计)下载考试结果数据出现错误");
            LOGGER.error("按考试统计-查看 (按用户统计)下载考试结果数据出现错误");
        }

        return uploadUrl;
    }
}
