package com.yizhi.statistics.application.controller.exam.export;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.exam.domain.po.UserExamGroupPO;
import com.yizhi.statistics.application.exam.domain.vo.UserExamByOrgVO;
import com.yizhi.statistics.application.service.StatisticsExamService;
import com.yizhi.statistics.application.util.CreateFormartExcelUtils;
import com.yizhi.statistics.application.util.WmyDateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Date 2020/9/21 8:53 上午
 * @Author lvjianhui
 **/
@Component
public class ExamByGroupOrgUserExport extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExamByGroupOrgUserExport.class);

    @Autowired
    private StatisticsExamService statisticsExamService;

    @Override
    protected String execute(Map<String, Object> param) {

        // TODO 友邦数据修改 下载部门考试明细 5
        String fileUrl = "";
        TaskContext taskContext = null;
        try {
            UserExamGroupPO po = (UserExamGroupPO) param.get("reportExamParam");
            Long accountId = (Long) param.get("accountId");
            Long siteId = po.getSiteId();
            Long companyId = po.getCompanyId();



            Date submitTime = new Date();
            Long taskId = (Long) param.get("taskId");
            String serialNo = (String) param.get("serialNo");
            SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMddHHmmss");
            SimpleDateFormat sdf3 = new SimpleDateFormat("yyyyMMdd");
            String taskName = (String) param.get("taskName");
            taskContext = new TaskContext(taskId, serialNo, taskName,
                    accountId, submitTime, siteId, companyId);
            working(taskContext);

            Date date = new Date();
            String fileName = new StringBuffer("【部门名称】用户考试统计").append(sdf2.format(date)).append(".xls").toString().replace("部门名称", po.getOrgName());
            String raw1 = new StringBuffer("【部门名称】用户考试统计").append(sdf3.format(date)).toString().replace("部门名称", po.getOrgName());
            String raw2 = new StringBuffer("统计周期：").append(po.getStartAt()).append("~").append(po.getEndAt()).toString();
            List<String> headNames = new ArrayList<String>();

            po.setPageIndex(1);
            po.setPageSize(Integer.MAX_VALUE);
            List<UserExamByOrgVO> list = statisticsExamService.getUserExam(po).getRecords();
            headNames.add("部门编码");
            headNames.add("部门名称");
            headNames.add("所在组织架构");
            headNames.add("用户名");
            headNames.add("姓名");
            headNames.add("应参加考试数");
            headNames.add("实际参加考试数");
            headNames.add("已通过考试数");
            headNames.add("未通过考试数");
            headNames.add("通过率");
            headNames.add("考试累计时长");

            List<JSONObject> data = new ArrayList<JSONObject>();
            for (UserExamByOrgVO vo : list) {
                JSONObject obj = new JSONObject();
                obj.put("部门编码", po.getOrgCode());
                obj.put("部门名称", po.getOrgName());
                obj.put("所在组织架构", vo.getOrgNames());
                obj.put("用户名", vo.getUserName());
                obj.put("姓名", vo.getFullName());
                obj.put("应参加考试数", vo.getPlanJoinCount());
                obj.put("实际参加考试数", vo.getActualJoinCount());
                obj.put("已通过考试数", vo.getPassCount());
                obj.put("未通过考试数", vo.getUnPassCount());
                obj.put("通过率", vo.getPassRate());
                obj.put("考试累计时长", WmyDateUtils.durationFormart(String.valueOf(vo.getDuration())));
                data.add(obj);
            }


            fileUrl = CreateFormartExcelUtils.createFormartExcel(
                    fileName,
                    raw1,
                    raw2,
                    headNames,
                    po.getStartAt(),
                    po.getEndAt(),
                    data
            );
            success(taskContext, "成功", fileUrl);
        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "按部门用户考试结果写入过程中发生错误");
            LOGGER.error("按部门用户考试结果写入过程中发生错误", e);
        }
        return fileUrl;
    }
}
