package com.yizhi.statistics.application.controller.exam.export;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.exam.domain.po.UserExamGroupPO;
import com.yizhi.statistics.application.exam.domain.vo.UserExamDetailOrgVO;
import com.yizhi.statistics.application.service.StatisticsExamService;
import com.yizhi.statistics.application.util.CreateFormartExcelUtils;
import com.yizhi.statistics.application.util.WmyDateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Date 2020/9/21 9:40 上午
 * @Author lvjianhui
 **/
@Component
public class ExamByOrgUserDetailExport extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExamByGroupOrgUserExport.class);

    @Autowired
    private StatisticsExamService statisticsExamService;

    @Override
    protected String execute(Map<String, Object> param) {
        String fileUrl = "";
        TaskContext taskContext = null;
        // TODO 友邦数据修改 下载用户开始明细 6
        try {
            UserExamGroupPO po = (UserExamGroupPO) param.get("reportExamParam");
            Long accountId = (Long) param.get("accountId");
            Long siteId = po.getSiteId();
            Long companyId = po.getCompanyId();

            Date submitTime = new Date();
            Long taskId = (Long) param.get("taskId");
            String serialNo = (String) param.get("serialNo");
            SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMddHHmmss");
            SimpleDateFormat sdf3 = new SimpleDateFormat("yyyyMMdd");
            String taskName = (String) param.get("taskName");
            taskContext = new TaskContext(taskId, serialNo, taskName,
                    accountId, submitTime, siteId, companyId);
            working(taskContext);

            Date date = new Date();
            String fileName = new StringBuffer("【部门名称】用户考试明细").append(sdf2.format(date)).append(".xls").toString().replace("部门名称", po.getOrgName());
            String raw1 = new StringBuffer("【部门名称】用户考试明细").append(sdf3.format(date)).toString().replace("部门名称", po.getOrgName());
            String raw2 = new StringBuffer("统计周期：").append(po.getStartAt()).append("~").append(po.getEndAt()).toString();
            List<String> headNames = new ArrayList<String>();

            headNames.add("用户名");
            headNames.add("姓名");
            headNames.add("所在部门");
            headNames.add("考试名称");
            headNames.add("考试开始时间");
            headNames.add("考试结束时间");
            headNames.add("考试时长");
            headNames.add("考试次数");
            headNames.add("最早交卷时间");
            headNames.add("最高分");
            headNames.add("考试状态");
            headNames.add("考试结果");

            po.setPageIndex(1);
            po.setPageSize(Integer.MAX_VALUE);
            List<UserExamDetailOrgVO> list = statisticsExamService.getExamUserDetail(po);

            List<JSONObject> data = new ArrayList<JSONObject>();
            for (UserExamDetailOrgVO vo : list) {
                JSONObject obj = new JSONObject();
                obj.put("用户名", vo.getName());
                obj.put("姓名", vo.getFullName());
                obj.put("所在部门", vo.getOrgName());
                obj.put("考试名称", vo.getExamName());
                obj.put("考试开始时间", vo.getExamStartAt());
                obj.put("考试结束时间", vo.getExamEndAt());
                obj.put("考试时长", WmyDateUtils.durationFormart(String.valueOf(vo.getDuration())));
                obj.put("考试次数", vo.getJoinExamCount());
                obj.put("最早交卷时间", vo.getFirstPushAt());
                obj.put("最高分", vo.getMaxScore());
                obj.put("考试状态", vo.getExamStatus());
                obj.put("考试结果", vo.getExamResult());
                data.add(obj);
            }

            fileUrl = CreateFormartExcelUtils.createFormartExcel(
                    fileName,
                    raw1,
                    raw2,
                    headNames,
                    po.getStartAt(),
                    po.getEndAt(),
                    data
            );
            success(taskContext, "成功", fileUrl);
        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "按部门用户考试明细写入过程中发生错误");
            LOGGER.error("按部门用户考试明细写入过程中发生错误", e);
        }
        return fileUrl;
    }
}
