package com.yizhi.statistics.application.controller.exam.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.refactor.exam.ReportExamParam;
import com.yizhi.statistics.application.refactor.exam.ReportExamVO;
import com.yizhi.statistics.application.service.StatisticsExamService;
import com.yizhi.statistics.application.util.WorkUtil;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Auther: dingxiaowei
 * @Date: 2019/04/18
 * @Description:按考试统计报表导出excel
 */
@Component
public class ExamGroupExport extends AbstractDefaultTask<String, Map<String, Object>> {
    private static final Logger LOGGER = LoggerFactory.getLogger(ExamGroupExport.class);

    @Autowired
    private StatisticsExamService statisticsExamService;

    @Autowired
    private WorkUtil workUtil;

    @Override
    protected String execute(Map<String, Object> map) {
        String uploadUrl = null;
        LOGGER.info("开始获取数据");
        ReportExamParam reportExamParam = (com.yizhi.statistics.application.refactor.exam.ReportExamParam) map.get("reportExamParam");
        Long accountId = (Long) map.get("accountId");
        Long siteId = reportExamParam.getSiteId();
        Long companyId = reportExamParam.getCompanyId();
        boolean isAdmin = (boolean) map.get("isAdmin");

        Date submitTime = new Date();
        Long taskId = (Long) map.get("taskId");
        String serialNo = (String) map.get("serialNo");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMddHHmmss");
        String taskName = (String) map.get("taskName");
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,
                accountId, submitTime, siteId, companyId);
        working(taskContext);

        Page<com.yizhi.statistics.application.refactor.exam.ReportExamVO> page = null;

        try {
            reportExamParam.setPageNo(1);
            reportExamParam.setPageSize(Integer.MAX_VALUE);
            reportExamParam.setAdmin(isAdmin);
            reportExamParam.setAccountId(accountId);
            reportExamParam.setSiteId(siteId);
            reportExamParam.setCompanyId(companyId);
            page = statisticsExamService.getExamGroup(reportExamParam);
        } catch (Exception e) {
            LOGGER.info("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }


        try {
            List<com.yizhi.statistics.application.refactor.exam.ReportExamVO> list = new ArrayList<>();
            if (null != page) {
                list = page.getRecords();
            } else {
                success(taskContext, "未找到数据", "");
            }
            if (CollectionUtils.isEmpty(list)) {
                success(taskContext, "未找到数据", "");
            }


            //创建excel文件
            XSSFWorkbook wb = new XSSFWorkbook();
            //创建报表页
            XSSFSheet sheet = wb.createSheet("按考试统计报表");
            //创建表头
            XSSFRow row = sheet.createRow(0);

//            XSSFCellStyle style = wb.createCellStyle();
//            style.setAlignment(HorizontalAlignment.CENTER);

            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 6);
            //表头合并单元格
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell(0);
            cell.setCellValue("按考试统计报表");//填写表头名
//            cell.setCellStyle(style);//表头居中

            row = sheet.createRow(1);
            CellRangeAddress cra2 = new CellRangeAddress(1, 1, 0, 6);
            sheet.addMergedRegion(cra2);
            cell = row.createCell(0);
            cell.setCellValue("统计周期:" + reportExamParam.getStartTime()
                    + "~" + reportExamParam.getEndTime());

            row = sheet.createRow(2);
            row.createCell(0).setCellValue("考试名称");
            row.createCell(1).setCellValue("考试时间");
            row.createCell(2).setCellValue("可参加人数");
            row.createCell(3).setCellValue("实际参加人数");
            row.createCell(4).setCellValue("通过人数");
            row.createCell(5).setCellValue("通过率");
            row.createCell(6).setCellValue("平均分");
            row.createCell(7).setCellValue("完成率");

            if (list != null && list.size() > 0) {
                for (int i = 0; i < list.size(); i++) {
                    ReportExamVO reportExamVO = list.get(i);

                    row = sheet.createRow(i + 3);
                    row.createCell(0).setCellValue(reportExamVO.getName());
                    String examTime = sdf.format(reportExamVO.getStartTime()) + " " + sdf.format(reportExamVO.getEndTime());
                    row.createCell(1).setCellValue(examTime);
                    row.createCell(2).setCellValue(reportExamVO.getTotalAsk());
                    row.createCell(3).setCellValue(reportExamVO.getTotalIn());
                    row.createCell(4).setCellValue(reportExamVO.getTotalPass());
                    row.createCell(5).setCellValue(reportExamVO.getPassRate() + "%");
                    row.createCell(6).setCellValue(reportExamVO.getAvgScore().doubleValue());
                    row.createCell(7).setCellValue(reportExamVO.getComplete() == null ? "0.00" : reportExamVO.getComplete().toString() + "%");
                }
            }

            String fileName = "按考试统计报表" + sdf2.format(submitTime) + ".xls";
            LOGGER.info("表名：{}",fileName);
//            String requestPath = "/Users/dingxiaowei/aaa";
            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }

            String path = requestPath + "/" + fileName;
            LOGGER.info("文件路径：{}",path);
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                uploadUrl = OssUpload.upload(path, fileName);//阿里云返回url
                LOGGER.info("url：{}",uploadUrl);
                file = new File(path);
                success(taskContext, "成功", uploadUrl);
            } catch (Exception e2) {
                e2.printStackTrace();
                fail(taskContext, "按考试统计写入过程中发生错误");
                LOGGER.error("按考试统计写入过程中发生错误");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "按考试统计报表下载发生错误");
            LOGGER.error("按考试统计报表下载发生错误，请查看日志");
        }

        return uploadUrl;
    }
}
