package com.yizhi.statistics.application.controller.exam.export;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.exam.domain.po.ExamGroupPO;
import com.yizhi.statistics.application.exam.domain.vo.ExamGroupOrgVO;
import com.yizhi.statistics.application.service.StatisticsExamService;
import com.yizhi.statistics.application.util.CreateFormartExcelUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Date 2020/9/20 11:23 下午
 * @Author lvjianhui
 **/
@Component
public class ExamGroupOrgExport extends AbstractDefaultTask<String, Map<String, Object>> {

    @Autowired
    StatisticsExamService statisticsExamService;
    private static final Logger LOGGER = LoggerFactory.getLogger(ExamGroupOrgExport.class);

    @Override
    protected String execute(Map<String, Object> param) {
        ExamGroupPO po = (ExamGroupPO) param.get("reportExamParam");
        Long accountId = (Long) param.get("accountId");
        Long siteId = po.getSiteId();
        Long companyId = po.getCompanyId();
        boolean isAdmin = (boolean) param.get("isAdmin");

        Date submitTime = new Date();
        Long taskId = (Long) param.get("taskId");
        String serialNo = (String) param.get("serialNo");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMddHHmmss");
        SimpleDateFormat sdf3 = new SimpleDateFormat("yyyyMMdd");
        String taskName = (String) param.get("taskName");
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,
                accountId, submitTime, siteId, companyId);
        working(taskContext);

        po.setPageIndex(1);
        po.setPageSize(Integer.MAX_VALUE);
        List<ExamGroupOrgVO> list = statisticsExamService.getExamGroupByOrg(po).getRecords();

        Date date = new Date();
        String fileName = new StringBuffer("按部门统计考试").append(sdf2.format(date)).append(".xls").toString();
        String raw1 = new StringBuffer("部门名称考试统计").append(sdf3.format(date)).toString();
        String raw2 = new StringBuffer("统计周期：").append(po.getStartAt()).append("~").append(po.getEndAt()).toString();
        List<String> headNames = new ArrayList<String>();
        // 部门编码、部门名称、所在组织架构、可参加考试数、可参加人数、实际参加人次、通过人次、通过率
        headNames.add("部门编码");
        headNames.add("部门名称");
        headNames.add("所在组织架构");
        headNames.add("可参加考试数");
        headNames.add("可参加人数");
        headNames.add("实际参加人次");
        headNames.add("通过人次");
        headNames.add("通过率");

        List<JSONObject> data = new ArrayList<JSONObject>();
        for (ExamGroupOrgVO vo : list) {
            JSONObject obj = new JSONObject();
            obj.put("部门编码", vo.getOrgCode());
            obj.put("部门名称", vo.getOrgName());
            obj.put("所在组织架构", vo.getOrgNames());
            obj.put("可参加考试数", vo.getExamCount());
            obj.put("可参加人数", vo.getPlanUserCount());
            obj.put("实际参加人次", vo.getActualUserCount());
            obj.put("通过人次", vo.getPassUserCount());
            obj.put("通过率", vo.getPassRate());
            data.add(obj);
        }

        String fileUrl = "";
        try {
            fileUrl = CreateFormartExcelUtils.createFormartExcel(
                    fileName,
                    raw1,
                    raw2,
                    headNames,
                    po.getStartAt(),
                    po.getEndAt(),
                    data
            );
            success(taskContext, "成功", fileUrl);
        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "按部门统计考试写入过程中发生错误");
            LOGGER.error("按部门统计考试写入过程中发生错误");
        }
        return fileUrl;
    }

}
