package com.yizhi.statistics.application.controller.forum;

import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.comment.application.constans.Constant;
import com.yizhi.comment.application.feign.ManageCommentClient;
import com.yizhi.comment.application.vo.StatisticsForumAccountVo;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.forum.application.feign.remote.RemotePostsClient;
import com.yizhi.forum.application.vo.domain.PostsVo;
import com.yizhi.statistics.application.entity.StatisticsForumAccount;
import com.yizhi.statistics.application.service.IStatisticsForumAccountService;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 跑批任务
 */
@Component
public class StatisticsForumAccountAbs extends AbstractDefaultTask<String, Map<String, Object>> {

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private IStatisticsForumAccountService statisticsForumAccountService;
    @Autowired
    private ReportClient reportClient;
    @Autowired
    private RemotePostsClient remotePostsClient;
    @Autowired
    private ManageCommentClient commentClient;


    final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");

    private static final Logger LOGGER = LoggerFactory.getLogger(StatisticsForumAccountAbs.class);


    @Override
    protected String execute(Map<String, Object> map) {
        Long taskId = idGenerator.generate();
        Long taskStartTime = System.currentTimeMillis();
        /**
         * 走异步任务
         */
        Date curDateWork = new Date();

        TaskContext taskContext = new TaskContext(taskId, sim.format(curDateWork) + " 号0:10论坛按用户统计报表分析前一天的数据", 999L, curDateWork);
        working(taskContext);

        String result = "";
        //先获取所有已上架的帖子
        List<PostsVo> allPosts = remotePostsClient.getAllPosts();
        if (CollectionUtils.isEmpty(allPosts)) {
            result = "***** 论坛按用户统计报表,帖子数量为0 正常退出 *****";
            taskDetail(taskId, result);
            LOGGER.info(result);
            return "error";
        }
        Set<Long> siteIds = allPosts.stream().map(a -> a.getSiteId()).collect(Collectors.toSet());
        if (CollectionUtils.isEmpty(siteIds)) {
            result = "***** 论坛按用户统计报表,站点为空 异常退出 *****";
            taskDetail(taskId, result);
            LOGGER.info(result);
            return "error";
        }

        //获取用户关于帖子的数据
        Map<Long, com.yizhi.forum.application.vo.forum.StatisticsForumAccountVo> postsAccountMap = remotePostsClient.getPersonalDataMap();
        //获取用户关于评论回复点赞举报的数据
        Map<Long, StatisticsForumAccountVo> commentAccountMap = commentClient.getPersonalDataMap(Constant.TASK_TYPE_POSTS);

        //先得清空statistics_forum_account表数据
        try {
            statisticsForumAccountService.deleteAllData();
            result = "***** 论坛用户统计数据开始前先删除数据*****";
            taskDetail(taskId, result);
            LOGGER.info(result);
        } catch (Exception e) {
            result = "***** 论坛用户统计数据开始前删除数据，出现异常*****";
            taskDetail(taskId, result);
            LOGGER.info(result);
            e.printStackTrace();
        }

        ReportRangeAccountReqVO tem = null;
        List<ReportAccountRespVO> accounts = null;
        tem = new ReportRangeAccountReqVO();
        for (Long siteId : siteIds) {
            tem.setSiteId(siteId);
            // 获得所有的用户
            accounts = reportClient.getRangeAccounts(tem);
            List<StatisticsForumAccount> statisticsForumAccounts = null;

            if (CollectionUtils.isNotEmpty(accounts)) {
                statisticsForumAccounts = new ArrayList<>(accounts.size());
                result = "***** 论坛用户统计数据 开始插入：" + siteId + " 数据*****";
                taskDetail(taskId, result);
                LOGGER.info(result);
                for (ReportAccountRespVO a : accounts) {
                    statisticsForumAccounts.add(buildRecords(a, siteId, postsAccountMap, commentAccountMap));
                }
                try {
                    statisticsForumAccountService.insertBatch(statisticsForumAccounts);
                } catch (Exception e) {
                    result = "***** 论坛用户统计数据 插入：" + siteId + " 数据出现异常*****";
                    taskDetail(taskId, result);
                    LOGGER.info(result);
                    e.printStackTrace();
                }
            }
        }
        Long taskEndTime = System.currentTimeMillis();
        Long t = (taskEndTime - taskStartTime);
        Integer count = accounts.size();
        success(taskContext, "执行成功,耗时:" + t + "s", "");
        taskDetail(taskId, "总数量:" + count + "。处理时间ms:" + t);
        LOGGER.info("***** 数据入库结束 ,数量:" + count + "。处理时间ms:" + t + " *****");
        return "success";

    }

    /**
     * 组建主体信息
     *
     * @param account
     * @param siteId
     * @param postsAccountMap
     * @param commentAccountMap
     * @return
     */
    public StatisticsForumAccount buildRecords(ReportAccountRespVO account, Long siteId,
                                               Map<Long, com.yizhi.forum.application.vo.forum.StatisticsForumAccountVo> postsAccountMap,
                                               Map<Long, StatisticsForumAccountVo> commentAccountMap
    ) {
        StatisticsForumAccount statisticsForumAccount = new StatisticsForumAccount();
        statisticsForumAccount.setId(idGenerator.generate());
        statisticsForumAccount.setAccountId(account.getUserId());
        statisticsForumAccount.setAccountName(account.getUserName());
        statisticsForumAccount.setFullName(account.getUserFullName());
        statisticsForumAccount.setOrgId(account.getOrgId());
        statisticsForumAccount.setOrgName(account.getOrgName());
        statisticsForumAccount.setPosition(account.getPosition());
        statisticsForumAccount.setStatus(account.getStatus() == 1 ? "启用" : "禁用");
        if (postsAccountMap != null) {
            com.yizhi.forum.application.vo.forum.StatisticsForumAccountVo vo = postsAccountMap.get(account.getUserId());
            if (vo != null) {
                statisticsForumAccount.setSendNum(vo.getSendNum());
                statisticsForumAccount.setReadNum(vo.getReadNum());
                statisticsForumAccount.setDraftNum(vo.getDraftNum());
                statisticsForumAccount.setHighlightNum(vo.getHighlightNum());
                statisticsForumAccount.setTopNum(vo.getTopNum());
                statisticsForumAccount.setBeForbiddenNum(vo.getBeForbiddenNum());
                statisticsForumAccount.setBeInformNum(vo.getBeInformNum());
                statisticsForumAccount.setBeAdmiredNum(vo.getBeAdmiredNum());
            }

        }
        if (commentAccountMap != null) {
            StatisticsForumAccountVo vo = commentAccountMap.get(account.getUserId());
            if (vo != null) {
                statisticsForumAccount.setCommentNum(vo.getCommentNum());
                statisticsForumAccount.setReplyNum(vo.getReplyNum());
                statisticsForumAccount.setAdmireNum(vo.getAdmireNum());
                statisticsForumAccount.setInformNum(vo.getInformNum());
            }
        }

        statisticsForumAccount.setSiteId(siteId);
        statisticsForumAccount.setCompanyId(account.getCompanyId());

        return statisticsForumAccount;
    }
}
