package com.yizhi.statistics.application.controller.forum.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.entity.StatisticsForum;
import com.yizhi.statistics.application.service.IStatisticsForumService;
import com.yizhi.statistics.application.util.WorkUtil;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class ForumDownload extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger logger = LoggerFactory.getLogger(ForumDownload.class);

    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
    SimpleDateFormat dateFormat1 = new SimpleDateFormat("yyyy-MM-dd");

    @Autowired
    private WorkUtil workUtil;
    @Autowired
    private IStatisticsForumService statisticsForumService;

    @Override
    protected String execute(Map<String, Object> arg0) {

        //下载路径
        String upLoadUrl = null;
        Long taskId = (Long) arg0.get("currentTaskId");
        String serialNo = (String) arg0.get("serialNo");
        String taskName = (String) arg0.get("currentTaskName");
        Long accountId = Long.valueOf(arg0.get("accountId").toString());
        Date submitTime = new Date((Long) arg0.get("submitTime"));
        Long siteId = Long.valueOf(arg0.get("siteId").toString());
        Long companyId = Long.valueOf(arg0.get("companyId").toString());
        String keyword = (String) arg0.get("keyword");
        String startDate = null;
        String endDate = null;
        if (arg0.get("startDate") != null && arg0.get("endDate") != null) {
            startDate = (String) arg0.get("startDate");
            endDate = (String) arg0.get("endDate");
        }

        /**
         * 走异步任务
         */
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);
        working(taskContext);

        //获取帖子数据
        Page<StatisticsForum> page = statisticsForumService.getPage(keyword, startDate, endDate, siteId, 1, Integer.MAX_VALUE);
        List<StatisticsForum> records = page.getRecords();
        if (CollectionUtils.isEmpty(records)) {
            logger.info("\"数据为空！\"");
            success(taskContext, "数据为空！", upLoadUrl);
            return "数据为空！";
        }


        try {
            //excel生成过程: excel-->sheet-->row-->cell

            // 第一步，创建一个Excel文件
            XSSFWorkbook wb = new XSSFWorkbook();
            String sheetName = "论坛数据-按帖子统计" + dateFormat.format(submitTime);

            // 第二步，在webbook中添加一个sheet,对应Excel文件中的sheet
            XSSFSheet sheet = wb.createSheet(sheetName);
            XSSFFont font = wb.createFont();
//			font.setFontName("宋体");
//			font.setFontHeightInPoints((short) 11);
            font.setBold(true);
            XSSFCellStyle style = wb.createCellStyle();
            style.setFont(font);
            //居中格式
            style.setAlignment(HorizontalAlignment.CENTER);
            style.setVerticalAlignment(VerticalAlignment.CENTER);

            // 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short
            XSSFRow row = sheet.createRow((int) 0);
            // 合并单元格
            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 11); // 起始行, 终止行, 起始列, 终止列
            sheet.addMergedRegion(cra);
            //第一行
            String title1 = "论坛统计报表--按帖子统计";
            XSSFCell cell = row.createCell((short) 0);
            cell.setCellValue(title1);
            cell.setCellStyle(style);

            //第二行
            row = sheet.createRow((int) 1);
            CellRangeAddress cra1 = new CellRangeAddress(1, 1, 0, 11); // 起始行, 终止行, 起始列, 终止列
            sheet.addMergedRegion(cra1);
            String title2 = "";
            if (startDate == null || endDate == null) {
                title2 = "截止时间：不限";
            } else {
                title2 = startDate+ "到" + endDate;
            }
            cell = row.createCell((short) 0);
            cell.setCellValue(title2);
            cell.setCellStyle(style);

            //第三行
            XSSFRow titleRow = sheet.createRow((int) 2);
//        row2.setRowStyle(style);
//        row2.setHeight((short) (14 * 20));
            titleRow.createCell((short) 0).setCellValue("帖子名称");
            titleRow.createCell((short) 1).setCellValue("用户名");
            titleRow.createCell((short) 2).setCellValue("姓名");
            titleRow.createCell((short) 3).setCellValue("发表时间");
            titleRow.createCell((short) 4).setCellValue("精选状态");
            titleRow.createCell((short) 5).setCellValue("置顶状态");
            titleRow.createCell((short) 6).setCellValue("点赞数");
            titleRow.createCell((short) 7).setCellValue("回复数");
            titleRow.createCell((short) 8).setCellValue("阅读数");
            titleRow.createCell((short) 9).setCellValue("收藏数");
            titleRow.createCell((short) 10).setCellValue("举报数");
            titleRow.createCell((short) 11).setCellValue("发表端口");

            for (int i = 0; i < records.size(); i++) {
                row = sheet.createRow(i + 3);
//            row.setRowStyle(style);
//            row.setHeight((short) (13 * 20));
                // 第四步，创建单元格，并设置值
                StatisticsForum vo = records.get(i);
                if (vo != null) {
                    row.createCell((short) 0).setCellValue(vo.getTitle() == null ? "" : vo.getTitle());
                    row.createCell((short) 1).setCellValue(vo.getAccountName() == null ? "" : vo.getAccountName());
                    row.createCell((short) 2).setCellValue(vo.getFullName() == null ? "" : vo.getFullName());
                    row.createCell((short) 3).setCellValue(vo.getPostsCreateTime() == null ? new Date() : vo.getPostsCreateTime());
                    row.createCell((short) 4).setCellValue(vo.getIsHighlight() == 1 ? "已加精" : "未加精");
                    row.createCell((short) 5).setCellValue(vo.getIsTop() == 1 ? "已置顶" : "未置顶");
                    row.createCell((short) 6).setCellValue(vo.getThumbsNum() == null ? 0 : vo.getThumbsNum());
                    row.createCell((short) 7).setCellValue(vo.getCommentNum() == null ? 0 : vo.getCommentNum());
                    row.createCell((short) 8).setCellValue(vo.getReadNum() == null ? 0 : vo.getReadNum());
                    row.createCell((short) 9).setCellValue(vo.getFavoriteNum() == null ? 0 : vo.getFavoriteNum());
                    row.createCell((short) 10).setCellValue(vo.getInformNum() == null ? 0 : vo.getInformNum());
                    if (vo.getTerminalType() != null) {
                        String terminalType = "";
                        switch (vo.getTerminalType()) {
                            case 1:
                                terminalType = "管理端";
                                break;
                            case 2:
                                terminalType = "pc端";
                                break;
                            case 3:
                                terminalType = "微信端";
                                break;
                        }
                        row.createCell((short) 11).setCellValue(terminalType);
                    }
                }

            }


            String requestPath = FileConstant.SAVE_PATH;
//            String requestPath = "D:\\";
            File fileDir = new File(requestPath);

            if (!fileDir.exists()) {
                fileDir.mkdir();
            }

            // 第五步，写入实体数据 实际应用中这些数据从数据库得到，
            StringBuffer fileNameSb = new StringBuffer().append(sheetName).append(".xlsx");
            String path = new StringBuffer().append(requestPath).append(fileNameSb).toString();
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                //阿里云返回url
                upLoadUrl = OssUpload.upload(path, fileNameSb.toString());
                file = new File(path);
                success(taskContext, "成功", upLoadUrl);
            } catch (Exception e1) {
                e1.printStackTrace();
                fail(taskContext, "写入数据到Excel的过程中或者上传到阿里云中发生错误" + e1.getMessage());
                logger.error("写入数据到Excel的过程中或者上传到阿里云中发生错误" + e1.getMessage());
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            fail(taskContext, "论坛报表--按帖子统计导出过程中发生错误，请查看日志" + e.getMessage());
            logger.error("论坛报表--按帖子统计导出过程中发生错误，请查看日志" + e.getMessage());
        }
        return upLoadUrl;
    }

}
