package com.yizhi.statistics.application.controller.point;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.point.application.feign.PointDetailsFeignClients;
import com.yizhi.point.application.vo.domain.PointDetailsVo;
import com.yizhi.statistics.application.entity.PointLog;
import com.yizhi.statistics.application.enums.PointChangeTypeEnum;
import com.yizhi.statistics.application.mapper.PointLogMapper;
import com.yizhi.statistics.application.service.IPracticeChatLogService;
import com.yizhi.statistics.application.service.PointLogService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Component
public class PointDetailsSyncAbs extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(PointDetailsSyncAbs.class);

    @Autowired
    IPracticeChatLogService chatLogService;
    @Autowired
    IdGenerator idGenerator;
    @Autowired
    PointDetailsFeignClients pointDetailsClients;
    @Autowired
    PointLogMapper pointLogMapper;
    @Autowired
    PointLogService pointLogService;
    @Autowired
    RedisCache redisCache;

    @Override
    protected String execute(Map<String, Object> map){

        String startDateStr = (String) map.get("startDate");
        String endDateStr = (String) map.get("endDate");
        Long syncCompanyId = (Long) map.get("companyId");
        //入参 需要同步的站点id
        Long syncSiteId = (Long) map.get("siteId");
        LOGGER.info("报表同步积分明细数据开始;companyId ={},siteId ={} ,startDate = {},endDate = {}=======",syncCompanyId,syncSiteId,startDateStr,endDateStr);
        List<PointDetailsVo> list = pointDetailsClients.syncPointDetails(startDateStr, endDateStr, syncCompanyId, syncSiteId);
        if (CollectionUtils.isEmpty(list)){
            LOGGER.info("同步积分明细数据完成；明细列表为空；统计参数= {}", JSONObject.toJSONString(map));
            return "ok";
        }
        list.parallelStream().forEach(pointDetailsVo -> {
            try {
                PointLog pointLog = new PointLog();
                //拷贝数据到pointLog类中
                copyPointDetailProperties(pointDetailsVo,pointLog);
                pointLog.setId(idGenerator.generate());
                pointLogMapper.insert(pointLog);
            }catch (Exception e){
                LOGGER.warn("积分同步异常: 信息如下: ",e);
            }
        });
        LOGGER.info("同步积分明细完成;参数={}",map);
        return "ok";
    }

    /**
     * 拷贝数据
     */
    private PointLog copyPointDetailProperties(PointDetailsVo source, PointLog target) {

        target.setAccountId(source.getAccountId());
        target.setBeforeCount(source.getChangeBefore());
        target.setAfterCount(source.getChangeAfter());
        target.setAccountId(source.getAccountId());
        target.setCompanyId(source.getCompanyId());
        target.setSiteId(source.getSiteId());
        target.setOrgId(source.getOrgId());
        target.setChangeCount(source.getPoint());
        target.setCreateById(source.getCreateById());
        target.setCreateByName(source.getCreateByName());
        target.setCreatedAt(source.getCreateTime());
        target.setPointDetailId(source.getId());
        //变动原因名称；例如课程名、项目名
        target.setRelationName(source.getLearnName());
        //变动类型
        Integer changeType = getChangeType(source);
        if (PointChangeTypeEnum.forum.equals(changeType)){
            String relationName = getRelationName(source.getLearnType(),source.getLearnName());
            target.setRelationName(relationName);
        }
        target.setChangeType(changeType);
        return target;
    }

    /**
     * 获取变动愿意
     * @param learnType detail表的变动类型
     * @return 变动原因
     */
    private String getRelationName(String learnType,String learnName) {

        //论坛变动原因单独解析
        if (StringUtils.isEmpty(learnType)){
            LOGGER.warn("论坛learnType为空;learnName = {}",learnName);
            return learnName;
        }else if (learnType.contains("回复")){
            if (StringUtils.isEmpty(learnName)){
                return "回复帖子";
            }
            return "回复帖子-"+learnName;
        }else {
            if (StringUtils.isEmpty(learnName)){
                return "发表帖子";
            }
            return "发表帖子-"+learnName;
        }
    }

    /**
     * 获取变动类型
     * @param source 源积分明细数据
     * 现有生产数据库对应字段如下：
     *  变动类型	            现有数据库learn_source
     *  在线课程	            课程学习	    课程	课程测验
     *  线下课程	            线下课程学习记录	学习记录
     *  项目	            培训项目	学习项目
     *  考试	            考试
     *  作业	            作业
     *  调研	            调研
     *  原创活动	            案例活动
     *  打卡	            打卡               //(签到打卡成功发放）
     *  论坛	            论坛
     *  签到	            项目签到
     *  导入	            外部导入	导入积分	空15条
     *  陪练	            陪练
     *
     */
    private Integer getChangeType(PointDetailsVo source) {
        String learnType = source.getLearnType();
        Long learnSourceId = source.getLearnSourceId();
        String learnSource = source.getLearnSource();
        //根据元数据不同；返回对应的变动类型
        if (StringUtils.isEmpty(learnSource) || learnSource.equals("外部导入") || learnSource.equals("导入积分")){
            return PointChangeTypeEnum.externalImport;
        }else if (learnSource.equals("课程学习") || learnSource.equals("课程") || learnSource.equals("课程测验")){
            return PointChangeTypeEnum.onlineCourse;
        }else if (learnSource.equals("线下课程学习记录") || learnSource.equals("学习记录")){
            return PointChangeTypeEnum.offlineCourse;
        }else if (learnSource.equals("培训项目") || learnSource.equals("学习项目")){
            return PointChangeTypeEnum.trainingProject;
        }else if (learnSource.equals("考试")){
            return PointChangeTypeEnum.exam;
        }else if (learnSource.equals("作业")){
            return PointChangeTypeEnum.assignment;
        }else if (learnSource.equals("调研")){
            return PointChangeTypeEnum.research;
        }else if (learnSource.equals("案例活动")){
            return PointChangeTypeEnum.caselibrary;
        }else if (learnSource.equals("打卡")){
            //签到打卡
            return PointChangeTypeEnum.clockIn;
        }else if (learnSource.equals("论坛")){
            return PointChangeTypeEnum.forum;
        }else if (learnSource.equals("项目签到")){
            return PointChangeTypeEnum.sign;
        }else if (learnSource.equals("陪练")){
            return PointChangeTypeEnum.practice;
        }
        LOGGER.warn("报表同步积分明细;没有匹配到现有积分源类型!!learnSource={},learnType={},learnSourceId={}",learnSource,learnType,learnSourceId);
        return null;
    }
}