package com.yizhi.statistics.application.controller.point.export;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.request.point.StatisticsPointDetailQO;
import com.yizhi.statistics.application.service.PointLogService;
import com.yizhi.statistics.application.service.StatisticsAccountService;
import com.yizhi.statistics.application.vo.point.StatisticsPointDetailVO;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: PointExport
 * @author: zjl
 * @date: 2021/1/25  9:55
 */
@Component
public class PointPersonalExport extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(PointPersonalExport.class);

    @Autowired
    private PointLogService pointLogService;

    @Autowired
    private StatisticsAccountService statisticsAccountService;

    @Override
    protected String execute(Map<String, Object> map) {
        RequestContext context = (RequestContext) map.get("context");
        Long companyId = context.getCompanyId();
        Long siteId = context.getSiteId();
        Long accountId = context.getAccountId();
        StatisticsPointDetailQO qo = (StatisticsPointDetailQO)map.get("qo");
        String accountName = statisticsAccountService.getStatisticsAccountByAccountId(qo.getAccountId()).getAccountName();

        qo.setPageSize(Integer.MAX_VALUE);

        Long taskId = (Long) map.get("taskId");
        String taskName = (String) map.get("taskName");
        String serialNo = (String) map.get("serialNo");


        String upLoadUrl = null;
        String requestPath = FileConstant.SAVE_PATH;
        File fileDir = new File(requestPath);
        if (!fileDir.exists()) {
            fileDir.mkdir();
        }
        Date submitTime = new Date();
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);
        working(taskContext);

        Page<StatisticsPointDetailVO> page = null;
        List<StatisticsPointDetailVO> list = null;
        try {
            page = pointLogService.getStatisticsPoint(qo, context);
            if (null != page) {
                list = page.getRecords();
                if (CollectionUtils.isEmpty(list)) {
                    success(taskContext, "未找到数据", "");
                }
            }
        } catch (Exception e) {
            LOGGER.error("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }
        try {
            //excel生成：excel-sheet-row-cell
            HSSFWorkbook workbook = new HSSFWorkbook();
            HSSFSheet sheet = workbook.createSheet("积分明细表" + "-"+ accountName + "-" + DateUtil.format(submitTime, "yyyyMMddHHmmss"));
            // 在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short,
            // 创建第1行
            //创建单元格格式
            HSSFCellStyle style = workbook.createCellStyle();
            style.setAlignment(HorizontalAlignment.LEFT);//创建居左格式
            HSSFRow row = sheet.createRow((int) 0);
            //合并单元格
            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 5); // 起始行, 终止行, 起始列, 终止列
            sheet.addMergedRegion(cra);
            HSSFCell cell = row.createCell((short) 0);
            cell.setCellValue("积分明细列表" + "-"+ accountName + "-" + DateUtil.format(submitTime, "yyyyMMdd"));

            row = sheet.createRow((int) 1);
            cra = new CellRangeAddress(1, 1, 0, 5); // 起始行, 终止行, 起始列, 终止列
            sheet.addMergedRegion(cra);
            String startDate = DateUtil.formatDate(qo.getStartAt());
            String endDate = DateUtil.formatDate(qo.getEndAt());
            row.createCell((short) 0).setCellValue("统计周期:" + startDate + "～" + endDate);

            row = sheet.createRow((int) 2);
            row.createCell((short) 0).setCellValue("时间");
            row.createCell((short) 1).setCellValue("变动量");
            row.createCell((short) 2).setCellValue("变动类型");
            row.createCell((short) 3).setCellValue("变动原因");
            row.createCell((short) 4).setCellValue("当期剩余");
            row.createCell((short) 5).setCellValue("累计剩余");

            int index = 3;
            int s = 0;
            for (StatisticsPointDetailVO statisticsPointDetailVO : list) {
                if (statisticsPointDetailVO == null) {
                    continue;
                }
                row = sheet.createRow((int) index);
                cell = row.createCell((short) 0);
                if (statisticsPointDetailVO.getCreatedAt() != null) {
                    cell.setCellValue(DateUtil.formatDateTime(statisticsPointDetailVO.getCreatedAt()));
                }
                cell = row.createCell((short) 1);
                if (statisticsPointDetailVO.getChangeCount() != null) {
                    cell.setCellValue(statisticsPointDetailVO.getChangeCount());
                }
                cell = row.createCell((short) 2);
                if (statisticsPointDetailVO.getChangeType() != null) {
                    cell.setCellValue(statisticsPointDetailVO.getChangeType());
                }
                cell = row.createCell((short) 3);
                if (statisticsPointDetailVO.getRelationName() != null) {
                    cell.setCellValue(statisticsPointDetailVO.getChangeType() + ":" + statisticsPointDetailVO.getRelationName());
                }
                cell = row.createCell((short) 4);
                if (statisticsPointDetailVO.getCurrentRemain() != null) {
                    cell.setCellValue(statisticsPointDetailVO.getCurrentRemain());
                }
                cell = row.createCell((short) 5);
                if (statisticsPointDetailVO.getCumulativeRemain() != null) {
                    cell.setCellValue(statisticsPointDetailVO.getCumulativeRemain());
                }
                index++;
                s++;
            }
            StringBuffer fileNameSb = new StringBuffer().append("积分明细表").append("-").append(accountName).append("-").append(DateUtil.format(submitTime, "yyyyMMddHHmmss")).append(".xls");
            String fileName = fileNameSb.toString();
            String path = new StringBuffer().append(requestPath).append(fileNameSb).toString();
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                workbook.write(os);
                //阿里云返回url
                upLoadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功", upLoadUrl);
            } catch (Exception e1) {
                LOGGER.error("写入数据到Excel的过程中或者上传到阿里云中发生错误", e1);
                fail(taskContext, "写入过程中发生错误");

            } finally {
                if (os != null) {
                    os.close();
                }
                if (workbook != null) {
                    workbook.close();
                }
                if (file != null) {
                    file.delete();
                }
            }
        } catch (Exception e) {
            LOGGER.error("创建Excel的过程中发生错误", e);
            fail(taskContext, "签到统计导出过程中发生错误，请查看日志");
        }
        return upLoadUrl;
    }
}
