package com.yizhi.statistics.application.controller.practice;

import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.practice.application.feign.PracticeConfigClient;
import com.yizhi.practice.application.pojo.vo.PracticeVo;
import com.yizhi.statistics.application.entity.PracticeConfig;
import com.yizhi.statistics.application.service.IPracticeConfigService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Component
public class PracticeAbs extends AbstractDefaultTask<String, PracticeTaskParam> {

    private static final Logger logger = LoggerFactory.getLogger(PracticeAbs.class);

    final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");


    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private PracticeConfigClient practiceConfigClient;
    @Autowired
    private IPracticeConfigService practiceConfigService;


    @Override
    public String execute(PracticeTaskParam param) {

    	logger.info("PracticeAbs - execute start.");
        Date curDateWork = new Date();
        Long taskId = idGenerator.generate();
        String taskName = "智能陪练同步前一天的陪练表数据"+sim.format(curDateWork);
        Long t1 = System.currentTimeMillis();

        TaskContext taskContext = new TaskContext(taskId, taskName, 999L, curDateWork);
        working(taskContext);
        try {
            List<PracticeVo> practiceVoList = practiceConfigClient.selectRangeList(param.getStartDate(), param.getEndDate());
            if (null != practiceVoList && !practiceVoList.isEmpty()){
                taskDetail(taskId, "智能陪练同步前一天的陪练数据,practiceListSize="+practiceVoList.size());
                List<PracticeConfig> practiceConfigList = new ArrayList<>();
                for (PracticeVo practiceVo : practiceVoList) {
                    PracticeConfig practiceConfig = new PracticeConfig();
                    BeanUtils.copyProperties(practiceVo,practiceConfig);
                    practiceConfigList.add(practiceConfig);
                }
                logger.info("Ready to batch handle {} datas.", practiceConfigList.size());

                practiceConfigList.stream().forEach(it -> {
                    try{
                        practiceConfigService.insertOrUpdate(it);
                    }catch (Exception e) {
                        logger.error("智能陪练 陪练配置 同步异常: ", e);
                    }
                });
                taskDetail(taskId, "智能陪练同步前一天的陪练数据完成,practiceListSize="+practiceVoList.size());
            }
        } catch (Exception e) {
            taskDetail(taskId, "智能陪练同步practiceConfig异常: "+e.getMessage());
           logger.error("智能陪练同步practiceConfig异常: ",e);
        }
        
        logger.info("PracticeAbs - execute end.");
        return null;
    }


}
