package com.yizhi.statistics.application.controller.practice;

import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.practice.application.enums.PracticeVisibleTypeEnum;
import com.yizhi.practice.application.feign.PracticeConfigClient;
import com.yizhi.practice.application.pojo.vo.PracticeVisibleVo;
import com.yizhi.statistics.application.entity.PracticeVisible;
import com.yizhi.statistics.application.entity.StatisticsAccount;
import com.yizhi.statistics.application.service.IPracticeVisibleService;
import com.yizhi.statistics.application.service.StatisticsAccountService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Component
public class PracticeVisibleAbs extends AbstractDefaultTask<String, PracticeTaskParam> {

    private static final Logger logger = LoggerFactory.getLogger(PracticeVisibleAbs.class);

    final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");
    @Autowired
    private StatisticsAccountService statisticsAccountService;


    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private PracticeConfigClient practiceConfigClient;
    @Autowired
    private IPracticeVisibleService visibleService;


    @Override
    public String execute(PracticeTaskParam param) {

        logger.info("========PracticeVisibleAbs start==========");
        Long taskId = idGenerator.generate();
        try {
            Date curDateWork = new Date();
            String taskName = "智能陪练同步前一天陪练可见范围表数据" + sim.format(curDateWork);

            TaskContext taskContext = new TaskContext(taskId, taskName, 999L, curDateWork);
            working(taskContext);

            List<PracticeVisibleVo> practiceVisibleVoList = practiceConfigClient.selectVisibleRangeList(param.getStartDate(), param.getEndDate());
            if (null == practiceVisibleVoList || practiceVisibleVoList.isEmpty()) {
                logger.info("========PracticeVisibleAbs running========== size:0");
                return null;
            }
            logger.info("========PracticeVisibleAbs running========== size:{}",practiceVisibleVoList.size());
            taskDetail(taskId, "智能陪练同步前一天陪练可见范围表数据,practiceVisibleVoListSize=" + practiceVisibleVoList.size());
            Map<Boolean, List<PracticeVisibleVo>> visibleMap = practiceVisibleVoList.stream().collect(Collectors.groupingBy(PracticeVisibleVo::getDeleted));

            // 1，删除deleted=1
            List<PracticeVisibleVo> deleteList = visibleMap.get(true);
            Map<Long, List<PracticeVisibleVo>> deletedMap = deleteList.stream().collect(Collectors.groupingBy(PracticeVisibleVo::getPracticeId));
            deletedMap.forEach((key, value) -> {
                List<Long> deletedOrgIds = new ArrayList<>();
                List<Long> deletedAccountIds = new ArrayList<>();
                value.stream().forEach(practiceVisibleVo -> {
                    deletedOrgIds.add(practiceVisibleVo.getVisibleOrgId());
                    deletedAccountIds.add(practiceVisibleVo.getVisibleAccountId());
                });
                visibleService.physicalDeletedVisible(deletedOrgIds, deletedAccountIds, key);
            });

            // 2，新增
            List<PracticeVisibleVo> list = visibleMap.get(false);
            Map<Integer, List<PracticeVisibleVo>> visibleTypeMap = list.stream().collect(Collectors.groupingBy(PracticeVisibleVo::getVisibleType));

            // 2-1查询用户查询部门后新增
            List<PracticeVisibleVo> accountVisibleList = visibleTypeMap.get(PracticeVisibleTypeEnum.PART.getStatus());
            List<PracticeVisible> accountVisible = new ArrayList<>();
            accountVisibleList.stream().forEach(it -> {
                StatisticsAccount account = statisticsAccountService.getStatisticsAccountByAccountId(it.getVisibleAccountId());
                PracticeVisible visible = new PracticeVisible();
                BeanUtils.copyProperties(it, visible);
                visible.setVisibleId(it.getId());
                visible.setId(null);
                visible.setVisibleOrgId(account.getOrgId());
                visible.setVisibleOrgName(account.getOrgName());
                accountVisible.add(visible);
            });

            accountVisible.stream().forEach(it -> {
                try {
                    visibleService.insert(it);
                } catch (Exception e) {
                    logger.error("新增 visible error", e);
                }
            });

            // 2-2部门，查询用户
            List<PracticeVisibleVo> orgVisibleList = visibleTypeMap.get(PracticeVisibleTypeEnum.ORG.getStatus());
            List<PracticeVisible> orgAccountVisible = new ArrayList<>();
            orgVisibleList.stream().forEach(it -> {
                List<StatisticsAccount> accountList = statisticsAccountService.getAccountByOrgId(it.getCompanyId(), it.getVisibleOrgId());
                accountList.stream().forEach(account -> {
                    PracticeVisible visible = new PracticeVisible();
                    BeanUtils.copyProperties(it, visible);
                    visible.setVisibleId(it.getId());
                    visible.setVisibleAccountId(account.getAccountId());
                    visible.setVisibleAccountFullName(account.getFullName());
                    visible.setVisibleAccountName(account.getAccountName());
                    visible.setVisibleAccountWorkNum(account.getWorkNum());
                    orgAccountVisible.add(visible);
                });
            });

            orgAccountVisible.stream().forEach(it -> {
                try {
                    visibleService.insert(it);
                } catch (Exception e) {
                    logger.error("智能陪练 可见范围同步异常: ", e);
                }
            });
            logger.info("========PracticeVisibleAbs end==========");

            taskDetail(taskId, "跑批-智能陪练可见范围,accountSize:" + accountVisible.size() + "----orgSize:" + orgAccountVisible.size());
        } catch (Exception e) {
            taskDetail(taskId, "智能陪练同步前一天陪练可见范围表数据异常: " + e.getMessage());
            logger.error("智能陪练同步前一天陪练可见范围表数据异常: ", e);
        }
        return null;
    }

}
