package com.yizhi.statistics.application.controller.practice.export;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.request.practice.PracticeReportByUserReq;
import com.yizhi.statistics.application.service.IPracticeChatLogService;
import com.yizhi.statistics.application.util.CreateFormartExcelUtils;
import com.yizhi.statistics.application.vo.practice.PracticeReportByUserListVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 智能陪练统计-按部门统计导出
 *
 * @ClassName: ExportPracticeByOrg
 */

@Component
public class ExportPracticeByOrgListDetailExport extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExportPracticeByOrgListDetailExport.class);

    @Autowired
    private IPracticeChatLogService iPracticeChatLogService;

    @Override
    protected String execute(Map<String, Object> map) {

        String fileUrl = "";
        String taskName = "";
        TaskContext taskContext = null;
        try {
            RequestContext requestContext = (RequestContext) map.get("context");
            PracticeReportByUserReq param = (PracticeReportByUserReq) map.get("param");
            Long taskId = (Long) map.get("taskId");
            String serialNo = (String) map.get("serialNo");
            taskName = (String) map.get("taskName");
            taskContext = new TaskContext(taskId, serialNo, taskName,
                    requestContext.getAccountId(), new Date(), requestContext.getSiteId(), requestContext.getCompanyId());
            LOGGER.info(taskName + " 任务开始之行");
            working(taskContext);

            Date date = new Date();
            SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
            SimpleDateFormat sdf3 = new SimpleDateFormat("yyyyMMdd");

            String raw1 = new StringBuffer("部门用户陪练统计").append(sdf3.format(date)).toString();
            String fileName = raw1 + ".xlsx";
            String raw2 = new StringBuffer("统计周期：").append(DateUtil.formatDate(param.getDateFrom())).append("~").append(DateUtil.formatDate(param.getDateTo())).toString();
            param.setPageSize(Integer.MAX_VALUE);
            LOGGER.info(taskName + " 组装数据");
            List<PracticeReportByUserListVo> datas = iPracticeChatLogService.reportByUserList(param, requestContext);

            List<String> headNames = new ArrayList<String>();
            headNames.add("部门名称");
            headNames.add("所在组织架构");
            headNames.add("用户名");
            headNames.add("姓名");

            headNames.add("可参加训练数");
            headNames.add("实际参加训练数");
            headNames.add("参加训练次数");

            headNames.add("可参加考核数");
            headNames.add("实际参加考核数");
            headNames.add("参加考核次数");
            headNames.add("通过考核数");
            headNames.add("考核通过率%");

            List<JSONObject> data = new ArrayList<JSONObject>();
            for (PracticeReportByUserListVo vo : datas) {
                JSONObject obj = new JSONObject();
                obj.put("部门名称", vo.getDepart());
                obj.put("所在组织架构", vo.getOrgInfo());
                obj.put("用户名", vo.getAccount());
                obj.put("姓名", vo.getUserName());
                obj.put("可参加训练数", vo.getPracticeNumberShould());
                obj.put("实际参加训练数", vo.getPracticeNumberActual());
                obj.put("参加训练次数", vo.getPracticeCount());
                obj.put("可参加考核数", vo.getExamNumberShould());
                obj.put("实际参加考核数", vo.getExamNumberActual());
                obj.put("参加考核次数", vo.getExamCount());
                obj.put("通过考核数", vo.getExamNumberPassed());
                obj.put("考核通过率%", vo.getPassRate());
                data.add(obj);
            }
            fileUrl = CreateFormartExcelUtils.createFormartExcel(
                    fileName,
                    raw1,
                    raw2,
                    headNames,
                    null,
                    null,
                    data
            );
            LOGGER.info(taskName + " success");
            success(taskContext, "成功", fileUrl);
        } catch (Exception e) {
            LOGGER.info(taskName + " fail");
            fail(taskContext, "智能陪练-报表-按人员-详情-下载，写入过程中发生错误");
            LOGGER.error("智能陪练-报表-按人员-详情-下载,写入过程中发生错误.", e);
        }
        return fileUrl;
    }
}
