package com.yizhi.statistics.application.controller.practice.export;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.request.practice.PracticeByOrgPageReq;
import com.yizhi.statistics.application.service.PracticeByOrgService;
import com.yizhi.statistics.application.util.CreateFormartExcelUtils;
import com.yizhi.statistics.application.vo.practice.PracticeStatByOrgVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 智能陪练统计-按部门统计导出
 *
 * @ClassName: ExportPracticeByOrg
 */

@Component
public class ExportPracticeByOrgListExport extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExportPracticeByOrgListExport.class);
    @Autowired
    private PracticeByOrgService practiceByOrgService;

    @Override
    protected String execute(Map<String, Object> map) {
        String serialNo = null;
        String taskName = null;
        TaskContext taskContext = null;
        String fileUrl = "";
        try {
        RequestContext requestContext = (RequestContext) map.get("context");
        PracticeByOrgPageReq param = (PracticeByOrgPageReq) map.get("param");
        Long taskId = (Long) map.get("taskId");
        serialNo = (String) map.get("serialNo");
        taskName = (String) map.get("taskName");
        taskContext = new TaskContext(taskId, serialNo, taskName,
                requestContext.getAccountId(), new Date(), requestContext.getSiteId(), requestContext.getCompanyId());
        LOGGER.info(taskName +" 任务开始之行");
        working(taskContext);

        Date date = new Date();
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf3 = new SimpleDateFormat("yyyyMMdd");

        String raw1 = new StringBuffer("按部门统计报表").append(sdf3.format(date)).toString();
        String fileName = raw1 + ".xlsx";
        String raw2 = new StringBuffer("统计周期：").append(param.getStartDate()).append("~").append(param.getEndDate()).toString();
        param.setPageSize(Integer.MAX_VALUE);

        LOGGER.info(taskName +" 开始组装数据");
        List<PracticeStatByOrgVo> datas = practiceByOrgService.getLogByOrg(param, requestContext).getRecords();

        List<String> headNames = new ArrayList<String>();
        headNames.add("部门名称");
        headNames.add("可参加训练数");
        headNames.add("可参加训练人数");
        headNames.add("实际参加训练人次");

        headNames.add("可参加考核数");
        headNames.add("可参加考核人数");
        headNames.add("实际参加考核人次");
        headNames.add("考核通过人次");
        headNames.add("通过率(%)");

        List<JSONObject> data = new ArrayList<JSONObject>();
        for (PracticeStatByOrgVo vo : datas) {
            JSONObject obj = new JSONObject();
            obj.put("部门名称", vo.getOrgName());
            obj.put("可参加训练数", vo.getPlanJoinPracticeCount());
            obj.put("可参加训练人数", vo.getPlanJoinExamUserCount());
            obj.put("实际参加训练人次", vo.getActualJoinPracticeUserCount());

            obj.put("可参加考核数", vo.getPlanJoinExamCount());
            obj.put("可参加考核人数", vo.getPlanJoinExamUserCount());
            obj.put("实际参加考核人次", vo.getActualJoinExamUserCount());
            obj.put("考核通过人次", vo.getExamPassUserCount());
            obj.put("通过率(%)", vo.getExamPassRate());
            data.add(obj);
        }
        fileUrl = "";

            fileUrl = CreateFormartExcelUtils.createFormartExcel(
                    fileName,
                    raw1,
                    raw2,
                    headNames,
                    null,
                    null,
                    data
            );
            LOGGER.info(taskName +" success");
            success(taskContext, "成功", fileUrl);
        } catch (Exception e) {
            LOGGER.info(taskName +" fail");
            fail(taskContext, "智能陪练-报表-按部门导出发生错误"+e.getMessage());
            LOGGER.error("智能陪练-报表-按部门导出发生错误", e);
        }
        return fileUrl;
    }
}
