package com.yizhi.statistics.application.controller.practice.export;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.domain.PracticeByOrgUserDetailDto;
import com.yizhi.statistics.application.request.practice.PracticeReportByUserReq;
import com.yizhi.statistics.application.service.PracticeByOrgService;
import com.yizhi.statistics.application.util.CreateFormartExcelUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 智能陪练统计-按部门统计导出
 *
 * @ClassName: ExportPracticeByOrg
 */

@Component
public class ExportPracticeByOrgListUserDetailExport extends AbstractDefaultTask<String, Map<String, Object>> {

    @Autowired
    private PracticeByOrgService practiceByOrgService;
    private Logger LOGGER = LoggerFactory.getLogger(ExportPracticeByOrgListUserDetailExport.class);

    @Override
    protected String execute(Map<String, Object> map) {
        LOGGER.info("任务开始之行");
        String fileUrl = "";
        TaskContext taskContext = null;
        String taskName = null;
        try {
            RequestContext requestContext = (RequestContext) map.get("context");
            PracticeReportByUserReq param = (PracticeReportByUserReq) map.get("param");
            Long taskId = (Long) map.get("taskId");
            String serialNo = (String) map.get("serialNo");
            taskName = (String) map.get("taskName");
            taskContext = new TaskContext(taskId, serialNo, taskName,
                    requestContext.getAccountId(), new Date(), requestContext.getSiteId(), requestContext.getCompanyId());
            LOGGER.info(taskName + " 任务开始之行");
            working(taskContext);

            Date date = new Date();
            SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
            SimpleDateFormat sdf3 = new SimpleDateFormat("yyyyMMdd");

            String raw1 = new StringBuffer("部门用户陪练明细").append(sdf3.format(date)).toString();
            String fileName = raw1 + ".xlsx";
            String raw2 = new StringBuffer("统计周期：").append(sdf1.format(param.getDateFrom())).append("~").append(sdf1.format(param.getDateTo())).toString();

            LOGGER.info(taskName + "开始组装数据");
            List<PracticeByOrgUserDetailDto> list = practiceByOrgService.getUserPracticeByOrgId(param, requestContext);

            List<String> headNames = new ArrayList<String>();
            headNames.add("部门名称");
            headNames.add("所在组织架构");
            headNames.add("用户名");
            headNames.add("姓名");

            headNames.add("陪练名称");
            headNames.add("陪练模式");
            headNames.add("完成时间");
            headNames.add("得分");
            headNames.add("考核结果");

            List<JSONObject> data = new ArrayList<JSONObject>();

            for (PracticeByOrgUserDetailDto vo : list) {
                JSONObject obj = new JSONObject();
                obj.put("部门名称", vo.getOrgName());
                obj.put("所在组织架构", vo.getOrgNames());
                obj.put("用户名", vo.getAccountName());
                obj.put("姓名", vo.getFullName());

                obj.put("陪练名称", vo.getPracticeName());
                obj.put("陪练模式", vo.getUseTypeStr());
                obj.put("完成时间", DateUtil.formatDateTime(vo.getEndAt()));
                obj.put("得分", vo.getScore());
                obj.put("考核结果", vo.getExamResultStr());
                data.add(obj);
            }
            fileUrl = CreateFormartExcelUtils.createFormartExcel(
                    fileName,
                    raw1,
                    raw2,
                    headNames,
                    null,
                    null,
                    data
            );
            LOGGER.info(taskName + " success");
            success(taskContext, "成功", fileUrl);
        } catch (Exception e) {
            LOGGER.error("智能陪练-报表-用户加陪练明细导出失败", e);
            fail(taskContext, "智能陪练-报表-用户加陪练明细导出失败" + e.getMessage());
            LOGGER.info(taskName + " 导出失败");
        }
        return fileUrl;
    }

}
