package com.yizhi.statistics.application.controller.practice.export;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.request.practice.PracticeStatByPracticeReq;
import com.yizhi.statistics.application.service.IPracticeChatLogService;
import com.yizhi.statistics.application.util.CreateFormartExcelUtils;
import com.yizhi.statistics.application.vo.practice.PracticeStatByPracticeVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 智能陪练统计-按陪练统计导出
 * @ClassName: ExportPracticeByPrac
 * @author: zjl
 * @date: 2020/11/10  10:18
 */

@Component
public class ExportPracticeByPrac extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExportPracticeByPrac.class);

    @Autowired
    IPracticeChatLogService chatLogService;

    @Override
    protected String execute(Map<String, Object> map){
        RequestContext context = (RequestContext) map.get("context");
        PracticeStatByPracticeReq req = (PracticeStatByPracticeReq) map.get("req");
        Long companyId = context.getCompanyId();
        Long siteId = context.getSiteId();
        Long accountId = context.getAccountId();

        Long taskId = (Long) map.get("taskId");
        String taskName = (String) map.get("taskName");
        String serialNo = (String) map.get("serialNo");
        Date submitTime = new Date();
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);
        working(taskContext);

        Page<PracticeStatByPracticeVo> page = new Page<>();
        List<PracticeStatByPracticeVo> practiceList = new ArrayList<>();
        try {
            Integer pageSize = Integer.MAX_VALUE;
            req.setPageSize(pageSize);
            page = chatLogService.practiceGroup(context, req);
            if (null != page) {
                practiceList = page.getRecords();
                if (CollectionUtils.isEmpty(practiceList)) {
                    success(taskContext, "未找到数据", "");
                }
            }
        } catch (Exception e) {
            LOGGER.error("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }
        Date date = new Date();
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMddHHmmss");
        SimpleDateFormat sdf3 = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        String fileName = new StringBuffer().append("按陪练统计报表").append(sdf2.format(date)).append(".xlsx").toString();
        String raw1 = new StringBuffer().append("按陪练统计报表").toString();
        String raw2 = new StringBuffer("统计周期：").append(sdf1.format(req.getStartDate())).append(" ~ ").append(sdf1.format(req.getEndDate())).toString();

        List<String> headNames = new ArrayList<String>();
        headNames.add("陪练名称");
        headNames.add("练习时间");
        headNames.add("考核时间");
        headNames.add("可参加人数");
        headNames.add("训练人数");
        headNames.add("训练参与率");
        headNames.add("考核人数");
        headNames.add("考核通过人数");
        headNames.add("通过率");
        headNames.add("考核平均分");
        headNames.add("考核完成率");

        List<JSONObject> data = new ArrayList<JSONObject>();
        for (PracticeStatByPracticeVo byPracticeVo : practiceList) {
            JSONObject obj = new JSONObject();
            obj.put("陪练名称",byPracticeVo.getName() );
            obj.put("练习时间",getStringTime(sdf3,byPracticeVo.getPracticeStartAt(),byPracticeVo.getPracticeEndAt()) );
            obj.put("考核时间",getStringTime(sdf3,byPracticeVo.getExamStartAt(),byPracticeVo.getExamEndAt()) );
            obj.put("可参加人数",byPracticeVo.getCanJoinCount() );
            obj.put("训练人数", byPracticeVo.getPracticeTrainingCount());
            obj.put("训练参与率", byPracticeVo.getPracticeTrainingRate());
            obj.put("考核人数", byPracticeVo.getPracticeTestCount());
            obj.put("考核通过人数", byPracticeVo.getPracticeTestPassCount());
            obj.put("通过率", byPracticeVo.getPracticeTestPassRate());
            obj.put("考核平均分", byPracticeVo.getPracticeTestAverage());
            obj.put("考核完成率", byPracticeVo.getPracticeTestDoneRate());
            data.add(obj);
        }

        try {
            String fileUrl = CreateFormartExcelUtils.createFormartExcel(fileName, raw1, raw2, headNames, null, null, data);
            success(taskContext, "按陪练统计导出成功", fileUrl);
            LOGGER.info("按陪练统计导出下载地址: {}",fileUrl);
            return fileUrl;

        } catch (IOException e) {
            LOGGER.error("按陪练统计导出过程中出现错误", e);
            fail(taskContext, "按陪练统计导出过程中出现错误"+e.getMessage());
        }
        return null;
    }

    //获取报表显示时间
    private String getStringTime(SimpleDateFormat format, Date startDate,Date endDate) {
        if (null == startDate || null == endDate){
            return "/";
        }else {
            String startTime = format.format(startDate);
            String endTime = format.format(endDate);
            return startTime + " ~ " + endTime;
        }
    }
}
