package com.yizhi.statistics.application.controller.practice.export;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.entity.PracticeConfig;
import com.yizhi.statistics.application.mapper.PracticeConfigMapper;
import com.yizhi.statistics.application.request.practice.PracticeStatByPracticeIdReq;
import com.yizhi.statistics.application.service.IPracticeChatLogService;
import com.yizhi.statistics.application.util.CreateFormartExcelUtils;
import com.yizhi.statistics.application.vo.practice.PracticeStatViewByIdVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 智能陪练统计-按陪练统计-详情-下载陪练结果明细
 * @ClassName: ExportPracticeByPracView
 * @author: zjl
 * @date: 2020/11/11  11:02
 */
@Component
public class ExportPracticeByPracView extends AbstractDefaultTask<String, Map<String, Object>> {
    private static final Logger LOGGER = LoggerFactory.getLogger(ExportPracticeByPracView.class);

    @Autowired
    IPracticeChatLogService chatLogService;
    @Autowired
    PracticeConfigMapper practiceConfigMapper;

    @Override
    protected String execute(Map<String, Object> map){
        RequestContext context = (RequestContext) map.get("context");
        PracticeStatByPracticeIdReq req = (PracticeStatByPracticeIdReq) map.get("req");
        Long companyId = context.getCompanyId();
        Long siteId = context.getSiteId();
        Long accountId = context.getAccountId();

        Long taskId = (Long) map.get("taskId");
        String taskName = (String) map.get("taskName");
        String serialNo = (String) map.get("serialNo");
        Long practiceId = req.getPracticeId();
        PracticeConfig practiceConfig = practiceConfigMapper.selectById(practiceId);
        String practiceName = "";
        if (null != practiceConfig){
            practiceName = practiceConfig.getName();
        }

        Date submitTime = new Date();
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);
        working(taskContext);

        Page<PracticeStatViewByIdVo> page = new Page<>();
        List<PracticeStatViewByIdVo> practiceViewList = new ArrayList<>();
        try {
            Integer pageSize = Integer.MAX_VALUE;
            req.setPageSize(pageSize);
            page = chatLogService.practiceGroupView(req,context);
            if (null != page) {
                practiceViewList = page.getRecords();
                if (CollectionUtils.isEmpty(practiceViewList)) {
                    success(taskContext, "未找到数据", "");
                }
            }
        } catch (Exception e) {
            LOGGER.error("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }

        Date date = new Date();
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMddHHmmss");
        SimpleDateFormat sdf3 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String fileName = new StringBuffer().append(practiceName).append("-陪练结果统计报表").append(sdf2.format(date)).append(".xlsx").toString();
        String raw1 = new StringBuffer().append(practiceName).append("-陪练结果数据").toString();
        String raw2 = new StringBuffer("统计周期：").append(sdf1.format(req.getStartDate())).append(" ~ ").append(sdf1.format(req.getEndDate())).toString();

        List<String> headNames = new ArrayList<String>();
        headNames.add("用户名");
        headNames.add("姓名");
        headNames.add("训练次数");
        headNames.add("考核次数");
        headNames.add("首次考核时间");
        headNames.add("最高分");
        headNames.add("考核结果");
        headNames.add("获得积分");
        headNames.add("所在部门");
        headNames.add("所在组织架构");

        List<JSONObject> data = new ArrayList<JSONObject>();
        for (PracticeStatViewByIdVo practiceViewVo : practiceViewList) {
            JSONObject obj = new JSONObject();
            obj.put("用户名", practiceViewVo.getAccountName());
            obj.put("姓名", practiceViewVo.getFullName());
            obj.put("训练次数", practiceViewVo.getTrainingCount());
            obj.put("考核次数", practiceViewVo.getTestCount());
            Date firstTestDate = practiceViewVo.getFirstTestDate();
            if (null == firstTestDate){
                obj.put("首次考核时间","/");
            }else {
                obj.put("首次考核时间", sdf3.format(firstTestDate));
            }
            obj.put("最高分", practiceViewVo.getHighestScore());
            String passResult = practiceViewVo.getPassResult();
            obj.put("考核结果", "80".equals(passResult)? "通过": "不通过");
            obj.put("获得积分", practiceViewVo.getPoint());
            obj.put("所在部门", practiceViewVo.getOrgName());
            obj.put("所在组织架构", practiceViewVo.getOrgNames());
            data.add(obj);
        }
        try {
            String fileUrl = CreateFormartExcelUtils.createFormartExcel(fileName, raw1, raw2, headNames, null, null, data);
            success(taskContext, "陪练结果导出成功", fileUrl);
            LOGGER.info("陪练结果下载地址: {}",fileUrl);
            return fileUrl;

        } catch (IOException e) {
            LOGGER.error("陪练结果下载过程中出现错误", e);
            fail(taskContext, "陪练结果下载过程中出现错误"+e.getMessage());
        }
        return null;
    }
}
