package com.yizhi.statistics.application.controller.practice.export;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.request.practice.PracticeReportByUserReq;
import com.yizhi.statistics.application.service.IPracticeChatLogService;
import com.yizhi.statistics.application.util.CreateFormartExcelUtils;
import com.yizhi.statistics.application.vo.practice.PracticeReportByUserListVo;

import lombok.extern.slf4j.Slf4j;

/**
 * 智能陪练-报表-按用户-列表页-下载excel
 * @ClassName: ExportPracticeByUser
 */

@Slf4j
@Component
public class ExportPracticeByUser extends AbstractDefaultTask<String, Map<String, Object>> {

	@Autowired
	private IPracticeChatLogService chatLogService;
	
    @Override
    protected String execute(Map<String, Object> map) {
    	RequestContext requestContext = (RequestContext) map.get("context");
    	PracticeReportByUserReq param = (PracticeReportByUserReq) map.get("param");
    	Long taskId = (Long) map.get("taskId");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,
                requestContext.getAccountId(), new Date(), requestContext.getSiteId(), requestContext.getCompanyId());
        working(taskContext);
        
        List<PracticeReportByUserListVo> datas = chatLogService.reportByUserList(param, requestContext);
        
        Date date = new Date();
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy/MM/dd");
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMddHHmmss");
        SimpleDateFormat sdf3 = new SimpleDateFormat("yyyyMMdd");
        String fileName = new StringBuffer("按用户统计报表结果").append(sdf2.format(date)).append(".xlsx").toString();
        String raw1 = new StringBuffer("按用户统计报表结果").append(sdf3.format(date)).toString();
        String raw2 = new StringBuffer("统计周期：").append(sdf1.format(param.getDateFrom())).append("~").append(sdf1.format(param.getDateTo())).toString();
        
        List<String> headNames = new ArrayList<String>();
        headNames.add("用户名");
        headNames.add("姓名");
        headNames.add("可参加训练数");
        headNames.add("实际参加训练数");
        headNames.add("训练次数");
        headNames.add("应参加考核数");
        headNames.add("参加考核数");
        headNames.add("已通过考核数");
        headNames.add("通过率");
        headNames.add("所在部门");
        headNames.add("所在组织架构");
        
        List<JSONObject> data = new ArrayList<JSONObject>();
        for (PracticeReportByUserListVo vo : datas) {
            JSONObject obj = new JSONObject();
            obj.put("用户名", vo.getAccount());
            obj.put("姓名", vo.getUserName());
            obj.put("可参加训练数", vo.getPracticeNumberShould());
            obj.put("实际参加训练数", vo.getPracticeNumberActual());
            obj.put("训练次数", vo.getPracticeCount());
            obj.put("应参加考核数", vo.getExamNumberShould());
            obj.put("参加考核数", vo.getExamNumberActual());
            obj.put("已通过考核数", vo.getExamNumberPassed());
            obj.put("通过率", vo.getPassRate()+"%");
            obj.put("所在部门", vo.getDepart());
            obj.put("所在组织架构", vo.getOrgInfo());
            data.add(obj);
        }
        
        String fileUrl = "";
        try {
            fileUrl = CreateFormartExcelUtils.createFormartExcel(
                    fileName,
                    raw1,
                    raw2,
                    headNames,
                    null,
                    null,
                    data
            );
            success(taskContext, "成功", fileUrl);
        } catch (Exception e) {
            fail(taskContext, "智能陪练-报表-按人员-列表-下载，写入过程中发生错误");
            log.error("智能陪练-报表-按人员-列表-下载,写入过程中发生错误.", e);
        }
        return fileUrl;
    }
}
