package com.yizhi.statistics.application.controller.practice.export;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.practice.application.enums.PracticeLevelMapEnum;
import com.yizhi.practice.application.enums.PracticePassResultEnum;
import com.yizhi.practice.application.enums.PracticeUseTypeEnum;
import com.yizhi.statistics.application.request.practice.PracticeReportByUserDetailReq;
import com.yizhi.statistics.application.service.IPracticeChatLogService;
import com.yizhi.statistics.application.util.CreateFormartExcelUtils;
import com.yizhi.statistics.application.vo.practice.PracticeReportByUserDetailVo;

import lombok.extern.slf4j.Slf4j;

/**
 * 智能陪练-报表-按用户-详情页-下载excel
 * @ClassName: ExportPracticeByUserDetail
 */

@Slf4j
@Component
public class ExportPracticeByUserDetail extends AbstractDefaultTask<String, Map<String, Object>> {

	@Autowired
	private IPracticeChatLogService chatLogService;
	
    @Override
    protected String execute(Map<String, Object> map) {
    	RequestContext requestContext = (RequestContext) map.get("context");
    	PracticeReportByUserDetailReq param = (PracticeReportByUserDetailReq) map.get("param");
    	Long taskId = (Long) map.get("taskId");
        String serialNo = (String) map.get("serialNo");
        String taskName = (String) map.get("taskName");
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,
                requestContext.getAccountId(), new Date(), requestContext.getSiteId(), requestContext.getCompanyId());
        working(taskContext);
        
        List<PracticeReportByUserDetailVo> datas = chatLogService.reportByUserDetail(param, requestContext);
        
        Date date = new Date();
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy/MM/dd");
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyyMMddHHmmss");
        SimpleDateFormat sdf3 = new SimpleDateFormat("yyyyMMdd");
        SimpleDateFormat sdf4 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String fileName = new StringBuffer("用户陪练结果查询").append(sdf2.format(date)).append(".xlsx").toString();
        String raw1 = new StringBuffer("用户陪练结果查询").append(sdf3.format(date)).toString();
        String raw2 = new StringBuffer("统计周期：").append(sdf1.format(param.getDateFrom())).append("~").append(sdf1.format(param.getDateTo())).toString();
        
        List<String> headNames = new ArrayList<String>();
        headNames.add("用户名");
        headNames.add("姓名");
        headNames.add("陪练名称");
        headNames.add("陪练模式");
        headNames.add("完成时间");
        headNames.add("得分");
        headNames.add("考核结果");
        headNames.add("所在部门");
        headNames.add("所在组织架构");
        
        List<JSONObject> data = new ArrayList<JSONObject>();
        for (PracticeReportByUserDetailVo vo : datas) {
            JSONObject obj = new JSONObject();
            obj.put("用户名", vo.getAccount());
            obj.put("姓名", vo.getUserName());
            obj.put("陪练名称", vo.getPracticeName());
            obj.put("陪练模式", getMode(vo));
            obj.put("完成时间", sdf4.format(vo.getEndAt()));
            obj.put("得分", vo.getScore());
            obj.put("考核结果", getPassResult(vo)); 
            obj.put("所在部门", vo.getDepart());
            obj.put("所在组织架构", vo.getOrgInfo());
            data.add(obj);
        }
        
        String fileUrl = "";
        try {
            fileUrl = CreateFormartExcelUtils.createFormartExcel(
                    fileName,
                    raw1,
                    raw2,
                    headNames,
                    null,
                    null,
                    data
            );
            success(taskContext, "成功", fileUrl);
        } catch (Exception e) {
            fail(taskContext, "智能陪练-报表-按人员-详情-下载，写入过程中发生错误");
            log.error("智能陪练-报表-按人员-详情-下载,写入过程中发生错误.", e);
        }
        return fileUrl;
    }
    
    private String getMode(PracticeReportByUserDetailVo vo) {
    	Integer mode = vo.getPracticeMode();
    	Integer level = vo.getPracticeLevel() == null ? 0 : vo.getPracticeLevel();
    	return PracticeLevelMapEnum.getDescByStatus(new StringBuffer().append(mode).append(level).toString());
    }
    
    private String getPassResult(PracticeReportByUserDetailVo vo) {
    	Integer result = vo.getResult();
    	String resultString = null;
    	if(PracticeUseTypeEnum.PRACTICE.getCode().equals(vo.getPracticeMode())) {
    		resultString = "/";
    	}else {
    		if(PracticePassResultEnum.PASS.getStatus().equals(result)) {
    			resultString = new String(PracticePassResultEnum.PASS.getMsg()); 
    		}else {
    			resultString = new String(PracticePassResultEnum.UN_PASS.getMsg());
    		}
    	}
    	return resultString;
    }
    
}
