package com.yizhi.statistics.application.controller.report;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.statistics.application.entity.Report;
import com.yizhi.statistics.application.entity.ReportVisibleRange;
import com.yizhi.statistics.application.service.IReportService;
import com.yizhi.statistics.application.service.IReportVisibleRangleService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/statisticsReport")
public class ReportController {

	private Logger logger = LoggerFactory.getLogger(ReportController.class);

	@Autowired
	private IReportVisibleRangleService reportVisibleRangleService;

	@Autowired
	private IReportService reportService;

	@Autowired
	private IdGenerator idGenerator;

	/*@Autowired
	private SiteClient siteClient;*/
	
	@Autowired
	private ReportControllerAbs reportControllerAbs;

	@GetMapping("/save")
	public void save(@RequestParam("type") Integer type) {
		Map<String,Object> map = new HashMap<String,Object>();
		map.put("type", type);
		reportControllerAbs.execute(map, true);
	}

	@PostMapping("/update")
	public Report update(@RequestBody Report report) {
		reportService.updateById(report);
		return report;
	}

	@GetMapping("/up")
	public Boolean up(@RequestParam("reportId") Long reportId) {
		Date date = new Date();
		Report report = reportService.selectById(reportId);
		if (StringUtils.isEmpty(report)) {
			return Boolean.FALSE;
		}
		if (report.getState() == 0) {
			report.setState(date.before(report.getEndTime()) ? 2 : 1);
			reportService.updateById(report);
			return Boolean.TRUE;
		}
		return Boolean.FALSE;
	}

	@GetMapping("/down")
	public Boolean down(@RequestParam("reportId") Long reportId) {
		Report report = reportService.selectById(reportId);
		if (StringUtils.isEmpty(report)) {
			return Boolean.FALSE;
		}
		if (report.getState() == 1 || report.getState() == 2) {
			report.setState(0);
			reportService.updateById(report);
			return Boolean.TRUE;
		}
		return Boolean.FALSE;
	}

	@PostMapping("/authorize/insert")
	public Boolean insertReportVisibleRanges(@RequestBody List<ReportVisibleRange> reportVisibleRanges) {
		if (!CollectionUtils.isEmpty(reportVisibleRanges)) {
			RequestContext context = ContextHolder.get();
			Long reportId = reportVisibleRanges.get(0).getReportId();
			ReportVisibleRange reportVisibleRange = new ReportVisibleRange();
			reportVisibleRange.setState(1);
			reportVisibleRange.setReportId(reportId);
			EntityWrapper<ReportVisibleRange> entityWrapper = new EntityWrapper<>(reportVisibleRange);
			reportVisibleRangleService.delete(entityWrapper);
			for (ReportVisibleRange reportVisibleRange1 : reportVisibleRanges) {
				reportVisibleRange1.setId(idGenerator.generate());
				reportVisibleRange1.setSiteId(context.getSiteId());
				reportVisibleRange1.setCompanyId(context.getCompanyId());
			}
			return reportVisibleRangleService.insertBatch(reportVisibleRanges);
		} else {
			logger.error("列表为空 ");
			return Boolean.FALSE;
		}
	}

	@GetMapping("/list")
	public Page<Report> list(@RequestParam("pageSize") Integer pageSize, @RequestParam("pageNo") Integer pageNo, @RequestParam("type") Integer type,
			@RequestParam("siteId") Long siteId, @RequestParam("companyId") Long companyId, @RequestParam("kwd") String kwd) {
		Page<Report> page = new Page<Report>(pageNo, pageSize);
		Report report = new Report();
		report.setSiteId(siteId);
		report.setCompanyId(companyId);
		if (!type.equals(3)) {
			report.setState(type);
		}
		EntityWrapper<Report> wrapper = new EntityWrapper<Report>(report);
		wrapper.like("name", kwd);
		wrapper.orderBy("create_time", false);
		reportService.selectPage(page, wrapper);
		List<Report> list = page.getRecords();
		for(Report vo:list) {
			if (vo.getState().equals(2)&&vo.getEndTime().before(new Date())) {
				vo.setState(1);
				reportService.updateById(vo);
			}
		}
		return page;
	}
	
	@GetMapping("/VisibleRange")
	public List<ReportVisibleRange> VisibleRange(@RequestParam("reportId") Long reportId){
		ReportVisibleRange range = new ReportVisibleRange();
		range.setReportId(reportId);
		EntityWrapper<ReportVisibleRange> wrapper = new EntityWrapper<ReportVisibleRange>(range);
		List<ReportVisibleRange> list = reportVisibleRangleService.selectList(wrapper);
		return list;
	}
	
	@PostMapping("/relation/list")
	public List<Report> relationList(@RequestBody List<Long> ids) {
		RequestContext context = ContextHolder.get();
		return reportService.relationList(context.getSiteId(), context.getCompanyId(), ids);
	}
	
	@GetMapping("/get/id")
	public Report getById(@RequestParam("id") Long id) {
		Report report = reportService.selectById(id);
		return report;
	}
}
