package com.yizhi.statistics.application.controller.report;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.entity.Report;
import com.yizhi.statistics.application.enums.ReportType;
import com.yizhi.statistics.application.service.IReportService;
import com.yizhi.system.application.system.remote.SiteClient;
import com.yizhi.system.application.vo.CompanySimpleVO;
import com.yizhi.system.application.vo.SiteVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class ReportControllerAbs  extends AbstractDefaultTask<String, Map<String, Object>> {

	private Logger logger = LoggerFactory.getLogger(ReportControllerAbs.class);
	
    final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");
    
    final SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    
    final Date date = new Date();
	
    @Autowired
	private IReportService reportService;

    @Autowired
	private IdGenerator idGenerator;

    @Autowired
	private SiteClient siteClient;
	
	@Override
	protected String execute(Map<String, Object> param) {
		Integer type = (Integer) param.get("type");
        Long taskId = idGenerator.generate();
		Long t1=System.currentTimeMillis();
        /**
         * 走异步任务
         */
		logger.info("平台报告入库开始！！！");
        TaskContext taskContext = new TaskContext(taskId, sim.format(date) + " 号生成平台报告可编辑记录", 999L, date);
        working(taskContext);
		try {
			String year = new SimpleDateFormat("yyyy").format(date);
			Date January = format.parse(year + "-01-01 00:00:00");
			Date March = format.parse(year + "-03-31 23:59:59");
			Date April = format.parse(year + "-04-01 00:00:00");
			Date June = format.parse(year + "-06-30 23:59:59");
			Date July = format.parse(year + "-07-01 00:00:00");
			Date September = format.parse(year + "-09-30 23:59:59");
			Date October = format.parse(year + "-10-01 00:00:00");
			Date December = format.parse(year + "-12-31 23:59:59");
			List<CompanySimpleVO> companys = siteClient.list();
			if (!CollectionUtils.isEmpty(companys)) {
				for (CompanySimpleVO company : companys) {
					List<SiteVO> sites = siteClient.getSiteVOs(company.getId());
					if (!CollectionUtils.isEmpty(sites)) {
						switch (type) {
						case 0:
							// 生成一季度报告
							for (SiteVO site : sites) {
								insert2table( site, year, January, March, ReportType.firstQuarter.getType());
							}
							break;
						case 1:
							for (SiteVO site : sites) {
								// 生成二季度报告
								insert2table(site, year, April, June, ReportType.secondQuarter.getType());
								// 生成上半年度报告
								insert2table( site, year, January, June, ReportType.firstHalfYear.getType());
							}
							break;
						case 2:
							for (SiteVO site : sites) {
								// 生成三季度报告
								insert2table(site, year, July, September, ReportType.thirdQuarter.getType());
							}
							break;
						case 3:
							for (SiteVO site : sites) {
								// 生成四季度报告
								insert2table(site, year, October, December, ReportType.fourthQuarter.getType());
								// 生成下半年度报告
								insert2table( site, year, July, December, ReportType.secondHalfYear.getType());
								// 生成全年度报告
								insert2table( site, year, January, December, ReportType.annual.getType());
							}
							break;
							//2019年只生成全年报告
						case 4:
							for (SiteVO site : sites) {
								// 生成全年度报告
								insert2table( site, year, January, December, ReportType.annual.getType());
							}
							break;
						}
					}
				}
			}
			Long t2=System.currentTimeMillis();
			String result="处理时间ms:"+(t2-t1);
			logger.info("平台报告入库结束，"+result);
            success(taskContext, result);
		} catch (Exception e) {
			logger.error("平台报告生成异常", e);
            fail(taskContext, "平台报告生成异常");
		}
        return null;
	}
	
	/**
	 * 生成报告
	 * @return
	 */
	public Report insert2table(SiteVO site,String year,Date startDate,Date endDate,int type) {
		String name="";
		switch (type) {
		case 0:
			name=year + ReportType.firstQuarter.getTypeName();
			break;
		case 1:
			name=year + ReportType.secondQuarter.getTypeName();
			break;
		case 2:
			name=year + ReportType.firstHalfYear.getTypeName();
			break;
		case 3:
			name=year + ReportType.thirdQuarter.getTypeName();
			break;
		case 4:
			name=year + ReportType.fourthQuarter.getTypeName();
			break;
		case 5:
			name=year + ReportType.secondHalfYear.getTypeName();
			break;
		case 6:
			name=year + ReportType.annual.getTypeName();
			break;
		}
		Report report = new Report();
		report.setCompanyId(site.getCompanyId());
		report.setSiteId(site.getId());
		report.setType(type);
		report.setName(name);
		EntityWrapper<Report> wrapper = new EntityWrapper<Report>(report);
		Report one = reportService.selectOne(wrapper);
		if (StringUtils.isEmpty(one)) {
			report.setCreateById(site.getAdminAccountId());
			report.setStartTime(startDate);
			report.setEndTime(endDate);
			report.setCreateTime(new Date());
			report.setId(idGenerator.generate());
			report.setUrl("https://customproject.oss-cn-shanghai.aliyuncs.com/web_image/3a40264b0208c6a5865396cca239b70.png");
			report.setName(name);
			reportService.insert(report);
		}
		return report;
	}
}
