package com.yizhi.statistics.application.controller.report;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.calendar.application.feign.StudentCalendarClient;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.point.application.feign.PointUserFeignClients;
import com.yizhi.statistics.application.entity.Report;
import com.yizhi.statistics.application.entity.ReportDetail;
import com.yizhi.statistics.application.mapper.ReportDetailMapper;
import com.yizhi.statistics.application.service.IReportDetailService;
import com.yizhi.statistics.application.service.IReportService;
import com.yizhi.statistics.application.service.StatisticsAccountService;
import com.yizhi.statistics.application.vo.report.AccountVO;
import com.yizhi.statistics.application.vo.report.StudyDurationVO;
import com.yizhi.system.application.system.remote.SiteClient;
import com.yizhi.system.application.vo.CompanySimpleVO;
import com.yizhi.system.application.vo.SiteVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.datasource.DataSourceUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.sql.DataSource;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.text.SimpleDateFormat;
import java.util.*;

@Component
public class StudentReportControllerAbs extends AbstractDefaultTask<String, Map<String, Object>> {

	private Logger logger = LoggerFactory.getLogger(StudentReportControllerAbs.class);

	final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");

	final SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	final Date date = new Date();

	@Autowired
	private IdGenerator idGenerator;

	@Autowired
	private SiteClient siteClient;

	@Autowired
	private PointUserFeignClients pointUserFeignClients;

	@Autowired
	private IReportDetailService reportDetailService;

	@Autowired
	private IReportService reportService;

	@Autowired
	private com.yizhi.certificate.application.feign.CertificateAppClient CertificateAppClient;

	@Autowired
	private StudentCalendarClient studentCalendarClient;

	@Autowired
	private StatisticsAccountService statisticsAccountService;

	@Autowired
	private ReportDetailMapper reportDetailMapper;

	@Autowired
	private DataSource dataSource;

	// 字段分割符
	private final static String FIELD_STERMINATED = "','";
	// 行分割符
	private final static String LINE_STERMINATED = "'\r\n'";

	private final static String FENGEFU = ","; // tab键
	private final static String HUANHANGFU = "\r\n"; // 换行

	private StringBuilder strBuilder = new StringBuilder();

	private final static String REPORT_DETAIL = "(id,report_id,report_name,report_type,account_id,state,register_time,proportion,all_proportion,user_duration,site_name,sign_num,all_sign_num,study_duration,all_study_duration,finish_num,all_finish_num,point,all_point,first_certificate,all_first_certificate,latest_date,all_latest_date,max_study,all_max_study,max_duration,all_max_duration,type,all_type,create_time,create_by_id,company_id,site_id)";
	private final static String REPORT_DETAIL_EXECUTE_SQL = "load data local infile ''  into table report_detail fields terminated by "
			+ FIELD_STERMINATED + "enclosed by '' escaped by 'N'" + " lines terminated by " + LINE_STERMINATED + " "
			+ REPORT_DETAIL;

	@Override
	protected String execute(Map<String, Object> param) {
		Long taskId = idGenerator.generate();
		Long t1 = System.currentTimeMillis();
		int success = 0;
		int fail = 0;
		/**
		 * 走异步任务
		 */
		TaskContext taskContext = new TaskContext(taskId, sim.format(date) + " 号生成学员平台报告数据", 999L, date);
		working(taskContext);
		try {
			List<CompanySimpleVO> companys = siteClient.list();
			List<Long> siteId = new ArrayList<Long>();
			if (!CollectionUtils.isEmpty(companys)) {
				// 遍历所有企业
				for (CompanySimpleVO company : companys) {
					List<Long> allSiteId = siteClient.getSiteIds(company.getId()); // 全站点id
					List<SiteVO> sites = siteClient.getSiteVOs(company.getId());
					if (!CollectionUtils.isEmpty(sites)) {
						// 遍历所有站点
						for (SiteVO site : sites) {
							Report report = new Report();
							report.setSiteId(site.getId());
							report.setCompanyId(site.getCompanyId());
							EntityWrapper<Report> wrapper = new EntityWrapper<Report>(report);
							List<Report> list = reportService.selectList(wrapper);
							List<AccountVO> accounts = statisticsAccountService.getAccountIdsBySiteId(site.getId());
							siteId.add(site.getId()); // 当前站点id
							if (!CollectionUtils.isEmpty(list)) {
								// 遍历所有报告
								for (Report vo : list) {
									// 遍历所有用户
									Long count = reportDetailService.getCount(vo.getId(), site.getId());
									if (count > 0) {
										logger.info("当前站点：" + site.getId() + " 平台报告：" + vo.getId() + "学员数据已经入库！！！");
										continue;
									}
									if (!CollectionUtils.isEmpty(accounts)) {
										String startDate = format.format(vo.getStartTime());
										String endDate = format.format(vo.getEndTime());
										List<ReportDetail> details = new ArrayList<ReportDetail>();
										// 课程数据全站点
										Map<Long, Integer> allfinishMap = new HashMap<Long, Integer>();
										Map<Long, Integer> allproportionMap = new HashMap<Long, Integer>();
										Map<Long, Long> alltotalDurationMap = new HashMap<Long, Long>();
										Map<Long, Date> alllateDateMap = new HashMap<Long, Date>();
										Map<Long, Date> allmaxDateMap = new HashMap<Long, Date>();
										Map<Long, Long> allmaxDurationMap = new HashMap<Long, Long>();
										List<StudyDurationVO> allcountFinished = reportDetailMapper
												.countFinished(allSiteId, startDate, endDate);
										List<StudyDurationVO> allproportion = reportDetailMapper.proportion(allSiteId,
												startDate, endDate);
										List<StudyDurationVO> alltotalDuration = reportDetailMapper
												.totalDuration(allSiteId, startDate, endDate);
										List<StudyDurationVO> allstudyDuration = reportDetailMapper
												.studyDuration(allSiteId, startDate, endDate);
										List<StudyDurationVO> alllateDate = reportDetailMapper.lateDate(allSiteId,
												startDate, endDate);
										for (StudyDurationVO vo1 : allcountFinished) {
											allfinishMap.put(vo1.getAccountId(), vo1.getFinish());
										}
										for (StudyDurationVO vo1 : allproportion) {
											Integer dividend = (Integer.valueOf(accounts.size()) - vo1.getProportion());
											Integer divisor = accounts.size();
											BigDecimal res = new BigDecimal(0);
											BigDecimal dividendN = new BigDecimal(dividend.toString());
											BigDecimal divisorN = new BigDecimal(divisor.toString());
											BigDecimal a = dividendN.multiply(new BigDecimal("100"));
											res = a.divide(divisorN, 2, BigDecimal.ROUND_HALF_UP);
											allproportionMap.put(vo1.getAccountId(), Integer.valueOf(res.intValue()));
										}
										for (StudyDurationVO vo1 : alltotalDuration) {
											alltotalDurationMap.put(vo1.getAccountId(), vo1.getTotalDuration());
										}
										for (StudyDurationVO vo1 : allstudyDuration) {
											allmaxDateMap.put(vo1.getAccountId(), vo1.getMaxDate());
											allmaxDurationMap.put(vo1.getAccountId(), vo1.getMaxDuration());
										}
										for (StudyDurationVO vo1 : alllateDate) {
											alllateDateMap.put(vo1.getAccountId(), vo1.getLateDate());
										}
										// 积分数据全站点
										Map<Long, Long> allPoint = pointUserFeignClients.totalPoint(allSiteId,
												startDate, endDate);
										// 证书数据全站点
										Map<Long, Date> allFirst = CertificateAppClient.firstCertificate(allSiteId,
												startDate, endDate);
										// 签到数据全站点
										Map<Long, Long> allSign = studentCalendarClient.signNum(allSiteId, startDate,
												endDate);
										// 课程数据当前站点
										Map<Long, Integer> finishMap = new HashMap<Long, Integer>();
										Map<Long, Integer> proportionMap = new HashMap<Long, Integer>();
										Map<Long, Long> totalDurationMap = new HashMap<Long, Long>();
										Map<Long, Date> lateDateMap = new HashMap<Long, Date>();
										Map<Long, Date> maxDateMap = new HashMap<Long, Date>();
										Map<Long, Long> maxDurationMap = new HashMap<Long, Long>();
										List<StudyDurationVO> countFinished = reportDetailMapper.countFinished(siteId,
												startDate, endDate);
										List<StudyDurationVO> proportion = reportDetailMapper.proportion(siteId,
												startDate, endDate);
										List<StudyDurationVO> totalDuration = reportDetailMapper.totalDuration(siteId,
												startDate, endDate);
										List<StudyDurationVO> studyDuration = reportDetailMapper.studyDuration(siteId,
												startDate, endDate);
										List<StudyDurationVO> lateDate = reportDetailMapper.lateDate(siteId, startDate,
												endDate);
										for (StudyDurationVO vo1 : countFinished) {
											finishMap.put(vo1.getAccountId(), vo1.getFinish());
										}
										for (StudyDurationVO vo1 : proportion) {
											Integer dividend = (Integer.valueOf(accounts.size()) - vo1.getProportion());
											Integer divisor = accounts.size();
											BigDecimal res = new BigDecimal(0);
											BigDecimal dividendN = new BigDecimal(dividend.toString());
											BigDecimal divisorN = new BigDecimal(divisor.toString());
											BigDecimal a = dividendN.multiply(new BigDecimal("100"));
											res = a.divide(divisorN, 2, BigDecimal.ROUND_HALF_UP);
											proportionMap.put(vo1.getAccountId(), Integer.valueOf(res.intValue()));
										}
										for (StudyDurationVO vo1 : totalDuration) {
											totalDurationMap.put(vo1.getAccountId(), vo1.getTotalDuration());
										}
										for (StudyDurationVO vo1 : studyDuration) {
											maxDateMap.put(vo1.getAccountId(), vo1.getMaxDate());
											maxDurationMap.put(vo1.getAccountId(), vo1.getMaxDuration());
										}
										for (StudyDurationVO vo1 : lateDate) {
											lateDateMap.put(vo1.getAccountId(), vo1.getLateDate());
										}
										// 积分数据当前站点
										Map<Long, Long> point = pointUserFeignClients.totalPoint(siteId, startDate,
												endDate);
										// 证书数据当前站点
										Map<Long, Date> first = CertificateAppClient.firstCertificate(siteId, startDate,
												endDate);
										// 签到数据当前站点
										Map<Long, Long> sign = studentCalendarClient.signNum(siteId, startDate,
												endDate);
										Date creatTime = new Date();
										for (AccountVO user : accounts) {
											// 组装数据
											ReportDetail detail = new ReportDetail();
											long dayBetween = (date.getTime() - user.getCreateTime().getTime())
													/ (60 * 60 * 24 * 1000);
											detail.setReportId(vo.getId());
											detail.setSiteId(site.getId());
											detail.setCompanyId(site.getCompanyId());
											detail.setAccountId(user.getAccountId());
											detail.setId(idGenerator.generate());
											detail.setReportName(vo.getName());
											detail.setReportType(vo.getType());
											detail.setRegisterTime(user.getCreateTime());
											detail.setSiteName(site.getName());
											detail.setUserDuration(dayBetween);
											detail.setCreateById(site.getAdminAccountId());
											detail.setCreateTime(creatTime);
											detail.setProportion(proportionMap.get(user.getAccountId()) != null
													? proportionMap.get(user.getAccountId())
													: 0);
											detail.setSignNum(null != sign.get(user.getAccountId())
													? sign.get(user.getAccountId())
													: 0l);
											detail.setStudyDuration(totalDurationMap.get(user.getAccountId()) != null
													? totalDurationMap.get(user.getAccountId())
													: 0l);
											detail.setFinishNum(finishMap.get(user.getAccountId()) != null
													? finishMap.get(user.getAccountId())
													: 0);
											detail.setPoint(point.get(user.getAccountId()) != null
													? point.get(user.getAccountId())
													: 0l);
											detail.setFirstCertificate(
													StringUtils.isEmpty(first.get(user.getAccountId())) ? null
															: first.get(user.getAccountId()));
											detail.setLatestDate(lateDateMap.get(user.getAccountId()) != null
													? lateDateMap.get(user.getAccountId())
													: null);
											detail.setMaxStudy(maxDateMap.get(user.getAccountId()) != null
													? maxDateMap.get(user.getAccountId())
													: null);
											detail.setMaxDuration(maxDurationMap.get(user.getAccountId()) != null
													? maxDurationMap.get(user.getAccountId())
													: 0l);
											if (detail.getProportion() >= 80) {
												detail.setType(0);
											} else if (detail.getProportion() < 80 && detail.getProportion() >= 60) {
												detail.setType(1);
											} else if (detail.getProportion() < 60 && detail.getProportion() >= 40) {
												detail.setType(2);
											} else if (detail.getProportion() < 40 && detail.getProportion() >= 20) {
												detail.setType(3);
											} else if (detail.getProportion() < 20) {
												detail.setType(4);
											}
											detail.setAllProportion(allproportionMap.get(user.getAccountId()) != null
													? allproportionMap.get(user.getAccountId())
													: 0);
											detail.setAllSignNum(null != allSign.get(user.getAccountId())
													? allSign.get(user.getAccountId())
													: 0l);
											detail.setAllStudyDuration(
													alltotalDurationMap.get(user.getAccountId()) != null
															? alltotalDurationMap.get(user.getAccountId())
															: 0l);
											detail.setAllFinishNum(allfinishMap.get(user.getAccountId()) != null
													? allfinishMap.get(user.getAccountId())
													: 0);
											detail.setAllPoint(allPoint.get(user.getAccountId()) != null
													? allPoint.get(user.getAccountId())
													: 0l);
											detail.setAllFirstCertificate(
													StringUtils.isEmpty(allFirst.get(user.getAccountId())) ? null
															: allFirst.get(user.getAccountId()));
											detail.setAllLatestDate(alllateDateMap.get(user.getAccountId()) != null
													? alllateDateMap.get(user.getAccountId())
													: null);
											detail.setAllMaxStudy(allmaxDateMap.get(user.getAccountId()) != null
													? allmaxDateMap.get(user.getAccountId())
													: null);
											detail.setAllMaxDuration(allmaxDurationMap.get(user.getAccountId()) != null
													? allmaxDurationMap.get(user.getAccountId())
													: 0l);
											if (detail.getAllProportion() >= 80) {
												detail.setAllType(0);
											} else if (detail.getAllProportion() < 80
													&& detail.getAllProportion() >= 60) {
												detail.setAllType(1);
											} else if (detail.getAllProportion() < 60
													&& detail.getAllProportion() >= 40) {
												detail.setAllType(2);
											} else if (detail.getAllProportion() < 40
													&& detail.getAllProportion() >= 20) {
												detail.setAllType(3);
											} else if (detail.getAllProportion() < 20) {
												detail.setAllType(4);
											}
											details.add(detail);
										}
										if (null != details) {
											boolean b = this.insertBatchStatisticsReportDetail(details);
											if (!b) {
												logger.error("站点：" + site.getId() + " 报告：" + vo.getId() + " 生成用户数据失败！");
												fail(taskContext,
														"站点：" + site.getId() + " 报告：" + vo.getId() + " 生成用户数据失败！");
												fail += details.size();

											} else {
												success += details.size();
											}
										}
									}
								}
							}
							siteId.clear();
						}
					}
				}
				Long t2 = System.currentTimeMillis();
				String result = "总数量:" + (success + fail) + "。处理时间ms:" + (t2 - t1) + ",处理成功:" + success + ",处理失败:"
						+ fail;
				logger.info(result);
				success(taskContext, result);
			}
		} catch (Exception e) {
			logger.error("平台报告用户数据入库失败", e);
			fail(taskContext, "平台报告用户数据入库失败！");
		}
		return null;
	}

	public boolean insertBatchStatisticsReportDetail(List<ReportDetail> listStatisticsReport) {

		Connection conn = null;
		InputStream in = null;
		PreparedStatement preStatement = null;
		com.mysql.jdbc.PreparedStatement mysqlPreStatement = null;
		try {
			// sql编译
			conn = DataSourceUtils.getConnection(dataSource);
			preStatement = conn.prepareStatement(REPORT_DETAIL_EXECUTE_SQL);
			mysqlPreStatement = preStatement.unwrap(com.mysql.jdbc.PreparedStatement.class);
			in = switchStreamStatisticsReportDetail(listStatisticsReport);
			mysqlPreStatement.setLocalInfileInputStream(in);
			// 执行
			preStatement.executeQuery();
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		} finally {
			// 大对象手动显示的设置为空保证GC时被释放
			if (in != null) {
				in = null;
			}

			try {
				// 关闭数据库连接
				if (mysqlPreStatement != null) {
					mysqlPreStatement.close();
				}
				if (preStatement != null) {
					preStatement.close();
				}
				if (conn != null) {
					DataSourceUtils.releaseConnection(conn, dataSource);
				}
			} catch (Exception e) {
				e.printStackTrace();
				return false;
			}
		}

		return true;
	}

	public InputStream switchStreamStatisticsReportDetail(List<ReportDetail> list) {
		for (ReportDetail sc : list) {
			strBuilder.append(idGenerator.generate());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getReportId());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getReportName());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getReportType());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getAccountId());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getState());
			strBuilder.append(FENGEFU);

			if (sc.getRegisterTime() != null) {
				strBuilder.append(format.format(sc.getRegisterTime()));
			}
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getProportion());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getAllProportion());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getUserDuration());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getSiteName());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getSignNum());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getAllSignNum());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getStudyDuration());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getAllStudyDuration());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getFinishNum());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getAllFinishNum());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getPoint());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getAllPoint());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getFirstCertificate()!=null?format.format(sc.getFirstCertificate()):"1970-01-01 08:00:00");
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getAllFirstCertificate()!=null?format.format(sc.getAllFirstCertificate()):"1970-01-01 08:00:00");
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getLatestDate()!=null?format.format(sc.getLatestDate()):"1970-01-01 08:00:00");
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getAllLatestDate()!=null?format.format(sc.getAllLatestDate()):"1970-01-01 08:00:00");
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getMaxStudy()!=null?format.format(sc.getMaxStudy()):"1970-01-01 08:00:00");
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getAllMaxStudy()!=null?format.format(sc.getAllMaxStudy()):"1970-01-01 08:00:00");
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getMaxDuration());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getAllMaxDuration());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getType());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getAllType());
			strBuilder.append(FENGEFU);
			
			if (sc.getCreateTime() != null) {
				strBuilder.append(format.format(sc.getCreateTime()));
			}
			
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getCreateById());
			strBuilder.append(FENGEFU);

			strBuilder.append(sc.getCompanyId());
			strBuilder.append(FENGEFU);
			
			strBuilder.append(sc.getSiteId());
			strBuilder.append(FENGEFU);
			strBuilder.append(HUANHANGFU);

		}
		byte[] bytes = strBuilder.toString().getBytes();
		InputStream in = new ByteArrayInputStream(bytes);

		// 大对象手动显示的设置为空保证GC时被释放
		bytes = null;

		// 清空之前的内容
		strBuilder.delete(0, strBuilder.length());

		return in;
	}
}
