package com.yizhi.statistics.application.controller.research;

import com.alibaba.fastjson.JSON;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.course.application.vo.CourseAccountReport;
import com.yizhi.research.application.feign.ResearchAnswerClient;
import com.yizhi.research.application.feign.ResearchAuthorizeClient;
import com.yizhi.research.application.feign.ResearchClient;
import com.yizhi.research.application.vo.StatisticResearchMetadataVo;
import com.yizhi.research.application.vo.domain.ResearchVo;
import com.yizhi.research.application.vo.domain.TrResearchAuthorizeVo;
import com.yizhi.statistics.application.commands.IStatisticsMetadataTask;
import com.yizhi.statistics.application.entity.StatisticsResearch;
import com.yizhi.statistics.application.entity.StatisticsResearchLearnMetadata;
import com.yizhi.statistics.application.mapper.StatisticsResearchLearnMetadataMapper;
import com.yizhi.statistics.application.mapper.StatisticsResearchMapper;
import com.yizhi.statistics.application.scope.Scope;
import com.yizhi.statistics.application.scope.Type;
import com.yizhi.statistics.application.service.StatisticsResearchLearnMetadataService;
import com.yizhi.statistics.application.service.StatisticsResearchService;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.statistics.application.vo.research.CalculateTimeVo;
import com.yizhi.statistics.application.work.CourseService;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Component
public class ResearchControllerAbs extends AbstractDefaultTask<String, Map<String, Object>> implements IStatisticsMetadataTask<StatisticsResearch> {

    private Logger logger = LoggerFactory.getLogger(ResearchControllerAbs.class);


    @Autowired
    private CourseService researchService;
    @Autowired
    private StatisticsResearchService statisticsResearchService;


    @Autowired
    private StatisticsResearchLearnMetadataMapper learnMetadataMapper;

    @Autowired
    private ResearchAuthorizeClient researchAuthorizeClient;

    @Autowired
    private StatisticsResearchLearnMetadataService learnMetadataService;

    @Autowired
    private ResearchAnswerClient researchAnswerClient;

    @Autowired
    private StatisticsResearchMapper statisticsResearchMapper;
    @Autowired
    private ResearchClient researchClient;
    @Autowired
    private JedisUtil jedisUtil;
    @Autowired
    private IdGenerator idGenerator;

    final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");

   

    @Override
    public void exceptionLog(Long taskId,StatisticsResearch metadataObject, Exception e) {
        taskDetail(taskId, "" + e);
    }

    @Override
    protected String execute(Map<String, Object> map) {
    	Long taskId = idGenerator.generate();
        Long startRunTime = System.currentTimeMillis();
        /**
         * 走异步任务
         */
        Date curDateWork = new Date();

        TaskContext taskContext = new TaskContext(taskId, sim.format(curDateWork) + " 号0:10在线调研报表分析前一天的数据", 999L, curDateWork);
        working(taskContext);


        // 获得所有的调研(条件：未上架/已上架，没有被删除的调研)
        List<ResearchVo> getAllResearch = researchClient.getAllResearch();

        //获取合理执行的开始时间与结束时间
        CalculateTimeVo calculateTimeVo = null;
        try {
            calculateTimeVo = getStartAndEndTime(map);
        } catch (ParseException e) {
            e.printStackTrace();
        }


        //开始时间的String格式
        String startDateString = calculateTimeVo.getStartDateString();
        //开始时间的Date格式
        Date startDate = calculateTimeVo.getStartDate();
        String endDateString = calculateTimeVo.getEndDateString();
        Date endDate = calculateTimeVo.getEndDate();
        Long day = calculateTimeVo.getDay();

        if (CollectionUtils.isNotEmpty(getAllResearch)) {
            taskDetail(taskId, "***** 在线调研统计开始 ,调研数量:" + getAllResearch.size() + " *****");
            logger.info("***** 在线调研统计开始 ,调研数量:" + getAllResearch.size() + " *****");
        } else {
            taskDetail(taskId, "***** 在线调研统计开始 ,调研数量获取异常:0或者为空 *****");
            logger.info("***** 在线调研统计开始 ,调研数量获取异常:0或者为空 *****");
        }


        boolean isOk = true;
        String curResearchName = "";
        Long curResearchId = 0L;


        try {
            if (CollectionUtils.isNotEmpty(getAllResearch)) {
                for (ResearchVo research : getAllResearch) {
                    Long researchId = research.getId();
                    Long workId = research.getId();
                    String researchName = research.getName();
                    curResearchId = researchId;
                    curResearchName = researchName;
                    Long siteId = research.getSiteId();
                    Long companyId = research.getCompanyId();
                    Integer visib_range = research.getVisibleRange();
                    if (null == visib_range) {
                        taskDetail(taskId, "可见范围类型为null");
                        continue;
                    }

                    Integer switchScope = new Scope().Research(visib_range);
                    if (siteId == null || companyId == null || workId == null || (switchScope == -1)) {
                        logger.info("调研:" + researchId + ",参数异常，请注意。");
                        taskDetail(taskId, "调研:" + researchId + ",参数异常，请注意。");
                        continue;
                    }

                    // 指定范围
                    List<TrResearchAuthorizeVo> TrResearchAuthorizeList = null;
                    List<Long> accountIds = new ArrayList<>();
                    List<Long> orgIds = new ArrayList<>();
                    CourseAccountReport accountReport = new CourseAccountReport();
                    if (switchScope == 2) { // 如果是指定范围
                        TrResearchAuthorizeList = researchAuthorizeClient.getResearchAuthorize(researchId);
                        if (CollectionUtils.isNotEmpty(TrResearchAuthorizeList)) {
                            for (TrResearchAuthorizeVo authorize : TrResearchAuthorizeList) {
                                if (authorize.getType().equals(1)) {
                                    orgIds.add(authorize.getRelationId());
                                } else {
                                    accountIds.add(authorize.getRelationId());
                                }

                            }

                            accountReport.setAccountIds(accountIds);
                            accountReport.setOrgIds(orgIds);
                        }
                        if (CollectionUtils.isEmpty(accountIds) && CollectionUtils.isEmpty(orgIds)) {
                            logger.info("调研:" + workId + ",是指定人可见，但是原始数据没有查询到可见范围权限，请复查原始数据");
                            taskDetail(taskId, "调研:" + workId + ",是指定人可见，但是原始数据没有查询到可见范围权限，请复查原始数据");
                            continue;
                        }
                    }
                    try {
                        StatisticsResearch statisticsResearch = new StatisticsResearch();
                        BeanUtils.copyProperties(research, statisticsResearch);

                        researchService.executeTask(taskId,statisticsResearch, workId, startDate, endDate, Type.REARCH.getType(), companyId, siteId, accountReport, this, switchScope);
                    } catch (Exception e) {
                        e.printStackTrace();
                        isOk = false;
                        taskDetail(taskId, "统计调研 " + curResearchName + "(" + curResearchId + ")异常:" + "" + e.getMessage());
                    }
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            isOk = false;
            taskDetail(taskId, "统计调研 " + curResearchName + "(" + curResearchId + ")异常:" + e.getMessage());
        }

        Long endRunTime = System.currentTimeMillis();
        String con = "调研报表分析完成耗时 :" + ((endRunTime - startRunTime) / 1000) + "秒";
        if (isOk) {
            success(taskContext, con);
        } else {
            fail(taskContext, con + ",执行过程中有错误");
        }
        taskDetail(taskId, con);
        
        //设置可参加人数缓存
        jedisUtil.setWorkIsSuccess(2,curDateWork);
        
        return "success";
    }

    @Override
    public void executeMetadata(Long taskId,StatisticsResearch metadataObject, Date startDate, Date endDate, Long companId, Long siteId) {

        String startDateString = sim.format(startDate);
        String endDateString = sim.format(endDate);

        // 先删除这天的记录再添加，防止一天添加多次记录
        learnMetadataMapper.deleteRecordeByDate(metadataObject.getId(), startDateString, endDateString);

        if (null != metadataObject) {
            statisticsResearchService.insertMainResearch(metadataObject);
        }


        List<StatisticResearchMetadataVo> voList = new ArrayList<>();
        List<StatisticsResearchLearnMetadata> insertList = new ArrayList<>();

        // 批量从answer表中查询当前日期学习记录
        voList = researchAnswerClient.selectLearnRecord(metadataObject.getId(), startDateString, endDateString);

        // 没有学习记录，跳转
        if (CollectionUtils.isEmpty(voList)) {

        } else {
            for (StatisticResearchMetadataVo vo : voList) {
                StatisticsResearchLearnMetadata learnMetadata = new StatisticsResearchLearnMetadata();
                learnMetadata.setAccountId(vo.getAccountId());
                learnMetadata.setResearchId(vo.getResearchId());
                learnMetadata.setId(null);
                learnMetadata.setCanState(1);
                learnMetadata.setJoinState(1);
                learnMetadata.setFinishTime(vo.getFinishTime());
                learnMetadata.setResearchAnswerCreateTime(vo.getResearchAnswerCreateTime());
                learnMetadata.setRecordCreateTime(new Date());

                insertList.add(learnMetadata);
            }
            //批量插入
            if (!CollectionUtils.isEmpty(insertList)) {
                try {
                    // 批量添加当前日期学习记录
                    learnMetadataService.insertBatch(insertList);
                    taskDetail(taskId, "批量添加成功" + metadataObject.getName() + "(" + metadataObject.getId() + ")" + "实际添加学习记录:" + insertList.size());
                } catch (Exception e) {
                    logger.info("***insert data*********" + JSON.toJSONString(insertList));
                    e.printStackTrace();
                    taskDetail(taskId, "批量添加" + metadataObject.getName() + "(" + metadataObject.getId() + ")" + "数据错误:" + e.getMessage());
                }

            }
        }

    }


    private CalculateTimeVo getStartAndEndTime(Map<String, Object> map) throws ParseException {
        String startDate = null;
        String endDate = null;
        Date dCurDate = null;
        CalculateTimeVo calculateTimeVo = new CalculateTimeVo();
        if (map != null) {
            startDate = (String) map.get("startDate");
            endDate = (String) map.get("endDate");
        }

        if (startDate == null || "".equals(startDate)) {
            Date middleMaxTime = learnMetadataMapper.selectMaxDate();
            ;
            if (middleMaxTime != null) {
                dCurDate = addDay(middleMaxTime, 1);
                startDate = sim.format(dCurDate);
            } else {
                Date recordMinTime = statisticsResearchMapper.selectRecordMinTime();

                if (recordMinTime == null) {
                    recordMinTime = researchClient.selectRecordMinTime();
                }
                startDate = sim.format(recordMinTime);
            }
        }
        if (endDate == null || "".equals(endDate)) {
            dCurDate = addDay(new Date(), -1);
            endDate = sim.format(dCurDate);
        }

        Long day = 0L;
        Date beginDate = null;
        Date end = null;
        try {
            beginDate = sim.parse(startDate);
            end = sim.parse(endDate);
            day = (end.getTime() - beginDate.getTime()) / (24 * 60 * 60 * 1000);
        } catch (ParseException e1) {
            return null;
        }

        calculateTimeVo.setDay(day);
        calculateTimeVo.setStartDate(beginDate);
        calculateTimeVo.setStartDateString(startDate);
        calculateTimeVo.setEndDate(end);
        calculateTimeVo.setEndDateString(endDate);
        return calculateTimeVo;


    }


    private Date addDay(Date date, int num) {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        calendar.add(calendar.DATE, num);
        date = calendar.getTime();
        return date;
    }

    private String getCurDate(int d, Date startDate) {
        String startDateString = sim.format(startDate);
        String currentDate = startDateString;
        if (d > 0) {
            Date dCurDate = addDay(startDate, d);
            currentDate = sim.format(dCurDate);
        }
        return currentDate;
    }

}
