package com.yizhi.statistics.application.controller.sign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.statistics.application.controller.sign.export.*;
import com.yizhi.statistics.application.domain.StatisticsSignInformation;
import com.yizhi.statistics.application.feign.StatisticsSignClient;
import com.yizhi.statistics.application.refactor.sign.StatisticsSignVO;
import com.yizhi.statistics.application.service.IStatisticsSignService;
import com.yizhi.statistics.application.vo.sign.SignAccountGroupVo;
import com.yizhi.statistics.application.vo.sign.StatisticsSignVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

/**
 * @param
 * @author dingxiaowei
 * @date 2019/4/22
 * @return
 */
@RestController()
@RequestMapping("/statisticsSign")
public class SignController implements StatisticsSignClient {

    @Autowired
    private SignControllerAbs signControllerAbs;

    @Autowired
    private IStatisticsSignService statisticsSignService;

    @Autowired
    private DownloadSignGroup downloadSignGroup;

    @Autowired
    private DownloadSignDetails downloadSignDetails;

    @Autowired
    ExportSignAccountGroup exportSignAccountGroup;

    @Autowired
    ExportAllSignAccountGroup exportAllSignAccountGroup;

    @Autowired
    ExportSignAccountGroupView exportSignAccountGroupView;


    /**
     * 签到报表定时异步任务
     *
     * @param startDate
     * @param endDate
     */
    @Override
    @GetMapping("/asynchronous/load/date")
    public void AsynchronousSign(String startDate, String endDate) {
        Map<String, Object> map = new HashMap<String, Object>();
        if (StringUtils.isNotBlank(startDate)) {
            map.put("startDate", startDate);
        }
        if (StringUtils.isNotBlank(endDate)) {
            map.put("endDate", endDate);
        }
        signControllerAbs.execute(map);
    }

    @Override
    @GetMapping("/sign/group")
    public Page<StatisticsSignVO> signGroup(
            @RequestParam(name = "projectId", required = false) Long projectId,
            @RequestParam(name = "kwd", required = false) String kwd,
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate,
            @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo,
            @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize) {


        RequestContext context = ContextHolder.get();
        Long companyId = context.getCompanyId();
        Long siteId = context.getSiteId();
        boolean isAdmin = context.isAdmin();
        Long accountId = context.getAccountId();

        Page<StatisticsSignVO> page = statisticsSignService.signGroup(isAdmin, accountId, projectId, kwd, startDate, endDate, companyId, siteId, pageNo, pageSize);
        return page;

    }


    @Override
    @GetMapping("/sign/group/export")
    public String signGroupDownload(@RequestParam(name = "projectId", required = false) Long projectId,
                                    @RequestParam(name = "kwd", required = false) String kwd,
                                    @RequestParam(name = "startDate", required = false) String startDate,
                                    @RequestParam(name = "endDate", required = false) String endDate) {
        RequestContext context = ContextHolder.get();
        Map<String, Object> map = new HashMap<String, Object>(64);
        Long taskId = System.currentTimeMillis();
        String serialNo = "SIGN-GROUP-EXPORT-" + taskId.toString();
        String taskName = "签到统计列表-" + taskId.toString();
        map.put("taskId", taskId);
        map.put("taskName", taskName);
        map.put("serialNo", serialNo);
        map.put("projectId", projectId);
        if (StringUtils.isNotBlank(kwd)) {
            map.put("kwd", kwd);
        }
        map.put("context", context);
        map.put("startDate", startDate);
        map.put("endDate", endDate);
        downloadSignGroup.execute(map, true);
        return serialNo;
    }


    @Override
    @GetMapping("/sign/group/view")
    public Page<StatisticsSignVO> signView(
            @RequestParam(name = "signId", required = true) Long signId,
            @RequestParam(name = "signTimeId", required = true) Long signTimeId,
            @RequestParam(name = "startDate", required = false) String startDate,
            @RequestParam(name = "endDate", required = false) String endDate,
            @RequestParam(name = "orgNameOrCode", required = false) String orgNameOrCode,
            @RequestParam(name = "accountName", required = false) String accountName,
            @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo,
            @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize) {

        RequestContext res = ContextHolder.get();
        Page<StatisticsSignVO> page = statisticsSignService.signGroupView(res.isAdmin(), res.getAccountId(), signId, signTimeId,
                res.getCompanyId(), res.getSiteId(), orgNameOrCode, accountName, pageNo, pageSize);
        return page;

    }


    @Override
    @GetMapping("/sign/group/view/export")
    public String signViewDownload(@RequestParam(name = "signId", required = true) Long signId,
                                   @RequestParam(name = "signTimeId", required = true) Long signTimeId,
                                   @RequestParam(name = "startDate", required = false) String startDate,
                                   @RequestParam(name = "endDate", required = false) String endDate,
                                   @RequestParam(name = "orgNameOrCode", required = false) String orgNameOrCode,
                                   @RequestParam(name = "accountName", required = false) String accountName) {
        RequestContext context = ContextHolder.get();
        StatisticsSignInformation trSign = statisticsSignService.selectBySignTimeId(signTimeId);

        Map<String, Object> map = new HashMap<String, Object>(64);
        Long taskId = System.currentTimeMillis();
        String serialNo = "SIGN-DETAILS-" + taskId.toString();
        String taskName = "签到明细统计-" + taskId.toString();
        map.put("signId", signId);
        map.put("taskId", taskId);
        map.put("taskName", taskName);
        map.put("serialNo", serialNo);
        map.put("signTimeId", signTimeId);

        if (StringUtils.isNotBlank(orgNameOrCode)) {
            map.put("orgNameOrCode", orgNameOrCode);
        }
        if (StringUtils.isNotBlank("accountName")) {
            map.put("accountName", accountName);
        }

        map.put("trSign", trSign);
        map.put("context", context);
        map.put("startDate", startDate);
        map.put("endDate", endDate);
        downloadSignDetails.execute(map, true);
        return serialNo;
    }



    /**
     * 签到-按用户统计
     * @param accountKwd
     * @param orgKwd
     * @param startDate
     * @param endDate
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    @GetMapping("/sign/account/group")
    public Page<SignAccountGroupVo> signAccountGroup(@RequestParam(name = "accountKwd", required = false) String accountKwd,
                                                     @RequestParam(name = "orgKwd", required = false) String orgKwd,
                                                     @RequestParam(name = "startDate", required = false) String startDate,
                                                     @RequestParam(name = "endDate", required = false) String endDate,
                                                     @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo,
                                                     @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize){
        RequestContext context = ContextHolder.get();
        Long accountId = context.getAccountId();
        Long companyId = context.getCompanyId();
        Long siteId = context.getSiteId();

        Page<SignAccountGroupVo> page = statisticsSignService.signAccountGroup(context.isAdmin(), accountId, accountKwd, orgKwd, startDate, endDate, companyId, siteId, pageNo, pageSize);
        return page;
    }


    /**
     * 签到-按用户统计-导出签到统计数据
     * @param accountKwd
     * @param orgKwd
     * @param startDate
     * @param endDate
     * @return
     */
    @Override
    @GetMapping("/sign/account/group/export")
    public String signAccountGroupExport(@RequestParam(name = "accountKwd", required = false) String accountKwd,
                                         @RequestParam(name = "orgKwd", required = false) String orgKwd,
                                         @RequestParam(name = "startDate", required = false) String startDate,
                                         @RequestParam(name = "endDate", required = false) String endDate) {
        RequestContext context = ContextHolder.get();
        Map<String, Object> map = new HashMap<String, Object>(64);
        Long taskId = System.currentTimeMillis();
        String serialNo = "SIGN-ACCOUNT-GROUP-EXPORT-" + taskId.toString();
        String taskName = "报表系统-签到统计-按用户统计-下载签到统计数据-" + taskId.toString();
        map.put("taskId", taskId);
        map.put("taskName", taskName);
        map.put("serialNo", serialNo);

        if (StringUtils.isNotBlank(accountKwd)) {
            map.put("accountKwd", accountKwd);
        }
        if (StringUtils.isNotBlank(orgKwd)) {
            map.put("orgKwd", orgKwd);
        }
        map.put("context", context);
        map.put("startDate", startDate);
        map.put("endDate", endDate);
        exportSignAccountGroup.execute(map, true);
        return serialNo;
    }


    /**
     * 签到-按用户统计-导出全部签到统计数据和明细
     * @param accountKwd
     * @param orgKwd
     * @param startDate
     * @param endDate
     * @return
     */
    @Override
    @GetMapping("/sign/account/group/exportAll")
    public String signAccountGroupExportAll(@RequestParam(name = "accountKwd", required = false) String accountKwd,
                                            @RequestParam(name = "orgKwd", required = false) String orgKwd,
                                            @RequestParam(name = "startDate", required = false) String startDate,
                                            @RequestParam(name = "endDate", required = false) String endDate) {
        RequestContext context = ContextHolder.get();
        Map<String, Object> map = new HashMap<String, Object>(64);
        Long taskId = System.currentTimeMillis();
        String serialNo = "SIGN-ACCOUNT-GROUP-EXPORTALL-" + taskId.toString();
        String taskName = "报表系统-签到统计-按用户统计-下载全部数据明细-" + taskId.toString();
        map.put("taskId", taskId);
        map.put("taskName", taskName);
        map.put("serialNo", serialNo);
        if (StringUtils.isNotBlank(accountKwd)) {
            map.put("accountKwd", accountKwd);
        }
        if (StringUtils.isNotBlank(orgKwd)) {
            map.put("orgKwd", orgKwd);
        }
        map.put("context", context);
        map.put("startDate", startDate);
        map.put("endDate", endDate);
        exportAllSignAccountGroup.execute(map, true);
        return serialNo;
    }

    /**
     * 签到-按用户统计-查看详情
     * @param accountId
     * @param startDate
     * @param endDate
     * @param projectKwd
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    @GetMapping("/sign/account/group/view")
    public Page<StatisticsSignVo> signAccountView(@RequestParam(name = "accountId", required = true) Long accountId,
                                                  @RequestParam(name = "startDate", required = false) String startDate,
                                                  @RequestParam(name = "endDate", required = false) String endDate,
                                                  @RequestParam(name = "projectKwd", required = false) String projectKwd,
                                                  @RequestParam(name = "pageNo", required = false, defaultValue = "1") Integer pageNo,
                                                  @RequestParam(name = "pageSize", required = false, defaultValue = "10") Integer pageSize) {
        RequestContext context = ContextHolder.get();
        Long companyId = context.getCompanyId();
        Long siteId = context.getSiteId();
        Page<StatisticsSignVo> page = statisticsSignService.signAccountGroupView(accountId, siteId, companyId,
                startDate, endDate, projectKwd, pageNo, pageSize);
        return page;
    }





    /**
     * 签到-按用户统计-查看详情-下载全部数据
     * @param accountId
     * @param startDate
     * @param endDate
     * @param projectKwd
     * @return
     */
    @Override
    @GetMapping("/sign/account/group/view/export")
    public String signAccountViewExport(@RequestParam(name = "accountId", required = true) Long accountId,
                                        @RequestParam(name = "accountName", required = true) String accountName,
                                        @RequestParam(name = "startDate", required = false) String startDate,
                                        @RequestParam(name = "endDate", required = false) String endDate,
                                        @RequestParam(name = "projectKwd", required = false) String projectKwd) {
        RequestContext context = ContextHolder.get();

        Map<String, Object> map = new HashMap<String, Object>(64);
        Long taskId = System.currentTimeMillis();
        String serialNo = "SIGN-ACCOUNT-DETAILS-" + taskId.toString();
        String taskName = "报表系统-签到统计-按用户统计-下载全部数据明细-" + taskId.toString();
        map.put("accountId", accountId);
        map.put("accountName", accountName);
        map.put("taskId", taskId);
        map.put("taskName", taskName);
        map.put("serialNo", serialNo);

        if (StringUtils.isNotBlank(projectKwd)) {
            map.put("projectKwd", projectKwd);
        }

        map.put("context", context);
        map.put("startDate", startDate);
        map.put("endDate", endDate);
        exportSignAccountGroupView.execute(map, true);
        return serialNo;
    }
}














