package com.yizhi.statistics.application.controller.sign;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.course.application.vo.CourseAccountReport;
import com.yizhi.sign.application.feign.ReportSignClient;
import com.yizhi.sign.application.vo.domain.StatisticsSignLearn;
import com.yizhi.sign.application.vo.domain.TrSign;
import com.yizhi.statistics.application.commands.IStatisticsMetadataTask;
import com.yizhi.statistics.application.domain.StatisticsSignInformation;
import com.yizhi.statistics.application.domain.StatisticsSignRecorde;
import com.yizhi.statistics.application.entity.vo.ErrorMsg;
import com.yizhi.statistics.application.scope.Scope;
import com.yizhi.statistics.application.scope.Type;
import com.yizhi.statistics.application.service.IStatisticsSignRecordeService;
import com.yizhi.statistics.application.service.IStatisticsSignService;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.statistics.application.work.CourseService;
import com.yizhi.training.application.feign.TrainingProjectClient;
import com.yizhi.training.application.vo.domain.TpAuthorizationRangeVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author dingxiaowei
 * @date 2019/4/23
 * @auther ding
 */
@Component
public class SignControllerAbs extends AbstractDefaultTask<String, Map<String, Object>> implements IStatisticsMetadataTask<TrSign> {

    private static final Logger LOGGER = LoggerFactory.getLogger(SignControllerAbs.class);

    final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");


    @Autowired
    private CourseService courseService;


    @Autowired
    private IStatisticsSignService statisticsSignService;

    @Autowired
    private ReportSignClient reportSignClient;

    @Autowired
    private IStatisticsSignRecordeService statisticsSignRecordeService;

    @Autowired
    private TrainingProjectClient trainingProjectClient;

    @Autowired
    private JedisUtil jedisUtil;

    @Autowired
    IdGenerator idGenerator;


    @Override
    public String execute(Map<String, Object> map) {
        // TODO Auto-generated method stub
        Long taskId = idGenerator.generate();
        Long t1 = System.currentTimeMillis();
        /**
         * 走异步任务
         */
        Date curDateWork = new Date();

        TaskContext taskContext = new TaskContext(taskId, sim.format(curDateWork) + " 号签到报表分析前一天的数据", 999L, curDateWork);
        working(taskContext);

        String startDate = (String) map.get("startDate");
        String endDate = (String) map.get("endDate");

        SimpleDateFormat formatDate = new SimpleDateFormat("yyyy-MM-dd");

        Date dCurDate;
        boolean isRunDel = true;

        if (startDate == null || "".equals(startDate)) {
            //获取最近记录时间
            Date middleMaxTime = statisticsSignService.selectMaxDate();
            if (middleMaxTime != null) {
                dCurDate = addDay(middleMaxTime, 1);
                if (dCurDate.getTime() > System.currentTimeMillis()) {
                    dCurDate = new Date();
                }
                startDate = formatDate.format(dCurDate);
                //有记录,删当天记录,防止重复插入数据
                isRunDel = true;
            } else {
                //没有记录,获取所有签到的最早开始时间
                Date recordMinTime = reportSignClient.selectRecordMinTime();
                if (null != recordMinTime) {
                    startDate = formatDate.format(recordMinTime);
                } else {
                    startDate = "1970-01-01";
                }

                //没有记录,不用删除当天数据
                isRunDel = false;
            }
        }

        if (endDate == null || "".equals(endDate)) {
            dCurDate = addDay(new Date(), -1);
            endDate = formatDate.format(dCurDate);
        }

        // 得到所有的签到列表
        List<TrSign> trSignList = reportSignClient.getAllResearchs("1970-01-01", endDate);


        if (CollectionUtils.isNotEmpty(trSignList)) {
            taskDetail(taskId, "***** 签到统计开始 ,签到数量:" + trSignList.size() + " *****");
            LOGGER.info("***** 签到统计入库开始 ,签到数量:" + trSignList.size() + " *****");
        } else {
            taskDetail(taskId, "***** 签到统计开始 ,签到数量获取异常:0或者为空 *****");
            LOGGER.info("***** 签到统计入库开始 ,签到数量获取异常:0或者为空 *****");
        }

        ErrorMsg error = null;
        List<ErrorMsg> listError = new ArrayList<ErrorMsg>();
        //转换scope对象
        Scope scope = new Scope();

        Integer dealSuccess = 0;
        boolean isOk = true;

        if (CollectionUtils.isNotEmpty(trSignList)) {
            /*getAllCourse=getAllCourse.subList(0, 10);*/
            for (TrSign trSign : trSignList) {
                Long siteId = trSign.getSiteId();
                Long signId = trSign.getId();
                Long signTimeId = trSign.getSignTimeId();
                Long trainningProjectId = trSign.getTrainingProjectId();
                if (null == trainningProjectId) {
                    LOGGER.info("签到:" + signTimeId + "异常,跳过操作");
                    error = new ErrorMsg(signTimeId, "异常,跳过操作");
                    listError.add(error);
                    continue;
                }

                CourseAccountReport courseAccountReport = new CourseAccountReport();

                Date examCreateTime = trSign.getCreateTime();
                //  1 平台用户 2 指定范围
                Integer workScope = trSign.getVisibleRange();
                if (null == workScope) {
                    workScope = 1;
                }
                //每个业务都需要将本身的scope进行转换，下边所有的代码全部是switchScope
                Integer switchScope = scope.Training(workScope);
                Long companyId = trSign.getCompanyId();
                List<TpAuthorizationRangeVo> authorizationRanges = null;


                // 对该签到记录进行插入
                if (siteId == null || companyId == null || signId == null || (switchScope == -1)) {
                    LOGGER.info("签到:" + signTimeId + "异常,跳过操作");
                    error = new ErrorMsg(signTimeId, "异常,跳过操作");
                    listError.add(error);
                    continue;
                }
                if (switchScope == 2) {
                    authorizationRanges = trainingProjectClient.VisibleRange(trSign.getTrainingProjectId());
                    // set集合转换为arrayList集合
                    if (CollectionUtils.isEmpty(authorizationRanges)) {
                        LOGGER.info("签到:" + signId + ",是指定人可见，但是原始数据没有查询到可见范围权限");
                        error = new ErrorMsg(signId, ",是指定人可见，但是原始数据没有查询到可见范围权限");
                        listError.add(error);
                        continue;
                    } else {
                        List<Long> accountIds = new ArrayList<>();
                        List<Long> orgIds = new ArrayList<>();
                        for (TpAuthorizationRangeVo tpAuthorizationRange : authorizationRanges) {
                            //1部门 2用户
                            if (tpAuthorizationRange.getType() == 1) {
                                orgIds.add(tpAuthorizationRange.getRelationId());
                            } else if (tpAuthorizationRange.getType() == 2) {
                                accountIds.add(tpAuthorizationRange.getRelationId());
                            }
                        }
                        courseAccountReport.setCourseId(trainningProjectId);
                        courseAccountReport.setAccountIds(accountIds);
                        courseAccountReport.setOrgIds(orgIds);
                    }
                }

                Date start = null;
                Date end = null;

                try {
                    start = formatDate.parse(startDate);
                    end = formatDate.parse(endDate);
                } catch (ParseException e1) {
                    return null;
                }
                //需要删除当天记录,避免重复删除
                if (isRunDel) {
                    statisticsSignRecordeService.deleteRcordesByDate(signTimeId, formatDate.format(start), formatDate.format(end));
                }

                try {
                    courseService.executeTask(taskId, trSign, signTimeId, start, end, Type.SIGN.getType(), companyId, siteId, courseAccountReport, this, switchScope);
                    dealSuccess++;
                } catch (Exception e) {
                    isOk = false;
                    e.printStackTrace();
                    taskDetail(taskId, "可见范围处理过程中出现异常" + signTimeId);
                }

            }
        }

        Long t2 = System.currentTimeMillis();
        Long t = (t2 - t1) / 1000;
        System.out.println("耗时(s)：" + t);
        for (int i = 0; i < listError.size(); i++) {
            System.out.println(listError.get(i).toString());
        }

        if (isOk) {
            success(taskContext, "执行成功,耗时:" + t + "s", "");
        } else {
            success(taskContext, "执行成功,耗时:" + t + "s," + "执行过程中有数据异常", "");
        }

        taskDetail(taskId, "总数量:" + trSignList.size() + "。处理时间ms:" + (t2 - t1) + ",处理成功:" + dealSuccess + ",处理失败:" + (trSignList.size() - dealSuccess));
        LOGGER.info("***** 签到统计入库结束 ,签到数量:" + trSignList.size() + "。处理时间ms:" + (t2 - t1) + ",处理成功:" + dealSuccess + ",处理失败:" + (trSignList.size() - dealSuccess) + " *****");

        //设置可参加人数缓存
        jedisUtil.setWorkIsSuccess(6, new Date());

        return "success";
    }

    @Override
    public void executeMetadata(Long taskId, TrSign metadataObject, Date startDate, Date endDate, Long companId, Long siteId) {

        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        TrSign trSign = metadataObject;

        Long day = 0L;
        day = (endDate.getTime() - startDate.getTime()) / (24 * 60 * 60 * 1000);

        // 对这个签到记录开始操作
        Long signTimeId = trSign.getSignTimeId();

        StatisticsSignInformation statisticsSign = new StatisticsSignInformation();
        statisticsSign.setId(idGenerator.generate());
        statisticsSign.setSignId(trSign.getId());
        statisticsSign.setSignTimeId(trSign.getSignTimeId());
        statisticsSign.setTrainningProjectId(trSign.getTrainingProjectId());
        statisticsSign.setTrainningProjectName(trSign.getTrainningProjectName());
        statisticsSign.setSignName(trSign.getName());
        statisticsSign.setSignStartTime(trSign.getSignStartDate());
        statisticsSign.setSignEndTime(trSign.getSignEndDate());
        statisticsSign.setSignCompanyId(trSign.getCompanyId());
        statisticsSign.setSignSiteId(trSign.getSiteId());
        statisticsSign.setSignOrgId(trSign.getOrgId());
        statisticsSign.setSignCreateTime(trSign.getCreateTime());
        statisticsSign.setAddress(trSign.getAddress());
        statisticsSign.setRange(trSign.getRange());

//        StatisticsExam statisticsExam1 = statisticsExamService.selectById(examId);
//        if (null!=statisticsExam1){
//            statisticsExamService.updateById(statisticsExam);
//        }else {
//            statisticsExamService.insertAllColumn(statisticsExam);
//        }

        //或插入或更新
        try {
            StatisticsSignInformation statisticsSignInformationa = new StatisticsSignInformation();
            statisticsSignInformationa.setSignTimeId(signTimeId);
            List<StatisticsSignInformation> list = statisticsSignService.selectList(new EntityWrapper<>(statisticsSignInformationa));
            if (CollectionUtils.isNotEmpty(list)) {
                statisticsSignInformationa = list.get(0);

                if (null != statisticsSignInformationa && null != statisticsSignInformationa.getId()) {
                    statisticsSign.setId(statisticsSignInformationa.getId());
                    statisticsSignService.updateById(statisticsSign);
                }
            } else {
                statisticsSignService.insert(statisticsSign);
                LOGGER.info("签到主体添加成功signTimeId:" + signTimeId);
                taskDetail(taskId, "签到主体添加成功signTimeId:" + signTimeId);
            }

        } catch (Exception e) {
            e.printStackTrace();
            taskDetail(taskId, "签到主体插入过程中出现错误" + signTimeId);
        }


        // 插入考试记录
        try {

            //插入签到记录
            List<StatisticsSignLearn> signLearnList = null;
            // 获取签到记录
            signLearnList = reportSignClient
                    .getRecords(signTimeId, df.format(startDate), df.format(endDate));
            if (CollectionUtils.isNotEmpty(signLearnList)) {
                System.out.println("signLearnList" + signLearnList.size());
                List<StatisticsSignRecorde> list = new ArrayList<>();
                BeanUtils.copyProperties(signLearnList, list);
                list = JSON.parseArray(JSON.toJSONString(signLearnList), StatisticsSignRecorde.class);
                System.out.println("list" + list.size());
                statisticsSignRecordeService.insertBatch(list);
                System.out.println(Thread.currentThread().getName() +
                        "完成执行签到记录插入" + metadataObject.getName() + "  " + list.size() + "条记录");
                taskDetail(taskId, "完成执行签到记录插入" + signTimeId + list.size() + "条记录");
            } else {
                System.out.println(Thread.currentThread().getName() +
                        "未找到签到记录" + metadataObject.getName());
                taskDetail(taskId, "未找到签到记录" + signTimeId);
            }
        } catch (Exception e) {
            e.printStackTrace();
            taskDetail(taskId, "签到记录插入过程中出现错误" + signTimeId);
        }

    }


    @Override
    public void exceptionLog(Long taskId, TrSign metadataObject, Exception e) {
        // TODO Auto-generated method stub
        taskDetail(taskId, "" + e.getMessage());
    }


    protected Date addDay(Date date, int num) {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        calendar.add(calendar.DATE, num);
        date = calendar.getTime();
        return date;
    }


}
