package com.yizhi.statistics.application.controller.sign.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.service.IStatisticsSignService;
import com.yizhi.statistics.application.vo.sign.StatisticsSignVo;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 签到-按用户统计-全部数据明细导出
 *
 * @ClassName: ExportAllSignAccountGroup
 * @author: zjl
 * @date: 2020/9/15  18:07
 */
@Component
public class ExportAllSignAccountGroup extends AbstractDefaultTask<String, Map<String, Object>> {
    private static final Logger LOGGER = LoggerFactory.getLogger(ExportAllSignAccountGroup.class);


    @Autowired
    private IStatisticsSignService statisticsSignService;

    @Override
    protected String execute(Map<String, Object> map) {
        RequestContext context = (RequestContext) map.get("context");
        Long companyId = context.getCompanyId();
        Long siteId = context.getSiteId();
        Long accountId = context.getAccountId();
        boolean isAdmin = context.isAdmin();

        String orgKwd = (String) map.get("orgKwd");
        String accountKwd = (String) map.get("accountKwd");
        String startDate = (String) map.get("startDate");
        String endDate = (String) map.get("endDate");

        Long taskId = (Long) map.get("taskId");
        String taskName = (String) map.get("taskName");
        String serialNo = (String) map.get("serialNo");


        String upLoadUrl = null;
        String requestPath = FileConstant.SAVE_PATH;
        //String requestPath= "E\\";
        File fileDir = new File(requestPath);
        if (!fileDir.exists()) {
            fileDir.mkdir();
        }
        Date submitTime = new Date();
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);
        working(taskContext);

        Page<StatisticsSignVo> page = null;
        List<StatisticsSignVo> list = null;
        try {
            Integer pageNo = 1;
            Integer pageSize = Integer.MAX_VALUE;
            page = statisticsSignService.signAllAccountGroup(isAdmin, accountId, accountKwd, orgKwd, startDate, endDate, companyId, siteId, pageNo, pageSize);
            if (null != page) {
                list = page.getRecords();
                if (CollectionUtils.isEmpty(list)) {
                    success(taskContext, "未找到数据", "");
                }
            }
        } catch (Exception e) {
            LOGGER.error("获取数据过程中出现错误", e);
            fail(taskContext, "获取数据过程中出现错误");
        }
        try {
            //excel生成：excel-sheet-row-cell
            HSSFWorkbook workbook = new HSSFWorkbook();
            HSSFSheet sheet = workbook.createSheet("按用户统计签到明细" + DateFormatUtils.format(submitTime, "yyyyMMddHHmmss"));
            // 在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short,
            // 创建第1行
            //创建单元格格式
            HSSFCellStyle style = workbook.createCellStyle();
            style.setAlignment(HorizontalAlignment.LEFT);//创建居左格式
            HSSFRow row = sheet.createRow((int) 0);
            //合并单元格
            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 4); // 起始行, 终止行, 起始列, 终止列
            sheet.addMergedRegion(cra);
            HSSFCell cell = row.createCell((short) 0);
            cell.setCellValue("按用户统计签到明细" + DateFormatUtils.format(submitTime, "yyyyMMdd"));

            row = sheet.createRow((int) 1);
            cra = new CellRangeAddress(1, 1, 0, 4); // 起始行, 终止行, 起始列, 终止列
            sheet.addMergedRegion(cra);
            if (startDate == null) {
                startDate = "xxxx";
            }
            if (endDate == null) {
                endDate = "xxxx";
            }
            row.createCell((short) 0).setCellValue("统计周期:" + startDate + "～" + endDate);

            row = sheet.createRow((int) 2);
            row.createCell((short) 0).setCellValue("用户名");
            row.createCell((short) 1).setCellValue("姓名");
            row.createCell((short) 2).setCellValue("所在部门");
            row.createCell((short) 3).setCellValue("所在组织架构");
            row.createCell((short) 4).setCellValue("项目名称");
            row.createCell((short) 5).setCellValue("规定签到时间");
            row.createCell((short) 6).setCellValue("签到参考点");
            row.createCell((short) 7).setCellValue("签到范围");
            row.createCell((short) 8).setCellValue("签到状态");
            row.createCell((short) 9).setCellValue("签到时间");

            int index = 3;
            int s = 0;
            for (StatisticsSignVo statisticsSignVo : list) {
                if (statisticsSignVo == null) {
                    continue;
                }
                row = sheet.createRow((int) index);
                cell = row.createCell((short) 0);
                if (statisticsSignVo.getAccountName() != null) {
                    cell.setCellValue(statisticsSignVo.getAccountName());
                }
                cell = row.createCell((short) 1);
                if (statisticsSignVo.getAccountFullname() != null) {
                    cell.setCellValue(statisticsSignVo.getAccountFullname());
                }
                cell = row.createCell((short) 2);
                if (statisticsSignVo.getOrgName() != null) {
                    cell.setCellValue(statisticsSignVo.getOrgName());
                }
                cell = row.createCell((short) 3);
                if (statisticsSignVo.getOrgNames() != null) {
                    cell.setCellValue(statisticsSignVo.getOrgNames());
                }
                cell = row.createCell((short) 4);
                if (statisticsSignVo.getTrainningProjectName() != null) {
                    cell.setCellValue(statisticsSignVo.getTrainningProjectName());
                }
                cell = row.createCell((short) 5);
                if (statisticsSignVo.getSignStartTime() != null && statisticsSignVo.getSignEndTime() != null) {
                    cell.setCellValue(statisticsSignVo.getSignStartTime() + "~" + statisticsSignVo.getSignEndTime());
                }
                cell = row.createCell((short) 6);
                if (statisticsSignVo.getAddress() != null) {
                    cell.setCellValue(statisticsSignVo.getAddress());
                }
                cell = row.createCell((short) 7);
                if (statisticsSignVo.getRange() != null) {
                    cell.setCellValue(statisticsSignVo.getRange());
                }
                cell = row.createCell((short) 8);
                if (statisticsSignVo.getJoinState() != null) {
                    cell.setCellValue(statisticsSignVo.getJoinState() == 1 ? "已签到" : "未签到");
                }
                cell = row.createCell((short) 9);
                if (statisticsSignVo.getSignTime() != null) {
                    cell.setCellValue(statisticsSignVo.getSignTime());
                }
                index++;
                s++;
//                taskDetail(taskContext.getTaskId(),  "签到统计导出数据第" + s + "行成功");
            }
            StringBuffer fileNameSb = new StringBuffer().append("按用户统计签到明细").append(DateFormatUtils.format(submitTime, "yyyyMMddHHmmss")).append(".xls");
            String fileName = fileNameSb.toString();
            String path = new StringBuffer().append(requestPath).append(fileNameSb).toString();
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                workbook.write(os);
                //阿里云返回url
                upLoadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功", upLoadUrl);
            } catch (Exception e1) {
                LOGGER.error("写入数据到Excel的过程中或者上传到阿里云中发生错误", e1);
                fail(taskContext, "写入过程中发生错误");

            } finally {
                if (os != null) {
                    os.close();
                }
                if (workbook != null) {
                    workbook.close();
                }
                if (file != null) {
                    file.delete();
                }
            }
        } catch (Exception e) {
            LOGGER.error("创建Excel的过程中发生错误", e);
            fail(taskContext, "签到统计导出过程中发生错误，请查看日志");
        }
        return upLoadUrl;
    }
}
