package com.yizhi.statistics.application.controller.training;

import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.course.application.vo.CourseAccountReport;
import com.yizhi.statistics.application.commands.IStatisticsMetadataTask;
import com.yizhi.statistics.application.entity.vo.CourseCalculateTimeVo;
import com.yizhi.statistics.application.mapper.StatisticsTrainingStudentActivityRecordMapper;
import com.yizhi.statistics.application.scope.Scope;
import com.yizhi.statistics.application.scope.Type;
import com.yizhi.statistics.application.service.*;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.statistics.application.util.TrainingUtil;
import com.yizhi.statistics.application.work.CourseService;
import com.yizhi.training.application.feign.TrainingProjectClient;
import com.yizhi.training.application.feign.TrainingProjectReportClient;
import com.yizhi.training.application.vo.domain.*;
import com.yizhi.training.application.vo.manage.StatisticsTrainingRecorde;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 培训项目异步
 *
 * @author wangfeida
 */
@Component
public class TrainingControllerAbs extends AbstractDefaultTask<String, Map<String, Object>> implements IStatisticsMetadataTask<TrainingProjectVo> {


    private Logger logger = LoggerFactory.getLogger(TrainingControllerAbs.class);

    final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");

    @Autowired
    private TrainingUtil trainingUtil;

    @Autowired
    private JedisUtil jedisUtil;

    @Autowired
    private IdGenerator idGenerator;


    @Autowired
    private CourseService courseService;

    @Autowired
    private IStatisticsTrainingPlanService statisticsTrainingPlanService;


    @Autowired
    private TrainingProjectReportClient trainingProjectReportClient;

    @Autowired
    private TrainingProjectClient trainingProjectClient;

    @Autowired
    private IStatisticsTrainingStudentRecordeService statisticsTrainingStudentRecordeService;

    @Autowired
    private StatisticsTrainingStudentActivityRecordMapper statisticsTrainingStudentActivityRecordMapper;

    @Autowired
    private IStatisticsTrainingPlanActivityService statisticsTrainingPlanActivityService;

    @Autowired
    private IStatisticsTrainingStudentPlanRecordeService statisticsTrainingStudentPlanRecordeService;

    @Autowired
    private IStatisticsTrainingService statisticsTrainingService;

    @Autowired
    private IStatisticsTrainingStudentActivityRecordService statisticsTrainingStudentActivityRecordService;

    @Autowired
    private IStatisticsActivityPointService statisticsActivityPointService;

    @Override
    public String execute(Map<String, Object> map) {
        // TODO Auto-generated method stub
        Long taskId = idGenerator.generate();
        Long t1 = System.currentTimeMillis();
        /**
         * 走异步任务
         */
        Date curDateWork = new Date();
        // 如果是28号凌晨0：10开始跑 则curDate是27号的日期
        TaskContext taskContext = new TaskContext(taskId, sim.format(curDateWork) + " 号0:10培训项目报表分析前一天的数据", 999L, curDateWork);
        working(taskContext);

        // 得到执行项目
        List<TrainingProjectVo> allTrainingProject = trainingProjectReportClient.getAllTrainingProject();



        Integer countNum = allTrainingProject.size();
        Date middleMaxTime = statisticsTrainingStudentActivityRecordMapper.selectMaxDate();
        CourseCalculateTimeVo cctv = trainingUtil.calculateTime(map, middleMaxTime);
        Date startDate = cctv.getStartDate();
        Date endDate = cctv.getEndDate();
        String startDateString = cctv.getStartDateString();
        String endDateString = cctv.getEndDateString();


        if (CollectionUtils.isNotEmpty(allTrainingProject)) {
            taskDetail(taskId, "***** 培训项目统计开始 ,培训项目数量:" + allTrainingProject.size() + " *****");
            logger.info("***** 培训项目统计开始 ,培训项目数量:" + allTrainingProject.size() + " *****");
        } else {
            taskDetail(taskId, "***** 培训项目统计开始 ,培训项目数量获取异常:0或者为空 *****");
            logger.info("***** 培训项目统计开始 ,培训项目数量获取异常:0或者为空 *****");
        }

        int dealSuccess = 0;


        //转换scope对象
        Scope scope = new Scope();

        int i = 1;

        if (CollectionUtils.isNotEmpty(allTrainingProject)) {

            for (TrainingProjectVo tp : allTrainingProject) {

                Long temp1 = System.currentTimeMillis();
                Long siteId = tp.getSiteId();
                Long tpId = tp.getId();
                Long workId = tp.getId();
                Integer workScope = tp.getVisibleRange(); // 原始数据可见范围 0 游客 1 平台用户 2 指定范围//只能传1和2
                //每个业务都需要将本身的scope进行转换，下边所有的代码全部是switchScope
                Integer switchScope = scope.Training(workScope);
                Long companyId = tp.getCompanyId();


                List<TpAuthorizationRangeVo> authorizationRanges = null;
                CourseAccountReport courseAccountReport = new CourseAccountReport();

                // 对这个课程的学习记录开始操作
                if (siteId == null || companyId == null || workId == null || (switchScope == -1)) {
                    taskDetail(taskId, "培训项目ID:" + tp.getId() + "处理失败,原因:参数不正确");
                    continue;
                }
                if (switchScope == 2) {
                    authorizationRanges = trainingProjectClient.VisibleRange(tpId);
                    // set集合转换为arrayList集合
                    if (CollectionUtils.isEmpty(authorizationRanges)) {
                        logger.info("培训项目:" + tpId + ",是指定人可见，但是原始数据没有查询到可见范围权限");
                        continue;
                    } else {
                        List<Long> accountIds = new ArrayList<>();
                        List<Long> orgIds = new ArrayList<>();
                        for (TpAuthorizationRangeVo tpAuthorizationRange : authorizationRanges) {
                            //1部门 2用户
                            if (tpAuthorizationRange.getType() == 1) {
                                orgIds.add(tpAuthorizationRange.getRelationId());
                            } else if (tpAuthorizationRange.getType() == 2) {
                                accountIds.add(tpAuthorizationRange.getRelationId());
                            }
                        }
                        courseAccountReport.setCourseId(tpId);
                        courseAccountReport.setAccountIds(accountIds);
                        courseAccountReport.setOrgIds(orgIds);
                    }
                }
                courseService.executeTask(taskId, tp, workId, startDate, endDate, Type.TRAINNING.getType(), companyId, siteId, courseAccountReport, this, switchScope);
                dealSuccess++;
                Long temp2 = System.currentTimeMillis();
                String me = "统计完成" + (i++) + "培训项目,总数:" + countNum + "。项目id：" + workId + "。时间(ms):" + (temp2 - temp1);
                logger.info(me);
            }
            //所有培训项目进行循环完毕，把活动,实质上一个活动和培训项目事多对一的关系记录导入
            statisticsTrainingStudentActivityRecordService.dealGetAllRecordeByTimeLimit(startDateString, endDateString);

            //添加实际参加人数和人次
            statisticsTrainingStudentActivityRecordMapper.deleteTrainingActivitys();

            // 得到所有的培训项目
            List<Long> ids = statisticsTrainingStudentActivityRecordMapper.getTrainings();
            if (CollectionUtils.isNotEmpty(ids)) {
                // 添加实际参加人数和人次
                logger.info("***** deleteTrainingActivitys *****");
                statisticsTrainingStudentActivityRecordMapper.deleteTrainingActivitys();

                logger.info("***** addTrainingActivitys start *****");
                int pageSize = 10;
                double dcount = Double.valueOf(ids.size()) / Double.valueOf(pageSize);
                int pageCount = (int) Math.ceil(dcount);
                for (int index = 0; index < pageCount; index++) {
                    int fromIndex = index * pageSize;
                    int toIndex = (index + 1) * pageSize > ids.size() ? ids.size() : (index + 1) * pageSize;
                    statisticsTrainingStudentActivityRecordMapper.addTrainingActivitys(ids.subList(fromIndex, toIndex));
                }
                logger.info("***** addTrainingActivitys end *****");


                logger.info("***** deleteTrainingProjectActivitys *****");
                statisticsTrainingStudentActivityRecordMapper.deleteTrainingProjectActivitys();

                logger.info("***** addTrainingProjectActivitys start *****");
                pageSize = 10;
                dcount = Double.valueOf(ids.size()) / Double.valueOf(pageSize);
                pageCount = (int) Math.ceil(dcount);
                for (int index = 0; index < pageCount; index++) {
                    int fromIndex = index * pageSize;
                    int toIndex = (index + 1) * pageSize > ids.size() ? ids.size() : (index + 1) * pageSize;
                    statisticsTrainingStudentActivityRecordMapper.addTrainingProjectActivitys(ids.subList(fromIndex, toIndex));
                }
                logger.info("***** addTrainingProjectActivitys end *****");
            }
        }


        StringBuffer updateOrgIdResult = new StringBuffer("  ");
        taskDetail(taskId, updateOrgIdResult.toString());
        Long t2 = System.currentTimeMillis();
        String result = "总数量:" + allTrainingProject.size() + "。处理时间ms:" + (t2 - t1) + ",处理成功:" + dealSuccess + ",处理失败:" + (allTrainingProject.size() - dealSuccess);
        taskDetail(taskId, result);

        //被动接受更省时间，因为活动量少
        try {
            statisticsActivityPointService.dealBatchPointStatistics();
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            logger.info("培训项目跑完，跑积分出错。原因:" + e.getMessage());
        }

        try {
            //设置可参加人数缓存
            jedisUtil.setWorkIsSuccess(7, curDateWork);
            updateOrgIdResult.append(curDateWork + "培训项目和account跑成功。处理orgId成功。");

            success(taskContext, result);

        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            updateOrgIdResult.append(curDateWork + "培训项目和account跑成功。处理orgId失败。原因:" + e.getMessage());
        }

        return "success";
    }

    @Override
    public void executeMetadata(Long taskId, TrainingProjectVo metadataObject, Date startDate, Date endDate, Long companId, Long siteId) {
        // TODO Auto-generated method stub

        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        TrainingProjectVo tp = metadataObject;
        //对培训项目本身操作
        statisticsTrainingService.dealStatisticsTraining(tp);
        //对计划信息进行操作
        Long tpId = tp.getId();
        List<TpPlanVo> listTpPlanByTrainingProjectId = trainingProjectReportClient.getPlanMessageByTpId(tpId);
        statisticsTrainingPlanService.deleteStatisticsTrainingPlan(tpId);
        statisticsTrainingPlanService.selectSourcePlanToStatistics(listTpPlanByTrainingProjectId);
        // 对这个培训项目的学习记录开始操作
        Long id = tp.getId();
        String startDateString = df.format(startDate);
        String endDateString = df.format(endDate);
        StatisticsTrainingRecorde str = trainingProjectReportClient.getStatisticsTrainingRecordeById(id, startDateString, endDateString);
        if (str != null) {
            //项目和计划的对应关系，直接删除，重新插入
            List<TpPlanActivityVo> listTpPlanActivity = str.getListTpPlanActivity();
            statisticsTrainingPlanActivityService.dealStatisticsTrainingPlanActivityByTpId(id, listTpPlanActivity);
            //根据日期删除，进行插入 学员和培训项目的关系
            List<TpStudentProjectRecordVo> listTpStudentProjectRecord = str.getListTpStudentProjectRecord();
            statisticsTrainingStudentRecordeService.dealStatisticsTrainingStudentRecordeByTpIdAndTimeLimit(id, startDateString, endDateString, listTpStudentProjectRecord);

            //根据日期删除，进行插入 学员和计划的关系
            List<TpStudentPlanRecordVo> listTpStudentPlanRecord = str.getListTpStudentPlanRecord();

            statisticsTrainingStudentPlanRecordeService.dealIStatisticsTrainingStudentPlanRecordeByTpIdAndTimeLimit(id, startDateString, endDateString, listTpStudentPlanRecord);
        }
        taskDetail(taskId, "培训项目ID:" + metadataObject.getId() + "处理成功");
    }


    @Override
    public void exceptionLog(Long taskId, TrainingProjectVo metadataObject, Exception e) {
        // TODO Auto-generated method stub
        taskDetail(taskId, "培训项目ID:" + metadataObject.getId() + "处理失败" + e.getMessage());
    }


    protected Date addDay(Date date, int num) {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        calendar.add(calendar.DATE, num);
        date = calendar.getTime();
        return date;
    }
}
