package com.yizhi.statistics.application.controller.training;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.course.application.vo.CourseAccountReport;
import com.yizhi.statistics.application.commands.IStatisticsMetadataTask;
import com.yizhi.statistics.application.mapper.StatisticsTrainingStudentActivityRecordMapper;
import com.yizhi.statistics.application.scope.Scope;
import com.yizhi.statistics.application.scope.Type;
import com.yizhi.statistics.application.service.*;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.statistics.application.work.CourseService;
import com.yizhi.training.application.feign.TrainingProjectClient;
import com.yizhi.training.application.feign.TrainingProjectReportClient;
import com.yizhi.training.application.vo.domain.TpAuthorizationRangeVo;
import com.yizhi.training.application.vo.domain.TrainingProjectVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 培训项目手动指定时间 ，指定项目跑批
 *
 * @author wangfeida
 */
@Component
public class TrainingControllerHandAbs extends AbstractDefaultTask<String, Map<String, Object>> implements IStatisticsMetadataTask<TrainingProjectVo> {


    private Logger logger = LoggerFactory.getLogger(TrainingControllerHandAbs.class);

    @Autowired
    private JedisUtil jedisUtil;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private CourseService courseService;

    @Autowired
    private IStatisticsTrainingPlanService statisticsTrainingPlanService;


    @Autowired
    private TrainingProjectReportClient trainingProjectReportClient;

    @Autowired
    private TrainingProjectClient trainingProjectClient;

    @Autowired
    private IStatisticsTrainingStudentRecordeService statisticsTrainingStudentRecordeService;

    @Autowired
    private StatisticsTrainingStudentActivityRecordMapper statisticsTrainingStudentActivityRecordMapper;

    @Autowired
    private IStatisticsTrainingPlanActivityService statisticsTrainingPlanActivityService;

    @Autowired
    private IStatisticsTrainingStudentPlanRecordeService statisticsTrainingStudentPlanRecordeService;

    @Autowired
    private IStatisticsTrainingService statisticsTrainingService;

    @Autowired
    private IStatisticsTrainingStudentActivityRecordService statisticsTrainingStudentActivityRecordService;

    @Autowired
    private IStatisticsActivityPointService statisticsActivityPointService;

    @Override
    public String execute(Map<String, Object> map) {

        Long tpId = (Long) map.get("tpId");
        String startAt = (String) map.get("startAt");
        String endAt = (String) map.get("endAt");

        // 得到所有的培训项目
        List<TrainingProjectVo> allTrainingProject = trainingProjectReportClient.getAllTrainingProject();

        TrainingProjectVo trainingProjectVo = allTrainingProject.stream().filter(it -> it.getId().equals(tpId)).findAny().get();
        //转换scope对象
        Scope scope = new Scope();
        int i = 1;
        Long temp1 = System.currentTimeMillis();
        Long siteId = trainingProjectVo.getSiteId();
        Integer workScope = trainingProjectVo.getVisibleRange(); // 原始数据可见范围 0 游客 1 平台用户 2 指定范围//只能传1和2
        //每个业务都需要将本身的scope进行转换，下边所有的代码全部是switchScope
        Integer switchScope = scope.Training(workScope);
        Long companyId = trainingProjectVo.getCompanyId();


        List<TpAuthorizationRangeVo> authorizationRanges = null;
        CourseAccountReport courseAccountReport = new CourseAccountReport();

        if (switchScope == 2) {
            authorizationRanges = trainingProjectClient.VisibleRange(tpId);
            // set集合转换为arrayList集合
            if (CollectionUtils.isNotEmpty(authorizationRanges)) {
                List<Long> accountIds = new ArrayList<>();
                List<Long> orgIds = new ArrayList<>();
                for (TpAuthorizationRangeVo tpAuthorizationRange : authorizationRanges) {
                    //1部门 2用户
                    if (tpAuthorizationRange.getType() == 1) {
                        orgIds.add(tpAuthorizationRange.getRelationId());
                    } else if (tpAuthorizationRange.getType() == 2) {
                        accountIds.add(tpAuthorizationRange.getRelationId());
                    }
                }
                courseAccountReport.setCourseId(tpId);
                courseAccountReport.setAccountIds(accountIds);
                courseAccountReport.setOrgIds(orgIds);
            }
        }
        courseService.executeTask(idGenerator.generate(), trainingProjectVo, tpId, DateUtil.parseDate(startAt), DateUtil.parseDate(endAt), Type.TRAINNING.getType(), companyId, siteId, courseAccountReport, this, switchScope);

        //所有培训项目进行循环完毕，把活动,实质上一个活动和培训项目事多对一的关系记录导入
//        statisticsTrainingStudentActivityRecordService.dealGetAllRecordeByTimeAndIdLimit();
        // TODO 修改为只处理指定陪练
//        statisticsTrainingStudentActivityRecordService.dealGetAllRecordeByTimeLimit(startAt, endAt);

        //添加实际参加人数和人次
        statisticsTrainingStudentActivityRecordMapper.deleteTrainingActivitysById(tpId);

        logger.info("***** addTrainingActivitys start *****");
        List<Long> trainingProjectIds = new ArrayList<>();
        trainingProjectIds.add(tpId);
        statisticsTrainingStudentActivityRecordMapper.addTrainingActivitys(trainingProjectIds);

        statisticsTrainingStudentActivityRecordMapper.addTrainingProjectActivitys(trainingProjectIds);
        logger.info("***** addTrainingProjectActivitys end *****");

        //被动接受更省时间，因为活动量少
        try {
            statisticsActivityPointService.dealBatchPointStatistics();
        } catch (Exception e) {
            logger.error("培训项目跑完，跑积分出错。原因:" + e);
        }

        return "success";
    }

    @Override
    public void executeMetadata(Long taskId, TrainingProjectVo metadataObject, Date startDate, Date endDate, Long companId, Long siteId) {
//        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
//        TrainingProjectVo tp = metadataObject;
        //对培训项目本身操作
//        statisticsTrainingService.dealStatisticsTraining(tp);
        //对计划信息进行操作
//        Long tpId = tp.getId();
//        List<TpPlanVo> listTpPlanByTrainingProjectId = trainingProjectReportClient.getPlanMessageByTpId(tpId);
//        statisticsTrainingPlanService.deleteStatisticsTrainingPlan(tpId);
//        statisticsTrainingPlanService.selectSourcePlanToStatistics(listTpPlanByTrainingProjectId);
        // 对这个培训项目的学习记录开始操作
//        Long id = tp.getId();
//        String startDateString = df.format(startDate);
//        String endDateString = df.format(endDate);
//        StatisticsTrainingRecorde str = trainingProjectReportClient.getStatisticsTrainingRecordeById(id, startDateString, endDateString);
//        if (str != null) {
            //项目和计划的对应关系，直接删除，重新插入
//            List<TpPlanActivityVo> listTpPlanActivity = str.getListTpPlanActivity();
//            statisticsTrainingPlanActivityService.dealStatisticsTrainingPlanActivityByTpId(id, listTpPlanActivity);
            //根据日期删除，进行插入 学员和培训项目的关系
//            List<TpStudentProjectRecordVo> listTpStudentProjectRecord = str.getListTpStudentProjectRecord();
//            statisticsTrainingStudentRecordeService.dealStatisticsTrainingStudentRecordeByTpIdAndTimeLimit(id, startDateString, endDateString, listTpStudentProjectRecord);

            //根据日期删除，进行插入 学员和计划的关系
//            List<TpStudentPlanRecordVo> listTpStudentPlanRecord = str.getListTpStudentPlanRecord();

//            statisticsTrainingStudentPlanRecordeService.dealIStatisticsTrainingStudentPlanRecordeByTpIdAndTimeLimit(id, startDateString, endDateString, listTpStudentPlanRecord);
//        }
    }


    @Override
    public void exceptionLog(Long taskId, TrainingProjectVo metadataObject, Exception e) {
//        taskDetail(taskId, "培训项目ID:" + metadataObject.getId() + "处理失败" + e.getMessage());
    }
}
