package com.yizhi.statistics.application.controller.training.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.service.IStatisticsTrainingService;
import com.yizhi.statistics.application.training.vo.TrainingProjectGroupParam;
import com.yizhi.training.application.vo.manage.ReportStudyTrainingProjectAccountViewVo;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


/**
 * 按照用户统计-查看
 * @author wangfeida
 *
 */

@Component
public class TrainingProjectAccountGroupViewExport extends AbstractDefaultTask<String, TrainingProjectGroupParam> {

	private static final Logger logger = LoggerFactory.getLogger(TrainingProjectAccountGroupViewExport.class);

	
	 @Autowired
	 IStatisticsTrainingService statisticsTrainingProjectService;

	@Override
	protected String execute(TrainingProjectGroupParam tpgx) {
		// TODO Auto-generated method stub
		String sheetName="培训项目-按用户统计-查看";
		String upLoadUrl = null;
		Long adminId = tpgx.getAdminId();
		Boolean isAdmin=tpgx.getIsAdmin();
		Long companyId = tpgx.getCompanyId();
		Long siteId = tpgx.getSiteId();
		String startDate = tpgx.getStartDate();
		String endDate = tpgx.getEndDate();
		String kwd = tpgx.getKwd();
		Long taskId=tpgx.getTaskId();
		Date submitTime=tpgx.getSubmitTime();
		String serialNo=tpgx.getSerialNo();
		String taskName=tpgx.getTaskName();
		Long accountId=tpgx.getAccountId();
		String projectName=tpgx.getProjectName();
		Integer pageNo = 1;
		Integer pageSize = 10;
		Integer maxNumOneSheet=50000;   //一个sheet最多有多少数据量
		Integer numSheet=0;             //有多少个sheet
		Integer curSheet=1;
        
		TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,adminId, submitTime, siteId, companyId);// 任务id  任务名字      操作人id   操作时间
		working(taskContext);

		Page<ReportStudyTrainingProjectAccountViewVo> page=statisticsTrainingProjectService.trainingProjectAccountGroupView(companyId, siteId, pageNo, pageSize, accountId, projectName,2);
		List<ReportStudyTrainingProjectAccountViewVo> list = new ArrayList<ReportStudyTrainingProjectAccountViewVo>();
		
		if (page != null) {

			logger.info("培训项目-按照个人统计-查看-下载-总记录数：{}", page.getTotal());
			list.addAll(page.getRecords());
			int num=page.getTotal()/pageSize+1;
			for (int i = 2; i <= num; i++) {
				page= statisticsTrainingProjectService.trainingProjectAccountGroupView(companyId, siteId, i, pageSize, accountId, projectName,2);
				if(page == null || page.getRecords().size() == 0) {
					break;
				}
				list.addAll(page.getRecords());
			}
		}
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
		SimpleDateFormat df2 = new SimpleDateFormat("yyyyMMddHHmmss");
		try {
			String headerName=null;
			if(!CollectionUtils.isEmpty(list)) {
				headerName=list.get(0).getAccountName();
			}
			XSSFWorkbook wb = new XSSFWorkbook();
			if(CollectionUtils.isNotEmpty(list)) {
				numSheet = list.size() / maxNumOneSheet + 1;
				XSSFSheet sheet=null;
				XSSFRow row =null;
				XSSFCell cell=null;
				XSSFRow row2 =null;
				XSSFCell cell2 =null;
				XSSFRow row3 =null;
				for (int i = 1; i <= numSheet; i++) {
				
					sheet = wb.createSheet(sheetName+i);
					row = sheet.createRow((int) 0);
					XSSFCellStyle style = wb.createCellStyle();
					style.setAlignment(HorizontalAlignment.CENTER); 
					CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 7);
					sheet.addMergedRegion(cra);
					cell = row.createCell(0);
					cell.setCellValue(headerName + "项目学习明细统计");
					cell.setCellStyle(style);
					row2 = sheet.createRow(1);
					CellRangeAddress cra2 = new CellRangeAddress(1, 1, 0, 3);
					sheet.addMergedRegion(cra2);
					cell2 = row2.createCell(0);
					cell2.setCellValue("统计周期： " + startDate + "~" + endDate);

					row3 = sheet.createRow((int) 2);
					row3.createCell(0).setCellValue("用户名");
					row3.createCell(1).setCellValue("姓名");
					row3.createCell(2).setCellValue("项目名称");
					row3.createCell(3).setCellValue("开始时间");
					row3.createCell(4).setCellValue("最早完成时间");
					
					row3.createCell(5).setCellValue("获得积分");
					row3.createCell(6).setCellValue("学习状态");
					row3.createCell(7).setCellValue("用户当前状态");
					row3.createCell(8).setCellValue("所在部门");
					
					if (list != null && list.size() > 0) {
						ReportStudyTrainingProjectAccountViewVo rscv = null;
						for (int j = 0; j < maxNumOneSheet; j++) {
							row = sheet.createRow(j + 3);
							int index = j + (i - 1) * maxNumOneSheet;
							if (index >= list.size()) {
								break;
							}
							rscv = list.get(index);
						
							row.createCell(0).setCellValue(rscv.getAccountName());
							row.createCell(1).setCellValue(rscv.getAccountFullName());
							row.createCell(2).setCellValue(rscv.getProjectName());
							if (rscv.getStartTime() != null) {
								row.createCell(3).setCellValue(df.format(rscv.getStartTime()));
							}
							//完成时间就是首次完成时间
							if (rscv.getEndTime() != null) {
								row.createCell(4).setCellValue(df.format(rscv.getEndTime()));
							}
							if(rscv.getPoint()!=null) {
								row.createCell(5).setCellValue(rscv.getPoint());
							}
							
							row.createCell(6).setCellValue(rscv.getState());
							row.createCell(7).setCellValue(rscv.getAccountState() == 0 ? "禁用" : "启用");
							row.createCell(8).setCellValue(rscv.getOrgName());
							//taskDetail(taskContext.getTaskId(), sheetName+"导出数据第+" + (i+1) + "+行成功");
						}
					}
				}
			}
			String fileName = sheetName + df2.format(submitTime) + ".xlsx";
			String requestPath = FileConstant.SAVE_PATH;
			//String requestPath="E:\\home";
			File fileDir = new File(requestPath);
			if (!fileDir.exists()) {
				fileDir.mkdir();
			}
			String path = requestPath+File.separator + fileName;
			FileOutputStream os = null;
			File file = null;
			try {
				os = new FileOutputStream(path);
				wb.write(os);
				// 阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file = new File(path);
				success(taskContext, "成功", upLoadUrl);
			} catch (Exception e1) {
				e1.printStackTrace();
				fail(taskContext, "写入过程中发生错误");
				logger.error("上传过程中发生错误，请查看日志");
			} finally {
				if (os != null) {
					os.close();
				}
				if (wb != null) {
					wb.close();
				}
				if (file != null) {
					file.delete();
				}
			}

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, sheetName+"导出过程中发生错误，请查看日志"+e.getMessage());
			logger.error(sheetName+"导出过程中发生错误，请查看日志");
		}
		return upLoadUrl;
	 }
}
