package com.yizhi.statistics.application.controller.training.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.service.IStatisticsTrainingService;
import com.yizhi.statistics.application.training.vo.TrainingProjectGroupParam;
import com.yizhi.training.application.vo.manage.ReportStudyTrainingProjectVo;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


/**
 * 按照项目统计
 * @author wangfeida
 *
 */


@Component
public class TrainingProjectGroupExport extends AbstractDefaultTask<String, TrainingProjectGroupParam> {

	private static final Logger logger = LoggerFactory.getLogger(TrainingProjectGroupExport.class);

	
	 @Autowired
	 IStatisticsTrainingService statisticsTrainingProjectService;



	@Override
	protected String execute(TrainingProjectGroupParam tpgx) {
		// TODO Auto-generated method stub
		String sheetName="项目完成情况统计";
		String upLoadUrl = null;
		Long adminId = tpgx.getAdminId();
		Boolean isAdmin=tpgx.getIsAdmin();
		Long companyId = tpgx.getCompanyId();
		Long siteId = tpgx.getSiteId();
		String startDate = tpgx.getStartDate();
		String endDate = tpgx.getEndDate();
		String kwd = tpgx.getKwd();
		Long taskId=tpgx.getTaskId();
		Date submitTime=tpgx.getSubmitTime();
		String serialNo=tpgx.getSerialNo();
		String taskName=tpgx.getTaskName();
		Integer pageNo = 2;
		Integer pageSize = 10;
		
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,adminId, submitTime, siteId, companyId);// 任务id  任务名字      操作人id   操作时间
		working(taskContext);
		Page<ReportStudyTrainingProjectVo> page = statisticsTrainingProjectService.trainingProjectGroup(kwd, startDate, endDate, companyId, siteId, adminId, isAdmin, pageNo-1, pageSize);
		List<ReportStudyTrainingProjectVo> list = new ArrayList<ReportStudyTrainingProjectVo>();
		
		if (page != null) {
			logger.info("培训项目-项目完成情况统计-下载-总记录数：{}", page.getTotal());
			list.addAll(page.getRecords());
			int num=page.getTotal()/pageSize+1;
			for (int i = 2; i <= num; i++) {
				page=statisticsTrainingProjectService.trainingProjectGroup(kwd, startDate, endDate, companyId, siteId, adminId, isAdmin, i, pageSize);
				if(page == null || page.getRecords().size() == 0) {
					break;
				}
				list.addAll(page.getRecords());
			}
		}
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
		SimpleDateFormat df2 = new SimpleDateFormat("yyyyMMddHHmmss");
		
		
	try {
				   XSSFWorkbook wb = new XSSFWorkbook();
					XSSFSheet sheet = wb.createSheet(sheetName);
					XSSFRow row = sheet.createRow((int) 0);
					XSSFCellStyle style = wb.createCellStyle();
					style.setAlignment(HorizontalAlignment.CENTER); // 创建一个居中格式
					CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 7); // 起始行, 终止行, 起始列, 终止列
					sheet.addMergedRegion(cra);
					XSSFCell cell = row.createCell(0);
					cell.setCellValue(sheetName);
					cell.setCellStyle(style);
					XSSFRow row2 = sheet.createRow(1);
					CellRangeAddress cra2 = new CellRangeAddress(1, 1, 0, 3);
					sheet.addMergedRegion(cra2);
					XSSFCell cell2 = row2.createCell(0);
					cell2.setCellValue("统计周期： " + startDate + "~" + endDate);
					XSSFRow row3 = sheet.createRow((int) 2);
					row3.createCell(0).setCellValue("项目名称");
					row3.createCell(1).setCellValue("开始时间");
					row3.createCell(2).setCellValue("结束时间");
					row3.createCell(3).setCellValue("可参加人数");
					row3.createCell(4).setCellValue("实际参加人数");
					row3.createCell(5).setCellValue("完成人数");
					row3.createCell(6).setCellValue("完成率");
					// 第五步，写入实体数据 实际应用中这些数据从数据库得到，
					if (CollectionUtils.isNotEmpty(list)) {
						ReportStudyTrainingProjectVo rscv = null;
						for (int i = 0; i < list.size(); i++) {
							row = sheet.createRow(i+ 3);
							rscv = list.get(i);
							// 第四步，创建单元格，并设置值
							row.createCell(0).setCellValue(rscv.getName());
							if (rscv != null && rscv.getStartTime() != null) {
								row.createCell(1).setCellValue(df.format(rscv.getStartTime()));
							}
							if (rscv != null && rscv.getEndTime() != null) {
								row.createCell(2).setCellValue(df.format(rscv.getEndTime()));
							}
							row.createCell(3).setCellValue(rscv.getTotalAsk());
							row.createCell(4).setCellValue(rscv.getTotalIn());
							row.createCell(5).setCellValue(rscv.getTotalFinish());
							row.createCell(6).setCellValue(rscv.getAvgFinish() + "%");
						}
					}
			String fileName = sheetName + df2.format(submitTime) + ".xlsx";
			String requestPath = FileConstant.SAVE_PATH;
			File fileDir = new File(requestPath);
			if (!fileDir.exists()) {
				fileDir.mkdir();
			}
			String path = requestPath +File.separator+fileName;
			FileOutputStream os = null;
			File file = null;
			try {
				os = new FileOutputStream(path);
				wb.write(os);
				// 阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file = new File(path);
				success(taskContext, "成功", upLoadUrl);
			} catch (Exception e1) {
				e1.printStackTrace();
				fail(taskContext, "写入过程中发生错误");
				logger.error("上传过程中发生错误，请查看日志");
			} finally {
				if (os != null) {
					os.close();
				}
				if (wb != null) {
					wb.close();
				}
				if (file != null) {
					file.delete();
				}
			}

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, sheetName+"导出过程中发生错误，请查看日志"+e.getMessage());
			logger.error(sheetName+"导出过程中发生错误，请查看日志");
		}
		return upLoadUrl;
	 }
}
