package com.yizhi.statistics.application.controller.training.export;

import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.entity.StatisticsTraining;
import com.yizhi.statistics.application.mapper.StatisticsTrainingPlanActivityMapper;
import com.yizhi.statistics.application.service.IStatisticsTrainingService;
import com.yizhi.statistics.application.training.vo.TrainingProjectGroupParam;
import com.yizhi.training.application.vo.manage.TrainingProjectGroupViewExportVO1;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;


/**
 * 按照项目统计-查看-
 * @author wangfeida
 *
 */

@Component
public class TrainingProjectGroupViewExport1 extends AbstractDefaultTask<String, TrainingProjectGroupParam> {

	private static final Logger logger = LoggerFactory.getLogger(TrainingProjectGroupViewExport1.class);


	 @Autowired
	 IStatisticsTrainingService statisticsTrainingProjectService;

	 @Autowired
	 StatisticsTrainingPlanActivityMapper statisticsTrainingPlanActivityMapper;

	@Override
	protected String execute(TrainingProjectGroupParam tpgx) {
		// TODO Auto-generated method stub

		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
		SimpleDateFormat df2 = new SimpleDateFormat("yyyyMMddHHmmss");
		String sheetName="按项目统计-查看-下载项目学习结果-导出记录";
		String upLoadUrl = null;
		Long adminId = tpgx.getAdminId();
		Boolean isAdmin=tpgx.getIsAdmin();
		Long companyId = tpgx.getCompanyId();
		Long siteId = tpgx.getSiteId();
		Long taskId=tpgx.getTaskId();
		Date submitTime=tpgx.getSubmitTime();
		String serialNo=tpgx.getSerialNo();
		String taskName=tpgx.getTaskName();
		Long tpId=tpgx.getProjectId();
		Integer maxNumOneSheet=100*10000;   //一个sheet最多有多少数据量
		Integer numSheet=0;             //有多少个sheet
		StatisticsTraining statisticsTraining =statisticsTrainingProjectService.getStatisticsTrainingByTpId(tpId);

		String projectName=null;
		String startDate = null;
		String endDate = null;
		if(statisticsTraining!=null) {
			projectName=statisticsTraining.getTrainingProjectName();
			startDate=df.format(statisticsTraining.getStartTime());
			endDate=df.format(statisticsTraining.getEndTime());
		}

		TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,adminId, submitTime, siteId, companyId);// 任务id  任务名字      操作人id   操作时间
		working(taskContext);

		List<TrainingProjectGroupViewExportVO1> list = statisticsTrainingProjectService.export1(companyId, siteId, adminId, isAdmin, tpId);
		try {

			XSSFWorkbook wb = new XSSFWorkbook();
			if(list!=null&&list.size()>0) {
				numSheet = list.size() / maxNumOneSheet + 1;  //有这么多sheet
				for (int i = 1; i <= numSheet; i++) {
					String headerName = projectName + "完成情況统计";
					XSSFSheet sheet = wb.createSheet(sheetName + i);
					XSSFRow row = sheet.createRow((int) 0);
					XSSFCellStyle style = wb.createCellStyle();
					style.setAlignment(HorizontalAlignment.CENTER); // 创建一个居中格式
					CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 9); // 起始行, 终止行, 起始列, 终止列
					sheet.addMergedRegion(cra);
					XSSFCell cell = row.createCell(0);
					cell.setCellValue(headerName);
					cell.setCellStyle(style);
					XSSFRow row2 = sheet.createRow(1);
					CellRangeAddress cra2 = new CellRangeAddress(1, 1, 0, 3);
					sheet.addMergedRegion(cra2);
					XSSFCell cell2 = row2.createCell(0);
					cell2.setCellValue("项目周期： " + startDate + "~" + endDate);

					XSSFRow row3 = sheet.createRow(2);
					row3.createCell(0).setCellValue("项目名称");
					row3.createCell(1).setCellValue("用户名");
					row3.createCell(2).setCellValue("姓名");
					row3.createCell(3).setCellValue("开始时间");
					row3.createCell(4).setCellValue("完成时间");
					row3.createCell(5).setCellValue("获得积分");
					row3.createCell(6).setCellValue("学习状态");
					row3.createCell(7).setCellValue("用户状态");
					row3.createCell(8).setCellValue("所在部门");
					row3.createCell(9).setCellValue("所在组织架构");

					if (list != null && list.size() > 0) {
						TrainingProjectGroupViewExportVO1 rscv = null;
						for (int j = 0; j < maxNumOneSheet; j++) {
							row = sheet.createRow(j + 3);
							int index = j + (i - 1) * maxNumOneSheet;
							if (index >= list.size()) {
								break;
							}
							rscv = list.get(index);
							// 第四步，创建单元格，并设置值
							row.createCell(0).setCellValue(rscv.getProjectName());
							row.createCell(1).setCellValue(rscv.getAccountName());
							row.createCell(2).setCellValue(rscv.getAccountFullName());
							if (rscv.getStartTime() != null) {
								row.createCell(3).setCellValue(df.format(rscv.getStartTime()));
							}
							if (rscv.getEndTime() != null) {
								row.createCell(4).setCellValue(df.format(rscv.getEndTime()));
							}

                            if(rscv.getPoint()!=null) {
	                            row.createCell(5).setCellValue(rscv.getPoint());
                            }

							String studeyState = "不明确";
							if(rscv.getEndTime()!=null) {
								studeyState = "已完成";
							}
							if(rscv.getEndTime()==null&&rscv.getStartTime()!=null) {
								studeyState="进行中";
							}
							if (rscv.getEndTime()==null&&rscv.getStartTime()==null) {
								studeyState = "未开始";
							}
							row.createCell(6).setCellValue(studeyState);
							row.createCell(7).setCellValue(rscv.getAccountState() == 0 ? "禁用" : "启用");
							row.createCell(8).setCellValue(rscv.getOrgName());
							if (StringUtils.isNotBlank(rscv.getOrgNames())){
								row.createCell(9).setCellValue(rscv.getOrgNames());
							}else {
								row.createCell(9).setCellValue("--");
							}
						}
					}
				}
			}
			String fileName = sheetName + df2.format(submitTime) + ".xlsx";
			String requestPath = FileConstant.SAVE_PATH;
			//String requestPath= "E\\";
			File fileDir = new File(requestPath);
			if (!fileDir.exists()) {
				fileDir.mkdir();
			}
			String path = requestPath + File.separator + fileName;
			FileOutputStream os = null;
			File file = null;
			try {
				os = new FileOutputStream(path);
				wb.write(os);
				// 阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file = new File(path);
				success(taskContext, "成功", upLoadUrl);
			} catch (Exception e1) {
				e1.printStackTrace();
				fail(taskContext, "写入过程中发生错误");
				logger.error("上传过程中发生错误，请查看日志");
			} finally {
				if (os != null) {
					os.close();
				}
				if (wb != null) {
					wb.close();
				}
				if (file != null) {
					file.delete();
				}
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, sheetName+"导出过程中发生错误，请查看日志"+e.getMessage());
			logger.error(sheetName+"导出过程中发生错误，请查看日志");
		}
		return upLoadUrl;
	}
}
