package com.yizhi.statistics.application.controller.training.export;

import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.entity.StatisticsTraining;
import com.yizhi.statistics.application.service.IStatisticsTrainingService;
import com.yizhi.statistics.application.training.vo.TrainingProjectGroupParam;
import com.yizhi.training.application.feign.TrainingProjectReportClient;
import com.yizhi.training.application.vo.manage.TrainingProjectGroupViewExportVO2;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;


/**
 * 按照项目统计-查看-
 * @author wangfeida
 *
 */

@Component
public class TrainingProjectGroupViewExport2 extends AbstractDefaultTask<String, TrainingProjectGroupParam> {

	private static final Logger logger = LoggerFactory.getLogger(TrainingProjectGroupViewExport2.class);

	@Autowired
	private TrainingProjectReportClient trainingProjectReportClient;
	
	@Autowired
	 IStatisticsTrainingService statisticsTrainingProjectService;

	@Override
	protected String execute(TrainingProjectGroupParam tpgx) {
		// TODO Auto-generated method stub
		
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
		SimpleDateFormat df2 = new SimpleDateFormat("yyyyMMddHHmmss");
		String sheetName="按项目统计-查看-下载学习计划明细";
		String upLoadUrl = null;
		Long adminId = tpgx.getAdminId();
		Boolean isAdmin=tpgx.getIsAdmin();
		Long companyId = tpgx.getCompanyId();
		Long siteId = tpgx.getSiteId();
		Long taskId=tpgx.getTaskId();
		Date submitTime=tpgx.getSubmitTime();
		String serialNo=tpgx.getSerialNo();
		String taskName=tpgx.getTaskName();
		Long tpId=tpgx.getProjectId();
		
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,adminId, submitTime, siteId, companyId);// 任务id  任务名字      操作人id   操作时间
		working(taskContext);

		/**
		 * 项目主体信息-表头
		 */
	
		StatisticsTraining statisticsTraining =statisticsTrainingProjectService.getStatisticsTrainingByTpId(tpId);
		String projectName=null;
		String startDate = null;
		String endDate = null;
		if(statisticsTraining!=null) {
			projectName=statisticsTraining.getTrainingProjectName();
			startDate=df.format(statisticsTraining.getStartTime());
			endDate=df.format(statisticsTraining.getEndTime());
		}

		Integer maxNumOneSheet=1000000;   //一个sheet最多有多少数据量
		Integer numSheet=0;             //有多少个sheet
		List<TrainingProjectGroupViewExportVO2> list = statisticsTrainingProjectService.export2(companyId, siteId, adminId, isAdmin, tpId);
		try {
			XSSFWorkbook wb = new XSSFWorkbook();
			if(CollectionUtils.isNotEmpty(list)) {
				numSheet = list.size() / maxNumOneSheet + 1;
				String headerName=null;
				XSSFSheet sheet =null;
				XSSFCell cell2 =null;
				XSSFRow row3=null;
				for (int i = 1; i <= numSheet; i++) {
					headerName = projectName + "活动完成情况统计";
					sheet = wb.createSheet(sheetName + i);
					XSSFRow row = sheet.createRow((int) 0);
					XSSFCellStyle style = wb.createCellStyle();
					style.setAlignment(HorizontalAlignment.CENTER); 
					CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 7); 
					sheet.addMergedRegion(cra);
					XSSFCell cell = row.createCell(0);
					cell.setCellValue(headerName);
					cell.setCellStyle(style);
					XSSFRow row2 = sheet.createRow((int) 1);
					CellRangeAddress cra2 = new CellRangeAddress(1, 1, 0, 3);
					sheet.addMergedRegion(cra2);
					cell2 = row2.createCell(0);
					cell2.setCellValue("统计周期： " + startDate + "~" + endDate);
					row3 = sheet.createRow((int) 2);
					row3.createCell(0).setCellValue("项目名称");
					row3.createCell(1).setCellValue("学习计划名称");
					row3.createCell(2).setCellValue("活动类型");
					row3.createCell(3).setCellValue("活动名称");
					row3.createCell(4).setCellValue("可参加人数");
					row3.createCell(5).setCellValue("实际参加人数");
					row3.createCell(6).setCellValue("完成人数");
					row3.createCell(7).setCellValue("完成率");
					if (list != null && list.size() > 0) {
						TrainingProjectGroupViewExportVO2 rscv = null;
						for (int j = 0; j < maxNumOneSheet; j++) {
							row = sheet.createRow(j + 3);
							int index = j + (i - 1) * maxNumOneSheet;
							if (index >= list.size()) {
								break;
							}
							rscv = list.get(index);
							// 第四步，创建单元格，并设置值
							row.createCell(0).setCellValue(rscv.getProjectName());
							row.createCell(1).setCellValue(rscv.getPlanName());
							row.createCell(2).setCellValue(rscv.getActivityType());
							row.createCell(3).setCellValue(rscv.getActivityName());
							row.createCell(4).setCellValue(rscv.getTotalAskAccount());
							row.createCell(5).setCellValue(rscv.getTotalJoinAccount());
							row.createCell(6).setCellValue(rscv.getTotalFinishAccount());
							if (rscv.getAvgFinish() != null) {
								row.createCell(7).setCellValue(rscv.getAvgFinish() + "%");
							}
						}
					}
				}
			}
			String fileName = sheetName + df2.format(submitTime) + ".xlsx";
			String requestPath = FileConstant.SAVE_PATH;
			//String requestPath= "E\\";
			File fileDir = new File(requestPath);
			if (!fileDir.exists()) {
				fileDir.mkdir();
			}
			String path = requestPath +File.separator+ fileName;
			FileOutputStream os = null;
			File file = null;
			try {
				os = new FileOutputStream(path);
				wb.write(os);
				// 阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file = new File(path);
				success(taskContext, "成功", upLoadUrl);
			} catch (Exception e1) {
				e1.printStackTrace();
				fail(taskContext, "写入过程中发生错误");
				logger.error("上传过程中发生错误，请查看日志");
			} finally {
				if (os != null) {
					os.close();
				}
				if (wb != null) {
					wb.close();
				}
				if (file != null) {
					file.delete();
				}
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, sheetName+"导出过程中发生错误，请查看日志"+e.getMessage());
			logger.error(sheetName+"导出过程中发生错误，请查看日志");
		}
		return upLoadUrl;
	}
}
