package com.yizhi.statistics.application.controller.training.export;

import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.toolkit.StringUtils;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.domain.StatisticsExamRecorde;
import com.yizhi.statistics.application.entity.StatisticsTraining;
import com.yizhi.statistics.application.service.IStatisticsAccountRoleService;
import com.yizhi.statistics.application.service.IStatisticsTrainingService;
import com.yizhi.statistics.application.service.StatisticsExamRecordeService;
import com.yizhi.statistics.application.training.vo.TrainingProjectAccountRecords;
import com.yizhi.statistics.application.training.vo.TrainingProjectGroupParam;
import com.yizhi.statistics.application.util.WmyDateUtils;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.streaming.SXSSFCell;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 按照项目统计-查看-
 *
 * @author wangfeida
 */

@Component
public class TrainingProjectGroupViewExport3 extends AbstractDefaultTask<String, TrainingProjectGroupParam> {

    private static final Logger logger = LoggerFactory.getLogger(TrainingProjectGroupViewExport3.class);


    @Autowired
    IStatisticsTrainingService statisticsTrainingProjectService;
    @Autowired
    StatisticsExamRecordeService statisticsExamRecordeService;

    @Autowired
    private IStatisticsAccountRoleService statisticsAccountRoleService;

    private final int START_ROW_INDEX = 3;

    @Override
    protected String execute(TrainingProjectGroupParam tpgx) {
        // TODO 友邦数据修改 下载学员学习明细 7
        // TODO Auto-generated method stub
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat df2 = new SimpleDateFormat("yyyyMMddHHmmss");
        String sheetName = "按项目统计-查看-下载学员学习明细-导出记录";
        String upLoadUrl = null;
        Long adminId = tpgx.getAdminId();
        Boolean isAdmin = tpgx.getIsAdmin();
        Long companyId = tpgx.getCompanyId();
        Long siteId = tpgx.getSiteId();
        Long taskId = tpgx.getTaskId();
        Date submitTime = tpgx.getSubmitTime();
        String serialNo = tpgx.getSerialNo();
        String taskName = tpgx.getTaskName();
        Long tpId = tpgx.getProjectId();
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, adminId, submitTime, siteId, companyId);// 任务id  任务名字      操作人id   操作时间
        working(taskContext);

        //验证下管辖区是否是全平台的
        if (!isAdmin) {
            isAdmin = statisticsAccountRoleService.getIsAdmin(0, siteId, adminId);
        }


        StatisticsTraining statisticsTraining = statisticsTrainingProjectService.getStatisticsTrainingByTpId(tpId);
        String projectName = null;
        String startDate = null;
        String endDate = null;
        if (statisticsTraining != null) {
            projectName = statisticsTraining.getTrainingProjectName();
            startDate = df.format(statisticsTraining.getStartTime());
            endDate = df.format(statisticsTraining.getEndTime());
        }

        //获取人员
        List<TrainingProjectAccountRecords> accountList = statisticsTrainingProjectService.getTrainingAccounts(companyId, siteId, isAdmin, adminId, tpId);
        if (CollectionUtils.isEmpty(accountList)) {
            fail(taskContext, "查询不到人员信息");
            return "";
        }

        //查询项目活动
        List<TrainingProjectAccountRecords> activityList = statisticsTrainingProjectService.getTrainingPlanActivitys(companyId, siteId, tpId);
        if (CollectionUtils.isEmpty(activityList)) {
            fail(taskContext, "查询项目活动信息");
            return "";
        }

        try {
            SXSSFWorkbook wb = new SXSSFWorkbook(1000);
            SXSSFSheet sheet = null;
            Long count = 0L;
            int sheetNum = 1;
            boolean isAddEmptySheet = true;
            int startRowIndex = START_ROW_INDEX;
            // 查询活动学习记录
            TrainingProjectAccountRecords accountRelation = null;
            List<TrainingProjectAccountRecords> relationList = null;
            int icount = 0;

            List<Long> examIds = activityList.stream().filter(it -> it.getActivityType().equals("考试")).map(TrainingProjectAccountRecords::getRelationId).collect(Collectors.toList());
            List<StatisticsExamRecorde> examRecorde = new ArrayList<>();
            if (examIds != null && CollectionUtils.isNotEmpty(examIds)) {
                examRecorde = statisticsExamRecordeService.getDurationByExamId(examIds, statisticsTraining.getStartTime(), statisticsTraining.getEndTime());
            }

            for (TrainingProjectAccountRecords activity : activityList) {
                String planName = activity.getPlanName();
                String projectNames = activity.getProjectName();
                String activityType = activity.getActivityType();
                String activityName = activity.getActivityName();

                logger.info("run:" + (icount++) + "relationId:" + activity.getRelationId() + "total:" + activityList.size() + " accountList" + accountList.size());

                Long tt = System.currentTimeMillis() / 1000;

                relationList = statisticsTrainingProjectService.getTrainingRelationsNew(activity.getRelationId(), statisticsTraining.getStartTime(), statisticsTraining.getEndTime());
                if (CollectionUtils.isEmpty(relationList)) {
                    logger.info(activityName + "(" + activity.getRelationId() + "):查询时间：" + (System.currentTimeMillis() / 1000 - tt) + " 查询数据：0");
                } else {
                    logger.info(activityName + "(" + activity.getRelationId() + "):查询时间：" + (System.currentTimeMillis() / 1000 - tt) + " 查询数据：" + relationList.size());
                }

                Map<Long, TrainingProjectAccountRecords> accountRelationMaps = new HashMap<Long, TrainingProjectAccountRecords>();
                if (!CollectionUtils.isEmpty(relationList)) {
                    for (TrainingProjectAccountRecords relation : relationList) {
                        accountRelationMaps.put(relation.getAccountId(), relation);
                    }
                }

                // 遍历学习记录设置各学员的学习信息
                for (TrainingProjectAccountRecords account : accountList) {
                    account.setPlanName(planName);
                    account.setProjectName(projectNames);
                    account.setActivityType(activityType);
                    account.setActivityName(activityName);

                    accountRelation = accountRelationMaps.get(account.getAccountId());
                    if (accountRelation != null) {
                        account.setPoint(accountRelation.getPoint());
                        account.setScore(accountRelation.getScore());
                        account.setFirstFinishTime(accountRelation.getFirstFinishTime());
                        account.setStartTime(accountRelation.getStartTime());

                        account.setLearningTime(accountRelation.getLearningTime());

                        if (activityType.equals("考试")) {
                            Optional<StatisticsExamRecorde> optional = examRecorde.stream().filter(it-> it.getAccountId().equals(account.getAccountId()) && it.getExamId().equals(activity.getRelationId())).findFirst();
                            if (null != optional &&  optional.isPresent()) {
                                account.setLearningTime(optional.get().getSumDuration());
                            }else  {
                                account.setLearningTime(0);
                            }
                        }

                        account.setState(accountRelation.getState());
                    } else {
                        account.setPoint(null);
                        account.setScore(null);
                        account.setFirstFinishTime(null);
                        account.setStartTime(null);
                        account.setLearningTime(null);
                        account.setState(null);
                    }
                }

                if (count >= 200000) {
                    count = 0L;
                    startRowIndex = START_ROW_INDEX;
                }
                if (count.compareTo(0L) == 0) {
                    sheet = createSheet(wb, "学员学习明细统计" + (sheetNum++), projectName + "学员学习明细统计", startDate, endDate);
                    isAddEmptySheet = false;
                }

                accountRelationMaps.clear();
                accountRelationMaps = null;
                relationList.clear();
                relationList = null;

                writeSheetData(sheet, accountList, df, startRowIndex);

                count += accountList.size();
                startRowIndex = Integer.valueOf(String.valueOf(count)) + START_ROW_INDEX;

            }

            if (isAddEmptySheet) {
                createSheet(wb, "学员学习明细统计" + (sheetNum++), projectName + "学员学习明细统计", startDate, endDate);
            }

            String fileName = sheetName + df2.format(submitTime) + ".xlsx";
            String requestPath = FileConstant.SAVE_PATH;
            //String requestPath = "C:\\home\\file";
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }
            String path = requestPath + File.separator + fileName;
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                // 阿里云返回url
                upLoadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功", upLoadUrl);
            } catch (Exception e1) {
                e1.printStackTrace();
                fail(taskContext, "写入过程中发生错误");
                logger.error("上传过程中发生错误，请查看日志");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            fail(taskContext, sheetName + "导出过程中发生错误，请查看日志" + e.getMessage());
            logger.error(sheetName + "导出过程中发生错误，请查看日志");
        }
        return upLoadUrl;
    }

    /**
     * 在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short
     *
     * @param wb
     * @param sheetName
     * @param headerName
     * @param startDate
     * @param endDate
     * @return
     */
    @SuppressWarnings("unused")
    private SXSSFSheet createSheet(SXSSFWorkbook wb, String sheetName, String headerName, String startDate, String endDate) {

        SXSSFSheet sheet1 = null;
        SXSSFRow row = null;
        CellStyle style = null;
        SXSSFCell cell = null;
        SXSSFRow row2 = null;
        SXSSFCell cell2 = null;
        SXSSFRow row3 = null;

        // 第一步，在webbook中添加一个sheet,对应Excel文件中的sheet
        sheet1 = wb.createSheet(sheetName);
        // 第二步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short
        row = sheet1.createRow((int) 0);
        // 第三步，创建单元格，并设置值表头 设置表头居中
        style = wb.createCellStyle();
        style.setAlignment(HorizontalAlignment.CENTER); // 创建一个居中格式
        // 合并单元格
        CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 7); // 起始行, 终止行,
        // 起始列, 终止列
        sheet1.addMergedRegion(cra);
        cell = row.createCell(0);
        cell.setCellValue(headerName + "学员学习明细统计");
        cell.setCellStyle(style);

        // 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short
        row2 = sheet1.createRow((int) 1);
        // 第四步，创建单元格
        CellRangeAddress cra2 = new CellRangeAddress(1, 1, 0, 3);
        sheet1.addMergedRegion(cra2);
        cell2 = row2.createCell(0);
        cell2.setCellValue("项目周期： " + startDate + "~" + endDate);

        row3 = sheet1.createRow((int) 2);
        row3.createCell(0).setCellValue("用户名");
        row3.createCell(1).setCellValue("姓名");
        row3.createCell(2).setCellValue("学习计划名称");
        row3.createCell(3).setCellValue("学习活动名称");
        row3.createCell(4).setCellValue("学习活动类型");
        row3.createCell(5).setCellValue("开始时间");
        row3.createCell(6).setCellValue("最早完成时间");
        row3.createCell(7).setCellValue("课程学习时长");
        row3.createCell(8).setCellValue("考试时长");
        row3.createCell(9).setCellValue("所获积分");
        row3.createCell(10).setCellValue("完成状态");
        row3.createCell(11).setCellValue("用户当前状态");
        row3.createCell(12).setCellValue("所在部门");
        row3.createCell(13).setCellValue("所在组织架构");
        return sheet1;
    }

    private void writeSheetData(SXSSFSheet sheet, List<TrainingProjectAccountRecords> accountRelationList, SimpleDateFormat df, int startRowIndex) {
        SXSSFRow row = null;
        SXSSFCell sxssfCell = null;
        if (CollectionUtils.isNotEmpty(accountRelationList)) {
            TrainingProjectAccountRecords rscv = null;
            for (int j = 0; j < accountRelationList.size(); j++) {
                rscv = accountRelationList.get(j);

                // 创建单元格，并设置值
                row = sheet.createRow(startRowIndex++);
                sxssfCell = row.createCell(0);
                sxssfCell.setCellValue(rscv.getAccountName());

                sxssfCell = row.createCell(1);
                sxssfCell.setCellValue(rscv.getAccountFullName());

                sxssfCell = row.createCell(2);
                sxssfCell.setCellValue(rscv.getPlanName());

                sxssfCell = row.createCell(3);
                sxssfCell.setCellValue(rscv.getActivityName());

                sxssfCell = row.createCell(4);
                sxssfCell.setCellValue(rscv.getActivityType());

                if (rscv.getStartTime() != null) {
                    sxssfCell = row.createCell(5);
                    sxssfCell.setCellValue(df.format(rscv.getStartTime()));
                }
                if (rscv.getFirstFinishTime() != null) {
                    sxssfCell = row.createCell(6);
                    sxssfCell.setCellValue(df.format(rscv.getFirstFinishTime()));
                }

                // 课程时长和考试时长先社为0
                sxssfCell = row.createCell(7);
                sxssfCell.setCellValue(0);
                sxssfCell = row.createCell(8);
                sxssfCell.setCellValue(0);

                Integer totalLearningTime = rscv.getLearningTime();
                // 第一版只有课程时长，第二版增加考试时长
                if (null != totalLearningTime && totalLearningTime > 0) {
                    String time = WmyDateUtils.durationFormart(String.valueOf(totalLearningTime));
                    if ("课程".equals(rscv.getActivityType())) {
                        sxssfCell = row.createCell(7);
                        sxssfCell.setCellValue(time);
                    } else if ("考试".equals(rscv.getActivityType())) {
                        sxssfCell = row.createCell(8);
                        sxssfCell.setCellValue(time);
                    }
                }

                if (rscv.getPoint() != null) {
                    sxssfCell = row.createCell(9);
                    sxssfCell.setCellValue(rscv.getPoint());
                }

                String state = null;
                if (rscv.getState() != null) {
                    if (rscv.getState() == -1) {
                        state = "未开始";
                    }
                    if (rscv.getState() == 0) {
                        state = "进行中";
                    }
                    if (rscv.getState() == 1) {
                        state = "已完成";
                    }
                } else {
                    state = "未开始";
                }

                sxssfCell = row.createCell(10);
                sxssfCell.setCellValue(state);

                if (rscv.getAccountState() != null) {
                    sxssfCell = row.createCell(11);
                    sxssfCell.setCellValue(rscv.getAccountState() == 0 ? "禁用" : "启用");
                }

                sxssfCell = row.createCell(12);
                sxssfCell.setCellValue(rscv.getOrgName());
                if (StringUtils.isNotEmpty(rscv.getOrgNames())) {
                    sxssfCell = row.createCell(13);
                    sxssfCell.setCellValue(rscv.getOrgNames());
                } else {
                    sxssfCell = row.createCell(13);
                    sxssfCell.setCellValue("--");
                }

            }
        }
    }
}
