package com.yizhi.statistics.application.controller.training.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.toolkit.StringUtils;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.service.IStatisticsAccountRoleService;
import com.yizhi.statistics.application.service.IStatisticsTrainingService;
import com.yizhi.statistics.application.training.vo.TrainingProjectGroupParam;
import com.yizhi.training.application.feign.TrainingProjectReportClient;
import com.yizhi.training.application.vo.manage.ReportStudyTrainingProjectOrgVo;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Component
public class TrainingProjectOrgGroupExport  extends AbstractDefaultTask<String, TrainingProjectGroupParam> {

	private static final Logger logger = LoggerFactory.getLogger(TrainingProjectAccountGroupExport.class);

	@Autowired
	private TrainingProjectReportClient trainingProjectReportClient;

	 @Autowired
	 IStatisticsTrainingService statisticsTrainingProjectService;
	 
	@Autowired
	private IStatisticsAccountRoleService statisticsAccountRoleService;

	@Override
	protected String execute(TrainingProjectGroupParam tpgx) {
		// TODO Auto-generated method stub
		String sheetName="培训项目-按部门统计";
		String upLoadUrl = null;
		Long adminId = tpgx.getAdminId();
		Boolean isAdmin=tpgx.getIsAdmin();
		Long companyId = tpgx.getCompanyId();
		Long siteId = tpgx.getSiteId();
		String startDate = tpgx.getStartDate();
		String endDate = tpgx.getEndDate()+" 23:59:59";
		String kwd = tpgx.getKwd();
		Long taskId=tpgx.getTaskId();
		Date submitTime=tpgx.getSubmitTime();
		String serialNo=tpgx.getSerialNo();
		String taskName=tpgx.getTaskName();
		String orgKwd=tpgx.getOrgKwd();
		String accountKwd=tpgx.getAccountKwd();

		TaskContext taskContext = new TaskContext(taskId, serialNo, taskName,adminId, submitTime, siteId, companyId);// 任务id  任务名字      操作人id   操作时间
		working(taskContext);


		Integer pageNo = 1;
		Integer pageSize = 10;
		Integer maxNumOneSheet=50000;   //一个sheet最多有多少数据量
		Integer numSheet=0;             //有多少个sheet
		Integer curSheet=1;


		//验证下管辖区是否是全平台的
		if (!isAdmin) {
			isAdmin = statisticsAccountRoleService.getIsAdmin(0, siteId, adminId);
		}
		Page<ReportStudyTrainingProjectOrgVo> page = statisticsTrainingProjectService.trainingProjectOrgGroup(companyId, siteId, adminId, isAdmin, orgKwd, pageNo, pageSize,startDate,endDate);
		List<ReportStudyTrainingProjectOrgVo> list = new ArrayList<ReportStudyTrainingProjectOrgVo>();
		if (page != null) {



			list.addAll(page.getRecords());
			int num=page.getTotal()/pageSize+1;
			logger.info("培训项目-按照部门统计-下载-总记录数：{} 总页数：{}", page.getTotal(),num);
			for (int i = 2; i <= num; i++) {
				tpgx.setPageNo(i);
				page=statisticsTrainingProjectService.trainingProjectOrgGroup(companyId, siteId, adminId, isAdmin, orgKwd, i, pageSize,startDate,endDate);

				if(page == null || page.getRecords().size() == 0) {
					break;
				}
				list.addAll(page.getRecords());
			}

		}
		//SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		SimpleDateFormat df2 = new SimpleDateFormat("yyyyMMddHHmmss");


		try {

			XSSFWorkbook wb = new XSSFWorkbook();
			if(list!=null&&list.size()>0) {
				numSheet = list.size() / maxNumOneSheet + 1;  //有这么多sheet
				XSSFSheet sheet =null;
				XSSFRow row =null;
				XSSFCell cell =null;
				XSSFCell cell2 =null;
				XSSFRow row3=null;
				for (int i = 1; i <= numSheet; i++) {

					sheet = wb.createSheet(sheetName+i);

					row = sheet.createRow((int) 0);

					XSSFCellStyle style = wb.createCellStyle();
					style.setAlignment(HorizontalAlignment.CENTER); // 创建一个居中格式

					CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 7); // 起始行, 终止行, 起始列, 终止列
					sheet.addMergedRegion(cra);
					cell = row.createCell(0);
					cell.setCellValue(sheetName);
					cell.setCellStyle(style);


					XSSFRow row2 = sheet.createRow((int) 1);

					CellRangeAddress cra2 = new CellRangeAddress(1, 1, 0, 3);
					sheet.addMergedRegion(cra2);
					cell2 = row2.createCell(0);
					cell2.setCellValue("统计周期： " + startDate + "~" + endDate);

					row3 = sheet.createRow((int) 2);
					row3.createCell(0).setCellValue("部门编码");
					row3.createCell(1).setCellValue("部门名称");
					row3.createCell(2).setCellValue("所在组织架构");
					row3.createCell(3).setCellValue("可参加项目数");
					row3.createCell(4).setCellValue("可参加项目人数");
					row3.createCell(5).setCellValue("实际参加人数");
					row3.createCell(6).setCellValue("完成人数");
					row3.createCell(7).setCellValue("完成率");
					// 第五步，写入实体数据 实际应用中这些数据从数据库得到，
					if (CollectionUtils.isNotEmpty(list)) {
						ReportStudyTrainingProjectOrgVo rscv = null;
						for (int j = 0; j < maxNumOneSheet; j++) {
							row = sheet.createRow(j + 3);
							int a = j + (i - 1) * maxNumOneSheet;
							if (a >= list.size()) {
								break;
							}
							rscv = list.get(a);
							// 第四步，创建单元格，并设置值
							row.createCell(0).setCellValue(rscv.getOrgNo());
							row.createCell(1).setCellValue(rscv.getOrgName());
							if (StringUtils.isNotEmpty(rscv.getOrgNames())){
								row.createCell(2).setCellValue(rscv.getOrgNames());
							}else {
								row.createCell(2).setCellValue("--");
							}
							row.createCell(3).setCellValue(rscv.getTotalAskProject());
							row.createCell(4).setCellValue(rscv.getTotalAskAccount());
							row.createCell(5).setCellValue(rscv.getTotalIn());
							row.createCell(6).setCellValue(rscv.getTotalFinish());
							row.createCell(7).setCellValue(rscv.getAvgFinish().doubleValue());
							//taskDetail(taskContext.getTaskId(), sheetName+"导出数据第+" + (i+1) + "+行成功");
						}
					}
				}
			}
			String fileName = sheetName + df2.format(submitTime) + ".xlsx";
			String requestPath = FileConstant.SAVE_PATH;
			//String requestPath= "E\\";
			File fileDir = new File(requestPath);
			if (!fileDir.exists()) {
				fileDir.mkdir();
			}
			String path = requestPath+File.separator + fileName;
			FileOutputStream os = null;
			File file = null;
			try {
				os = new FileOutputStream(path);
				wb.write(os);
				// 阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file = new File(path);
				success(taskContext, "成功", upLoadUrl);
			} catch (Exception e1) {
				e1.printStackTrace();
				fail(taskContext, "写入过程中发生错误");
				logger.error("上传过程中发生错误，请查看日志");
			} finally {
				if (os != null) {
					os.close();
				}
				if (wb != null) {
					wb.close();
				}
				if (file != null) {
					file.delete();
				}
			}

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, sheetName+"导出过程中发生错误，请查看日志"+e.getMessage());
			logger.error(sheetName+"导出过程中发生错误，请查看日志");
		}
		return upLoadUrl;
	 }
}
