package com.yizhi.statistics.application.course.export;

import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.toolkit.StringUtils;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.course.vo.CourseAccountGroupParam;
import com.yizhi.statistics.application.course.vo.ReportStudyCourseAccountDetailsVo;
import com.yizhi.statistics.application.mapper.StatisticsAccountMapper;
import com.yizhi.statistics.application.service.IStatisticsAccountRoleService;
import com.yizhi.statistics.application.service.StatisticsCourseRecordeService;
import com.yizhi.statistics.application.util.ExcelUtil;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;



/**
 * 按照用户统计---（后边的按钮）---导出excel
 * @author wangfeida
 *
 */

@Component
public class CourseAccountGroupDetailsExportExcel extends AbstractDefaultTask<String, CourseAccountGroupParam> {

	private static final Logger logger = LoggerFactory.getLogger(CourseAccountGroupDetailsExportExcel.class);


	@Autowired
	private StatisticsCourseRecordeService statisticsCourseRecordeService;

	@Autowired
	private IdGenerator idGenerator;

	@Autowired
	private StatisticsAccountMapper statisticsAccountMapper;

	@Autowired
	ExcelUtil excelUtil;

	@Autowired
	private IStatisticsAccountRoleService statisticsAccountRoleService;

	@Override
	protected String execute(CourseAccountGroupParam courseAccountGroupParam) {
		// TODO Auto-generated method stub
		String upLoadUrl = null;
		Long companyId =courseAccountGroupParam.getCompanyId();

		Long siteId =courseAccountGroupParam.getSiteId();
		String startDate =courseAccountGroupParam.getStartDate();
		String endDate = courseAccountGroupParam.getEndDate();
		String orgKwd = courseAccountGroupParam.getOrgKwd();
		String accountKwd = courseAccountGroupParam.getAccountKwd();
		Long taskId=courseAccountGroupParam.getTaskId();
		Date submitTime=courseAccountGroupParam.getSubmitTime();
		String serialNo=courseAccountGroupParam.getSerialNo();
		String taskName=courseAccountGroupParam.getTaskName();
		Boolean isAdmin=courseAccountGroupParam.getIsAdmin();
		Long adminId=courseAccountGroupParam.getAdminId();
		Boolean isAllData=courseAccountGroupParam.getIsAllData();

		//验证下管辖区是否是全平台的
		if (!isAdmin) {
			isAdmin = statisticsAccountRoleService.getIsAdmin(0, siteId, adminId);
		}

		Long t1=System.currentTimeMillis()/1000;

		TaskContext taskContext = new TaskContext(taskId,serialNo, taskName, adminId, submitTime, siteId, companyId);// 任务id 任务编号 任务名字      操作人id   操作时间
		working(taskContext);


		List<Long> ids = statisticsAccountMapper.getCourseIdByCompanyIdAndSiteId(companyId, siteId);
		if(CollectionUtils.isEmpty(ids)){
			success(taskContext, "成功", "搜索不到课程信息");
		}

		List<ReportStudyCourseAccountDetailsVo> list = new ArrayList<ReportStudyCourseAccountDetailsVo>();

		List<ReportStudyCourseAccountDetailsVo>  courseAccountDetailsVos = null;
		int iy = 0;
		List<Long> courseIds = new ArrayList<Long>();
		Long count = 0L;
		Long totalCount = 0L;
		int sheetNum = 1;

		SXSSFWorkbook  wb = new SXSSFWorkbook();

		for(int index = 0; index < ids.size(); index++){
			if(iy < 50 && index < ids.size() - 1){
				iy++;
				courseIds.add(ids.get(index));
				continue;
			} else {
				courseIds.add(ids.get(index));
			}

			Long tt= System.currentTimeMillis()/1000;
			courseAccountDetailsVos = statisticsCourseRecordeService.listReportStudyCourseAccountDetailsVoExport(companyId, siteId, startDate, endDate,
					courseIds, adminId, isAdmin, isAllData, orgKwd, accountKwd);
			System.out.println("第" + (index) + "门课程查询时间："+(System.currentTimeMillis()/1000 -tt) +" 查询数据：" + courseAccountDetailsVos.size());
			iy = 0;
			count += courseAccountDetailsVos.size();
			if(count >= 200000){
				list.addAll(courseAccountDetailsVos);
				createSheet(wb, list, sheetNum++, startDate, endDate, taskContext);
				count = 0L;
				list.clear();
			}
			else {
				list.addAll(courseAccountDetailsVos);
			}

			if(CollectionUtils.isNotEmpty(courseAccountDetailsVos)) {
				totalCount += courseAccountDetailsVos.size();
			}
			courseIds.clear();
		}

		if(count > 0){
			createSheet(wb, list, sheetNum++, startDate, endDate, taskContext);
			count = 0L;
			list.clear();
		} else {
			//创建空的，不然文件打开错误
			if(list == null || list.size() == 0){
				String sheetName="课程学习明细"+sheetNum;
	            String headerName="课程学习明细"+sheetNum;
	            createExcel(wb,sheetName, headerName, startDate, endDate);
			}
		}
		count = 0L;

		String fileName = "学员维度学习明细(详情)数据_" +idGenerator.generate() + ".xlsx";
		String requestPath = FileConstant.SAVE_PATH;
		//String requestPath = "C:\\home\\file";
		String requestPathSuffix=requestPath+File.separator+idGenerator.generate();
		File fileDir = new File(requestPathSuffix);
		if (!fileDir.exists()) {
			fileDir.mkdir();
		}
		String path = requestPathSuffix +File.separator+ fileName;
		FileOutputStream os = null;
		File file = null;
		File fileDic=null;
		try {
			os = new FileOutputStream(path);
			wb.write(os);
			// 阿里云返回url
			upLoadUrl = OssUpload.upload(path, fileName);
			file = new File(path);
			fileDic=new File(requestPathSuffix);
			success(taskContext, "成功", upLoadUrl);
		} catch (Exception e1) {
			e1.printStackTrace();
			fail(taskContext, "写入过程中发生错误");
			logger.error("上传过程中发生错误，请查看日志");
		} finally {
			if (os != null) {
				try {
					os.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			if (wb != null) {
				try {
					wb.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			if (file != null) {
				file.delete();
			}
			if(fileDic!=null) {
				fileDic.delete();
			}
		}

		Long t2=System.currentTimeMillis()/1000;
		System.out.println("总数据量："+totalCount+"。总时间："+(t2-t1));


		return upLoadUrl;
	}


	private void createSheet(SXSSFWorkbook  wb, List<ReportStudyCourseAccountDetailsVo> listLimit, int sheetNum,
			String startDate, String endDate, TaskContext taskContext) {
		// TODO Auto-generated method stub
		if(CollectionUtils.isNotEmpty(listLimit)) {
			for (ReportStudyCourseAccountDetailsVo reportStudyCourseAccountDetailsVo : listLimit) {
				if(reportStudyCourseAccountDetailsVo.getFilstFinishTime()!=null) {
					reportStudyCourseAccountDetailsVo.setStudystate("已结束");
				}
				else if(reportStudyCourseAccountDetailsVo.getFilstFinishTime()==null&&reportStudyCourseAccountDetailsVo.getStartTime()!=null) {
					reportStudyCourseAccountDetailsVo.setStudystate("进行中");
				}
				else {
					reportStudyCourseAccountDetailsVo.setStudystate("未开始");
				}
				String duration=reportStudyCourseAccountDetailsVo.getTotalDuration();
				if(duration==null) {
					duration="0";
				}
				int totalDurationInt=Integer.valueOf(duration);
				//格式化总学习时长
				Integer hour=totalDurationInt/3600;
				Integer minute=(totalDurationInt%3600)/60;
				Integer seconds=totalDurationInt%60;
				String time=hour+":"+minute+":"+seconds;
				reportStudyCourseAccountDetailsVo.setTotalDuration(time);
			}
		}

		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		try {

			if(CollectionUtils.isNotEmpty(listLimit)) {

				String sheetName="课程学习明细"+sheetNum;
	            String headerName="课程学习明细"+sheetNum;
	            SXSSFSheet sheet=createExcel(wb,sheetName, headerName, startDate, endDate);


				//将整个数组分成sheet
				SXSSFRow row=null;
				ReportStudyCourseAccountDetailsVo rscv =null;
				for (int j = 0; j < listLimit.size(); j++) {
					//从第三行开始插入
					row = sheet.createRow(j + 3);
					rscv=listLimit.get(j);
					if(rscv!=null) {
						writeDate(row, rscv, df);
					}
				   }

			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, "按个人统计---下载学习明细数据导出过程中发生错误，请查看日志");
			logger.error("按个人统计---下载学习明细数据导出过程中发生错误，请查看日志");
		}
	}




	@SuppressWarnings("unused")
	private SXSSFSheet createExcel(SXSSFWorkbook  wb,String sheetName,String headerName,String startDate,String endDate) {
		String[] str= {"用户名","姓名","工号","课程编码","课程名称","总学习时长(HH:mm:ss)","学习开始时间","最后一次学习时间",
				"首次完成时间","学习次数","学习状态","课程得分","用户当前状态","所在部门","所在组织架构"};
		SXSSFSheet sheet=excelUtil.createExcel(str, wb, sheetName, headerName, startDate, endDate);
		return sheet;
	}


	@SuppressWarnings("unused")
	private void writeDate(SXSSFRow row,ReportStudyCourseAccountDetailsVo rscv,SimpleDateFormat df) {
		row.createCell(0).setCellValue(rscv.getAccountName());
		row.createCell(1).setCellValue(rscv.getAccountFullName());
		row.createCell(2).setCellValue(rscv.getWorkNum());
		row.createCell(3).setCellValue(rscv.getCourseCode());
		row.createCell(4).setCellValue(rscv.getCourseName());
		row.createCell(5).setCellValue(rscv.getTotalDuration());
		if(rscv.getStartTime()!=null) {
			row.createCell(6).setCellValue(df.format(rscv.getStartTime()));
		}

		if(rscv.getLastTime()!=null) {
			row.createCell(7).setCellValue(df.format(rscv.getLastTime()));
		}
		else {
			row.createCell(7).setCellValue("");
		}
		if(rscv.getFilstFinishTime()!=null) {
			row.createCell(8).setCellValue(df.format(rscv.getFilstFinishTime()));
		}
		else {
			row.createCell(8).setCellValue("");
		}
		row.createCell(9).setCellValue(rscv.getStudyNum());
		row.createCell(10).setCellValue(rscv.getStudystate());
		if(rscv.getLearnScore()!=null) {
			row.createCell(11).setCellValue(rscv.getLearnScore()+"");
		}
		else {
			row.createCell(11).setCellValue("");
		}
        row.createCell(12).setCellValue(rscv.getAccountState());
		row.createCell(13).setCellValue(rscv.getOrgName());
		if (StringUtils.isNotEmpty(rscv.getOrgNames())){
			row.createCell(14).setCellValue(rscv.getOrgNames());
		}else {
			row.createCell(14).setCellValue("--");
		}

	}

}
