package com.yizhi.statistics.application.course.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.course.vo.CourseAccountGroupParam;
import com.yizhi.statistics.application.course.vo.ReportStudyCourseAccountVo;
import com.yizhi.statistics.application.service.StatisticsCourseRecordeService;
import com.yizhi.statistics.application.util.ExcelUtil;
import com.yizhi.statistics.application.util.WorkUtil;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 按照用户统计导出Excel
 * @author wangfeida
 *
 */


@Component
public class CourseAccountGroupExportExcel extends AbstractDefaultTask<String, CourseAccountGroupParam> {

	private static final Logger logger = LoggerFactory.getLogger(CourseAccountGroupExportExcel.class);


    @Autowired
    private IdGenerator idGenerator;

	@Autowired
	private StatisticsCourseRecordeService statisticsCourseRecordeService;

	@Autowired
	private ExcelUtil excelUtil;

	@Autowired
	private WorkUtil workUtil;

	@Override
	protected String execute(CourseAccountGroupParam courseAccountGroupParam) {
		// TODO Auto-generated method stub
		String upLoadUrl = null;
		Long accountId =courseAccountGroupParam.getAdminId();
		Long companyId =courseAccountGroupParam.getCompanyId();

		Long siteId =courseAccountGroupParam.getSiteId();
		String startDate =courseAccountGroupParam.getStartDate();
		String endDate = courseAccountGroupParam.getEndDate();
		String orgKwd = courseAccountGroupParam.getOrgKwd();
		String accountKwd = courseAccountGroupParam.getAccountKwd();
		Long taskId=courseAccountGroupParam.getTaskId();
		Date submitTime=courseAccountGroupParam.getSubmitTime();
		String serialNo=courseAccountGroupParam.getSerialNo();
		String taskName=courseAccountGroupParam.getTaskName();
		Boolean isAdmin=courseAccountGroupParam.getIsAdmin();
		Long adminId=courseAccountGroupParam.getAdminId();
		Integer pageNo = 1;
		Integer pageSize = 100;

		TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);
		working(taskContext);


		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

		//算出来第一页
		Page<ReportStudyCourseAccountVo> page =statisticsCourseRecordeService.courseAccountGroup(orgKwd, accountKwd, startDate, endDate, companyId, siteId, isAdmin,adminId, pageNo, pageSize);
		List<ReportStudyCourseAccountVo> list =null;
		if(page!=null) {
			list=new ArrayList<ReportStudyCourseAccountVo>();
		    list.addAll(page.getRecords());
		    Integer pageNum=page.getTotal()/pageSize+1;
		    for (int i = 2; i <= pageNum; i++) {
		    	page =statisticsCourseRecordeService.courseAccountGroup(orgKwd, accountKwd, startDate, endDate, companyId, siteId, isAdmin,adminId, i, pageSize);
		    	if(page!=null) {
		    		list.addAll(page.getRecords());
		    	}
			}
		}

		try {
			SXSSFWorkbook wb = new SXSSFWorkbook();
			String sheetName="学员维度学习统计";
			String headerName="学员维度学习统计";
			//String[] str= {"用户名","姓名","工号","可学课程数","参加课程数","完成课程数","未完成课程数","总学习时长(HH:mm:ss)","最后学习时间","用户当前状态","所在部门","所在组织架构"};
			String[] str= {"用户名","姓名","工号","可学课程数","参加课程数","完成课程数","总学习时长(HH:mm:ss)","最后学习时间","用户当前状态","所在部门","所在组织架构"};
			SXSSFSheet sheet=excelUtil.createExcel(str, wb, sheetName, headerName, startDate, endDate);

			SXSSFRow row=null;
			if(!CollectionUtils.isEmpty(list)) {
			ReportStudyCourseAccountVo rscav =null;
			for (int i = 0; i < list.size(); i++) {
				row = sheet.createRow(i + 3);
				rscav = list.get(i);
				if(rscav!=null) {
					row.createCell(0).setCellValue(rscav.getAccountName());
					row.createCell(1).setCellValue(rscav.getAccountFullName());
					row.createCell(2).setCellValue(rscav.getWorkNum());
					row.createCell(3).setCellValue(rscav.getTotalAskCourse());
					row.createCell(4).setCellValue(rscav.getTotalInCourse());
					row.createCell(5).setCellValue(rscav.getTotalFinishCourse());
					//row.createCell(6).setCellValue(rscav.getTotalNoFinishCourse());
					if(rscav.getTotalDuration() == null){
						row.createCell(6).setCellValue("0.0");
					}else {
						row.createCell(6).setCellValue(rscav.getTotalDuration().toString());
					}
					if(rscav.getLastTime()!=null) {
						row.createCell( 7).setCellValue(df.format(rscav.getLastTime()));
					}
					else {
						row.createCell( 7).setCellValue("");
					}
					row.createCell(8).setCellValue(rscav.getAccountState()==1?"启用":"禁用");
					row.createCell(9).setCellValue(rscav.getOrgName());
					if (StringUtils.isEmpty(rscav.getOrgNames())){
						row.createCell(10).setCellValue("--");
					}else {
						row.createCell(10).setCellValue(rscav.getOrgNames());
					}
				}
			}
			}
			String fileName = "用户维度学习统计数据"+idGenerator.generate() + ".xlsx";
			//String requestPath="F:\\home\\file";
			String requestPath = FileConstant.SAVE_PATH;
			String requestPathSuffix=requestPath+File.separator+idGenerator.generate();
			File fileDir = new File(requestPathSuffix);
			if (!fileDir.exists()) {
				fileDir.mkdir();
			}
			String path = requestPath +File.separator+ fileName;
			FileOutputStream os = null;
			File file = null;
			 File fileDic=null;
			try {
				os = new FileOutputStream(path);
				wb.write(os);
				// 阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file = new File(path);
				fileDic=new File(requestPathSuffix);
				success(taskContext, "成功", upLoadUrl);
			} catch (Exception e1) {
				e1.printStackTrace();
				fail(taskContext, "写入过程中发生错误");
				logger.error("上传过程中发生错误，请查看日志");
			} finally {
				if (os != null) {
					os.close();
				}
				if (wb != null) {
					wb.close();
				}
				if (file != null) {
					file.delete();
				}
				if(fileDic!=null) {
	                fileDic.delete();
	            }
			}

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, "按用户统计-下载学习统计数据导出过程中发生错误，请查看日志");
			logger.error("按用户统计-下载学习统计数据导出过程中发生错误，请查看日志");
		}
		return upLoadUrl;
	}
}
