package com.yizhi.statistics.application.course.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.course.vo.CourseAccountGroupViewParam;
import com.yizhi.statistics.application.course.vo.ReportStudyCourseAccountViewVo;
import com.yizhi.statistics.application.entity.StatisticsAccount;
import com.yizhi.statistics.application.service.StatisticsAccountService;
import com.yizhi.statistics.application.service.StatisticsCourseRecordeService;
import com.yizhi.statistics.application.util.ExcelUtil;
import com.yizhi.statistics.application.util.WorkUtil;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 按照用户统计---详情---导出Excel
 * @author wangfeida
 *
 */


@Component
public class CourseAccountGroupViewExportExcel extends AbstractDefaultTask<String,CourseAccountGroupViewParam> {

	private static final Logger logger = LoggerFactory.getLogger(CourseAccountGroupViewExportExcel.class);

	@Autowired
	private StatisticsCourseRecordeService statisticsCourseRecordeService;

	@Autowired
	private StatisticsAccountService statisticsAccountService;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ExcelUtil excelUtil;

    @Autowired
    private WorkUtil workUtil;

	@Override
	protected String execute(CourseAccountGroupViewParam courseAccountGroupViewParam) {
		// TODO Auto-generated method stub
		String upLoadUrl = null;
		Long accountId =courseAccountGroupViewParam.getAccountId();
		Long companyId =courseAccountGroupViewParam.getCompanyId();
		Long siteId = courseAccountGroupViewParam.getSiteId();
		String startDate = courseAccountGroupViewParam.getStartDate();
		String endDate = courseAccountGroupViewParam.getEndDate();
		String kwd = courseAccountGroupViewParam.getKwd();
		Long taskId=courseAccountGroupViewParam.getTaskId();
		Date submitTime=courseAccountGroupViewParam.getSubmitTime();
		String serialNo=courseAccountGroupViewParam.getSerialNo();
		String taskName=courseAccountGroupViewParam.getTaskName();
		Long adminId=courseAccountGroupViewParam.getAdminId();

		Integer pageNo=1;
		Integer pageSize=100;

        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, adminId, submitTime, siteId, companyId);
		working(taskContext);

		Page<ReportStudyCourseAccountViewVo> page=statisticsCourseRecordeService.courseAccountGroupView(companyId,accountId, siteId, kwd, startDate, endDate, pageNo, pageSize);
		List<ReportStudyCourseAccountViewVo> list=null;
		if(page!=null) {
			list=new ArrayList<ReportStudyCourseAccountViewVo>();
			if (courseAccountGroupViewParam.isAllData()) {
				list.addAll(page.getRecords());
			}
			// 如果是已学习数据
			else {
				for (ReportStudyCourseAccountViewVo vo : page.getRecords()) {
					if (vo.getTotalCount() != 0) {
						list.add(vo);
					}
				}
			}
		    Integer pageNum=page.getTotal()/pageSize+1;
		    for (int i = 2; i <= pageNum; i++) {
		    	page =statisticsCourseRecordeService.courseAccountGroupView(companyId,accountId, siteId, kwd, startDate, endDate, i, pageSize);
		    	if(page!=null) {
		    		// 修改之前
//		    		list.addAll(page.getRecords());
					// 修改之后
					// 如果是所有数据
					if (courseAccountGroupViewParam.isAllData()) {
						list.addAll(page.getRecords());
					}
					// 如果是已学习数据
					else {
						for (ReportStudyCourseAccountViewVo vo : page.getRecords()) {
							if (vo.getTotalCount() != 0) {
								list.add(vo);
							}
						}
					}
		    	}
			}
		}
		SimpleDateFormat df2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		StatisticsAccount sa=statisticsAccountService.getStatisticsAccountByAccountId(accountId);
		String accountName=" ";
		String accountFullName=" ";
		String orgName="";
		String orgNames = "";
		Integer accountState=0;
		if(sa!=null) {
			accountName=sa.getAccountName();
			accountFullName=sa.getFullName();
			orgName=sa.getOrgName();
			orgNames = sa.getOrgNames();
			accountState=sa.getAccountState();
		}

		try {
			SXSSFWorkbook wb = new SXSSFWorkbook();
			String sheetName=workUtil.chechPattonPattern(accountName)+"学习明细";
			String headerName=accountName+"学习明细";
			SXSSFSheet sheet = createExcel(wb,sheetName, headerName, startDate, endDate);
			ReportStudyCourseAccountViewVo rscv=null;
			if(!CollectionUtils.isEmpty(list)) {
			SXSSFRow row=null;
			for (int i = 0; i < list.size(); i++) {
				row = sheet.createRow(i + 3);
				rscv = list.get(i);
				if(rscv!=null) {
					writeDate(row, rscv, df2, accountName, accountFullName,orgName,accountState,orgNames);
				}
			}
			}
//			else {
//				createExcel(wb,sheetName, headerName, startDate, endDate);
//			}

			String fileName = workUtil.chechExcelPattern(accountName)+"学习明细统计_" + idGenerator.generate() + ".xlsx";
			String requestPath = FileConstant.SAVE_PATH;
			//String requestPath = "E:\\home";
			String requestPathSuffix=requestPath+File.separator+idGenerator.generate();
			File fileDir = new File(requestPathSuffix);
			if (!fileDir.exists()) {
				fileDir.mkdir();
			}
			String path = requestPath +File.separator+ fileName;
			FileOutputStream os = null;
			File file = null;
			File fileDic=null;
			try {
				os = new FileOutputStream(path);
				wb.write(os);
				// 阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file = new File(path);
				fileDic=new File(requestPathSuffix);
				success(taskContext, "成功", upLoadUrl);
			} catch (Exception e1) {
				e1.printStackTrace();
				fail(taskContext, "写入过程中发生错误");
				logger.error("上传过程中发生错误，请查看日志");
			} finally {
				if (os != null) {
					os.close();
				}
				if (wb != null) {
					wb.close();
				}
				if (file != null) {
					file.delete();
				}
				if(fileDic!=null) {
					fileDic.delete();
				}
			}

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, "按用户统计---详情---下载学习明细导出过程中发生错误，请查看日志");
			logger.error("按用户统计---详情---下载学习明细导出过程中发生错误，请查看日志");
		}
		return upLoadUrl;
	}


	@SuppressWarnings("unused")
	private SXSSFSheet createExcel(SXSSFWorkbook  wb,String sheetName,String headerName,String startDate,String endDate) {
		String[] str= {"用户名","姓名","课程名称","课程编码","学习状态","学习次数","开始学习时间","课程得分","学习时长(HH:mm:ss)","可见状态","用户状态","所在部门","所在组织架构"};
		SXSSFSheet sheet=excelUtil.createExcel(str, wb, sheetName, headerName, startDate, endDate);
		return sheet;
	}


	@SuppressWarnings("unused")
	private void writeDate(SXSSFRow row,ReportStudyCourseAccountViewVo rscv,SimpleDateFormat df,String accountName,String accountFullName,String orgName,Integer accountState,String orgNames) {
		row.createCell(0).setCellValue(accountName);
		row.createCell(1).setCellValue(accountFullName);
		row.createCell(2).setCellValue(rscv.getCourseName());
		row.createCell(3).setCellValue(rscv.getCourseNo());
		String state=null;
		if(rscv.getState()==0) {
			state="未开始";
		}
		if(rscv.getState()==1) {
			state="进行中";
		}
		if(rscv.getState()==2) {
			state="已完成";
		}

		row.createCell(4).setCellValue(state);
		row.createCell(5).setCellValue(rscv.getTotalCount());
		if(rscv.getStartTime()!=null) {
			row.createCell(6).setCellValue(df.format(rscv.getStartTime()));
		}
		else {
			row.createCell(6).setCellValue("");
		}
		if(rscv.getLearnScore()!=null) {
			row.createCell(7).setCellValue(rscv.getLearnScore());
		}
		else {
			row.createCell(7).setCellValue("");
		}

		row.createCell(8).setCellValue(rscv.getTotalDuration());
		row.createCell(9).setCellValue(rscv.getCanState()==0?"可见":"不可见");
		row.createCell(10).setCellValue(accountState==0?"禁用":"启用");
		row.createCell(11).setCellValue(orgName);
		if (StringUtils.isEmpty(orgNames)){
			row.createCell(12).setCellValue("--");
		}else {
			row.createCell(12).setCellValue(orgNames);
		}
	}
}
