package com.yizhi.statistics.application.course.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.course.vo.CourseGroupParam;
import com.yizhi.statistics.application.course.vo.ReportStudyCourseVo;
import com.yizhi.statistics.application.service.StatisticsCourseRecordeService;
import com.yizhi.statistics.application.util.ExcelUtil;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 按照课程统计导出Excel
 *
 * @author wangfeida
 */


@Component
public class CourseGroupExportExcel extends AbstractDefaultTask<String,CourseGroupParam> {

    private static final Logger logger = LoggerFactory.getLogger(CourseGroupExportExcel.class);

 
    
    @Autowired
    StatisticsCourseRecordeService statisticsCourseRecordeService;
    
    @Autowired
    ExcelUtil excelUtil;

    @Override
    protected String execute(CourseGroupParam courseGroupExportExcelVo) {
        // TODO Auto-generated method stub
    	SimpleDateFormat df2 = new SimpleDateFormat("yyyyMMddHHmmss");        
        String upLoadUrl = null;
        Long adminId = courseGroupExportExcelVo.getAdminId();
        Long companyId = courseGroupExportExcelVo.getCompanyId();
        Long siteId = courseGroupExportExcelVo.getSiteId();
        String startDate = courseGroupExportExcelVo.getStartDate();
        String endDate = courseGroupExportExcelVo.getEndDate();
        String kwd = courseGroupExportExcelVo.getKwd();
        Long taskId = courseGroupExportExcelVo.getTaskId();
        Date submitTime = courseGroupExportExcelVo.getSubmitTime();
        String serialNo = courseGroupExportExcelVo.getSerialNo();
        String taskName = courseGroupExportExcelVo.getTaskName();
        Boolean isAdmin=courseGroupExportExcelVo.getIsAdmin();
        
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, adminId, submitTime, siteId, companyId);
        working(taskContext);
        
        Integer pageNo = 2;
        Integer pageSize = 100;
     
        //全量数据
        List<ReportStudyCourseVo> list = new ArrayList<ReportStudyCourseVo>();
        Page<ReportStudyCourseVo> tmpPage =statisticsCourseRecordeService.courseGroup(kwd, startDate, endDate, companyId, siteId, isAdmin, adminId, pageNo-1, pageSize);
        if (tmpPage != null) {
            list.addAll(tmpPage.getRecords());
            int num = tmpPage.getTotal() / pageSize + 1;
            for (int i = 2; i <= num; i++) {
                tmpPage = statisticsCourseRecordeService.courseGroup(kwd, startDate, endDate, companyId, siteId, isAdmin, adminId, i, pageSize);
                if (tmpPage == null || tmpPage.getRecords().size() == 0) {
                    break;
                }
                list.addAll(tmpPage.getRecords());
            }
        }
        
        try {
            SXSSFWorkbook  wb = new SXSSFWorkbook ();
            String[] str= {"课程编码","课程名称","是否关联证书","获证张数","可学习人数","已学习人数","学习次数","课程时长","总学习时长(HH:mm:ss)","平均学习时长(s)","评分人数","课程评分","评论数","收藏数","课程当前状态"};
            String sheetName="课程维度学习统计";
            String headerName="课程维度学习统计";
            SXSSFSheet sheet=excelUtil.createExcel(str, wb, sheetName, headerName, startDate, endDate);
            if (CollectionUtils.isNotEmpty(list)) {
            	SXSSFRow row=null;
                for (int i = 0; i < list.size(); i++) {
                    row = sheet.createRow(i + 3);
                    ReportStudyCourseVo rscv = list.get(i);
                    // 第四步，创建单元格，并设置值
                    if(rscv!=null) {
                    	 row.createCell(0).setCellValue(rscv.getCourseNo());
                         row.createCell(1).setCellValue(rscv.getCourseName());
                        Boolean grantCertificate = rscv.getGrantCertificate();
                        if(grantCertificate == null) grantCertificate = Boolean.FALSE;
                        row.createCell(2).setCellValue(grantCertificate.booleanValue()?"是":"否");
                        Integer certificateCount = rscv.getCertificateCount();
                        row.createCell(3).setCellValue(certificateCount==null?"":certificateCount.toString());

                         row.createCell(4).setCellValue(rscv.getTotalAskAccount()!=null?rscv.getTotalAskAccount():0);
                         row.createCell(5).setCellValue(rscv.getTotalInAccount()!=null?rscv.getTotalInAccount():0);
                         row.createCell(6).setCellValue(rscv.getTotalCount()!=null?rscv.getTotalCount():0);
                         row.createCell(7).setCellValue(rscv.getTotalCourseTime());
                         row.createCell(8).setCellValue(rscv.getTotalDuration());
                         row.createCell(9).setCellValue(rscv.getAvgDuration());
                         row.createCell(10).setCellValue(rscv.getCommentAccountCount());
                         if (rscv.getCourseScore() != null) {
                             row.createCell(11).setCellValue(rscv.getCourseScore() + "");
                         } else {
                             row.createCell(11).setCellValue("");
                         }
                    
                         row.createCell(12).setCellValue(rscv.getNumComment()!=null?rscv.getNumComment():0);
                         row.createCell(13).setCellValue(rscv.getCollectNum()!=null?rscv.getCollectNum():0);
                         row.createCell(14).setCellValue(rscv.getState() == 1 ? "已上架" : "未上架");
                    }
                }
            }

            logger.info("第七步：给Excel命名");
            String fileName = "课程维度统计" + df2.format(submitTime) + ".xlsx";
            //String requestPath = "F:\\home\\file";
            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }
            String path = requestPath +File.separator+ fileName;
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                // 阿里云返回url
                upLoadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功", upLoadUrl);
                logger.info("第八步：成功上传到阿里云");
            } catch (Exception e1) {
                e1.printStackTrace();
                fail(taskContext, "写入过程中发生错误");
                logger.error("上传过程中发生错误，请查看日志");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }

        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            fail(taskContext, "按课程统计---下载结果数据导出过程中发生错误，请查看日志");
            logger.error("按课程统计---下载结果数据导出过程中发生错误，请查看日志{}", e);
        }
        logger.info("第九步：返回阿里云地址:" + upLoadUrl);
        return upLoadUrl;
    }
}
