package com.yizhi.statistics.application.course.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.point.application.feign.PointUserFeignClients;
import com.yizhi.statistics.application.course.vo.CourseGroupViewParam;
import com.yizhi.statistics.application.course.vo.ReportStudyCourseViewVo;
import com.yizhi.statistics.application.entity.StatisticsCourse;
import com.yizhi.statistics.application.service.StatisticsCourseRecordeService;
import com.yizhi.statistics.application.service.StatisticsCourseService;
import com.yizhi.statistics.application.util.ExcelUtil;
import com.yizhi.statistics.application.util.WorkUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 按照课程统计-----详情---导出Excel
 *
 * @author wangfeida
 */
@Component
public class CourseGroupViewExportExcel extends AbstractDefaultTask<String, CourseGroupViewParam> {


    private static final Logger logger = LoggerFactory.getLogger(CourseGroupViewExportExcel.class);

    @Autowired
    PointUserFeignClients pointUserFeignClients;

    @Autowired
    private StatisticsCourseRecordeService statisticsCourseRecordeService;

    @Autowired
    private StatisticsCourseService statisticsCourseService;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ExcelUtil excelUtil;

    @Autowired
    private WorkUtil workUtil;


    @Override
    protected String execute(CourseGroupViewParam courseGroupViewParam) {

        SimpleDateFormat simFormate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String upLoadUrl = null;
        String accountKwd = courseGroupViewParam.getAccountKwd();
        String orgKwd = courseGroupViewParam.getOrgKwd();
        Long courseId = courseGroupViewParam.getCourseId();
        // Long accountId=courseGroupViewParam.getAccountId();
        Boolean isAdmin = courseGroupViewParam.getIsAdmin();
        String startDate = courseGroupViewParam.getStartDate();
        String endDate = courseGroupViewParam.getEndDate();
        Long companyId = courseGroupViewParam.getCompanyId();
        Long siteId = courseGroupViewParam.getSiteId();

        Long taskId = courseGroupViewParam.getTaskId();
        String serialNo = courseGroupViewParam.getSerialNo();
        String taskName = courseGroupViewParam.getTaskName();
        Long adminId = courseGroupViewParam.getAdminId();
        Date submitTime = courseGroupViewParam.getSubmitTime();

        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, adminId, submitTime, siteId, companyId);
        working(taskContext);

        StatisticsCourse sc = statisticsCourseService.getStatisticsCourse(courseId);


        Integer pageNo = 2;
        Integer pageSize = 300000;
        List<ReportStudyCourseViewVo> list = new ArrayList<ReportStudyCourseViewVo>();
        Page<ReportStudyCourseViewVo> tmpPage = statisticsCourseRecordeService.courseGroupViewExport(orgKwd, accountKwd, startDate, endDate,
        		courseId, pageNo - 1, pageSize, companyId, siteId, isAdmin, courseGroupViewParam.isAllData(), adminId);
        if (tmpPage != null) {
            list.addAll(tmpPage.getRecords());
            /*if (courseGroupViewParam.isAllData()) {
                list.addAll(tmpPage.getRecords());
            }
            // 已学习数据
            else {
                if (!CollectionUtils.isEmpty(tmpPage.getRecords())) {
                    for (ReportStudyCourseViewVo vo : tmpPage.getRecords()) {
                        if (vo.getLearnNum() > 0) {
                            list.add(vo);
                        }
                    }
                }
            }*/

            int num = tmpPage.getTotal() / pageSize + 1;
            for (int i = 2; i <= num; i++) {
                tmpPage = statisticsCourseRecordeService.courseGroupViewExport(orgKwd, accountKwd, startDate, endDate,
                		courseId, i, pageSize, companyId, siteId, isAdmin, courseGroupViewParam.isAllData(), adminId);
                if (tmpPage == null || tmpPage.getRecords().size() == 0) {
                    break;
                }
                list.addAll(tmpPage.getRecords());
                // 修改后
                // 如果是所有数据
                /*if (courseGroupViewParam.isAllData()) {
                    list.addAll(tmpPage.getRecords());
                }
                // 已学习数据
                else {
                    if (!CollectionUtils.isEmpty(tmpPage.getRecords())) {
                        for (ReportStudyCourseViewVo vo : tmpPage.getRecords()) {
                            if (vo.getLearnNum() > 0) {
                                list.add(vo);
                            }
                        }
                    }
                }*/
            }
        }


        try {
            SXSSFWorkbook wb = new SXSSFWorkbook();

            String[] str= {"用户名","姓名","工号","学习时长(HH:mm:ss)","学习次数","开始时间","完成时间","学习状态","课程得分","获得积分","用户状态","所在部门","所在组织架构"};
            String cname = "";
            if (sc != null) {
                cname = sc.getCourseName();
            }//sheetName需要判断，headerName不需要判断，因为headerName是第一行的数据
            String sheetName=workUtil.chechPattonPattern(cname) + "学习明细";
            String headerName=cname + "学习明细";
            SXSSFSheet sheet=excelUtil.createExcel(str, wb, sheetName, headerName, startDate, endDate);
            ReportStudyCourseViewVo rscv = null;
            SXSSFRow row3=null;
            if(CollectionUtils.isNotEmpty(list)) {
            	 for (int i = 0; i < list.size(); i++) {
            		 row3 = sheet.createRow(i + 3);
                     rscv = list.get(i);
                     if (rscv != null) {
                         row3.createCell(0).setCellValue(rscv.getAccountName());
                         row3.createCell(1).setCellValue(rscv.getAccountFullName());
                         row3.createCell(2).setCellValue(rscv.getWorkNum());
                         row3.createCell(3).setCellValue(rscv.getTotalDuration());
                         row3.createCell(4).setCellValue(rscv.getLearnNum());
                         if (rscv.getStartTime() != null) {
                             row3.createCell(5).setCellValue(simFormate.format(rscv.getStartTime()));
                         } else {
                             row3.createCell(5).setCellValue("");
                         }
                         if (rscv.getEndTime() != null) {
                             row3.createCell(6).setCellValue(simFormate.format(rscv.getEndTime()));
                         } else {
                             row3.createCell(6).setCellValue("");
                         }
                         if (rscv.getState() == 0) {
                             row3.createCell(7).setCellValue("未开始");
                         }
                         if (rscv.getState() == 1) {
                             row3.createCell(7).setCellValue("进行中");
                         }
                         if (rscv.getState() == 2) {
                             row3.createCell(7).setCellValue("已完成");
                         }
                         if (rscv.getLearnScore() != null) {
                             row3.createCell(8).setCellValue(rscv.getLearnScore());
                         }

                         if (rscv.getLearnPoint() != null) {
                             row3.createCell(9).setCellValue(rscv.getLearnPoint());
                         }
                         row3.createCell(10).setCellValue(rscv.getAccountState() == 1 ? "启用" : "禁用");
                         row3.createCell(11).setCellValue(rscv.getOrgName());
                         if (StringUtils.isNotBlank(rscv.getOrgNames())){
                             row3.createCell(12).setCellValue(rscv.getOrgNames());
                         }else {
                             row3.createCell(12).setCellValue("--");
                         }
                     }
            }
            }
            String fileName = workUtil.chechExcelPattern(cname) + "学习明细_" +idGenerator.generate()+ ".xlsx";
            String requestPath = FileConstant.SAVE_PATH;
            //String requestPath="F:\\home\\file";
            String requestPathSuffix=requestPath+File.separator+idGenerator.generate();
            File fileDir = new File(requestPathSuffix);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }
            String path = requestPathSuffix +File.separator+ fileName;
            FileOutputStream os = null;
            File file = null;
            File fileDic=null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                upLoadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                fileDic=new File(requestPathSuffix);
                success(taskContext, "成功", upLoadUrl);
            } catch (Exception e1) {
                e1.printStackTrace();
                fail(taskContext, "写入过程中发生错误");
                logger.error("上传过程中发生错误，请查看日志");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
                if(fileDic!=null) {
                	fileDic.delete();
                }
            }

        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            fail(taskContext, "按课程统计---详情--- 下载课程学习明细导出过程中发生错误，请查看日志");
            logger.error("按课程统计---详情--- 下载课程学习明细导出过程中发生错误，请查看日志");
        }
        return upLoadUrl;
    }

}

