package com.yizhi.statistics.application.course.export;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.toolkit.StringUtils;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.course.vo.CourseOrgGroupParam;
import com.yizhi.statistics.application.course.vo.ReportStudyCourseOrgVo;
import com.yizhi.statistics.application.service.IStatisticsAccountRoleService;
import com.yizhi.statistics.application.service.StatisticsCourseRecordeService;
import com.yizhi.statistics.application.util.ExcelUtil;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 按照部门统计导出Excel
 * @author wangfeida
 *
 */



@Component
public class CourseOrgGroupExportExcel extends AbstractDefaultTask<String, CourseOrgGroupParam> {

	private static final Logger logger = LoggerFactory.getLogger(CourseOrgGroupExportExcel.class);


	@Autowired
	ExcelUtil excelUtil;

	@Autowired
	private StatisticsCourseRecordeService  statisticsCourseRecordeService;

	@Autowired
	private IdGenerator idGenerator;

	@Autowired
	private IStatisticsAccountRoleService statisticsAccountRoleService;

	@Override
	protected String execute(CourseOrgGroupParam courseOrgGroupParam) {
		// TODO Auto-generated method stub
		String upLoadUrl = null;
		Long adminId = courseOrgGroupParam.getAdminId();
		Long companyId =courseOrgGroupParam.getCompanyId();

		Long siteId = courseOrgGroupParam.getSiteId();
		String startDate =courseOrgGroupParam.getStartDate();
		String endDate =courseOrgGroupParam.getEndDate();
		String kwd =courseOrgGroupParam.getKwd();
		Long taskId=courseOrgGroupParam.getTaskId();
		Date submitTime=courseOrgGroupParam.getSubmitTime();
		String serialNo=courseOrgGroupParam.getSerialNo();
		String taskName=courseOrgGroupParam.getTaskName();
		Boolean isAdmin=courseOrgGroupParam.getIsAdmin();

		//验证下管辖区是否是全平台的
		if (!isAdmin) {
			isAdmin = statisticsAccountRoleService.getIsAdmin(0, siteId, adminId);
		}

        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, adminId, submitTime, siteId, companyId);
		working(taskContext);


		Integer pageNo = 1;
		Integer pageSize = 100;
		List<ReportStudyCourseOrgVo> list=new ArrayList<ReportStudyCourseOrgVo>();
		Page<ReportStudyCourseOrgVo> page = statisticsCourseRecordeService.courseOrgGroup(kwd, startDate, endDate, companyId, siteId, pageNo, pageSize,isAdmin,adminId);
		if(page!=null) {
			list.addAll(page.getRecords());
			int num=page.getTotal()/pageSize+1;
			for (int  i= 2; i <= num; i++) {
				page = statisticsCourseRecordeService.courseOrgGroup(kwd, startDate, endDate, companyId, siteId, i, pageSize,isAdmin,adminId);
				if(page == null || page.getRecords().size() == 0) {
					break;
				}
				list.addAll(page.getRecords());
			}
		}
		try {

			SXSSFWorkbook wb = new SXSSFWorkbook();
			String sheetName="部门学习情况统计";
			String headerName="部门学习情况统计";
			SXSSFSheet sheet=createExcel(wb, sheetName, headerName, startDate, endDate);

			// 第五步，写入实体数据 实际应用中这些数据从数据库得到，
			if(CollectionUtils.isNotEmpty(list)) {
			SXSSFRow row=null;
			ReportStudyCourseOrgVo rscv=null;
			for (int i = 0; i < list.size(); i++) {
				row = sheet.createRow(i + 3);
				rscv = list.get(i);
				// 第四步，创建单元格，并设置值
				if(rscv!=null) {
					row.createCell(0).setCellValue(rscv.getOrgNo());
					row.createCell(1).setCellValue(rscv.getOrgName());
					if (StringUtils.isNotEmpty(rscv.getOrgNames())){
						row.createCell(2).setCellValue(rscv.getOrgNames());
					}else {
						row.createCell(2).setCellValue("--");
					}
					row.createCell(3).setCellValue(rscv.getTotalInCourse());
					row.createCell(4).setCellValue(rscv.getTotalInAccount());
					row.createCell(5).setCellValue(rscv.getTotalCount());
					row.createCell(6).setCellValue(rscv.getTotalDuration());
					if(rscv.getAvgDuration() == null){
						row.createCell(7).setCellValue("0.0");
					}else{
						row.createCell(7).setCellValue(rscv.getAvgDuration());
					}
				}
			}
			}
			String fileName = "部门学习情况统计" + idGenerator.generate() + ".xlsx";
			//String requestPath = "E:\\home";
			String requestPath = FileConstant.SAVE_PATH;
			File fileDir = new File(requestPath);
			if (!fileDir.exists()) {
				fileDir.mkdir();
			}
			String path = requestPath + fileName;
			FileOutputStream os = null;
			File file = null;
			try {
				os = new FileOutputStream(path);
				wb.write(os);
				// 阿里云返回url
				upLoadUrl = OssUpload.upload(path, fileName);
				file = new File(path);
				success(taskContext, "成功", upLoadUrl);
			} catch (Exception e1) {
				e1.printStackTrace();
				fail(taskContext, "写入过程中发生错误");
				logger.error("上传过程中发生错误，请查看日志");
			} finally {
				if (os != null) {
					os.close();
				}
				if (wb != null) {
					wb.close();
				}
				if (file != null) {
					file.delete();
				}
			}

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(taskContext, "部门学习情况统计导出过程中发生错误，请查看日志");
			logger.error("部门学习情况统计导出过程中发生错误，请查看日志");
		}
		return upLoadUrl;
	}



	@SuppressWarnings("unused")
	private SXSSFSheet createExcel(SXSSFWorkbook  wb,String sheetName,String headerName,String startDate,String endDate) {
		String[] str= {"部门编码","部门名称","所在组织架构","参加课程数","参加人数","总学习次数","总学习时长(HH:mm:ss)","平均学习时长(秒)"};
		SXSSFSheet sheet=excelUtil.createExcel(str, wb, sheetName, headerName, startDate, endDate);
		return sheet;
	}
}
