package com.yizhi.statistics.application.export.research;

import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.toolkit.StringUtils;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.statistics.application.mapper.StatisticsResearchLearnMetadataMapper;
import com.yizhi.statistics.application.service.StatisticsResearchLearnMetadataService;
import com.yizhi.statistics.application.service.StatisticsResearchService;
import com.yizhi.statistics.application.util.WorkUtil;
import com.yizhi.statistics.application.vo.research.DownloadParamsVo;
import com.yizhi.statistics.application.vo.research.ResearchGroupViewVo;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Component
public class DownloadResearch extends AbstractDefaultTask<String, DownloadParamsVo> {

    private static final Logger logger = LoggerFactory.getLogger(DownloadResearch.class);

    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");

    @Autowired
    private StatisticsResearchLearnMetadataService learnMetadataService;
    @Autowired
    private StatisticsResearchLearnMetadataMapper learnMetadataMapper;
    @Autowired
    private StatisticsResearchService researchService;

    @Autowired
    private WorkUtil workUtil;

    @Override
    protected String execute(DownloadParamsVo vo) {

        Long researchId = vo.getResearchId();
        String researchName = vo.getResesrchName();
        RequestContext context = vo.getContext();
        String startDate = vo.getStartTime();
        String endDate = vo.getEndTime();
        String orgNameorOrgCode = vo.getOrgNameorOrgCode();
        String accountName = vo.getAccountName();
        Integer joinState = vo.getJoinState();
        String taskName = vo.getTaskName();
        String serialNo = vo.getSerialNo();
        Long taskId = vo.getTaskId();

        Date submitTime = new Date();
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, context.getAccountId(), submitTime, context.getSiteId(), context.getCompanyId());
        working(taskContext);

        int pageNo = 1;
        int pageSize = 20;
        Integer pageTotal = 0;

        if (context.isAdmin()) {
            pageTotal = learnMetadataMapper.researchGroupViewCount2(researchId, orgNameorOrgCode, accountName, joinState, context.getCompanyId(), context.getAccountId(), context.getSiteId(), startDate, endDate);
        } else {
            pageTotal = learnMetadataMapper.researchGroupViewCount1(researchId, orgNameorOrgCode, accountName, joinState, context.getCompanyId(), context.getAccountId(), context.getSiteId(), startDate, endDate);
        }
        int pageCount = 0;
        if (pageTotal != null) {
            pageCount = (int) Math.ceil(Double.valueOf(pageTotal) / pageSize);
        }

        List<ResearchGroupViewVo> list = new ArrayList<>();
        List<ResearchGroupViewVo> listOne;


        Page<ResearchGroupViewVo> page = new Page<>(1, Integer.MAX_VALUE);


        if (context.isAdmin()) {
            listOne = learnMetadataMapper.researchGroupView2(researchId, orgNameorOrgCode, accountName, joinState, context.getCompanyId(), context.getAccountId(), context.getSiteId(), startDate, endDate, page);
        } else {
            listOne = learnMetadataMapper.researchGroupView1(researchId, orgNameorOrgCode, accountName, joinState, context.getCompanyId(), context.getAccountId(), context.getSiteId(), startDate, endDate, page);
        }
        list.addAll(listOne);


        String upLoadUrl = null;
        String requestPath = FileConstant.SAVE_PATH;
//        String requestPath = "E\\";
        File fileDir = new File(requestPath);
        if (!fileDir.exists()) {
            fileDir.mkdir();
        }

        //excel生成：excel-sheet-row-cell
        XSSFWorkbook workbook = new XSSFWorkbook();
        String sheetName = workUtil.chechPattonPattern(researchName + "_" + dateFormat.format(submitTime));
        XSSFSheet sheet = workbook.createSheet(sheetName);
        // 在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short,
        // 创建第1行
        XSSFRow row = sheet.createRow((int) 0);
        //创建单元格格式
        XSSFCellStyle style = workbook.createCellStyle();
        style.setAlignment(HorizontalAlignment.LEFT);//创建居左格式

        //合并单元格
        CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 12); // 起始行, 终止行, 起始列, 终止列
        sheet.addMergedRegion(cra);
        //合并前6列当做第0行的第0列
        XSSFCell cell = row.createCell((short) 0);
        cell.setCellValue("调研参与人员清单");
        cell.setCellStyle(style);

        //创建第2行
        row = sheet.createRow((int) 1);
        CellRangeAddress cra1 = new CellRangeAddress(1, 1, 0, 12); // 起始行, 终止行, 起始列, 终止列
        sheet.addMergedRegion(cra1);
        cell = row.createCell((short) 0);
        cell.setCellValue("统计周期:" + startDate + "～" + endDate);
        cell.setCellStyle(style);

        //创建第三行
        row = sheet.createRow((int) 2);
        row.createCell((short) 0).setCellValue("用户名");
        row.createCell((short) 1).setCellValue("姓名");
        row.createCell((short) 2).setCellValue("是否完成");
        row.createCell((short) 3).setCellValue("提交时间");
        row.createCell((short) 4).setCellValue("用户状态");
        row.createCell((short) 5).setCellValue("所在部门");
        row.createCell((short) 6).setCellValue("所在组织架构");
        Integer state;
        String finish = "";
        String accountState = "";
        ResearchGroupViewVo researchGroupViewVo;
        String[] parentNames;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:ss:mm");
        try {
            //for循环为每一行设值
            int size;
            int s = 0;
            for (int i = 0; i < list.size(); i++) {
                size = 0;
                researchGroupViewVo = list.get(i);
                if (researchGroupViewVo == null) {
                    i--;
                    continue;
                }
                row = sheet.createRow((int) i + 3);
                cell = row.createCell((short) size);
                if (researchGroupViewVo.getAccountName() != null) {
                    cell.setCellValue(researchGroupViewVo.getAccountName());
                }
                size++;
                cell = row.createCell((short) size);
                if (researchGroupViewVo.getAccountFullName() != null) {
                    cell.setCellValue(researchGroupViewVo.getAccountFullName());
                }
                state = researchGroupViewVo.getJoinState();
                if (state == null || state == 0) {
                    finish = "否";
                } else {
                    finish = "是";
                }
                size++;
                row.createCell((short) size).setCellValue(finish);
                size++;
                row.createCell((short) size);
                if (researchGroupViewVo.getFinishTime() != null) {
                    row.getCell((short) size).setCellValue(sdf.format(researchGroupViewVo.getFinishTime()));
                }

                if (researchGroupViewVo.getAccountState() != null && researchGroupViewVo.getAccountState() == 1) {
                    accountState = "启用";
                } else {
                    accountState = "禁用";
                }
                size++;
                row.createCell((short) size).setCellValue(accountState);
                size++;

                if (researchGroupViewVo.getOrgName() != null) {
                    cell = row.createCell((short) size);
                    cell.setCellValue(researchGroupViewVo.getOrgName());
                } else {
                    row.createCell((short) size);
                }
                size++;
                if (StringUtils.isNotEmpty(researchGroupViewVo.getOrgParentNames())) {
                    cell = row.createCell((short) size);
                    cell.setCellValue(researchGroupViewVo.getOrgParentNames());
                } else {
                    row.createCell((short) size).setCellValue("--");
                }

                s++;
//                taskDetail(taskContext.getTaskId(), research.getName() + "导出数据第" + s + "行成功");
            }
            //导出

            StringBuffer fileNameSb = new StringBuffer().append(vo.getTaskName()).append(".xls");
            String fileName = fileNameSb.toString();
            String path = new StringBuffer().append(requestPath).append(fileNameSb).toString();
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                workbook.write(os);
                //阿里云返回url
                upLoadUrl = OssUpload.upload(path, fileName);
                file = new File(path);
                success(taskContext, "成功", upLoadUrl);
            } catch (Exception e1) {
                e1.printStackTrace();
                fail(taskContext, "写入过程中发生错误");
                logger.error("写入数据到Excel的过程中或者上传到阿里云中发生错误");
            } finally {
                if (os != null) {
                    os.close();
                }
                if (workbook != null) {
                    workbook.close();
                }
                if (file != null) {
                    file.delete();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, researchName + "导出过程中发生错误，请查看日志");
            logger.error("创建Excel的过程中发生错误");
        }
        return upLoadUrl;
    }

}
