package com.yizhi.statistics.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.statistics.application.entity.PracticeConfig;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 陪练配置 Mapper 接口
 * </p>
 */
public interface PracticeConfigMapper extends BaseMapper<PracticeConfig> {

    /**
     * 通过陪练名称获取陪练配置
     * @param nameKwd 陪练名称
     * @param startDate 筛选开始时间
     * @param endDate 筛选结束时间
     * @param companyId 公司id
     * @param siteId 站点id
     * @param configPage 分页对象
     */
    List<PracticeConfig> selectPracticeByName(String nameKwd, Date startDate, Date endDate, Long companyId, Long siteId, Page<PracticeConfig> configPage);
    
    /**
     * 应参加的陪练（训练模式）的数量
     * @param accountId
     * @param orgId
     * @param searchStart
     * @param searchEnd
     * @return
     */
    public Integer shouldPracticeNum(@Param("accountId") Long accountId, @Param("orgId") Long orgId
    		, @Param("searchStart") Date searchStart, @Param("searchEnd") Date searchEnd
    		, @Param("managers") List<Long> managers, @Param("isAdmin")Boolean isAdmin
    		, @Param("siteId")Long siteId);
    
    /**
     * 应参加的陪练（考核模式）的数量
     * @param accountId
     * @param orgId
     * @param searchStart
     * @param searchEnd
     * @return
     */
    public Integer shouldExamNum(@Param("accountId") Long accountId, @Param("orgId") Long orgId
    		, @Param("searchStart") Date searchStart, @Param("searchEnd") Date searchEnd
    		, @Param("managers") List<Long> managers, @Param("isAdmin")Boolean isAdmin
    		, @Param("siteId")Long siteId);



    /**
     * 报表通过陪练统计-陪练列表
     * @param nameKwd 陪练名称
     * @param startDate 筛选开始时间
     * @param endDate 筛选结束时间
     * @param manageIds 用户和管理用户id
     * @param companyId 公司id
     * @param siteId 站点id
     * @param configPage 分页
     */
    List<PracticeConfig> selectPracticesLimit(
            @Param("nameKwd") String nameKwd
            ,@Param("startDate") Date startDate
            ,@Param("endDate") Date endDate
            ,@Param("manageIds") List<Long> manageIds
            ,@Param("companyId") Long companyId
            ,@Param("siteId") Long siteId
            ,@Param("isAdmin") boolean isAdmin
            , Page<PracticeConfig> configPage);
}
