package com.yizhi.statistics.application.mapper;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.statistics.application.course.vo.ReportStudyCourseVo;
import com.yizhi.statistics.application.domain.ExamGroupOrgDto;
import com.yizhi.statistics.application.domain.OrgCountDto;
import com.yizhi.statistics.application.entity.StatisticsAccount;
import com.yizhi.statistics.application.entity.StatisticsCourseVisibleRange;
import com.yizhi.statistics.application.vo.practice.PracticeReportByUserListVo;
import com.yizhi.statistics.application.vo.practice.PracticeStatByOrgVo;
import com.yizhi.statistics.application.vo.report.AccountVO;

/**
 * <p>
 * Mapper 接口
 * </p>
 *
 * @author yinyuyan123
 * @since 2019-03-20
 */
public interface StatisticsAccountMapper extends BaseMapper<StatisticsAccount> {


    /**
     * 根据站点查询到此站点下有多少人
     *
     * @param siteId
     * @return
     */
    Integer siteIdJoinNum(@Param("siteId") Long siteId);


    /**
     * 查询所有的companyId，已经去重
     *
     * @return
     */
    List<Long> selectDistinctCompanyIdList();


    /**
     * 根据companyId查询所有的siteId，已经去重
     *
     * @param companyId
     * @return
     */
    List<Long> selectDistinctSiteIdListByCompanyId(@Param("companyId") Long companyId);


    /**
     * 每个业务的全平台可见，超级管理员可见的情况下，可参加人数
     *
     * @param companyId
     * @param siteId
     * @return
     */
    List<ReportStudyCourseVo> selectTotalAskAccountByCompanyIdAndSiteId(@Param("companyId") Long companyId, @Param("siteId") Long siteId);

    /**
     * 每个站点应该参加的人数
     *
     * @return
     */

    @MapKey("siteId")
    Map<Long, StatisticsCourseVisibleRange> getSiteVoAskNum();


    List<Long> getAccountsBycourseIdLimitIsAdmin(@Param("orgKwd") String orgKwd, @Param("accountKwd") String accountKwd, @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("courseId") Long courseId, Page<Long> page);

    List<Long> getAccountsBycourseIdLimitNotAdmin(@Param("orgKwd") String orgKwd, @Param("accountKwd") String accountKwd, @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("courseId") Long courseId, @Param("adminId") Long adminId, Page<Long> page);


    List<Long> getAccountsLimitIsAdmin(@Param("orgKwd") String orgKwd, @Param("accountKwd") String accountKwd, @Param("companyId") Long companyId, @Param("siteId") Long siteId, Page<Long> page);

    List<Long> getAccountsLimitNotAdmin(@Param("orgKwd") String orgKwd, @Param("accountKwd") String accountKwd, @Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("adminId") Long adminId, Page<Long> page);


    List<Long> accountIdsByCompanyIdAndSiteId(@Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgKwd") String orgKwd, @Param("accountKwd") String accountKwd);

    List<Long> accountIdsByCompanyIdAndSiteIdNotAdmin(@Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("adminId") Long adminId, @Param("orgKwd") String orgKwd, @Param("accountKwd") String accountKwd);

    public List<AccountVO> getAccountIdsBySiteId(@Param("siteId") Long siteId);

    /**
     * 获取企业站点课程ID
     *
     * @param companyId
     * @param siteId
     * @return
     */
    List<Long> getCourseIdByCompanyIdAndSiteId(@Param("companyId") Long companyId, @Param("siteId") Long siteId);

    /**
     * 获取部门下用户数量
     *
     * @param companyId
     * @param siteId
     * @return
     */
    List<OrgCountDto> getCountByOrgId(@Param("companyId") Long companyId, @Param("siteId") Long siteId);

    List<OrgCountDto> getCountByOrgIdNoAdmin(@Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("adminId") Long adminId);

    List<OrgCountDto> getRoleOrgCount(@Param("siteId") Long siteId, @Param("adminId")  Long adminI);

    List<ExamGroupOrgDto> getOrgByCompanyAndSiteId(@Param("companyId") Long companyId,
                                                   @Param("siteId") Long siteId,
                                                   @Param("pageStart") Long pageStart,
                                                   @Param("pageSize") Long pageSize,
                                                   @Param("orgName") String orgName);

    List<PracticeStatByOrgVo> getOrgByCompanyAndSiteIdNoAdmin(@Param("companyId") Long companyId,
                                                              @Param("siteId") Long siteId,
                                                              @Param("pageStart") Long pageStart,
                                                              @Param("pageSize") Long pageSize,
                                                              @Param("orgName") String orgName,
                                                              @Param("orgIds") List<Long> orgIds);

    List<AccountVO> getUserIdsByNameLike(@Param("companyId") Long companyId,
                                         @Param("siteId") Long siteId,
                                         @Param("orgId") Long orgId,
                                         @Param("userName") String userName,
                                         @Param("accountId") Long accountId,
                                         @Param("admin") boolean admin,
                                         @Param("pageStart") Long pageStart,
                                         @Param("pageSize") Long pageSize);

    Long getUserCountByNameLike(
            @Param("companyId") Long companyId,
            @Param("siteId") Long siteId,
            @Param("orgId") Long orgId,
            @Param("userName") String userName,
            @Param("accountId") Long accountId,
            @Param("admin") boolean admin
    );


    @Select("select distinct account_id \n" +
            "from \n" +
            "statistics_account_role\n" +
            "where \n" +
            "account_site_id  = #{siteId}\n" +
            "and admin_id = #{adminId}\n" +
            "and type = 1\n" +
            "and account_org_id = #{orgId}")
    List<Long> getUserByManager(@Param("adminId") Long adminId, @Param("siteId") Long siteId, @Param("orgId") Long orgId);


    Long getOrgCount(@Param("companyId") Long companyId,
                     @Param("siteId") Long siteId,
                     @Param("orgCode") String orgCode);


    AccountVO getOrgInfoByOrgId(@Param("companyId") Long companyId,@Param("siteId") Long siteId,@Param("orgId") Long orgId);

    Long getVisibleAccountByNoAdmin(
            @Param("companyId") Long companyId,
            @Param("siteId") Long siteId,
            @Param("orgId") Long orgI,
            @Param("adminId") Long adminId,
            @Param("startAt") Date startAt,
            @Param("endAt") Date endAt);

    List<StatisticsAccount> getAccountByIds(@Param("ids") List<Long> ids,@Param("companyId") Long companyId);

    Long getUserCount(@Param("orgIds") List<Long> orgIds, @Param("companyId")Long companyId);
    
    /**
     * 智能演练-报表-列表-主数据-用户列表,分页
     * @param isAdmin
     * @param siteId 当前登录人的site id
     * @param orgIds 管辖区内的部门
     * @param userIds 管辖区内的用户
     * @param orgName 搜索条件 - 部门
     * @param orgId 搜索条件 - 部门ID
     * @param name 搜索条件 - 用户、用户名
     * @param page
     * @return
     */
    public List<PracticeReportByUserListVo> practiceReportByUserList(
    		@Param("isAdmin")Boolean isAdmin
    		, @Param("siteId")Long siteId
    		, @Param("orgIds")List<Long> orgIds
    		, @Param("userIds")List<Long> userIds
    		, @Param("orgName") String orgName
    		, @Param("orgId")Long orgId
    		, @Param("name") String name
    		, Page<PracticeReportByUserListVo> page);
    
    /**
     * 智能演练-报表-列表-主数据-用户列表, 不是分页，获取当前登陆人管理的所有用户
     * @param isAdmin
     * @param siteId 当前登录人的site id
     * @param orgIds 管辖区内的部门
     * @param userIds 管辖区内的用户
     * @param orgName 搜索条件 - 部门
     * @param orgId 搜索条件 - 部门ID
     * @param name 搜索条件 - 用户、用户名
     * @return
     */
    public List<PracticeReportByUserListVo> practiceReportByUserAll(
    		@Param("isAdmin")Boolean isAdmin
    		, @Param("siteId")Long siteId
    		, @Param("orgIds")List<Long> orgIds
    		, @Param("userIds")List<Long> userIds
    		, @Param("orgName") String orgName
    		, @Param("orgId")Long orgId
    		, @Param("name") String name);

    /**
     * 智能演练-报表-列表-主数据，获取总数据量
     * @param isAdmin
     * @param siteId 当前登录人的site id
     * @param orgIds 管辖区内的部门
     * @param userIds 管辖区内的用户
     * @param orgName 搜索条件 - 部门
     * @param orgId 搜索条件 - 部门ID
     * @param name 搜索条件 - 用户、用户名
     * @return
     */
    public Long practiceReportByUserCount(@Param("isAdmin")Boolean isAdmin
    		, @Param("siteId")Long siteId
    		, @Param("orgIds")List<Long> orgIds
    		, @Param("userIds")List<Long> userIds
    		, @Param("orgName") String orgName
    		, @Param("orgId")Long orgId
    		, @Param("name") String name);

    Long getManageOrgCount(@Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgNameLike") String orgNameLike, @Param("orgIds") List<Long> orgIds);
    
    /**
     * 根据部门ID获取用户
     * @param companyId
     * @param visibleOrgId
     * @return
     */
    List<StatisticsAccount> getAccountByOrgId(@Param("companyId") Long companyId, @Param("visibleOrgId") Long visibleOrgId);

    Long getOrgCountByPractice(@Param("companyId") Long companyId, @Param("siteId") Long siteId, @Param("orgNameLike")  String orgNameLike, @Param("orgIds") List<Long> orgIds);

    /**
     * 获取管辖区的人数
     * @param userIds 管辖区用户id集合
     * @param orgIds 管辖区组织id集合
     */
    Long getManageCount(@Param("siteId")Long siteId, @Param("userIds") List<Long> userIds,@Param("orgIds") List<Long> orgIds,@Param("isAdmin") boolean isAdmin);
}
