package com.yizhi.statistics.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.yizhi.statistics.application.domain.OrgCountDto;
import com.yizhi.statistics.application.entity.StatisticsAccountRole;
import com.yizhi.statistics.application.entity.StatisticsCourseVisibleRange;
import com.yizhi.statistics.application.scope.JoinNum;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * <p>
 *  Mapper 接口
 * </p>
 *
 * @author wfd123
 * @since 2019-04-18
 */
public interface StatisticsAccountRoleMapper extends BaseMapper<StatisticsAccountRole> {

	/**
	 * 管辖区删除一部分人
	 * @param adminId
	 * @param accountDeleteIds
	 * @return
	 */
	Boolean deleteRoleByAdminId(@Param("adminId")Long adminId, @Param("accountDeleteIds")List<Long> accountDeleteIds) ;


	/**
	 * 根据管理人查找此管理人管辖区的人数
	 * @param adminId
	 * @return
	 */
    Integer selectCountByAdminId(@Param("adminId")Long adminId);

    /**
     * 每个管理员当前管理员的权限
     * @return
     */
    List<StatisticsAccountRole> listStatisticsAccountRole();


    /**
     * 普通管理员对应各个模块的可参加人数  业务是全平台可见
     * @return
     */

    List<StatisticsAccountRole> getAdminIdsAndWorkIdsAskNum();



    /**
     * 通过sql  -- 管辖区的超级管理员 全平台指定范围
     * @return
     */
    List<JoinNum> isAdminAllPlatform(@Param("workIdsLimit")List<Long> workIdsLimit);

    /**
     * 通过sql  -- 管辖区的普通管理员 全平台指定范围
     * @return
     */
    List<JoinNum> notAdminLimitPlatform(@Param("workIdsLimit")List<Long> workIdsLimit);


    List<JoinNum> isAdminByAdminId(@Param("accountId")Long accountId);

    List<JoinNum> notAdminByAdminId(@Param("accountId")Long accountId);

    void deleteAdmin(@Param("accountId")Long accountId);

    /**
     * 普通管理员验证管辖区是否是全平台的
     * @param type
     * @param siteId
     * @param adminId
     * @return
     */
    Integer getIsAdmin(@Param("type") Integer type,@Param("siteId") Long siteId,@Param("adminId") Long adminId);
    
    /**
     * 获取所有可见范围的企业ID和站点ID
     * @return
     */
    List<StatisticsCourseVisibleRange> getVisibleRangeAllCompanyIdAndSiteId();
    
    /**
     * 获取普通管理员信息
     * @return
     */
    List<Long> getAdminAccountIdBySiteId(@Param("siteId") Long siteId);
    
    List<JoinNum> isAdminAllPlatformNew(@Param("companyId") Long companyId, @Param("siteId") Long siteId);
    
    List<JoinNum> notAdminLimitPlatformNew(@Param("companyId") Long companyId, @Param("siteId") Long siteId);

    List<OrgCountDto> getAccountIdByAdminId(@Param("adminId") Long adminId,
                                            @Param("siteId") Long siteId);

    List<Long> getVisibleAccount(
            @Param("companyId") Long companyId,
            @Param("siteId") Long siteId,
            @Param("orgId") Long orgId,
            @Param("startAt") Date startAt,
            @Param("endAt") Date endAt
    );
}
