package com.yizhi.statistics.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.statistics.application.entity.StatisticsResearchLearnMetadata;
import com.yizhi.statistics.application.vo.research.ResearchGroupViewVo;
import com.yizhi.statistics.application.vo.research.ResearchGroupVo;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.session.RowBounds;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 学员调研明细 Mapper 接口
 * </p>
 *
 * @author hutao123
 * @since 2019-04-18
 */
public interface StatisticsResearchLearnMetadataMapper extends BaseMapper<StatisticsResearchLearnMetadata> {
    Date selectMaxDate();

    void deleteRecordeByDate(@Param ("researchId") Long researchId,
                             @Param ("startDate") String startDate,
                             @Param ("endDate") String endDate);


    Integer selectStatisticsResearchCount(@Param ("startDate") String startDate,
                                          @Param ("endDate") String endDate,
                                          @Param ("kwd") String kwd,
                                          @Param ("companyId") Long companyId,
                                          @Param ("orgIds") List<Long> orgIds,
                                          @Param ("siteId") Long siteId);

    List<ResearchGroupViewVo> selectResearchView(
            @Param ("researchId") Long researchId,
            @Param ("startDate") String startDate,
            @Param ("endDate") String endDate,
            @Param ("orgNameorOrgCode") String orgNameorOrgCode,
            @Param ("accountName") String accountName,
            @Param ("joinState") Integer joinState,
            Page<ResearchGroupViewVo> page);


    ResearchGroupVo selectJoinCount(@Param ("researchId") Long researchId,
                                    @Param ("startDate") String startDate,
                                    @Param ("endDate") String endDate,
                                    @Param ("orgNameorOrgCode") String orgNameorOrgCode,
                                    @Param ("accountName") String accountName,
                                    @Param ("joinState") Integer joinState);


    Integer selectResearchViewCount(@Param ("researchId") Long researchId,
                                    @Param ("startDate") String startDate,
                                    @Param ("endDate") String endDate,
                                    @Param ("orgNameorOrgCode") String orgNameorOrgCode,
                                    @Param ("accountName") String accountName,
                                    @Param ("joinState") Integer joinState);


    /**
     * 普通管理员   先查询researchIds
     *
     * @param startDate
     * @param endDate
     * @param kwd
     * @param companyId
     * @param orgIds
     * @param siteId
     * @param adminId
     * @param rowBounds
     * @return
     */
    List<Long> researchGroup1Ids(@Param ("startDate") String startDate,
                                 @Param ("endDate") String endDate,
                                 @Param ("kwd") String kwd,
                                 @Param ("companyId") Long companyId,
                                 @Param ("orgIds") List<Long> orgIds,
                                 @Param ("siteId") Long siteId,
                                 @Param ("adminId") Long adminId,
                                 RowBounds rowBounds);

    /**
     * 普通管理员   查询列表
     *
     * @param idList    调研ids
     * @param companyId
     * @param orgIds
     * @param siteId
     * @param adminId
     * @param rowBounds
     * @return
     */
    List<ResearchGroupVo> researchGroup1(@Param ("idList") List<Long> idList,
                                         @Param ("companyId") Long companyId,
                                         @Param ("orgIds") List<Long> orgIds,
                                         @Param ("siteId") Long siteId,
                                         @Param ("adminId") Long adminId,
                                         RowBounds rowBounds);




    /**
     * 普通管理员   查询列表总记录数
     *
     * @param startDate
     * @param endDate
     * @param kwd
     * @param companyId
     * @param orgIds
     * @param siteId
     * @param adminId
     * @return
     */
    Integer researchGroupListCount1(
            @Param ("startDate") String startDate,
            @Param ("endDate") String endDate,
            @Param ("kwd") String kwd,
            @Param ("companyId") Long companyId,
            @Param ("orgIds") List<Long> orgIds,
            @Param ("siteId") Long siteId,
            @Param ("adminId") Long adminId);

    /**
     * 普通管理员 查询参加人的学习记录列表
     *
     * @param researchId
     * @param orgNameorOrgCode
     * @param accountName
     * @param joinState
     * @param companyId
     * @param adminId
     * @param siteId
     * @param startDate  暂时没有用到
     * @param endDate   暂时没有用到
     * @param page
     * @return
     */
    List<ResearchGroupViewVo> researchGroupView1(
            @Param ("researchId") Long researchId,
            @Param ("orgNameorOrgCode") String orgNameorOrgCode,
            @Param ("accountName") String accountName,
            @Param ("joinState") Integer joinState,
            @Param ("companyId") Long companyId,
            @Param ("adminId") Long adminId,
            @Param ("siteId") Long siteId,
            @Param ("startDate") String startDate,
            @Param ("endDate") String endDate,
            Page<ResearchGroupViewVo> page);


    /**
     * 普通管理员 用户列表记录总数
     *
     * @param researchId
     * @param orgNameorOrgCode
     * @param accountName
     * @param joinState
     * @param companyId
     * @param adminId
     * @param siteId
     * @param startDate  暂时没有用到
     * @param endDate   暂时没有用到
     * @return
     */
    Integer researchGroupViewCount1(
            @Param ("researchId") Long researchId,
            @Param ("orgNameorOrgCode") String orgNameorOrgCode,
            @Param ("accountName") String accountName,
            @Param ("joinState") Integer joinState,
            @Param ("companyId") Long companyId,
            @Param ("adminId") Long adminId,
            @Param ("siteId") Long siteId,
            @Param ("startDate") String startDate,
            @Param ("endDate") String endDate);


    /**
     * 超级管理员   先查出列表ids
     *
     * @param startDate
     * @param endDate
     * @param kwd
     * @param companyId
     * @param orgIds
     * @param siteId
     * @param adminId   弃用
     * @param rowBounds
     * @return
     */
    List<Long> researchGroup2Ids(@Param ("startDate") String startDate,
                                 @Param ("endDate") String endDate,
                                 @Param ("kwd") String kwd,
                                 @Param ("companyId") Long companyId,
                                 @Param ("orgIds") List<Long> orgIds,
                                 @Param ("siteId") Long siteId,
                                 @Param ("adminId") Long adminId, RowBounds rowBounds);


    /**
     * 超级管理员   查询调研统计列表
     *
     * @param companyId
     * @param orgIds
     * @param siteId
     * @param adminId
     * @param rowBounds
     * @return
     * @pqram idList
     */
    List<ResearchGroupVo> researchGroup2(@Param ("idList") List<Long> idList,
                                         @Param ("companyId") Long companyId,
                                         @Param ("orgIds") List<Long> orgIds,
                                         @Param ("siteId") Long siteId,
                                         @Param ("adminId") Long adminId, RowBounds rowBounds);

    /**
     * 超级管理员   查询列表总记录数
     *
     * @param startDate
     * @param endDate
     * @param kwd
     * @param companyId
     * @param orgIds
     * @param siteId
     * @param adminId
     * @return
     */
    Integer researchGroupListCount2(
            @Param ("startDate") String startDate,
            @Param ("endDate") String endDate,
            @Param ("kwd") String kwd,
            @Param ("companyId") Long companyId,
            @Param ("orgIds") List<Long> orgIds,
            @Param ("siteId") Long siteId,
            @Param ("adminId") Long adminId);




    /**
     * 超级管理员 查询参加人的学习记录列表
     *
     * @param researchId
     * @param orgNameorOrgCode
     * @param accountName
     * @param joinState
     * @param companyId
     * @param adminId
     * @param siteId
     * @param startDate
     * @param endDate
     * @param page
     * @return
     */
    List<ResearchGroupViewVo> researchGroupView2(
            @Param ("researchId") Long researchId,
            @Param ("orgNameorOrgCode") String orgNameorOrgCode,
            @Param ("accountName") String accountName,
            @Param ("joinState") Integer joinState,
            @Param ("companyId") Long companyId,
            @Param ("adminId") Long adminId,
            @Param ("siteId") Long siteId,
            @Param ("startDate") String startDate,
            @Param ("endDate") String endDate,
            Page<ResearchGroupViewVo> page);


    /**
     * 超级管理员  用户列表记录总数
     *
     * @param researchId
     * @param orgNameorOrgCode
     * @param accountName
     * @param joinState
     * @param companyId
     * @param adminId
     * @param siteId
     * @param startDate
     * @param endDate
     * @return
     */
    Integer researchGroupViewCount2(
            @Param ("researchId") Long researchId,
            @Param ("orgNameorOrgCode") String orgNameorOrgCode,
            @Param ("accountName") String accountName,
            @Param ("joinState") Integer joinState,
            @Param ("companyId") Long companyId,
            @Param ("adminId") Long adminId,
            @Param ("siteId") Long siteId,
            @Param ("startDate") String startDate,
            @Param ("endDate") String endDate);
}
