package com.yizhi.statistics.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.statistics.application.domain.StatisticsSignInformation;
import com.yizhi.statistics.application.refactor.sign.StatisticsSignVO;
import com.yizhi.statistics.application.vo.sign.SignAccountGroupVo;
import com.yizhi.statistics.application.vo.sign.StatisticsSignVo;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * Mapper 接口
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-04-22
 */
public interface StatisticsSignMapper extends BaseMapper<StatisticsSignInformation> {

    /**
     * 签到统计列表
     *x
     * @param isAdmin
     * @param signTimeIds
     * @param companyId
     * @param siteId
     * @return
     */
    List<StatisticsSignVO> signGroup(@Param("isAdmin") boolean isAdmin, @Param("accountId") Long accountId,
                                   @Param("signTimeIds") List<Long> signTimeIds,
                                   @Param("companyId") Long companyId, @Param("siteId") Long siteId);

    /**
     * 签到统计查看超级管理员
     *
     * @param isAllVisible
     * @param trainningProjectId
     * @param companyId
     * @param siteId
     * @param signId
     * @param signTimeId
     * @param orgNameOrCode
     * @param accountName
     * @param page
     * @return
     */
    List<StatisticsSignVO> signGroupViewAdmin(@Param("isAllVisible") boolean isAllVisible, @Param("trainningProjectId") Long trainningProjectId,
                                            @Param("signId") Long signId, @Param("signTimeId") Long signTimeId,
                                            @Param("companyId") Long companyId, @Param("siteId") Long siteId,
                                            @Param("orgNameOrCode") String orgNameOrCode, @Param("accountName") String accountName, Page<StatisticsSignVO> page);

    /**
     * 签到统计查看普通管理员
     *
     * @param isAllVisible
     * @param trainningProjectId
     * @param signId
     * @param signTimeId
     * @param companyId
     * @param siteId
     * @param orgNameOrCode
     * @param accountName
     * @param page
     * @return
     */
    List<StatisticsSignVO> signGroupViewNoAdmin(@Param("isAllVisible") boolean isAllVisible, @Param("accountId") Long accountId,
                                                @Param("trainningProjectId") Long trainningProjectId,
                                                @Param("signId") Long signId, @Param("signTimeId") Long signTimeId,
                                                @Param("companyId") Long companyId, @Param("siteId") Long siteId,
                                                @Param("orgNameOrCode") String orgNameOrCode, @Param("accountName") String accountName, Page<StatisticsSignVO> page);

    /**
     * 获取签到时间段信息
     *
     * @param signTimeId
     * @return
     */
    StatisticsSignInformation selectBySignTimeId(@Param("signTimeId") Long signTimeId);

    /**
     * 获取签到记录最近的时间
     *
     * @return
     */
    Date selectMaxDate();

    /**
     * 获取签到列表分页id
     *
     * @param kwd
     * @param startDate
     * @param endDate
     * @param companyId
     * @param siteId
     * @param signTimeIdPage
     * @return
     */
    List<Long> getSignTimeIds(@Param("kwd") String kwd, @Param("startDate") String startDate, @Param("endDate") String endDate,
                              @Param("companyId") Long companyId, @Param("siteId") Long siteId, Page<Long> signTimeIdPage);


    /**
     * 按条件获取签到用户ids
     * @return
     */
    List<Long> getAccountIds(@Param("isAdmin") boolean isAdmin,
                             @Param("accountId") Long accountId,
                             @Param("accountKwd") String accountKwd,
                             @Param("orgKwd") String orgKwd, @Param("companyId") Long companyId, @Param("siteId") Long siteId, Page<Long> accountGroupPage);


    /**
     * 获取所有全平台可见的签到数
     *
     * @param companyId
     * @param siteId
     * @return
     */
    Integer getAllVisibleSignNum(Long companyId, Long siteId, String startDate, String endDate);

    /**
     *  获取按用户统计签到信息
     *
     * @param isAdmin
     * @param companyId
     * @param siteId
     * @param startDate
     * @param endDate
     * @param accountIds
     * @param signTimeIds
     * @return
     */
    List<SignAccountGroupVo> getSignAccountGroup(boolean isAdmin, Long accountId, Long companyId, Long siteId, String startDate, String endDate, List<Long> accountIds, List<Long> signTimeIds);

    /**
     * 获取用户签到详情
     * @param accountId
     * @param siteId
     * @param companyId
     * @param startDate
     * @param endDate
     * @param projectKwd
     * @param accountGroupViewPage
     * @return
     */
    List<StatisticsSignVo> getSignAccountGroupView(Long accountId, Long siteId, Long companyId, String startDate, String endDate, String projectKwd, Page<StatisticsSignVo> accountGroupViewPage);

}
