package com.yizhi.statistics.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.statistics.application.domain.StatisticsPointDto;
import com.yizhi.statistics.application.entity.PointLog;
import com.yizhi.statistics.application.enums.PointChangeTypeEnum;
import com.yizhi.statistics.application.mapper.PointLogMapper;
import com.yizhi.statistics.application.request.point.StatisticsPointDetailQO;
import com.yizhi.statistics.application.request.point.StatisticsPointQO;
import com.yizhi.statistics.application.service.PointLogService;
import com.yizhi.statistics.application.vo.point.StatisticsPointDetailVO;
import com.yizhi.statistics.application.vo.point.StatisticsPointVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 积分明细 服务实现类
 * </p>
 *
 * @author hutao123
 * @since 2021-01-25
 */
@Service
public class PointLogServiceImpl extends ServiceImpl<PointLogMapper, PointLog> implements PointLogService {

    private static final Logger logger = LoggerFactory.getLogger(PointLogServiceImpl.class);
    /**
     * 获取用户积分明细
     * @param qo
     * @return
     */
    @Override
    public Page<StatisticsPointDetailVO> getStatisticsPoint(StatisticsPointDetailQO qo, RequestContext context) {
        Page<StatisticsPointDetailVO> voPage = new Page<>(qo.getPageNo(), qo.getPageSize());
        Page<PointLog> page = new Page<>(qo.getPageNo(), qo.getPageSize());
        EntityWrapper<PointLog> ew = new EntityWrapper<>();
        ew.eq("account_id", qo.getAccountId());
        ew.eq("site_id", context.getSiteId());
        ew.eq("company_id", context.getCompanyId());
        ew.between("created_at", qo.getStartAt(), qo.getEndAt());
        if (Objects.nonNull(qo.getChangeType())) {
            ew.eq("change_type", qo.getChangeType());
        }
        ew.orderBy("created_at", true);
        page = this.selectPage(page, ew);
        List<PointLog> pointLogs = page.getRecords();
        if (CollectionUtils.isNotEmpty(pointLogs)) {
            //传入用户的第一条记录的beforeCount
            Integer beforeCount = baseMapper.selectList(ew).get(0).getBeforeCount();
            voPage.setRecords(transformVOList(pointLogs, beforeCount));
            voPage.setTotal(page.getTotal());
        }
        return voPage;
    }

    /**
     * 转换list-volist
     * @param pointLogList
     * @return
     */
    private List<StatisticsPointDetailVO> transformVOList(List<PointLog> pointLogList, Integer beforeCount) {
        List<StatisticsPointDetailVO> voList = new ArrayList<>(pointLogList.size());
        StatisticsPointDetailVO vo = null;
        for (PointLog pointLog : pointLogList) {
            vo = new StatisticsPointDetailVO();
            vo.setChangeType(PointChangeTypeEnum.relation.get(pointLog.getChangeType()));
            vo.setChangeCount(pointLog.getChangeCount());
            vo.setRelationName(pointLog.getRelationName());
            vo.setCreatedAt(pointLog.getCreatedAt());
            vo.setCurrentRemain(pointLog.getAfterCount() - beforeCount);//当前剩余 = 当前记录变动后数量 - 第一条记录的变动前数量
            vo.setCumulativeRemain(pointLog.getAfterCount());
            voList.add(vo);
        }
        return voList;
    }

    /**
     * 获取所有用户积分明细
     * @param qo
     * @return
     */
    @Override
    public Page<StatisticsPointVO> getAllStatisticsPoint(StatisticsPointQO qo, RequestContext context) {
        Page<StatisticsPointVO> voPage = new Page<>(qo.getPageNo(), qo.getPageSize());
        List<StatisticsPointDto> dtoList = baseMapper.getAllStatisticsPointList(context.getCompanyId(), context.getSiteId(), qo.getAccountKwd(),qo.getOrgKwd(), qo.getStartAt(), qo.getEndAt(),  voPage);
        List<StatisticsPointVO> voList = new ArrayList<>(dtoList.size());
        StatisticsPointVO vo = null;
        if (CollectionUtils.isNotEmpty(dtoList)) {
            Map<Long, Integer> map = new HashMap<>();
            //因为dtolist已经按时间正序排 只取第一条获取每个用户最早的beforecount
            for (StatisticsPointDto dto : dtoList) {
                if (!map.containsKey(dto.getAccountId())) {
                    map.put(dto.getAccountId(), dto.getBeforeCount());
                }
                continue;
            }
            for (StatisticsPointDto statisticsPointDto : dtoList) {
                vo = new StatisticsPointVO();
                BeanUtils.copyProperties(statisticsPointDto, vo);
                vo.setCurrentRemain(statisticsPointDto.getAfterCount() - map.get(statisticsPointDto.getAccountId()));
                vo.setCumulativeRemain(statisticsPointDto.getAfterCount());
                voList.add(vo);
            }
            voPage.setRecords(voList);
            voPage.setTotal(voList.size());
        }

        return voPage;
    }
}
