package com.yizhi.statistics.application.service.impl;

import cn.hutool.core.date.DateUtil;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.practice.application.enums.PracticeModelStatusEnum;
import com.yizhi.statistics.application.domain.OrgAccountDto;
import com.yizhi.statistics.application.domain.OrgCountDto;
import com.yizhi.statistics.application.domain.PracticeByOrgUserDetailDto;
import com.yizhi.statistics.application.domain.PracticeOrgCountDto;
import com.yizhi.statistics.application.mapper.PracticeChatLogByOrgMapper;
import com.yizhi.statistics.application.request.practice.PracticeByOrgPageReq;
import com.yizhi.statistics.application.request.practice.PracticeReportByUserReq;
import com.yizhi.statistics.application.service.PracticeByOrgService;
import com.yizhi.statistics.application.service.StatisticsAccountService;
import com.yizhi.statistics.application.system.AccountManage;
import com.yizhi.statistics.application.vo.practice.PracticeStatByOrgVo;
import com.yizhi.system.application.vo.practice.AccountOrgVo;
import com.yizhi.util.application.page.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Date 2020/11/12 3:03 下午
 * @Author lvjianhui
 **/
@Service
public class PracticeByOrgImpl implements PracticeByOrgService {

    @Autowired
    private AccountManage accountManage;

    @Autowired
    private PracticeChatLogByOrgMapper practiceChatLogByOrgMapper;
    @Autowired
    private StatisticsAccountService statisticsAccountService;

    @Override
    public List<PracticeByOrgUserDetailDto> getUserPracticeByOrgId(PracticeReportByUserReq res, RequestContext context) {

        List<Long> authOrgList = null;
        List<Long> authUserList = null;
        if (!context.isAdmin()) {
            OrgAccountDto orgUser = accountManage.getManageOrgAndUser(context.getAccountId(), context.getSiteId());
            authOrgList = orgUser.getOrgs();
            authUserList = orgUser.getUsers().stream().map(AccountOrgVo::getAccountId).collect(Collectors.toList());
        }
        List<PracticeByOrgUserDetailDto> list = practiceChatLogByOrgMapper.getUserDetailByOrgId(
                res.getOrgId(),
                res.getUser(),
                context.getSiteId(),
                context.getCompanyId(),
                context.isAdmin(),
                authOrgList,
                authUserList,
                res.getDateFrom(),
                res.getDateTo());
        return list;
    }

    @Override
    public PageInfo<PracticeStatByOrgVo> getLogByOrg(PracticeByOrgPageReq res, RequestContext context) {

        Long adminId = context.getAccountId();
        Long siteId = context.getSiteId();
        Long companyId = context.getCompanyId();

        Integer pageSize = res.getPageSize();
        Integer pageIndex = res.getPageNo();
        Boolean adminUser = context.isAdmin();
        Date startAt = DateUtil.parseDate(res.getStartDate()).toJdkDate();
        Date endAt = DateUtil.parseDate(res.getEndDate()).toJdkDate();
        String orgName = res.getOrgName();

        List<Long> orgIds = getAllManageOrgIds(adminId, siteId);

        PageInfo<PracticeStatByOrgVo> page = new PageInfo();
        page.setPageNo(pageIndex);
        page.setPageSize(pageSize);
        page.setPageTotal(0);
        if (!adminUser && orgIds.isEmpty()) {
            return page;
        }

        Long count = getOrgCount(adminUser, orgIds, siteId, companyId, orgName);
        if (count <= 0) {
            return page;
        }

        Integer pageTotal = count.intValue() / pageSize + 1;
        page.setPageTotal(pageTotal);
        page.setPageRecords(count.intValue());

        List<PracticeStatByOrgVo> pageList = getOrgPage(adminUser, orgIds, siteId, companyId, pageIndex.longValue(), pageSize.longValue(), orgName);
        List<Long> pageOrgIds = pageList.stream().map(PracticeStatByOrgVo::getOrgId).collect(Collectors.toList());

        OrgAccountDto authUser = accountManage.getManageOrgAndUser(adminId, siteId);
        List<Long> authOrgIds = authUser.getOrgs();
        List<Long> authUserIds = authUser.getUsers().stream().map(AccountOrgVo::getAccountId).collect(Collectors.toList());
        if (authOrgIds.isEmpty() && authUserIds.isEmpty()) {
            return page;
        }

        // 可参加考核数
        List<OrgCountDto> planJoinExamCount = getPlanJoinPracticeCount(pageOrgIds, siteId, companyId, adminUser,
                authOrgIds, authUserIds, PracticeModelStatusEnum.ASSESSMENT.getStatus(), startAt, endAt,adminId);
        Map<Long, Long> planJoinExamMap = planJoinExamCount.stream().collect(Collectors.toMap(OrgCountDto::getOrgId, OrgCountDto::getCount));
        Map<Long, Long> planJoinExamUserMap = planJoinExamCount.stream().collect(Collectors.toMap(OrgCountDto::getOrgId, OrgCountDto::getUserCount));

        // 可参加陪练数
        List<OrgCountDto> planJoinPracticeCount = getPlanJoinPracticeCount(pageOrgIds, siteId, companyId, adminUser,
                authOrgIds, authUserIds, PracticeModelStatusEnum.PRACTICE.getStatus(), startAt, endAt,adminId);
        Map<Long, Long> planJoinPracticeMap = planJoinPracticeCount.stream().collect(Collectors.toMap(OrgCountDto::getOrgId, OrgCountDto::getCount));
        Map<Long, Long> planJoinPracticeUserMap = planJoinPracticeCount.stream().collect(Collectors.toMap(OrgCountDto::getOrgId, OrgCountDto::getUserCount));

        // 实际参加考核数
        List<PracticeOrgCountDto> actualExamCount =
                practiceChatLogByOrgMapper.getActualJoinPracticeCount(pageOrgIds, siteId, companyId, adminUser, orgIds,
                        authUserIds, PracticeModelStatusEnum.ASSESSMENT.getStatus(), startAt, endAt, null);
        Map<Long, Long> actualExamUserMap = actualExamCount.stream().collect(Collectors.toMap(PracticeOrgCountDto::getOrgId, PracticeOrgCountDto::getUserCount));

        // 实际参加陪练数
        List<PracticeOrgCountDto> actualPracticeCount = practiceChatLogByOrgMapper.getActualJoinPracticeCount(pageOrgIds, siteId, companyId, adminUser, orgIds,
                authUserIds, PracticeModelStatusEnum.PRACTICE.getStatus(), startAt, endAt, null);
        Map<Long, Long> actualPracticeMap = actualPracticeCount.stream().collect(Collectors.toMap(PracticeOrgCountDto::getOrgId, PracticeOrgCountDto::getCount));

        // 通过考核人数
        List<PracticeOrgCountDto> actualPassExamUserCount =
                practiceChatLogByOrgMapper.getActualJoinPracticeCount(pageOrgIds, siteId, companyId, adminUser, orgIds,
                        authUserIds, PracticeModelStatusEnum.ASSESSMENT.getStatus(), startAt, endAt, 80);
        Map<Long, Long> actualPassExamUserMap = actualPassExamUserCount.stream().collect(Collectors.toMap(PracticeOrgCountDto::getOrgId, PracticeOrgCountDto::getUserCount));

        for (PracticeStatByOrgVo it : pageList) {
            Long orgId = it.getOrgId();
            if (null != actualExamUserMap.get(orgId)) {
                it.setActualJoinExamUserCount(actualExamUserMap.get(orgId));
            }
            if (null != actualPassExamUserMap.get(orgId)) {
                it.setExamPassUserCount(actualPassExamUserMap.get(orgId));
            }
            if (null != actualPracticeMap.get(orgId)) {
                it.setActualJoinPracticeUserCount(actualPracticeMap.get(orgId));
            }
            if (null != planJoinExamMap.get(orgId)) {
                it.setPlanJoinExamCount(planJoinExamMap.get(orgId));
            }
            if (null != planJoinPracticeMap.get(orgId)) {
                it.setPlanJoinPracticeCount(planJoinPracticeMap.get(orgId));
            }
            if (null != planJoinExamUserMap.get(orgId)) {
                it.setPlanJoinExamUserCount(planJoinExamUserMap.get(orgId));
            }
            if (null != planJoinPracticeUserMap.get(orgId)) {
                it.setPlanJoinPracticeUserCount(planJoinPracticeUserMap.get(orgId));
            }

        }
        page.setRecords(pageList);
        return page;
    }

    /**
     * @param pageOrgIds 分页后的部门ID
     * @param siteId     站点Id
     * @param companyId  公司Id
     * @param adminUser  是否为admin
     * @param orgIds     管辖区 的部门集合
     * @param userIds    管辖区的 用户集合
     * @return
     */
    private List<OrgCountDto> getPlanJoinPracticeCount(List<Long> pageOrgIds,
                                                       Long siteId,
                                                       Long companyId,
                                                       Boolean adminUser,
                                                       List<Long> orgIds,
                                                       List<Long> userIds,
                                                       Integer useType,
                                                       Date startAt,
                                                       Date endAt,
                                                       Long adminId) {
        // 指定用户可见陪练数
        List<OrgCountDto> visibleCount = practiceChatLogByOrgMapper.getPlanJoinPracticeCount(pageOrgIds, siteId, companyId, adminUser, orgIds, userIds, useType, startAt, endAt);

        // 全平台可见陪练
        Long allVisibleCount = practiceChatLogByOrgMapper.selectAllVisibleConfig(siteId, companyId, adminUser, useType, startAt, endAt);
        if (allVisibleCount <= 0) {
            return visibleCount;
        }


        Map<Long, Long> orgUserCount = practiceChatLogByOrgMapper.getCountByOrgId(siteId, companyId, pageOrgIds).stream().collect(Collectors.toMap(OrgCountDto::getOrgId, OrgCountDto::getUserCount));

        // 从可见范围查到的部门 设置 可见人数与可见陪练数
        for (OrgCountDto orgCountDto : visibleCount) {
            Long tmpOrgId = orgCountDto.getOrgId();
            orgCountDto.setCount(orgCountDto.getCount() + allVisibleCount);
            Long authOrgUserCount = orgUserCount.get(tmpOrgId);
            Integer authUserCount = getAuthUserCount(adminId,siteId,tmpOrgId);

            if (!adminUser && null !=  authUserCount) {
                authOrgUserCount = authUserCount.longValue();
            }
            orgCountDto.setUserCount(authOrgUserCount);
            orgUserCount.remove(orgCountDto.getOrgId());
        }

        // 从可见范围没有查到的部门，设置 可见人数与可见陪练数
        orgUserCount.forEach((key, value) -> {
            OrgCountDto dto = new OrgCountDto();
            dto.setUserCount(value);
            Integer authUserCount = getAuthUserCount(adminId,siteId,key);
            if (!adminUser && null !=  authUserCount) {
                dto.setUserCount(authUserCount.longValue());
            }
            dto.setCount(allVisibleCount);
            dto.setOrgId(key);
            visibleCount.add(dto);
        });
        return visibleCount;
    }

    private List<Long> getAllManageOrgIds(Long adminId, Long siteId) {
        OrgAccountDto dto = accountManage.getManageOrgAndUser(adminId, siteId);
        List<Long> manageOrgIds = dto.getOrgs();
        if (manageOrgIds == null) {
            manageOrgIds = new ArrayList();
        }

        Set<Long> allOrgIds = new HashSet(manageOrgIds);
        List<AccountOrgVo> manageUsers = dto.getUsers();
        for (AccountOrgVo manageUser : manageUsers) {
            if (!manageOrgIds.contains(manageUser.getOrgId())) {
                allOrgIds.add(manageUser.getOrgId());
            }
        }
        manageOrgIds.clear();
        manageOrgIds.addAll(allOrgIds);
        manageOrgIds.stream().sorted();
        return manageOrgIds;

    }

    private List<PracticeStatByOrgVo> getOrgPage(Boolean adminUser, List<Long> manageOrgIds, Long siteId, Long companyId, Long pageIndex, Long pageSize, String orgNameLike) {
        List<PracticeStatByOrgVo> list = new ArrayList();
        if (adminUser) {
            manageOrgIds = null;
        }
        list = statisticsAccountService.getOrgByCompanyAndSiteIdNoadmin(
                companyId,
                siteId,
                pageIndex,
                pageSize,
                orgNameLike,
                manageOrgIds);
        return list;
    }

    private Long getOrgCount(Boolean adminUser, List<Long> manageOrgIds, Long siteId, Long companyId, String orgNameLike) {
        if (adminUser) {
            manageOrgIds = null;
        }
        return statisticsAccountService.getOrgCountByPractice(companyId, siteId, orgNameLike, manageOrgIds);
    }

    private Integer getAuthUserCount(Long adminId, Long siteId, Long tipOrgId) {
        OrgAccountDto dto = accountManage.getManageOrgAndUser(adminId, siteId);
        List<Long> orgIdList = dto.getOrgs();
        if (orgIdList.contains(tipOrgId)) {
            return null;
        }
        List<AccountOrgVo> list = dto.getUsers();
        Map<Long, Integer> map = new HashMap();
        for (AccountOrgVo it : list) {
            Long orgId = it.getOrgId();
            if (orgIdList.contains(orgId)) {
                return null;
            }
            Integer userCount = map.get(orgId);
            if (userCount == null) {
                userCount = 1;
            } else {
                userCount++;
            }
            map.put(orgId, userCount);
        }
        Integer authOrgUserCount = map.get(tipOrgId);
        if (authOrgUserCount == null) {
            return 0;
        }
        return authOrgUserCount;
    }
}
