package com.yizhi.statistics.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.statistics.application.domain.ExamGroupOrgDto;
import com.yizhi.statistics.application.domain.OrgCountDto;
import com.yizhi.statistics.application.entity.StatisticsAccount;
import com.yizhi.statistics.application.exam.domain.po.UserExamGroupPO;
import com.yizhi.statistics.application.mapper.StatisticsAccountMapper;
import com.yizhi.statistics.application.service.StatisticsAccountService;
import com.yizhi.statistics.application.vo.practice.PracticeStatByOrgVo;
import com.yizhi.statistics.application.vo.report.AccountVO;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author yinyuyan123
 * @since 2019-03-20
 */
@Service("statisticsAccount")
public class StatisticsAccountServiceImpl extends ServiceImpl<StatisticsAccountMapper, StatisticsAccount> implements StatisticsAccountService {

	@Autowired
	IdGenerator idGenerator;

	@Override
	public Long getVisibleAccountByNoAdmin(Long companyId, Long siteId, Long orgId, Long adminId, Date startAt , Date endAt) {
		return this.baseMapper.getVisibleAccountByNoAdmin(companyId,siteId,orgId,adminId,startAt,endAt);
	}

	@Override
	public AccountVO getOrgInfoByOrgId(Long companyId,Long siteId,Long orgId) {
		return this.baseMapper.getOrgInfoByOrgId(companyId,siteId,orgId);
	}

	@Override
	public Long getOrgCount(Long companyId, Long siteId,String orgNameLike) {
		return this.baseMapper.getOrgCount(companyId,siteId,orgNameLike);
	}

	@Override
	public Long getManageOrgCount(Long companyId, Long siteId, String orgNameLike, List<Long> orgIds) {
		return this.baseMapper.getManageOrgCount(companyId,siteId,orgNameLike,orgIds);
	}

	@Override
	public List<ExamGroupOrgDto> getOrgByCompanyAndSiteId(Long companyId, Long siteId, Long pageIndex, Long pageSize,String orgName) {
		Long pageStart = (pageIndex - 1) * pageSize;
		return this.baseMapper.getOrgByCompanyAndSiteId(companyId,siteId,pageStart,pageSize,orgName);
	}

	@Override
	public List<OrgCountDto> getUserCountByOrgIdAndSiteIdAndCompanyId(Long companyId, Long siteId) {
		return this.baseMapper.getCountByOrgId(companyId,siteId);
	}

	@Override
	public List<OrgCountDto> getUserCountByOrgIdAndSiteIdAndCompanyIdNoAdmin(Long companyId, Long siteId, Long adminId) {
		return this.baseMapper.getCountByOrgIdNoAdmin(companyId,siteId,adminId);
	}

	@Override
	public List<OrgCountDto> getRoleOrgCount(Long siteId, Long adminId) {
		return this.baseMapper.getRoleOrgCount(siteId,adminId);
	}

	@Override
	public List<Long> getUserByManager(Long adminId, Long siteId, Long orgId) {
		return this.baseMapper.getUserByManager(adminId,siteId,orgId);
	}

	@Override
	public List<AccountVO> getUserIdByNameLike(UserExamGroupPO po) {

		Integer pageStart = (po.getPageIndex() - 1) * po.getPageSize();
		return this.baseMapper.getUserIdsByNameLike(
				po.getCompanyId(),
				po.getSiteId(),
				po.getOrgId(),
				po.getUserName(),
				po.getAccountId(),
				po.isAdmin(),
				pageStart.longValue(),
				po.getPageSize().longValue()
		);
	}
	@Override
	public Long getUserCountByNameLike(UserExamGroupPO po) {
		return this.baseMapper.getUserCountByNameLike(
				po.getCompanyId(),
				po.getSiteId(),
				po.getOrgId(),
				po.getUserName(),
				po.getAccountId(),
				po.isAdmin());
	}

	@Override
	public Boolean insertToStatisticsAccount(String relationId, List<ReportAccountRespVO> list, Long siteId, Long companyId) {
		// TODO Auto-generated method stub
		if(CollectionUtils.isNotEmpty(list)) {
			StatisticsAccount statisticsAccount=null;
			List<StatisticsAccount> listStatisticsAccount=new ArrayList<StatisticsAccount>();
			for (ReportAccountRespVO rarv:list) {
				statisticsAccount=new StatisticsAccount();
				statisticsAccount.setId(idGenerator.generate());
				statisticsAccount.setCompanyId(companyId);
				statisticsAccount.setSiteId(siteId);
				statisticsAccount.setOrgId(rarv.getOrgId());
				statisticsAccount.setOrgCode(rarv.getOrgCode());
				statisticsAccount.setOrgName(rarv.getOrgName());
				String orgParentNames=null;
				if(rarv.getParentOrgNames()!=null) {
					for (String orgName : rarv.getParentOrgNames()) {
						orgParentNames=orgParentNames+"/"+orgName;
					} 
					statisticsAccount.setOrgNames(orgParentNames);
				}
				else {
					statisticsAccount.setOrgNames(rarv.getOrgName());
				}
				statisticsAccount.setAccountId(rarv.getUserId());
				statisticsAccount.setAccountName(rarv.getUserName());
				statisticsAccount.setFullName(rarv.getUserFullName());
				statisticsAccount.setWorkNum(rarv.getWorkNum());
				statisticsAccount.setAccountState(rarv.getStatus());
				listStatisticsAccount.add(statisticsAccount);
			}
			return this.insertBatch(listStatisticsAccount);
		}
		return false;
	}

	

	@Override
	public List<StatisticsAccount> getStatisticsAccountBySiteId(Long siteId) {
		// TODO Auto-generated method stub
		StatisticsAccount sa=new StatisticsAccount();
		sa.setSiteId(siteId);
		EntityWrapper<StatisticsAccount> wrapper=new EntityWrapper<StatisticsAccount>(sa);
		return this.selectList(wrapper);
	}



	@Override
	public StatisticsAccount getStatisticsAccountByAccountId(Long accountId) {
		// TODO Auto-generated method stub
		StatisticsAccount sa=new StatisticsAccount();
		sa.setAccountId(accountId);
		return this.selectOne(new EntityWrapper<StatisticsAccount>(sa));
	}



	@Override
	public void deleteStatisticsAccount(Long siteId) {
		// TODO Auto-generated method stub
		
		StatisticsAccount sa=new StatisticsAccount();
		sa.setSiteId(siteId);
		EntityWrapper<StatisticsAccount> wrapper=new EntityWrapper<StatisticsAccount>(sa);
		this.delete(wrapper);
		
		
	}



	@Override
	public List<AccountVO> getAccountIdsBySiteId(Long siteId) {
		// TODO Auto-generated method stub
		return this.baseMapper.getAccountIdsBySiteId(siteId);
	}

	@Override
	public List<StatisticsAccount> getAccountByIds(List<Long> ids, Long companyId) {
		return this.baseMapper.getAccountByIds(ids,companyId);
	}

	@Override
	public Long getUserCount(List<Long> manageOrgIds, Long companyId) {
		return this.baseMapper.getUserCount(manageOrgIds,companyId);
	}

	@Override
	public List<StatisticsAccount> getAccountByOrgId(Long companyId, Long visibleOrgId) {
		return this.baseMapper.getAccountByOrgId(companyId,visibleOrgId);
	}

	@Override
	public List<PracticeStatByOrgVo> getOrgByCompanyAndSiteIdNoadmin(Long companyId, Long siteId, Long pageIndex, Long pageSize, String orgNameLike, List<Long> manageOrgIds) {
		Long pageStart = (pageIndex - 1) * pageSize;
		return this.baseMapper.getOrgByCompanyAndSiteIdNoAdmin(companyId,siteId,pageStart,pageSize,orgNameLike,manageOrgIds);
	}

	@Override
	public Long getOrgCountByPractice(Long companyId, Long siteId, String orgNameLike, List<Long> manageOrgIds) {
		return this.baseMapper.getOrgCountByPractice(companyId,siteId,orgNameLike,manageOrgIds);

	}
}
