package com.yizhi.statistics.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.album.application.feign.AlStudentActivityRecordClient;
import com.yizhi.album.application.vo.domain.AlStudentActivityRecord;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.statistics.application.album.vo.*;
import com.yizhi.statistics.application.entity.StatisticsAccount;
import com.yizhi.statistics.application.entity.StatisticsAlbum;
import com.yizhi.statistics.application.entity.StatisticsAlbumActivityRecord;
import com.yizhi.statistics.application.mapper.*;
import com.yizhi.statistics.application.service.IStatisticsAccountRoleService;
import com.yizhi.statistics.application.service.IStatisticsAlbumActivityRecordService;
import com.yizhi.statistics.application.util.AlbumInsertBatchUtil;
import com.yizhi.statistics.application.util.JedisUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

@Service
public class StatisticsAlbumActivityRecordServiceImpl extends ServiceImpl<StatisticsAlbumActivityRecordMapper, StatisticsAlbumActivityRecord> implements IStatisticsAlbumActivityRecordService {

	@Autowired
	private AlStudentActivityRecordClient alStudentActivityRecordClient;
	
	@Autowired
	private AlbumInsertBatchUtil albumInsertBatchUtil;
	
	@Autowired
	IdGenerator idGenerator;
	
	@Autowired
	private JedisUtil jedisUtil;
	
	@Autowired
	private StatisticsAccountMapper statisticsAccountMapper;
	
	@Autowired
	private IStatisticsAccountRoleService statisticsAccountRoleService;

	@Autowired
	private StatisticsAlbumMapper statisticsAlbumMapper;
	
	@Autowired
	private StatisticsAlbumActivityMapper statisticsAlbumActivityMapper;
	
	@Autowired
	private StatisticsCourseMapper statisticsCourseMapper;
	
	@Autowired
	private StatisticsAlbumActivityRecordMapper statisticsAlbumActivityRecordMapper;
	
	@Override
	public void dealGetAllRecordeByTimeLimit(String startDate, String endDate) {
		//先删除
		this.baseMapper.deleteLimitTime(startDate, endDate);
		//然后添加
		//初始化的时候因为数据量太大，因此分业务id导入;
		Long ttt=System.currentTimeMillis();
		List<Long> allWorkId = alStudentActivityRecordClient.getRecordeAllWorkId();
		if (CollectionUtils.isNotEmpty(allWorkId)) {
			for(Long relationId:allWorkId) {
				List<AlStudentActivityRecord> records = alStudentActivityRecordClient.getAllRecordeByTimeLimit(startDate, endDate, relationId);
				if (CollectionUtils.isNotEmpty(records)) {
					List<StatisticsAlbumActivityRecord> list = new ArrayList<StatisticsAlbumActivityRecord>();
					for(AlStudentActivityRecord record:records) {
						StatisticsAlbumActivityRecord insertRecord = new StatisticsAlbumActivityRecord();
						insertRecord.setId(idGenerator.generate());
						insertRecord.setAlbumId(record.getAlbumId());
						insertRecord.setAccountId(record.getAccountId());
						insertRecord.setRelationId(record.getRelationId());
						insertRecord.setSeconds(record.getSeconds());
						insertRecord.setFinished(record.getFinished());
						insertRecord.setFinishDate(record.getFinishDate());
						insertRecord.setSiteId(record.getSiteId());
						list.add(insertRecord);
					}
					Long t1=System.currentTimeMillis();
					try {
						albumInsertBatchUtil.insertBatchStatisticsAlbumRecorde(list);
					} catch (SQLException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					Long t2=System.currentTimeMillis();
					System.out.println("业务Id："+relationId+"。单独耗时(ms):"+(t2-t1)+"。总共时长"+(t2-ttt));
				}
			}
		}
	}

	@Override
	public List<ReportStudyAlbumChartsVo> chartGroup(String startDate, String endDate, Long companyId, Long siteId,Long adminId, Boolean isAdmin) {
		//验证下管辖区是否是全平台的
		boolean isAdmin1;
		if (!isAdmin) {
			isAdmin1 = statisticsAccountRoleService.getIsAdmin(15, siteId, adminId);
		} else {
			isAdmin1 = isAdmin;
		}
		List<ReportStudyAlbumChartsVo> list=this.baseMapper.chartGroup(startDate, endDate, companyId, siteId,adminId,isAdmin1);
		List<ReportStudyAlbumChartsVo> listAllRecord = new ArrayList<ReportStudyAlbumChartsVo>();
		Date startDate1, endDate1;
		ReportStudyAlbumChartsVo rsccvUpdate =null;
		int num = 0;
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
		try {
			startDate1 = dateFormat.parse(startDate);
			endDate1 = dateFormat.parse(endDate);
			num = (int) ((endDate1.getTime() - startDate1.getTime()) / (1000 * 3600 * 24));
			Date updateDate = startDate1;
			String updateDateValue = dateFormat.format(updateDate);
			Boolean isTrue = false;
			int j=0;
			for (int i = 0; i <= num; i++) {
				if (list != null&&list.size()>0) {
					if(j<list.size()) {
						ReportStudyAlbumChartsVo rsccv = list.get(j);
						if (rsccv.getValue().equals(updateDateValue)) {
							j=j+1;
							listAllRecord.add(rsccv);
							isTrue = true;
						}
						else {
							isTrue=false;
						}
					}
				}
				if (!isTrue) {
					rsccvUpdate = new ReportStudyAlbumChartsVo();
					rsccvUpdate.setValue(updateDateValue);
					rsccvUpdate.setTotalAccount(0);
					listAllRecord.add(rsccvUpdate);

				}
				Calendar c = Calendar.getInstance();
				c.setTime(updateDate);
				c.add(Calendar.DAY_OF_MONTH, 1); // 利用Calendar 实现 Date日期+1天
				updateDate = c.getTime();
				updateDateValue = dateFormat.format(updateDate);
				isTrue=false;
			}

		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return listAllRecord;
	}

	@Override
	public Page<ReportStudyAlbumVo> albumGroup(String kwd, String startDate, String endDate, Long companyId,
			Long siteId, Integer pageNo, Integer pageSize,Long adminId, Boolean isAdmin) {
		String kwdTrim=null;
		if(kwd!=null&&kwd.length()>0) {
			kwdTrim=kwd.trim();
			if(kwdTrim.length()==0) {
				kwdTrim=null;
			}
		}
		Page<Long> pageIds=new Page<Long>(pageNo,pageSize);
		StatisticsAlbum album = new StatisticsAlbum();
		album.setSiteId(siteId);
		album.setCompanyId(companyId);
		EntityWrapper<StatisticsAlbum> wrapper = new EntityWrapper<StatisticsAlbum>(album);
		wrapper.like("album_name", kwdTrim);
		wrapper.orderBy("album_create_time", false);
		List<StatisticsAlbum> list = statisticsAlbumMapper.selectPage(pageIds, wrapper);
		List<ReportStudyAlbumVo> listStatisticsAlbum=new ArrayList<ReportStudyAlbumVo>();
		if(CollectionUtils.isNotEmpty(list)) {
			//验证下管辖区是否是全平台的
			boolean isAdmin1;
			if (!isAdmin) {
				isAdmin1 = statisticsAccountRoleService.getIsAdmin(15, siteId, adminId);
			} else {
				isAdmin1 = isAdmin;
			}
				for (StatisticsAlbum albumVo : list) {
					//可参加人数
					ReportStudyAlbumVo vo = new ReportStudyAlbumVo();
					if(isAdmin1) {
						vo=statisticsAlbumActivityRecordMapper.totalInAccountIsAdmin(startDate, endDate, companyId, siteId, isAdmin, albumVo.getAlbumId());
					}else {
						vo = statisticsAlbumActivityRecordMapper.totalInAccountNotAdmin(startDate, endDate, companyId, siteId, isAdmin, albumVo.getAlbumId(), adminId);
					}
					//完成人数
					if (albumVo.getOvered().equals(1)&&albumVo.getLockNum().equals(0)) {
						vo.setFinish(statisticsAlbumActivityRecordMapper.finishNum(startDate, endDate, companyId, siteId, adminId, isAdmin, albumVo.getAlbumId()));
					}else {
						vo.setFinish(0);
					}
					vo.setAlbumId(albumVo.getAlbumId());
					vo.setAlbumName(albumVo.getAlbumName());
					vo.setOvered(albumVo.getOvered());
					vo.setDuration(albumVo.getDuration()/3600+":"+(albumVo.getDuration()%3600)/60+":"+albumVo.getDuration()%60);
					vo.setCommentNum(albumVo.getCommentCount());
					vo.setSubscribeNum(albumVo.getSubscribeCount());
					Long totalDuration = Long.parseLong(vo.getTotalDuration());
					vo.setTotalDuration(totalDuration/3600+":"+(totalDuration%3600)/60+":"+totalDuration%60);
					//平均学习时长
					if (totalDuration!=0) {
						vo.setAvgDuration(String.valueOf(totalDuration/vo.getTotalInAccount()));
					}else {
						vo.setAvgDuration("0");
					}
					listStatisticsAlbum.add(vo);
				}
		}
		Page<ReportStudyAlbumVo> page=new Page<ReportStudyAlbumVo>(pageNo,pageSize);
		if(pageIds!=null) {
			page.setTotal(pageIds.getTotal());
		}
		if(listStatisticsAlbum!=null) {
			page.setRecords(listStatisticsAlbum);
		}
		return page;
	}

	@Override
	public List<ReportStudyAlbumViewVo1> albumGroupView1(Long albumId, String startDate, String endDate, Long companyId,
			Long siteId, Long adminId, Boolean isAdmin) {
		//验证下管辖区是否是全平台的
		boolean isAdmin1;
		if (!isAdmin) {
			isAdmin1 = statisticsAccountRoleService.getIsAdmin(15, siteId, adminId);
		} else {
			isAdmin1 = isAdmin;
		}
		List<ReportStudyAlbumViewVo1> list = statisticsAlbumMapper.export1(startDate, endDate, companyId, siteId, adminId, albumId,isAdmin1);
		if (CollectionUtils.isNotEmpty(list)) {
			for(ReportStudyAlbumViewVo1 vo:list) {
				// 计算完成百分比
				Integer finished = statisticsAlbumActivityMapper.finished(endDate, endDate, albumId, vo.getAccountId(), siteId);
				Integer unLockNum = statisticsAlbumActivityMapper.unLockNum(albumId);
				if (unLockNum!=0) {
					BigDecimal res = new BigDecimal(0);
					BigDecimal dividendN = new BigDecimal(finished);
					BigDecimal divisorN = new BigDecimal(unLockNum);
					BigDecimal a = dividendN.multiply(new BigDecimal("100"));
					res = a.divide(divisorN, 2, BigDecimal.ROUND_HALF_UP);
					vo.setCompleteness(res.intValue());
				}else {
					vo.setCompleteness(0);
				}
				String totalDuration = vo.getTotalDuration();
				vo.setTotalDuration(Long.parseLong(totalDuration)/3600+":"+(Long.parseLong(totalDuration)%3600)/60+":"+Long.parseLong(totalDuration)%60);
			}
		}
		return list;
	}

	@Override
	public List<ReportStudyAlbumViewVo2> albumGroupView2(Long albumId, String startDate, String endDate, Long companyId,
			Long siteId, Long adminId, Boolean isAdmin) {
		//验证下管辖区是否是全平台的
		boolean isAdmin1;
		if (!isAdmin) {
			isAdmin1 = statisticsAccountRoleService.getIsAdmin(15, siteId, adminId);
		} else {
			isAdmin1 = isAdmin;
		}
		return statisticsAlbumMapper.export2(startDate, endDate, companyId, siteId, adminId, albumId,isAdmin1);
	}

	@Override
	public List<ReportStudyAlbumViewVo3> albumGroupView3(Long albumId, String startDate, String endDate, Long companyId,
			Long siteId, Long adminId, Boolean isAdmin) {
		//验证下管辖区是否是全平台的
		boolean isAdmin1;
		if (!isAdmin) {
			isAdmin1 = statisticsAccountRoleService.getIsAdmin(15, siteId, adminId);
		} else {
			isAdmin1 = isAdmin;
		}	
		return statisticsAlbumMapper.export3(startDate, endDate, companyId, siteId, adminId, albumId,isAdmin1);
	}

	@Override
	public Page<ReportStudyAlbumAccountVo> albumAccountGroup(String orgKwd, String accountKwd, String startDate,
			String endDate, Long companyId, Long siteId, Integer pageNo, Integer pageSize, Long adminId, Boolean isAdmin) {
		//验证下管辖区是否是全平台的
		boolean isAdmin1;
		if (!isAdmin) {
			isAdmin1 = statisticsAccountRoleService.getIsAdmin(15, siteId, adminId);
		} else {
			isAdmin1 = isAdmin;
		}
		Page<Long> pageAccountIds=new Page<Long>(pageNo,pageSize);
		List<Long> accountIds = statisticsAlbumMapper.accountIdsLimit(orgKwd, accountKwd, companyId, siteId, adminId,isAdmin1, pageAccountIds);
		Page<ReportStudyAlbumAccountVo> page=new Page<ReportStudyAlbumAccountVo>(pageNo,pageSize);
		if(CollectionUtils.isNotEmpty(accountIds)) {
			List<ReportStudyAlbumAccountVo> list=statisticsAlbumMapper.albumAccountGroup(startDate,endDate,companyId, siteId, accountIds);
			page.setRecords(list);
			page.setTotal(pageAccountIds.getTotal());
		}
		return page;
	}
	@Override
	public Page<ReportStudyAlbumAccountViewVo> albumAccountGroupView(Long companyId, Long accountId, Long siteId,
			String kwd, String startDate, String endDate, Integer pageNo, Integer pageSize, Long adminId, Boolean isAdmin) {
		Page<ReportStudyAlbumAccountViewVo> page=new Page<ReportStudyAlbumAccountViewVo>(pageNo,pageSize);
		StatisticsAccount account = new StatisticsAccount();
		account.setAccountId(accountId);
		account.setSiteId(siteId);
		account.setCompanyId(companyId);
		account = statisticsAccountMapper.selectOne(account);
		List<ReportStudyAlbumAccountViewVo> albumIds = statisticsAlbumMapper.albumIdsAccountGroupView(companyId, siteId, accountId, kwd,startDate,endDate, page);
		if (CollectionUtils.isNotEmpty(albumIds)) {
			for(ReportStudyAlbumAccountViewVo vo:albumIds) {
				// 计算完成百分比
				Integer finished = statisticsAlbumActivityMapper.finished(startDate, endDate, vo.getAlbumId(), accountId, siteId);
				Integer unLockNum = statisticsAlbumActivityMapper.unLockNum( vo.getAlbumId());
				if (unLockNum!=0) {
					BigDecimal res = new BigDecimal(0);
					BigDecimal dividendN = new BigDecimal(finished);
					BigDecimal divisorN = new BigDecimal(unLockNum);
					BigDecimal a = dividendN.multiply(new BigDecimal("100"));
					res = a.divide(divisorN, 2, BigDecimal.ROUND_HALF_UP);
					vo.setCompleteness(res.intValue());
				}else {
					vo.setCompleteness(0);
				}
				vo.setAccountId(accountId);
				vo.setAccountName(account.getAccountName());
				vo.setAccountFullName(account.getFullName());
				vo.setAccountState(account.getAccountState());
				vo.setEndTime(vo.getCompleteness()==100?vo.getEndTime():null);
				vo.setOrgName(account.getOrgName());
			}
			page.setRecords(albumIds);
			page.setTotal(page.getTotal());
		}
		return page;
		
	}

	@Override
	public List<ReportStudyAlbumViewVo2> albumGroupView(Long albumId, String startDate, String endDate, Long companyId,
			Long siteId, Long adminId, Boolean isAdmin) {
		//验证下管辖区是否是全平台的
		boolean isAdmin1;
		if (!isAdmin) {
			isAdmin1 = statisticsAccountRoleService.getIsAdmin(15, siteId, adminId);
		} else {
			isAdmin1 = isAdmin;
		}
		List<ReportStudyAlbumViewVo2> list=statisticsAlbumMapper.export2(startDate, endDate, companyId, siteId, adminId, albumId,isAdmin1);
		return list;
	}
}
