package com.yizhi.statistics.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.course.application.vo.CourseAccountReport;
import com.yizhi.statistics.application.entity.StatisticsCourseVisibleRange;
import com.yizhi.statistics.application.mapper.StatisticsCourseVisibleRangeMapper;
import com.yizhi.statistics.application.service.StatisticsCourseVisibleRangeService;
import com.yizhi.system.application.system.remote.OrganizationClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author yinyuyan123
 * @since 2019-03-20
 */
@Service
public class StatisticsCourseVisibleRangeServiceImpl extends ServiceImpl<StatisticsCourseVisibleRangeMapper, StatisticsCourseVisibleRange> implements StatisticsCourseVisibleRangeService {

	@Autowired
	IdGenerator idGenerator;
	
	@Autowired
	StatisticsCourseVisibleRangeMapper statisticsCourseVisibleRangeMapper;
	
	@Autowired
	OrganizationClient organizationClient;


	@Override
	public Long getUserCanJoinExamCountByDate(Long companyId, Long siteId, Long orgId,Date startAt, Date endAt,Long userId) {
		return statisticsCourseVisibleRangeMapper.getUserVisibleCountByDate(companyId,siteId,startAt,endAt,userId) +
				statisticsCourseVisibleRangeMapper.getOrgVisibleCountByDate(companyId,siteId,orgId,startAt,endAt);
	}

	@Override
	public List<StatisticsCourseVisibleRange> getStatisticsCourseVisibleRangeByCourseIdAndIsVirtual(Long workId,Integer type,Integer newScope) {
		// TODO Auto-generated method stub
		StatisticsCourseVisibleRange statisticsCourseVisibleRange=new StatisticsCourseVisibleRange();
		statisticsCourseVisibleRange.setCourseId(workId);
		statisticsCourseVisibleRange.setType(type);
		
		
		//statisticsCourseVisibleRange.setIsVirtual(newScope);
		EntityWrapper<StatisticsCourseVisibleRange> wrapper=new EntityWrapper<StatisticsCourseVisibleRange>(statisticsCourseVisibleRange);
		
		if(newScope==1) {
			wrapper.eq("is_virtual", 1);
		}
		if(newScope==2) {
			//如果是可见范围 is_virtual=2组织，is_virtual=3账号 都属于可见范围  因此暂时采用gt(>)
			wrapper.gt("is_virtual", 1);
		}
		return this.selectList(wrapper);
	}

	@Override
	public Boolean updateVisibleRange(Integer newScope, Integer oldScope, Long courseId,Date curDate,String cachKey) {
		// TODO Auto-generated method stub
		
		StatisticsCourseVisibleRange scvr=new StatisticsCourseVisibleRange();
		StatisticsCourseVisibleRange wrapperEntity=new StatisticsCourseVisibleRange();
		wrapperEntity.setCourseId(courseId);
		EntityWrapper<StatisticsCourseVisibleRange> wrapper=new EntityWrapper<StatisticsCourseVisibleRange>(wrapperEntity);
		//指定范围更改为全平台
		if(newScope==1&&oldScope==2) {
			//需要更改的值
			scvr.setIsVirtual(1);
		}
		//全平台更改为指定范围
		if(newScope==2&&oldScope==1) {
			//需要更改的值
			scvr.setIsVirtual(2);
		}
		//scvr.setRelationId(cachKey);
		scvr.setCreateTime(curDate);
		return this.update(scvr, wrapper);
	}

	@Override
	public List<StatisticsCourseVisibleRange> insertVisibleRange(Long companyId, Long courseId, Integer scope, Integer type, Date curDate, Long siteId, CourseAccountReport orgIdsAndAccountIds) {
		// TODO Auto-generated method stub
		List<StatisticsCourseVisibleRange> listStatisticsCourseVisibleRangeAccountds=new ArrayList<StatisticsCourseVisibleRange>();
		
		//如果是全平台的，直接插入一条记录
		if(scope==1) {
			StatisticsCourseVisibleRange scvr=new StatisticsCourseVisibleRange();
			scvr.setId(idGenerator.generate());
			scvr.setCompanyId(companyId);
			scvr.setSiteId(siteId);
			scvr.setCourseId(courseId);
			scvr.setRelationId(siteId);
			scvr.setType(type);
			scvr.setIsVirtual(1);
			scvr.setCreateTime(curDate);
			listStatisticsCourseVisibleRangeAccountds.add(scvr);
			//通过SQL脚本批量插入
			//return this.insert(scvr);
		}
		//如果是指定范围的，将指定范围进行插入
		if(scope==2) {
			
			if(orgIdsAndAccountIds!=null) {
				
				List<Long> orgIds=orgIdsAndAccountIds.getOrgIds();
				List<Long> accountIds=orgIdsAndAccountIds.getAccountIds();
				StatisticsCourseVisibleRange scvr=null;
				//循环个人的
				if(CollectionUtils.isNotEmpty(accountIds)) {
					for (Long accountId:accountIds) {
						scvr=new StatisticsCourseVisibleRange();
						scvr.setId(idGenerator.generate());
						scvr.setCompanyId(companyId);
						scvr.setSiteId(siteId);
						scvr.setCourseId(courseId);
						scvr.setRelationId(accountId);
						scvr.setType(type);
						scvr.setIsVirtual(3);
						scvr.setCreateTime(curDate);
						listStatisticsCourseVisibleRangeAccountds.add(scvr);
					}
				}
				
				//循环组织的 
				if(CollectionUtils.isNotEmpty(orgIds)) {
					Set<Long> setTempt=new HashSet<Long>();
					Set<Long>childIds=null;
					for (Long curOrgId : orgIds) {
						childIds=organizationClient.getChildIds(curOrgId);
						if(CollectionUtils.isNotEmpty(childIds)) {
							setTempt.addAll(childIds);
						}
					}
					
					
					for (Long accountId:setTempt) {
						scvr=new StatisticsCourseVisibleRange();
						scvr.setId(idGenerator.generate());
						scvr.setCompanyId(companyId);
						scvr.setSiteId(siteId);
						scvr.setCourseId(courseId);
						scvr.setRelationId(accountId);
						scvr.setType(type);
						scvr.setIsVirtual(2);
						scvr.setCreateTime(curDate);
						listStatisticsCourseVisibleRangeAccountds.add(scvr);
					}
				}
				
				//通过SQL脚本批量插入
				/*if(CollectionUtils.isNotEmpty(listStatisticsCourseVisibleRangeAccountds)) {
					 return this.insertBatch(listStatisticsCourseVisibleRangeAccountds);
				}*/
			}
		}
		return listStatisticsCourseVisibleRangeAccountds;
	}
	
	
	
	
	
	
	
	
	@Override
	public StatisticsCourseVisibleRange getgetStatisticsCourseVisibleRange(Long courseId) {
		// TODO Auto-generated method stub
		StatisticsCourseVisibleRange sc=new StatisticsCourseVisibleRange();
		sc.setCourseId(courseId);
		sc.setIsVirtual(1);
		EntityWrapper<StatisticsCourseVisibleRange> wrapper=new EntityWrapper<StatisticsCourseVisibleRange>(sc);
		return this.selectOne(wrapper);
	}

	@Override
	public void deleteAllStatisticsCourseVisibleRange() {
		statisticsCourseVisibleRangeMapper.deleteAllStatisticsCourseVisibleRange();
	}
	
	
}
