package com.yizhi.statistics.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.exam.application.vo.report.*;
import com.yizhi.statistics.application.domain.*;
import com.yizhi.statistics.application.entity.StatisticsCourseVisibleRange;
import com.yizhi.statistics.application.exam.domain.po.ExamGroupPO;
import com.yizhi.statistics.application.exam.domain.po.UserExamGroupPO;
import com.yizhi.statistics.application.exam.domain.vo.ExamGroupOrgVO;
import com.yizhi.statistics.application.exam.domain.vo.UserExamByOrgVO;
import com.yizhi.statistics.application.exam.domain.vo.UserExamDetailOrgVO;
import com.yizhi.statistics.application.mapper.StatisticsAccountRoleMapper;
import com.yizhi.statistics.application.mapper.StatisticsExamMapper;
import com.yizhi.statistics.application.refactor.exam.ExamCountByUserVO;
import com.yizhi.statistics.application.refactor.exam.StatisticsExamVO;
import com.yizhi.statistics.application.service.IStatisticsAccountRoleService;
import com.yizhi.statistics.application.service.StatisticsAccountService;
import com.yizhi.statistics.application.service.StatisticsCourseVisibleRangeService;
import com.yizhi.statistics.application.service.StatisticsExamService;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.statistics.application.util.WmyDateUtils;
import com.yizhi.statistics.application.vo.report.AccountVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author ding
 * @since 2019-04-17
 */
@Service
public class StatisticsExamServiceImpl extends ServiceImpl<StatisticsExamMapper, StatisticsExam> implements StatisticsExamService {

    private static final Logger LOGGER = LoggerFactory.getLogger(StatisticsExamServiceImpl.class);

    @Autowired
    private StatisticsExamMapper statisticsExamMapper;
    @Autowired
    private StatisticsAccountRoleMapper statisticsAccountRoleMapper;
    @Autowired
    private JedisUtil jedisUtil;
    @Autowired
    StatisticsCourseVisibleRangeService statisticsCourseVisibleRangeService;
    @Autowired
    IStatisticsAccountRoleService statisticsAccountRoleService;
    @Autowired
    StatisticsAccountService statisticsAccountService;
    @Autowired
    StatisticsAccountService statisticsAccountServiceImpl;
    @Autowired
    IStatisticsAccountRoleService iStatisticsAccountRoleService;

    @Override
    public List<UserExamDetailOrgVO> getExamUserDetail(UserExamGroupPO po) {

        List<UserExamDetailOrgVO> userList = statisticsExamMapper.getUserDetail(
                po.getCompanyId(),
                po.getSiteId(),
                po.getOrgId(),
                po.getUserName(),
                po.isAdmin(),
                po.getAccountId(),
                WmyDateUtils.stringToDate(po.getStartAt()),
                WmyDateUtils.stringToDate(po.getEndAt()));

        List<UserExamDetailOrgVO> examDetail = getUserExam(po, userList);
        Map<Long, UserExamDetailOrgVO> userInfoMap = userList.stream().collect(
                Collectors.toMap(UserExamDetailOrgVO::getAccountId, (p) -> p));

        for (UserExamDetailOrgVO vo : examDetail) {
            UserExamDetailOrgVO userInfo = userInfoMap.get(vo.getAccountId());
            if (userInfo != null) {
                BeanUtils.copyProperties(vo, userInfo);
            }
            switch (vo.getExamState()) {
//                case 1 : // 走默认值
                case 0:
                case 2: {
                    vo.setExamResult("未通过");
                }
                break;
                case -1: {
                    vo.setExamResult("");
                    vo.setExamStatus("未参加");
                }
                break;
            }
        }
        return examDetail;
    }

    /**
     * 获取用户的考试信息
     *
     * @param po
     * @param userList
     * @return
     */
    private List<UserExamDetailOrgVO> getUserExam(UserExamGroupPO po, List<UserExamDetailOrgVO> userList) {

        List<Long> userIds = userList.stream().map(u -> u.getAccountId()).collect(Collectors.toList());

        List<UserExamDetailOrgVO> examList = statisticsExamMapper.getExamUserDetail(
                po.getCompanyId(),
                po.getSiteId(),
                po.getOrgId(),
                WmyDateUtils.stringToDate(po.getStartAt()),
                WmyDateUtils.stringToDate(po.getEndAt()),
                userIds);


        List<ExamDto> userUnJoinExam = new ArrayList();
        for (Long userId : userIds) {
            List<ExamDto> list = statisticsExamMapper.getUserCanJoinExam(
                    po.getCompanyId(),
                    po.getSiteId(),
                    po.getOrgId(),
                    userId,
                    WmyDateUtils.stringToDate(po.getStartAt()),
                    WmyDateUtils.stringToDate(po.getEndAt())
            );
            userUnJoinExam.addAll(list);
        }
        for (ExamDto examDto : userUnJoinExam) {
            UserExamDetailOrgVO vo = new UserExamDetailOrgVO();
            BeanUtils.copyProperties(examDto, vo);
            vo.setOrgName(po.getOrgName());
            vo.setExamState(-1);
            examList.add(vo);
        }

        examList.sort(Comparator.comparing(UserExamDetailOrgVO::getName));
        return examList;
    }

    @Override
    public Page<UserExamByOrgVO> getUserExam(UserExamGroupPO po) {
        Page<UserExamByOrgVO> page = new Page(po.getPageIndex(), po.getPageSize());
        Long count = statisticsAccountService.getUserCountByNameLike(po);
        page.setTotal(count.intValue());
        List<AccountVO> userList = statisticsAccountService.getUserIdByNameLike(po);

        if (userList.isEmpty()) return page;

        List<UserExamByOrgVO> list = new ArrayList();
        for (AccountVO it : userList) {
            UserExamByOrgVO vo = new UserExamByOrgVO();
            BeanUtils.copyProperties(it, vo);
            vo.setUserName(it.getName());
            list.add(vo);
        }

        Map<Long, AccountVO> userIds = new HashMap<>();
        for (AccountVO accountVO : userList) {
            userIds.put(accountVO.getAccountId(), accountVO);
        }

        // 可见范围为站点的考试数量
        Long siteVisible = statisticsExamMapper.getExamCountBySitNoAcount(
                po.getCompanyId(),
                po.getSiteId(),
                WmyDateUtils.stringToDate(po.getStartAt()),
                WmyDateUtils.stringToDate(po.getEndAt())
        );
        if (siteVisible == null) {
            siteVisible = 0L;
        }

        Map<Long, Long> userPassCount = getUserPassExamCount(po, new ArrayList(userIds.keySet()));
        Map<Long, Long> actualJoinCount = getActualJoinCount(po, new ArrayList(userIds.keySet()));
        Map<Long, Integer> examDuration = getExamDuration(po, new ArrayList(userIds.keySet()));
        for (UserExamByOrgVO it : list) {
            Long accountId = it.getAccountId();
            Long passCount = userPassCount.get(accountId);
            if (passCount != null) {
                it.setPassCount(passCount);
            }
            Long joinCount = actualJoinCount.get(accountId);
            if (joinCount != null) {
                it.setActualJoinCount(joinCount);
            }
            Integer duration = examDuration.get(accountId);
            if (duration != null) {
                it.setDuration(duration);
            }
            it.setUnPassCount(it.getActualJoinCount() - it.getPassCount());
            Long orgOrUserVisibleExamCount = statisticsCourseVisibleRangeService.getUserCanJoinExamCountByDate(
                    po.getCompanyId(),
                    po.getSiteId(),
                    po.getOrgId(),
                    WmyDateUtils.stringToDate(po.getStartAt()),
                    WmyDateUtils.stringToDate(po.getEndAt()),
                    accountId);

            if (orgOrUserVisibleExamCount != null && orgOrUserVisibleExamCount > 0) {
                siteVisible = siteVisible + orgOrUserVisibleExamCount;
            }
            it.setPlanJoinCount(siteVisible);
            AccountVO vo = userIds.get(accountId);
            if (vo != null) {
                it.setUserName(vo.getName());
                it.setOrgNames(vo.getOrgNames());
                it.setOrgName(vo.getOrgName());
                it.setOrgCode(vo.getOrgCode());
            }
        }
        page.setRecords(list);
        return page;
    }

    private Map<Long, Long> getActualJoinCount(UserExamGroupPO po, List<Long> ids) {
        List<UserExamByOrgVO> data = statisticsExamMapper.getUserExam(
                po.getCompanyId(),
                po.getSiteId(),
                po.getOrgId(),
                ids,
                WmyDateUtils.stringToDate(po.getStartAt()),
                WmyDateUtils.stringToDate(po.getEndAt()));

        Map<Long, Long> map = new HashMap();
        for (UserExamByOrgVO it : data) {
            map.put(it.getAccountId(), it.getActualJoinCount());
        }
        return map;
    }

    private Map<Long, Integer> getExamDuration(UserExamGroupPO po, List<Long> ids) {
        List<UserExamByOrgVO> data = statisticsExamMapper.getExamDuration(
                po.getCompanyId(),
                po.getSiteId(),
                po.getOrgId(),
                ids,
                WmyDateUtils.stringToDate(po.getStartAt()),
                WmyDateUtils.stringToDate(po.getEndAt()));

        Map<Long, Integer> map = new HashMap();
        for (UserExamByOrgVO it : data) {
            map.put(it.getAccountId(), it.getDuration());
        }
        return map;
    }

    private Map<Long, Long> getUserPassExamCount(UserExamGroupPO po, List<Long> ids) {
        List<UserExamByOrgVO> list = statisticsExamMapper.getUserPassExamCount(po.getCompanyId(),
                po.getSiteId(),
                po.getOrgId(),
                ids,
                WmyDateUtils.stringToDate(po.getStartAt()),
                WmyDateUtils.stringToDate(po.getEndAt()));
        Map<Long, Long> map = new HashMap();
        for (UserExamByOrgVO userExamByOrgVO : list) {
            map.put(userExamByOrgVO.getAccountId(), userExamByOrgVO.getPassCount().longValue());
        }
        return map;
    }

    @Override
    public Page<ExamGroupOrgVO> getExamGroupByOrg(ExamGroupPO po) {

        Page page = new Page(po.getPageIndex(), po.getPageSize());
        // 查询时间范围内的考试记录

        Long count = statisticsAccountServiceImpl.getOrgCount(po.getCompanyId(), po.getSiteId(), po.getOrgNameLike());
        // 部门数量为0,直接返回
        if (count <= 0) {
            return page;
        }
        page.setTotal(count.intValue());

        List<ExamGroupOrgDto> orgs = getAccountOrginfo(po);

        List<ExamGroupOrgVO> vos = new ArrayList();
        // 考试数据封装
        orgs = getPageByOrgs(orgs, po);
        for (ExamGroupOrgDto org : orgs) {
            ExamGroupOrgVO vo = new ExamGroupOrgVO();
            BeanUtils.copyProperties(org, vo);
            vos.add(vo);
        }
        page.setRecords(vos);

        return page;
    }

    /**
     * @param orgs
     * @param po
     * @return
     */
    private List<ExamGroupOrgDto> getPageByOrgs(List<ExamGroupOrgDto> orgs, ExamGroupPO po) {
        po.setSiteId(po.getSiteId());
        po.setCompanyId(po.getCompanyId());

        Map<Long, Long> orgPassCount;
        Map<Long, Long> orgUserCount;
        Map<Long, Long> actualJoinCount = null;
        Map<Long, Long> canJoinExamCount;

        // 通过考试人数
        orgPassCount = getPassOrNoExamCountGroupByOr(po, 1);

        // 可参加考试人数 有考试为全员可见,则不考虑考试可见范围。
        // 所有考试均为指定用户可见，则可见人数为可见范围与考试指定人数的交集。
        // 可参加人数

        List<Long> orgIdList = orgs.stream().map(ExamGroupOrgDto::getOrgId).collect(Collectors.toList());
        orgUserCount = getUserCountByOrg(po, orgIdList);

        // 可参加考试数
        canJoinExamCount = getOrgExamCountByAccount(po, orgIdList);

        // 非管理员的实际参加开始人数查询
        // 管理员的实际参加人数可已直接查询
        // 实际参加考试数
        actualJoinCount = getPassOrNoExamCountGroupByOr(po, null);

        // 组装数据
        for (ExamGroupOrgDto org : orgs) {
            Long orgId = org.getOrgId();
            org.setPlanUserCount(orgUserCount.get(orgId));
            org.setPassUserCount(orgPassCount.get(orgId));
            org.setActualUserCount(actualJoinCount.get(orgId));
            org.setExamCount(canJoinExamCount.get(orgId));
            if (org.getActualUserCount() != null && org.getActualUserCount() > 0 && org.getPassUserCount() != null) {
                org.setPassRate((org.getPassUserCount() * 100) / org.getActualUserCount());
            }

            if (org.getPlanUserCount() == null) {
                org.setPlanUserCount(0L);
            }
            if (org.getActualUserCount() == null) {
                org.setActualUserCount(0L);
            }
            if (org.getExamCount() == null) {
                org.setExamCount(0L);
            }
            if (org.getPassUserCount() == null) {
                org.setPassUserCount(0L);
            }
            if (org.getPassRate() == null) {
                org.setPassRate(0L);
            }
        }
        return orgs;
    }

    /**
     * 获取可以参加的考试数
     *
     * @param po
     * @return
     */
    private Map<Long, Long> getOrgExamCountByAccount(ExamGroupPO po, List<Long> orgIdList) {

        Map<Long, Long> orgVisible = getOrgVisibleExamByDate(po, orgIdList);
        List<OrgCountDto> orgList = statisticsAccountService.getRoleOrgCount(po.getSiteId(), po.getAccountId());
        Map<Long, Long> map = new HashMap();
        if (!po.isAdmin()) {
            for (Long orgId : orgIdList) {
                Long count = statisticsAccountService.getVisibleAccountByNoAdmin(
                        po.getCompanyId(),
                        po.getSiteId(),
                        orgId,
                        po.getAccountId(),
                        WmyDateUtils.stringToDate(po.getStartAt()),
                        WmyDateUtils.stringToDate(po.getEndAt()));
                Long visibleCount = orgVisible.get(orgId);
                if (visibleCount != null) {
                    count = count + visibleCount;
                }
                map.put(orgId, count);
            }
            return map;
        }

        //人可见的考试数量  还需加上站点或部门可见的考试数
        List<OrgCountDto> list = statisticsExamMapper.getAccountcVisibleExamCountByOrg(
                po.getCompanyId(),
                po.getSiteId(),
                WmyDateUtils.stringToDate(po.getStartAt()),
                WmyDateUtils.stringToDate(po.getEndAt())
        );
        Map<Long, OrgCountDto> usersVisibleCount = list.stream().collect(Collectors.toMap(OrgCountDto::getOrgId, p -> (p)));

        for (Long orgId : orgIdList) {
            Long count = 0L;
            OrgCountDto userVisibleCount = usersVisibleCount.get(orgId);
            Long orgVisibleCount = orgVisible.get(orgId);
            if (userVisibleCount != null) {
                count = count + userVisibleCount.getCount();
            }
            if (orgVisibleCount != null) {
                count = count + orgVisibleCount;
            }

            map.put(orgId, count);
        }
        return map;
    }

    /**
     * 获取部门可参加考试的用户数
     *
     * @param po
     * @return
     */
    private Map<Long, Long> getUserCountByOrg(ExamGroupPO po, List<Long> orgIdList) {
        List<OrgCountDto> list;
        if (po.isAdmin()) {
            list = statisticsAccountService.getUserCountByOrgIdAndSiteIdAndCompanyId(po.getCompanyId(), po.getSiteId());
            HashMap<Long, Long> orgPassCount = new HashMap<Long, Long>();
            Map<Long, Long> orgExamAbleUnAccountMap = getOrgVisibleExamByDate(po, orgIdList);

            for (OrgCountDto it : list) {
                Long orgId = it.getOrgId();
                Long canJoinExamUserCount = it.getCount();
                Long count = orgExamAbleUnAccountMap.get(orgId);
                if (count == null || count < 1) {
                    canJoinExamUserCount = getCanJoinExamUser(po, orgId).longValue();
                }
                orgPassCount.put(orgId, canJoinExamUserCount);
            }
            return orgPassCount;
        }

        // 分组可见范围
        Map<Long, Long> orgVisible = getOrgVisibleExamByDate(po, orgIdList);

        List<OrgCountDto> orgCount = iStatisticsAccountRoleService.getAccountByAdminId(
                po.getAccountId(),
                po.getSiteId()
        );

        HashMap<Long, Long> canJoinUserCount = new HashMap<Long, Long>();
        for (OrgCountDto it : orgCount) {
            Long orgId = it.getOrgId();
            Long visibleCount = orgVisible.get(orgId);
            if (visibleCount != null && visibleCount > 0) {
                canJoinUserCount.put(orgId, it.getCount());
            } else {
                Long count = statisticsExamMapper.getRoleJoinExamCount(
                        po.getCompanyId(),
                        po.getSiteId(),
                        orgId,
                        po.getAccountId(),
                        WmyDateUtils.stringToDate(po.getStartAt()),
                        WmyDateUtils.stringToDate(po.getEndAt())
                );
                canJoinUserCount.put(orgId, count);
            }
        }

        return canJoinUserCount;

    }

    /**
     * 求用户的可见范围 与 可参加考试用户的交集
     *
     * @param po
     * @return
     */
    private Integer getCanJoinExamUser(ExamGroupPO po, Long orgId) {
        List<Long> examVisible = statisticsAccountRoleMapper.getVisibleAccount(
                po.getCompanyId(),
                po.getSiteId(),
                orgId,
                WmyDateUtils.stringToDate(po.getStartAt()),
                WmyDateUtils.stringToDate(po.getEndAt()));
        LOGGER.info("查询部门用户考试的可见范围" + po.isAdmin() + examVisible.size());
        if (po.isAdmin()) {
            return examVisible.size();
        }
        List<Long> managerUser = statisticsAccountService.getUserByManager(
                po.getAccountId(),
                po.getSiteId(),
                orgId);

        managerUser.retainAll(examVisible);
        return managerUser.size();
    }

    /**
     * 获取站点下的部门信息
     *
     * @param po
     * @return
     */
    private List<ExamGroupOrgDto> getAccountOrginfo(ExamGroupPO po) {
        List<ExamGroupOrgDto> list = statisticsAccountService.getOrgByCompanyAndSiteId(
                po.getCompanyId(),
                po.getSiteId(),
                po.getPageIndex().longValue(),
                po.getPageSize().longValue(),
                po.getOrgNameLike());
        return list;
    }

    /**
     * 查询时间段内 部门下 '可见范围不为指定用户'可见的考试数量
     *
     * @param po
     * @return
     */
    private Map<Long, Long> getOrgVisibleExamByDate(ExamGroupPO po, List<Long> orgIds) {
        // 部门考试数
        List<OrgCountDto> list = statisticsExamMapper.getExamCountByOrgNoAcount(
                po.getCompanyId(),
                po.getSiteId(),
                WmyDateUtils.stringToDate(po.getStartAt()),
                WmyDateUtils.stringToDate(po.getEndAt())
        );
        Map<Long, OrgCountDto> map = list.stream().collect(Collectors.toMap(OrgCountDto::getOrgId, (u) -> u));

        // 站点考试数
        Long siteExamCount = statisticsExamMapper.getExamCountBySitNoAcount(
                po.getCompanyId(),
                po.getSiteId(),
                WmyDateUtils.stringToDate(po.getStartAt()),
                WmyDateUtils.stringToDate(po.getEndAt())
        );
        HashMap<Long, Long> orgExamCount = new HashMap<Long, Long>();

        for (Long orgId : orgIds) {
            Long count = siteExamCount;
            OrgCountDto orgCount = map.get(orgId);
            if (orgCount != null && orgCount.getCount() != null && orgCount.getCount() > 0) {
                count = orgCount.getCount();
            }
            orgExamCount.put(orgId, count);
        }
        return orgExamCount;
    }

    /**
     * 按部门统计通过考试的用户数
     *
     * @param po
     * @param examRresult 1:考试通过,0：参加考试
     * @return
     */
    private Map<Long, Long> getPassOrNoExamCountGroupByOr(ExamGroupPO po, Integer examRresult) {
        List<ExamPassCountGroupOrgDto> list;
        if (po.isAdmin()) {
            list = statisticsExamMapper.getPassExamCountGroupByOrg(
                    po.getCompanyId(),
                    po.getSiteId(),
                    WmyDateUtils.stringToDate(po.getStartAt()),
                    WmyDateUtils.stringToDate(po.getEndAt()),
                    examRresult);
        } else {
            list = statisticsExamMapper.getPassExamCountGroupByOrgNoAdmin(
                    po.getCompanyId(),
                    po.getSiteId(),
                    WmyDateUtils.stringToDate(po.getStartAt()),
                    WmyDateUtils.stringToDate(po.getEndAt()),
                    po.getAccountId(),
                    examRresult);
        }

        HashMap<Long, Long> orgPassCount = new HashMap<Long, Long>();
        for (ExamPassCountGroupOrgDto it : list) {
            orgPassCount.put(it.getOrgId(), it.getCount());
        }
        return orgPassCount;
    }

    private Boolean getIsAdmin(ExamGroupPO po) {
        return statisticsAccountRoleService.getIsAdmin(1, po.getSiteId(), po.getAccountId());
    }

    /**
     * 按考试统计
     *
     * @param reportExamParam
     * @return
     */
    @Override
    public Page<com.yizhi.statistics.application.refactor.exam.ReportExamVO> getExamGroup(com.yizhi.statistics.application.refactor.exam.ReportExamParam reportExamParam) {
        Page<com.yizhi.statistics.application.refactor.exam.ReportExamVO> page = new Page<>(reportExamParam.getPageNo(), reportExamParam.getPageSize());
        Page<Long> examPage = new Page<>(reportExamParam.getPageNo(), reportExamParam.getPageSize());
        List<com.yizhi.statistics.application.refactor.exam.ReportExamVO> list = null;

        Integer total = 0;
        //验证下管辖区是否是全平台的
        boolean isAdmin;

        ReportExamParam param = new ReportExamParam();
        BeanUtils.copyProperties(reportExamParam, param);
        if (!reportExamParam.isAdmin()) {
            isAdmin = statisticsAccountRoleService.getIsAdmin(1, reportExamParam.getSiteId(), reportExamParam.getAccountId());
        } else {
            isAdmin = reportExamParam.isAdmin();
        }
        if (isAdmin) {

            List<Long> idList = statisticsExamMapper.getExamGroupCount(reportExamParam.getCompanyId(), reportExamParam.getSiteId(),
                    reportExamParam.getExamNameLike(), reportExamParam.getStartTime(), reportExamParam.getEndTime(), examPage);
            if (CollectionUtils.isEmpty(idList)) {
                return page;
            }
            list = statisticsExamMapper.getExamGroupAdmin(param, idList);
            total = examPage.getTotal();
        } else {
            List<Long> idList = statisticsExamMapper.getExamGroupCount(reportExamParam.getCompanyId(), reportExamParam.getSiteId(),
                    reportExamParam.getExamNameLike(), reportExamParam.getStartTime(), reportExamParam.getEndTime(), examPage);
            if (CollectionUtils.isEmpty(idList)) {
                return page;
            }

            list = statisticsExamMapper.getExamGroupNoAdmin(param, idList);
            total = examPage.getTotal();
        }

        if (CollectionUtils.isNotEmpty(list)) {
            for (com.yizhi.statistics.application.refactor.exam.ReportExamVO reportExamVO : list) {
                Integer askNum = jedisUtil.getAskNumByAdminIdAndWorkId(reportExamParam.getAccountId(), reportExamParam.getSiteId(), reportExamParam.isAdmin(), reportExamVO.getId());
                BigDecimal complete = divide(reportExamVO.getTotalIn(), askNum);
                reportExamVO.setComplete(complete);
                reportExamVO.setTotalAsk(askNum);
            }
            page.setRecords(list);
            page.setTotal(total);
        }
        return page;
    }

    @Override
    public Page<com.yizhi.statistics.application.refactor.exam.ReportExamAccountsViewVo> getExamAndAccountGroup(com.yizhi.statistics.application.refactor.exam.ExamCountByUserVO vo) {
        Page<com.yizhi.statistics.application.refactor.exam.ReportExamAccountsViewVo> page = new Page<>(vo.getPageNo(), vo.getPageSize());

        //查找是否是全平台可见
        boolean isAllVisible = false;
        StatisticsCourseVisibleRange statisticsCourseVisibleRange = new StatisticsCourseVisibleRange();
        statisticsCourseVisibleRange.setCourseId(vo.getExamId());
        statisticsCourseVisibleRange.setType(1);
        statisticsCourseVisibleRange.setIsVirtual(1);
        List<StatisticsCourseVisibleRange> rangeList =
                statisticsCourseVisibleRangeService.selectList(new EntityWrapper<>(statisticsCourseVisibleRange));
        if (CollectionUtils.isNotEmpty(rangeList)) {
            isAllVisible = true;
        }
        List<com.yizhi.statistics.application.refactor.exam.ReportExamAccountsViewVo> list = null;
        //验证下管辖区是否是全平台的
        boolean isAdmin;
        if (!vo.isAdmin()) {
            isAdmin = statisticsAccountRoleService.getIsAdmin(1, vo.getSiteId(), vo.getAccountId());
        } else {
            isAdmin = vo.isAdmin();
        }
        if (isAdmin) {
            list = statisticsExamMapper.getExamAndAccountGroupAdmin(vo, isAllVisible, page);
        } else {
            list = statisticsExamMapper.getExamAndAccountGroupNoAdmin(vo, isAllVisible, page);
        }
        if (CollectionUtils.isNotEmpty(list)) {
            page.setRecords(list);
        }
        return page;
    }

    @Override
    public Page<com.yizhi.statistics.application.refactor.exam.ReportExamAccountVO> getAccountGroup(ExamCountByUserVO vo) {
        Page<com.yizhi.statistics.application.refactor.exam.ReportExamAccountVO> page = new Page<>(vo.getPageNo(), vo.getPageSize());
        Page<Long> accountPage = new Page<>(vo.getPageNo(), vo.getPageSize());

        //获取全平台可见的考试数
        Integer num = statisticsExamMapper.getAllVibleNum(vo.getCompanyId(), vo.getSiteId(), vo.getStartTime(), vo.getEndTime());

        List<Long> examIds = statisticsExamMapper.getExamIds(vo.getCompanyId(), vo.getSiteId(), vo.getStartTime(), vo.getEndTime());

        //验证下管辖区是否是全平台的
        boolean isAdmin;
        if (!vo.isAdmin()) {
            isAdmin = statisticsAccountRoleService.getIsAdmin(1, vo.getSiteId(), vo.getAccountId());
            vo.setAdmin(isAdmin);
        }

        List<Long> accountIds = null;
        if (vo.getPageSize() != Integer.MAX_VALUE) {
            //查询每次分面id列表
            accountIds = statisticsExamMapper.getAccountIds(vo, accountPage);

            if (CollectionUtils.isEmpty(accountIds)) {
                return page;
            }
        }


        List<com.yizhi.statistics.application.refactor.exam.ReportExamAccountVO> list = statisticsExamMapper.getAccountGroup(vo, accountIds, examIds);
        if (CollectionUtils.isNotEmpty(list)) {
            for (com.yizhi.statistics.application.refactor.exam.ReportExamAccountVO reportExamAccountVO : list) {
                reportExamAccountVO.setTotalAsk(reportExamAccountVO.getTotalAsk() + num);
            }
            page.setRecords(list);
            page.setTotal(accountPage.getTotal());
        }
        return page;
    }

    @Override
    public Page<com.yizhi.statistics.application.refactor.exam.ReportExamAccountViewVo> getAccountGroupView(Long companyId, Long siteId, Long accountId, String startTime, String endTime, String examNameLike, Integer pageNo, Integer pageSize) {

        if (null == companyId || null == siteId) {
            companyId = ContextHolder.get().getCompanyId();
            siteId = ContextHolder.get().getSiteId();
        }
        Page<com.yizhi.statistics.application.refactor.exam.ReportExamAccountViewVo> page = new Page<>(pageNo, pageSize);
        List<com.yizhi.statistics.application.refactor.exam.ReportExamAccountViewVo> list = statisticsExamMapper.getAccountGroupView(companyId, siteId, accountId, examNameLike, startTime, endTime, page);
        if (CollectionUtils.isNotEmpty(list)) {
            page.setRecords(list);
        }
        return page;
    }

    @Override
    public StatisticsExamVO getExam(Long examId) {
        return statisticsExamMapper.getExam(examId);
    }

    public BigDecimal divide(Integer totalIn, Integer totalAsk) {
        if (totalIn == 0 || totalAsk == 0) {
            return BigDecimal.valueOf(0.00);
        } else {
            return BigDecimal.valueOf(totalIn).multiply(BigDecimal.valueOf(100)).divide(BigDecimal.valueOf(totalAsk), 2, BigDecimal.ROUND_HALF_UP);
        }
    }
}
