package com.yizhi.statistics.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.statistics.application.entity.StatisticsResearchLearnMetadata;
import com.yizhi.statistics.application.mapper.StatisticsResearchLearnMetadataMapper;
import com.yizhi.statistics.application.service.IStatisticsAccountRoleService;
import com.yizhi.statistics.application.service.StatisticsResearchLearnMetadataService;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.statistics.application.util.WorkUtil;
import com.yizhi.statistics.application.vo.research.ResearchGroupViewVo;
import com.yizhi.statistics.application.vo.research.ResearchGroupVo;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 学员调研明细 服务实现类
 * </p>
 *
 * @author hutao123
 * @since 2019-04-18
 */
@Service
@Log4j2
public class StatisticsResearchLearnMetadataServiceImple extends ServiceImpl<StatisticsResearchLearnMetadataMapper, StatisticsResearchLearnMetadata> implements StatisticsResearchLearnMetadataService {
    @Autowired
    private StatisticsResearchLearnMetadataMapper learnMetadataMapper;
    @Autowired
    private WorkUtil workUtil;
    @Autowired
    private JedisUtil jedisUtil;
    @Autowired
    private IStatisticsAccountRoleService statisticsAccountRoleService;

    @Override
    public Page<ResearchGroupVo> researchGroup(String startDate, String endDate, String kwd, Integer pageSize, Integer pageNo, Long companyId, List<Long> orgIds, Long siteId) {
        Page<ResearchGroupVo> page = new Page<>(pageNo, pageSize);
        RequestContext context = ContextHolder.get();
        if (kwd == null || kwd.equals("")) {
            kwd = null;
        }
        List<ResearchGroupVo> statisticsResearches = new ArrayList<>();
        Integer count = 0;

        //验证下管辖区是否是全平台的
        boolean isAdmin;
        if (!context.isAdmin()) {
            isAdmin = statisticsAccountRoleService.getIsAdmin(2, context.getSiteId(), context.getAccountId());
        } else {
            isAdmin = context.isAdmin();
        }

        //判断是否是super管理员
        if (isAdmin) {
            List<Long> researchIds = learnMetadataMapper.researchGroup2Ids(startDate, endDate, kwd, companyId, null, siteId, context.getAccountId(), page);
            if (CollectionUtils.isNotEmpty(researchIds)) {
                statisticsResearches = learnMetadataMapper.researchGroup2(researchIds, companyId, null, siteId, context.getAccountId(), new Page<>());
                count = learnMetadataMapper.researchGroupListCount2(startDate, endDate, kwd, companyId, null, siteId, context.getAccountId());
            }
        } else {
            List<Long> researchIds = learnMetadataMapper.researchGroup1Ids(startDate, endDate, kwd, companyId, null, siteId, context.getAccountId(), page);
            if (CollectionUtils.isNotEmpty(researchIds)) {
                statisticsResearches = learnMetadataMapper.researchGroup1(researchIds, companyId, null, siteId, context.getAccountId(), new Page<>());
                count = learnMetadataMapper.researchGroupListCount1(startDate, endDate, kwd, companyId, null, siteId, context.getAccountId());
            }
        }


        if (CollectionUtils.isNotEmpty(statisticsResearches)) {
            log.info("上下文：{}", JSON.toJSON(context));
            log.info("站点id:"+siteId);
            log.info("查询的列表：{}",JSON.toJSON(statisticsResearches));
            for (int i = 0; i < statisticsResearches.size(); i++) {
                if (statisticsResearches.get(i) != null) {
                    statisticsResearches.get(i).setIndex((pageNo - 1) * pageSize + i + 1);
                    Integer askNum = jedisUtil.getAskNumByAdminIdAndWorkId(context.getAccountId(), siteId, context.isAdmin(), statisticsResearches.get(i).getResearchId());
                    log.info("调研可参加的人数：{}",askNum);

                    statisticsResearches.get(i).setCanStateCount(askNum);
                }
            }

        }


        page.setTotal(count == null ? 0 : count);
        page.setRecords(statisticsResearches);
        return page;
    }

    public Page<ResearchGroupViewVo> selectResearchView(Long researchId, String startDate, String endDate, String orgNameorOrgCode, String accountName, Integer joinState,
                                                        Integer pageNo, Integer pageSize) {
        Page<ResearchGroupViewVo> page = new Page<>(pageNo, pageSize);
        RequestContext context = ContextHolder.get();
        if (accountName == null || accountName.equals("")) {
            accountName = null;
        }
        if (orgNameorOrgCode == null || orgNameorOrgCode.equals("")) {
            orgNameorOrgCode = null;
        }
        List<Long> accountIds = new ArrayList<>();
        List<ResearchGroupViewVo> statisticsResearches = new ArrayList<>();
        Integer count = 0;

        //验证下管辖区是否是全平台的
        boolean isAdmin;
        if (!context.isAdmin()) {
            isAdmin = statisticsAccountRoleService.getIsAdmin(2, context.getSiteId(), context.getAccountId());
        } else {
            isAdmin = context.isAdmin();
        }

        //判断是否为站点管理员
        if (isAdmin) {
            statisticsResearches = learnMetadataMapper.researchGroupView2(researchId, orgNameorOrgCode, accountName, joinState, context.getCompanyId(), context.getAccountId(), context.getSiteId(), startDate, endDate, page);
            count = learnMetadataMapper.researchGroupViewCount2(researchId, orgNameorOrgCode, accountName, joinState, context.getCompanyId(), context.getAccountId(), context.getSiteId(), startDate, endDate);
        } else {
            statisticsResearches = learnMetadataMapper.researchGroupView1(researchId, orgNameorOrgCode, accountName, joinState, context.getCompanyId(), context.getAccountId(), context.getSiteId(), startDate, endDate, page);
            count = learnMetadataMapper.researchGroupViewCount1(researchId, orgNameorOrgCode, accountName, joinState, context.getCompanyId(), context.getAccountId(), context.getSiteId(), startDate, endDate);

        }


        page.setTotal(count);
        page.setRecords(statisticsResearches);
        return page;
    }

    public Integer selectResearchViewCount(Long researchId, String startDate, String endDate, String orgNameorOrgCode, String accountName, Integer joinState) {
        if (accountName == null || accountName.equals("")) {
            accountName = null;
        }
        if (orgNameorOrgCode == null || orgNameorOrgCode.equals("")) {
            orgNameorOrgCode = null;
        }
        return learnMetadataMapper.selectResearchViewCount(researchId, startDate, endDate, orgNameorOrgCode, accountName, joinState);
    }

    public ResearchGroupVo selectJoinCount(Long researchId, String startDate, String endDate, String orgNameorOrgCode, String accountName, Integer joinState
    ) {
        if (accountName == null || accountName.equals("")) {
            accountName = null;
        }
        if (orgNameorOrgCode == null || orgNameorOrgCode.equals("")) {
            orgNameorOrgCode = null;
        }
        return learnMetadataMapper.selectJoinCount(researchId, startDate, endDate, orgNameorOrgCode, accountName, joinState);
    }

    @Override
    public List<ResearchGroupViewVo> researchViews(Long researchId, String startDate, String endDate, String orgNameorOrgCode, String accountName, Integer joinState, Integer pageNo, Integer pageSize) {
        if (accountName == null || accountName.equals("")) {
            accountName = null;
        }
        if (orgNameorOrgCode == null || orgNameorOrgCode.equals("")) {
            orgNameorOrgCode = null;
        }
        Page<ResearchGroupViewVo> page = new Page<>(pageNo, pageSize);
        return learnMetadataMapper.selectResearchView(researchId, startDate, endDate, orgNameorOrgCode, accountName, joinState, page);
    }


}
