package com.yizhi.statistics.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.statistics.application.domain.StatisticsSignInformation;
import com.yizhi.statistics.application.entity.StatisticsCourseVisibleRange;
import com.yizhi.statistics.application.mapper.StatisticsSignMapper;
import com.yizhi.statistics.application.refactor.sign.StatisticsSignVO;
import com.yizhi.statistics.application.service.IStatisticsAccountRoleService;
import com.yizhi.statistics.application.service.IStatisticsSignService;
import com.yizhi.statistics.application.service.StatisticsCourseVisibleRangeService;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.statistics.application.vo.sign.SignAccountGroupVo;
import com.yizhi.statistics.application.vo.sign.StatisticsSignVo;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-04-22
 */
@Service
public class StatisticsSignServiceImpl extends ServiceImpl<StatisticsSignMapper, StatisticsSignInformation> implements IStatisticsSignService {

    private static final Logger LOGGER = LoggerFactory.getLogger(StatisticsSignServiceImpl.class);

    @Autowired
    private StatisticsSignMapper statisticsSignMapper;

    @Autowired
    private StatisticsCourseVisibleRangeService courseVisibleRangeService;

    @Autowired
    private JedisUtil jedisUtil;

    @Autowired
    private IStatisticsAccountRoleService statisticsAccountRoleService;

    @Override
    public Page<StatisticsSignVO> signGroup(boolean isAdmin, Long accountId, Long projectId, String kwd, String startDate, String endDate, Long companyId, Long siteId, Integer pageNo, Integer pageSize) {
        Page<StatisticsSignVO> page = new Page<>(pageNo, pageSize);
        Page<Long> signTimeIdPage = new Page<>(pageNo, pageSize);
        List<Long> signTimeIds = statisticsSignMapper.getSignTimeIds(kwd, startDate, endDate, companyId, siteId, signTimeIdPage);
        if (CollectionUtils.isEmpty(signTimeIds)) {
            return page;
        }

        //验证下管辖区是否是全平台的
        boolean isAdmin1;
        if (!isAdmin) {
            isAdmin1 = statisticsAccountRoleService.getIsAdmin(6, siteId, accountId);
        } else {
            isAdmin1 = isAdmin;
        }

        List<StatisticsSignVO> list = statisticsSignMapper.signGroup(isAdmin1, accountId, signTimeIds, companyId, siteId);
        if (CollectionUtils.isNotEmpty(list)) {
            for (StatisticsSignVO statisticsSign : list) {
                Integer askNum = jedisUtil.getAskNumByAdminIdAndWorkId(accountId, siteId, isAdmin1, statisticsSign.getSignId());
                statisticsSign.setCanStateCount(askNum);
                statisticsSign.setNotSignCount(askNum - statisticsSign.getJoinStateCount());
                if (StringUtils.isNotBlank(statisticsSign.getRange())) {
					statisticsSign.setRange(statisticsSign.getRange()+"米");
				}
            }
            page.setRecords(list);
            page.setTotal(signTimeIdPage.getTotal());
        }
        return page;
    }

    @Override
    public Page<StatisticsSignVO> signGroupView(boolean isAdmin, Long accountId, Long signId, Long signTimeId, Long companyId, Long siteId, String orgNameOrCode, String accountName, Integer pageNo, Integer pageSize) {
        Page<StatisticsSignVO> page = new Page<>(pageNo, pageSize);

        StatisticsSignInformation statisticsSignInformation = statisticsSignMapper.selectBySignTimeId(signTimeId);

        if (null == statisticsSignInformation || null == statisticsSignInformation.getTrainningProjectId()) {
            return page;
        }

        StatisticsCourseVisibleRange statisticsCourseVisibleRange = new StatisticsCourseVisibleRange();
        statisticsCourseVisibleRange.setCourseId(signTimeId);
        statisticsCourseVisibleRange.setType(6);
        statisticsCourseVisibleRange.setIsVirtual(1);

        List<StatisticsCourseVisibleRange> rangeList = courseVisibleRangeService.selectList(new EntityWrapper<>(statisticsCourseVisibleRange));

        boolean isAllVisible = false;

        if (CollectionUtils.isNotEmpty(rangeList)) {
            isAllVisible = true;
        }

        List<StatisticsSignVO> list = null;

        //验证下管辖区是否是全平台的
        boolean isAdmin1;
        if (!isAdmin) {
            isAdmin1 = statisticsAccountRoleService.getIsAdmin(6, siteId, accountId);
        } else {
            isAdmin1 = isAdmin;
        }

        if (isAdmin1) {
            list = statisticsSignMapper.signGroupViewAdmin(isAllVisible, null, signId, signTimeId, companyId, siteId, orgNameOrCode, accountName, page);
        } else {
            list = statisticsSignMapper.signGroupViewNoAdmin(isAllVisible, accountId, null, signId, signTimeId, companyId, siteId, orgNameOrCode, accountName, page);
        }

        if (CollectionUtils.isNotEmpty(list)) {
            page.setRecords(list);
        }
        return page;
    }

    @Override
    public StatisticsSignInformation selectBySignTimeId(Long signTimeId) {
        return statisticsSignMapper.selectBySignTimeId(signTimeId);
    }

    @Override
    public Date selectMaxDate() {
        return statisticsSignMapper.selectMaxDate();
    }

    /**
     * 签到-按用户统计
     * todo
     * @return
     */
    @Override
    public Page<SignAccountGroupVo> signAccountGroup(boolean isAdmin, Long accountId, String accountKwd, String orgKwd, String startDate, String endDate, Long companyId, Long siteId, Integer pageNo, Integer pageSize) {
        Page<SignAccountGroupVo> accountGroupPage = null;
        try {
            accountGroupPage = new Page<>(pageNo, pageSize);
            Page<Long> signTimeIdPage = new Page<>(1, Integer.MAX_VALUE);
            Page<Long> accountPage = new Page<>(pageNo, pageSize);

            //获取全平台可见的签到数
            //Integer num = statisticsSignMapper.getAllVisibleSignNum(companyId, siteId, startDate, endDate);

            List<Long> signTimeIds = statisticsSignMapper.getSignTimeIds("", startDate, endDate, companyId, siteId, signTimeIdPage);


            //验证下管辖区是否是全平台的
            boolean isAllAdmin;
            if (!isAdmin) {
                isAllAdmin = statisticsAccountRoleService.getIsAdmin(6, siteId, accountId);
            } else {
                isAllAdmin = isAdmin;
            }

            List<Long> accountIds = statisticsSignMapper.getAccountIds(isAllAdmin, accountId, accountKwd, orgKwd, companyId, siteId, accountPage);

            if (CollectionUtils.isEmpty(accountIds)) {
                return accountGroupPage;
            }

            List<SignAccountGroupVo> list = statisticsSignMapper.getSignAccountGroup(isAdmin, accountId, companyId, siteId, startDate, endDate, accountIds, signTimeIds);
            if (CollectionUtils.isNotEmpty(list)) {
                for (SignAccountGroupVo signAccountGroupVo : list) {
                    signAccountGroupVo.setCanStateCount(signAccountGroupVo.getCanStateCount());
                    if(!StringUtils.isEmpty(signAccountGroupVo.getLastSignTime())){
                        signAccountGroupVo.setLastSignTime(signAccountGroupVo.getLastSignTime().substring(0,signAccountGroupVo.getLastSignTime().length()-2));
                    }
                }
                accountGroupPage.setRecords(list);
                accountGroupPage.setTotal(accountPage.getTotal());
            }
        } catch (Exception e) {
            LOGGER.error("签到获取按用户统计失败", e);
        }
        return accountGroupPage;
    }


    /**
     * 签到-按用户统计-获取全部用户   用于导出全部数据
     * todo
     *
     * @return
     */
    public Page<StatisticsSignVo> signAllAccountGroup(boolean isAdmin, Long accountId, String accountKwd, String orgKwd, String startDate, String endDate, Long companyId, Long siteId, Integer pageNo, Integer pageSize) {
        Page<StatisticsSignVo> accountGroupViewPage = null;
        try {
            accountGroupViewPage = new Page<>(pageNo, pageSize);
            Page<Long> accountPage = new Page<>(pageNo, pageSize);
            List<Long> accountIds = new ArrayList<>();
            //验证下管辖区是否是全平台的
            boolean isAllAdmin;
            if (!isAdmin) {
                isAllAdmin = statisticsAccountRoleService.getIsAdmin(6, siteId, accountId);
            } else {
                isAllAdmin = isAdmin;
            }
            accountIds = statisticsSignMapper.getAccountIds(isAllAdmin, accountId, accountKwd, orgKwd, companyId, siteId, accountPage);
            if (CollectionUtils.isEmpty(accountIds)) {
                return accountGroupViewPage;
            }
            List<StatisticsSignVo> accountGroupViewList = new ArrayList<>();
            for (Long aid : accountIds){
                List<StatisticsSignVo> list = statisticsSignMapper.getSignAccountGroupView(aid, siteId, companyId, startDate, endDate, "", accountGroupViewPage);
                if(null!= list&&list.size()>0){
                    accountGroupViewList.addAll(list);
                }
            }
            if (CollectionUtils.isNotEmpty(accountGroupViewList)) {
                for (StatisticsSignVo s : accountGroupViewList) {
                    if (!StringUtils.isEmpty(s.getRange())) {
                        s.setRange(s.getRange() + "米");
                    }
                    if(!StringUtils.isEmpty(s.getSignStartTime())){
                        s.setSignStartTime(s.getSignStartTime().substring(0,s.getSignStartTime().length()-2));
                    }
                    if(!StringUtils.isEmpty(s.getSignEndTime())){
                        s.setSignEndTime(s.getSignEndTime().substring(0,s.getSignEndTime().length()-2));
                    }
                    if(!StringUtils.isEmpty(s.getSignTime())){
                        s.setSignTime(s.getSignTime().substring(0,s.getSignTime().length()-2));
                    }

                }
                accountGroupViewPage.setRecords(accountGroupViewList);
            }
        } catch (Exception e) {
            LOGGER.error("签到获取全部用户统计失败", e);
        }
        return accountGroupViewPage;
    }


    /**
     * 签到-按用户统计-查看详情
     *
     * @param accountId
     * @param siteId
     * @param companyId
     * @param startDate
     * @param endDate
     * @param projectKwd
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    public Page<StatisticsSignVo> signAccountGroupView(Long accountId, Long siteId, Long companyId, String startDate, String endDate, String projectKwd, Integer pageNo, Integer pageSize) {
        Page<StatisticsSignVo> accountGroupViewPage = null;
        try {
            accountGroupViewPage = new Page<>(pageNo, pageSize);
            List<StatisticsSignVo> accountGroupViewList = statisticsSignMapper.getSignAccountGroupView(accountId, siteId, companyId, startDate, endDate, projectKwd, accountGroupViewPage);
            if (CollectionUtils.isNotEmpty(accountGroupViewList)) {
                for (StatisticsSignVo s : accountGroupViewList) {
                    if (!StringUtils.isEmpty(s.getRange())) {
                        s.setRange(s.getRange() + "米");
                    }
                    if(!StringUtils.isEmpty(s.getSignStartTime())){
                        s.setSignStartTime(s.getSignStartTime().substring(0,s.getSignStartTime().length()-2));
                    }
                    if(!StringUtils.isEmpty(s.getSignEndTime())){
                        s.setSignEndTime(s.getSignEndTime().substring(0,s.getSignEndTime().length()-2));
                    }
                    if(!StringUtils.isEmpty(s.getSignTime())){
                        s.setSignTime(s.getSignTime().substring(0,s.getSignTime().length()-2));
                    }
                }
                accountGroupViewPage.setRecords(accountGroupViewList);
                accountGroupViewPage.setTotal(accountGroupViewPage.getTotal());
            }
        } catch (Exception e) {
            LOGGER.error("签到获取按用户统计详情失败", e);
        }
        return accountGroupViewPage;
    }
}
