package com.yizhi.statistics.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.statistics.application.entity.StatisticsTraining;
import com.yizhi.statistics.application.mapper.StatisticsTrainingMapper;
import com.yizhi.statistics.application.mapper.StatisticsTrainingPlanActivityMapper;
import com.yizhi.statistics.application.mapper.StatisticsTrainingStudentActivityRecordMapper;
import com.yizhi.statistics.application.service.IStatisticsAccountRoleService;
import com.yizhi.statistics.application.service.IStatisticsTrainingService;
import com.yizhi.statistics.application.training.vo.TrainingProjectAccountRecords;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.training.application.vo.domain.TrainingProjectVo;
import com.yizhi.training.application.vo.manage.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author wfd123
 * @since 2019-05-13
 */
@Service
public class StatisticsTrainingServiceImpl extends ServiceImpl<StatisticsTrainingMapper, StatisticsTraining> implements IStatisticsTrainingService {

    private Logger logger = LoggerFactory.getLogger(StatisticsTrainingServiceImpl.class);

    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private StatisticsTrainingMapper statisticsTrainingMapper;
    @Autowired
    private JedisUtil jedisUtil;

    @Autowired
    private StatisticsTrainingPlanActivityMapper statisticsTrainingPlanActivityMapper;

    @Autowired
    private StatisticsTrainingStudentActivityRecordMapper statisticsTrainingStudentActivityRecordMapper;

    @Autowired
    private IStatisticsAccountRoleService statisticsAccountRoleService;

    @Override
    public void dealStatisticsTraining(TrainingProjectVo tp) {
        // TODO Auto-generated method stub
        Long tpId = tp.getId();
        StatisticsTraining stWrapper = new StatisticsTraining();
        stWrapper.setTrainingProjectId(tpId);
        EntityWrapper<StatisticsTraining> wrapper = new EntityWrapper<StatisticsTraining>(stWrapper);
        stWrapper = this.selectOne(wrapper);
        if (stWrapper == null) {
            StatisticsTraining st = new StatisticsTraining();
            st.setId(idGenerator.generate());
            st.setTrainingProjectId(tp.getId());
            st.setTrainingProjectName(tp.getName());
            st.setStartTime(tp.getStartTime());
            st.setEndTime(tp.getEndTime());
            st.setState(tp.getStatus());
            st.setCreateTime(tp.getCreateTime());
            st.setCompanyId(tp.getCompanyId());
            st.setSiteId(tp.getSiteId());
            st.setOrgId(tp.getOrgId());
            this.insert(st);
        } else {
            StatisticsTraining st = new StatisticsTraining();
            st.setId(stWrapper.getId());//根据id进行区分
            st.setTrainingProjectId(tp.getId());
            st.setTrainingProjectName(tp.getName());
            st.setStartTime(tp.getStartTime());
            st.setEndTime(tp.getEndTime());
            st.setState(tp.getStatus());
            st.setCreateTime(tp.getCreateTime());
            st.setCompanyId(tp.getCompanyId());
            st.setSiteId(tp.getSiteId());
            st.setOrgId(tp.getOrgId());
            this.updateById(st);
        }
    }


    @Override
    public List<TrainingProjectDataChartsVo> chartGroup(Long companyId, Long siteId, Long adminId, Boolean isAdmin,
                                                        String startDate, String endDate) {
        // TODO Auto-generated method stub
        //List<TrainingProjectDataChartsVo> list=statisticsTrainingMapper.chartGroupNew(companyId, siteId, adminId, isAdmin, startDate, endDate+" 23:59:59");

        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date end = new Date();
        if ("".equals(endDate) || endDate == null) {
            endDate = dateFormat.format(end);
        }
        if ("".equals(startDate) || startDate == null) {
            Date s = addDay(end, -30);
            startDate = dateFormat.format(s);
        }
        List<TrainingProjectDataChartsVo> list = statisticsTrainingMapper.chartGroupNew(companyId, siteId, startDate, endDate + " 23:59:59");

        if (CollectionUtils.isNotEmpty(list)) {
            Map<String, TrainingProjectDataChartsVo> mapList = new HashMap<String, TrainingProjectDataChartsVo>();
            for (TrainingProjectDataChartsVo item : list) {
                mapList.put(item.getValue(), item);
            }
            //统计参加人数
            List<TrainingProjectDataChartsVo> listTotalIn = statisticsTrainingMapper.chartGroupNewTotalIn(companyId, siteId, adminId, isAdmin, startDate, endDate + " 23:59:59");
            if (CollectionUtils.isNotEmpty(listTotalIn)) {
                for (TrainingProjectDataChartsVo item : listTotalIn) {
                    TrainingProjectDataChartsVo tmp = mapList.get(item.getValue());
                    if (tmp != null) {
                        tmp.setTotalJoin(item.getTotalJoin());
                    }
                }
            }

            //统计完成人数
            List<TrainingProjectDataChartsVo> listTotalFinish = statisticsTrainingMapper.chartGroupNewTotalFinish(companyId, siteId, adminId, isAdmin, startDate, endDate + " 23:59:59");
            if (CollectionUtils.isNotEmpty(listTotalFinish)) {
                for (TrainingProjectDataChartsVo item : listTotalFinish) {
                    TrainingProjectDataChartsVo tmp = mapList.get(item.getValue());
                    if (tmp != null) {
                        tmp.setTotalFinish(item.getTotalFinish());
                    }
                }
            }
        }

        List<TrainingProjectDataChartsVo> listAllRecord = new ArrayList<TrainingProjectDataChartsVo>();
        Date startDate1, endDate1;
        TrainingProjectDataChartsVo rsccvUpdate = null;
        int num = 0;

        try {
            startDate1 = dateFormat.parse(startDate);
            endDate1 = dateFormat.parse(endDate);
            num = (int) ((endDate1.getTime() - startDate1.getTime()) / (1000 * 3600 * 24));
            Date updateDate = startDate1;
            String updateDateValue = dateFormat.format(updateDate);
            Boolean isTrue = false;
            int j = 0;
            for (int i = 0; i <= num; i++) {
                if (list != null && list.size() > 0) {
                    if (j < list.size()) {
                        TrainingProjectDataChartsVo rsccv = list.get(j);
                        if (rsccv.getValue().equals(updateDateValue)) {
                            j = j + 1;
                            listAllRecord.add(rsccv);
                            isTrue = true;
                        } else {
                            isTrue = false;
                        }
                    }
                }
                if (!isTrue) {
                    rsccvUpdate = new TrainingProjectDataChartsVo();
                    rsccvUpdate.setValue(updateDateValue);
                    rsccvUpdate.setTotalFinish(0);
                    rsccvUpdate.setTotalJoin(0);
                    rsccvUpdate.setTotalProject(0);
                    listAllRecord.add(rsccvUpdate);

                }
                Calendar c = Calendar.getInstance();
                c.setTime(updateDate);
                c.add(Calendar.DAY_OF_MONTH, 1); // 利用Calendar 实现 Date日期+1天
                updateDate = c.getTime();
                updateDateValue = dateFormat.format(updateDate);
                isTrue = false;
            }

        } catch (ParseException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return listAllRecord;
    }

    @Override
    public Page<ReportStudyTrainingProjectVo> trainingProjectGroup(String kwd, String startDate, String endDate,
                                                                   Long companyId, Long siteId, Long adminId, Boolean isAdmin, Integer pageNo, Integer pageSize) {
        // TODO Auto-generated method stub
        Long t1 = System.currentTimeMillis();
        Page<ReportStudyTrainingProjectVo> pageDate = new Page<ReportStudyTrainingProjectVo>(pageNo, pageSize);
        //先查出来十个业务id
        Page<Long> pageLimit = new Page<Long>(pageNo, pageSize);
        List<Long> trainingProjectIds = statisticsTrainingMapper.trainingGroupLimit(startDate, endDate, companyId, siteId, kwd, pageLimit);
        Long t2 = System.currentTimeMillis();
        logger.info("按照培训项目统计，花费时间(ms):第一步" + (t2 - t1) + "参数信息。companyId:" + companyId + "_siteId:" + siteId + "_adminId:" + adminId + "_isAdmin:" + isAdmin);
        if (CollectionUtils.isNotEmpty(trainingProjectIds)) {
            pageDate.setTotal(pageLimit.getTotal());//总页数
            List<ReportStudyTrainingProjectVo> listDate = statisticsTrainingStudentActivityRecordMapper.trainingProjectGroupBySQLNew(companyId, siteId, trainingProjectIds);
            if (CollectionUtils.isNotEmpty(listDate)) {
                Map<Long, ReportStudyTrainingProjectVo> stumaps = new HashMap<Long, ReportStudyTrainingProjectVo>();
                //统计可参加人数
                for (ReportStudyTrainingProjectVo reportStudyTrainingProjectVo : listDate) {
                    Long tpId = reportStudyTrainingProjectVo.getId();
                    Integer askNum = jedisUtil.getAskNumByAdminIdAndWorkId(adminId, siteId, isAdmin, tpId);
                    reportStudyTrainingProjectVo.setTotalAsk(askNum);
                    stumaps.put(tpId, reportStudyTrainingProjectVo);
                }

                //统计实际参加人数
                List<ReportStudyTrainingProjectVo> stuTotalIns = statisticsTrainingStudentActivityRecordMapper.trainingProjectGroupBySQLNewTotalIn(companyId, siteId, adminId, isAdmin,
                        trainingProjectIds, startDate, endDate);
                ReportStudyTrainingProjectVo temp = null;
                for (ReportStudyTrainingProjectVo reportStudyTrainingProjectVo : stuTotalIns) {
                    temp = stumaps.get(reportStudyTrainingProjectVo.getId());
                    temp.setTotalIn(reportStudyTrainingProjectVo.getTotalIn());
                }

                //统计完成人数
                List<ReportStudyTrainingProjectVo> stuTotalFinishs = statisticsTrainingStudentActivityRecordMapper.trainingProjectGroupBySQLNewTotalFinish(companyId, siteId, adminId, isAdmin,
                        trainingProjectIds, startDate, endDate);
                for (ReportStudyTrainingProjectVo reportStudyTrainingProjectVo : stuTotalFinishs) {
                    temp = stumaps.get(reportStudyTrainingProjectVo.getId());
                    temp.setTotalFinish(reportStudyTrainingProjectVo.getTotalFinish());
                    if (temp.getTotalIn() > 0) {
                        BigDecimal a = new BigDecimal(temp.getTotalFinish());
                        BigDecimal b = new BigDecimal(temp.getTotalIn());
                        temp.setAvgFinish(a.divide(b, 2, BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(100)));
                    }
                }

                pageDate.setRecords(listDate);
            }
        }
        Long t3 = System.currentTimeMillis();
        logger.info("按照培训项目统计，花费时间(ms):第二步:" + (t3 - t2));
        return pageDate;
    }

    @Override
    public Page<TrainingProjectDataChartsVo> trainingProjectGroupView(Long companyId, Long siteId, Long adminId, Boolean isAdmin, Long tpId, Integer pageNo, Integer pageSize) {
        // TODO Auto-generated method stub
        Page<TrainingProjectDataChartsVo> page = new Page<TrainingProjectDataChartsVo>(pageNo, pageSize);
        //List<TrainingProjectDataChartsVo> list=statisticsTrainingMapper.trainingGroupView(companyId, siteId, adminId, isAdmin, tpId,page);
        List<TrainingProjectDataChartsVo> list = new ArrayList<TrainingProjectDataChartsVo>();
        page.setRecords(list);
        return page;
    }

    @Override
    public Page<ReportStudyTrainingProjectAccountVo> trainingProjectAccountGroup(
            Long companyId, Long siteId, Long adminId, Boolean isAdmin, String orgKwd, String accountKwd, Integer pageNo, Integer pageSize, String startDate, String endDate) {
        // TODO Auto-generated method stub
        Page<Long> pageAccountIds = new Page<Long>(pageNo, pageSize);
        List<Long> accountIds = statisticsTrainingMapper.accountIdsLimit(orgKwd, accountKwd, companyId, siteId, adminId, isAdmin, pageAccountIds);
        Page<ReportStudyTrainingProjectAccountVo> page = new Page<ReportStudyTrainingProjectAccountVo>(pageNo, pageSize);
        if (CollectionUtils.isNotEmpty(accountIds)) {
            //List<ReportStudyTrainingProjectAccountVo> list=statisticsTrainingMapper.trainingProjectAccountGroupNew(companyId, siteId, accountIds,startDate,endDate+" 23:59:59");
            List<ReportStudyTrainingProjectAccountVo> list = statisticsTrainingMapper.trainingProjectAccountGroupNew(accountIds);

            Map<Long, ReportStudyTrainingProjectAccountVo> traAccounts = new HashMap<Long, ReportStudyTrainingProjectAccountVo>();
            for (ReportStudyTrainingProjectAccountVo item : list) {
                traAccounts.put(item.getAccountId(), item);
            }

            //统计参与项目数量
            List<ReportStudyTrainingProjectAccountVo> listTotalAsk = statisticsTrainingMapper.trainingProjectAccountGroupNewTotalAsk(companyId, siteId, accountIds, startDate, endDate + " 23:59:59");
            if (CollectionUtils.isNotEmpty(listTotalAsk)) {
                ReportStudyTrainingProjectAccountVo tmp = null;
                for (ReportStudyTrainingProjectAccountVo item : listTotalAsk) {
                    tmp = traAccounts.get(item.getAccountId());
                    if (tmp != null) {
                        tmp.setTotalAsk(item.getTotalAsk());
                    }
                }
            }

            //统计参加数量
            List<ReportStudyTrainingProjectAccountVo> listTotalIn = statisticsTrainingMapper.trainingProjectAccountGroupNewTotalIn(companyId, siteId,
                    accountIds, startDate, endDate + " 23:59:59", isAdmin, adminId);
            if (CollectionUtils.isNotEmpty(listTotalIn)) {
                ReportStudyTrainingProjectAccountVo tmp = null;
                for (ReportStudyTrainingProjectAccountVo item : listTotalIn) {
                    tmp = traAccounts.get(item.getAccountId());
                    if (tmp != null) {
                        tmp.setTotalIn(item.getTotalIn());
                    }
                }
            }

            //统计完成数量
            List<ReportStudyTrainingProjectAccountVo> listTotalFinish = statisticsTrainingMapper.trainingProjectAccountGroupNewTotalFinish(companyId,
                    siteId, accountIds, startDate, endDate + " 23:59:59", isAdmin, adminId);
            if (CollectionUtils.isNotEmpty(listTotalIn)) {
                ReportStudyTrainingProjectAccountVo tmp = null;
                for (ReportStudyTrainingProjectAccountVo item : listTotalFinish) {
                    tmp = traAccounts.get(item.getAccountId());
                    if (tmp != null) {
                        tmp.setTotalFinish(item.getTotalFinish());
                    }
                }
            }

            page.setRecords(list);
            page.setTotal(pageAccountIds.getTotal());
        }
        return page;
    }

    @Override
    public Page<ReportStudyTrainingProjectAccountViewVo> trainingProjectAccountGroupView(
            Long companyId, Long siteId, Integer pageNo, Integer pageSize, Long accountId, String projectName, Integer type) {
        // TODO Auto-generated method stub
        Page<Long> pageLimitTpIds = new Page<Long>(pageNo, pageSize);

        List<Long> tpIds = statisticsTrainingMapper.trainingProjectIdsAccountGroupView(companyId, siteId, accountId, projectName, pageLimitTpIds);
        Page<ReportStudyTrainingProjectAccountViewVo> page = new Page<ReportStudyTrainingProjectAccountViewVo>(pageNo, pageSize);
        if (CollectionUtils.isNotEmpty(tpIds)) {
            List<ReportStudyTrainingProjectAccountViewVo> list = null;
            if (type == 1) {
                list = statisticsTrainingMapper.trainingProjectAccountGroupView(companyId, siteId, accountId, tpIds);
            }
            if (type == 2) {
                //添加积分信息
                list = statisticsTrainingMapper.trainingProjectAccountGroupViewExport(companyId, siteId, accountId, tpIds);
            }

            if (CollectionUtils.isNotEmpty(list)) {
                for (ReportStudyTrainingProjectAccountViewVo rstpavv : list) {
                    if (rstpavv.getEndTime() != null) {
                        rstpavv.setState("已完成");
                    } else if (rstpavv.getStartTime() != null) {
                        rstpavv.setState("未完成");
                    } else {
                        rstpavv.setState("未开始");
                    }
                }
                page.setRecords(list);
                page.setTotal(pageLimitTpIds.getTotal());
            }
        }
        return page;
    }

    @Override
    public StatisticsTraining getStatisticsTrainingByTpId(Long tpId) {
        // TODO Auto-generated method stub
        StatisticsTraining st = new StatisticsTraining();
        st.setTrainingProjectId(tpId);
        return this.selectOne(new EntityWrapper<StatisticsTraining>(st));
    }

    @Override
    public List<TrainingProjectGroupViewExportVO1> export1(Long companyId, Long siteId, Long adminId, Boolean isAdmin,
                                                           Long tpId) {
        // TODO Auto-generated method stub
        /*System.out.println(statisticsTrainingMapper.export1(companyId, siteId, adminId, tpId).size()+"--------");*/
        return statisticsTrainingMapper.export1(companyId, siteId, adminId, tpId, isAdmin);
    }

    @Override
    public List<TrainingProjectGroupViewExportVO2> export2(Long companyId, Long siteId, Long adminId, Boolean isAdmin,
                                                           Long tpId) {
        // TODO Auto-generated method stub
        return statisticsTrainingMapper.export2(companyId, siteId, adminId, tpId, isAdmin);
    }


    @Override
    public List<TrainingProjectGroupViewExportVO3> export3(Long companyId, Long siteId, Long adminId, Boolean isAdmin,
                                                           Long tpId) {
        // TODO Auto-generated method stub
        return statisticsTrainingMapper.export3(companyId, siteId, adminId, tpId, isAdmin);
    }


    @Override
    public Page<ReportStudyTrainingProjectOrgVo> trainingProjectOrgGroup(Long companyId, Long siteId, Long adminId,
                                                                         Boolean isAdmin, String kwd, Integer pageNo, Integer pageSize, String startDate, String endDate) {
        // TODO Auto-generated method stub
        //验证下管辖区是否是全平台的
        if (!isAdmin) {
            isAdmin = statisticsAccountRoleService.getIsAdmin(0, siteId, adminId);
        }
        Page<ReportStudyTrainingProjectOrgVo> page = new Page<ReportStudyTrainingProjectOrgVo>(pageNo, pageSize);
        Page<Long> pageLimit = new Page<Long>(pageNo, pageSize);
        List<ReportStudyTrainingProjectOrgVo> orgs = statisticsTrainingMapper.orgIdsLimitNew(companyId, siteId, adminId, isAdmin, kwd, pageLimit);
        List<Long> orgIds = new ArrayList<Long>();
        orgs.forEach(e -> orgIds.add(e.getOrgId()));

        Map<Long, ReportStudyTrainingProjectOrgVo> map = new HashMap<Long, ReportStudyTrainingProjectOrgVo>();
        if (CollectionUtils.isNotEmpty(orgIds)) {
            List<ReportStudyTrainingProjectOrgVo> listDate = statisticsTrainingStudentActivityRecordMapper.trainingProjectOrgGroupNew(companyId, siteId, adminId, orgIds, isAdmin, startDate, endDate + " 23:59:59");
            if (CollectionUtils.isNotEmpty(listDate)) {
                for (ReportStudyTrainingProjectOrgVo vo : listDate) {
                    map.put(vo.getOrgId(), vo);
                }
            }

            //统计参加项目数量和成绩人数
            List<ReportStudyTrainingProjectOrgVo> task = statisticsTrainingMapper.trainingProjectOrgGroupNewTotalAsk(companyId, siteId, orgIds);
            if (CollectionUtils.isNotEmpty(task)) {
                ReportStudyTrainingProjectOrgVo tmp = null;
                for (ReportStudyTrainingProjectOrgVo item : task) {
                    tmp = map.get(item.getOrgId());
                    if (tmp != null) {
                        tmp.setTotalAskProject(item.getTotalAskProject());
                        tmp.setTotalAskAccount(item.getTotalAskAccount());
                    }
                }
            }

            //统计参加项目参加数
            List<ReportStudyTrainingProjectOrgVo> totalIns = statisticsTrainingMapper.trainingProjectOrgGroupNewTotalIn(companyId, siteId, orgIds, startDate, endDate);
            if (CollectionUtils.isNotEmpty(totalIns)) {
                ReportStudyTrainingProjectOrgVo tmp = null;
                for (ReportStudyTrainingProjectOrgVo item : totalIns) {
                    tmp = map.get(item.getOrgId());
                    if (tmp != null) {
                        tmp.setTotalIn(item.getTotalIn());
                    }
                }
            }

            //统计参加项目完成数
            List<ReportStudyTrainingProjectOrgVo> totalFinishs = statisticsTrainingMapper.trainingProjectOrgGroupNewTotalFinish(companyId, siteId, orgIds, startDate, endDate);
            if (CollectionUtils.isNotEmpty(totalFinishs)) {
                ReportStudyTrainingProjectOrgVo tmp = null;
                for (ReportStudyTrainingProjectOrgVo item : totalFinishs) {
                    tmp = map.get(item.getOrgId());
                    if (tmp != null) {
                        tmp.setTotalFinish(item.getTotalFinish());
                    }
                }
            }

            //统计完成率
            for (ReportStudyTrainingProjectOrgVo item : listDate) {
                if (item.getTotalIn() > 0 && item.getTotalFinish() != null) {
                    BigDecimal a = new BigDecimal(item.getTotalFinish());
                    BigDecimal b = new BigDecimal(item.getTotalIn());
                    item.setAvgFinish(a.divide(b, 2, BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(100)));
                } else {
                    item.setAvgFinish(BigDecimal.valueOf(0.00));
                }
            }

            page.setTotal(pageLimit.getTotal());
            page.setRecords(listDate);
        }
        return page;
    }


    @Override
    public Integer findTrainingCourseLearningTime(Long courseId) {
        return statisticsTrainingMapper.selectTrainingCourseLearningTime(courseId);
    }


    @Override
    public List<TrainingProjectAccountRecords> getTrainingAccounts(Long companyId, Long siteId, boolean adminType,
                                                                   Long adminId, Long tpId) {
        return statisticsTrainingMapper.getTrainingAccounts(companyId, siteId, adminType, adminId, tpId);
    }


    @Override
    public List<TrainingProjectAccountRecords> getTrainingPlanActivitys(Long companyId, Long siteId, Long tpId) {
        return statisticsTrainingMapper.getTrainingPlanActivitys(companyId, siteId, tpId);
    }


    @Override
    public List<TrainingProjectAccountRecords> getTrainingRelations(Long relationId) {
        return statisticsTrainingMapper.getTrainingRelations(relationId);
    }

    @Override
    public List<TrainingProjectAccountRecords> getTrainingRelationsNew(Long relationId, Date startAt, Date endAt) {
        return statisticsTrainingMapper.getTrainingRelationsNew(relationId,startAt,endAt);
    }

    private Date addDay(Date date, int num) {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        calendar.add(calendar.DATE, num);
        date = calendar.getTime();
        return date;
    }

}


