package com.yizhi.statistics.application.system;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.statistics.application.domain.OrgAccountDto;
import com.yizhi.system.application.feign.AuthFeignClient;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.system.application.vo.domain.Organization;
import com.yizhi.system.application.vo.practice.OrgAccountVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Date 2020/11/11 10:59 上午
 * @Author lvjianhui
 **/
@Component
public class AccountManagerHandler implements AccountManage {

    @Autowired
    private AuthFeignClient authFeignClient;
    @Autowired
    private RedisCache redisCache;
    @Autowired
    private OrganizationClient organizationClient;

    private String REDIS_KEY_MANAGE = "STATISTIC_YZ_ADMIN_MANAGE_ORG_OR_USER";
    private String REDIS_KEY_ORG_CHIRDS = "STATISTIC_YZ_ORG_CHIRD";
    // 缓存两小时
    private Long duration = 2 * 3600L;
    private Logger logger = LoggerFactory.getLogger(AccountManagerHandler.class);

    public OrgAccountDto getManageOrgAndUser(Long adminId, Long siteId) {
        String key = REDIS_KEY_MANAGE + adminId;
        Object cache = redisCache.get(key);
        if (cache != null) {
            logger.info("查询用户管辖区 缓存命中 :{}", adminId);
            return JSON.parseObject(cache.toString(), OrgAccountDto.class);
        } else {
            logger.info("查询用户管辖区 未命中缓存 :{}", adminId);
            OrgAccountVo vo = authFeignClient.getOrgAndUserOfAdmin(adminId, siteId);
            redisCache.set(key, JSON.toJSONString(vo), duration);
            return JSON.parseObject(JSONObject.toJSONString(vo), OrgAccountDto.class);
        }
    }

    @Override
    public List<Long> selectAllByPId(Long orgId) {
        String key = REDIS_KEY_ORG_CHIRDS + orgId;
        Object cache = redisCache.get(key);
        if (cache != null) {
            logger.info("查询部门的子部门 缓存命中 :{}", orgId);
            return JSON.parseArray(cache.toString(), Long.class);
        }else {
            logger.info("查询部门的子部门 未命中缓存 :{}", orgId);
            List<Long> chirdIds = organizationClient.selectAllByPId(orgId).stream().map(Organization::getId).collect(Collectors.toList());
            redisCache.set(key, JSON.toJSONString(chirdIds), duration);
            return chirdIds;
        }
    }
}
