package com.yizhi.statistics.application.system;

import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.entity.StatisticsAccount;
import com.yizhi.statistics.application.service.StatisticsAccountService;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.system.remote.SiteClient;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @ClassName: DealAssignSiteSystemAbs 单独跑批指定站点数据
 * @author: zjl
 * @date: 2020/12/24  11:08
 */
@Component
public class DealAssignSiteSystemAbs extends AbstractDefaultTask<String, Map<String, Object>> {

    private Logger logger = LoggerFactory.getLogger(DealAssignSiteSystemAbs.class);

    @Autowired
    private JedisUtil jedisUtil;

    @Autowired
    private SiteClient siteClient;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ReportClient reportClient;

    @Autowired
    private StatisticsAccountService statisticsAccountService;

    @Autowired
    private OrganizationClient organizationClient;

    final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");

    Map<Long, String> orgNameMap = new HashMap<>();


    @Override
    protected String execute(Map<String, Object> arg0) {
        // TODO Auto-generated method stub
        Long t1 = System.currentTimeMillis();

        Long taskId = idGenerator.generate();

        /**
         * 走异步任务
         */
        Date curDateWork = new Date();
        String curDateWorkString = sim.format(curDateWork);
        TaskContext taskContext = new TaskContext(taskId, curDateWorkString + " 号单独跑statistics-account信息", 999L, curDateWork);
        taskDetail(taskId, "***** 开始单独统计statistics_account表信息:" + curDateWorkString + "号 *****");
        // 全平台的站点
        List<Long> siteIds = siteClient.getSiteIds((Long) arg0.get("companyId"));

        Integer siteIdsNum = 0;
        Integer siteIdsErrorNum = 0;
        if (CollectionUtils.isNotEmpty(siteIds)) {
            siteIdsNum = siteIds.size();
        }
        Set<Long> set = new HashSet<Long>(siteIds);
        ReportRangeAccountReqVO rrarv = null;

        List<StatisticsAccount> listDate = null;
        for (Long siteId : set) {
            if (siteId == null) {
                continue;
            }
            // 先删除这个站点下的人
            statisticsAccountService.deleteStatisticsAccount(siteId);
            rrarv = new ReportRangeAccountReqVO();
            rrarv.setSiteId(siteId);
            List<ReportAccountRespVO> listSystem = reportClient.getRangeAccounts(rrarv);
            if (CollectionUtils.isEmpty(listSystem)) {
                return "";
            }

            listDate = new ArrayList<StatisticsAccount>(listSystem.size());
            List<StatisticsAccount> finalListDate = listDate;
            logger.warn("开始组装数据 listSystem:{}", listSystem.size());
            listSystem.parallelStream().forEach(system -> {
                StatisticsAccount statisticsAccount = new StatisticsAccount();
                statisticsAccount.setAccountId(system.getUserId());
                statisticsAccount.setCompanyId(system.getCompanyId());
                statisticsAccount.setSiteId(siteId);
                statisticsAccount.setOrgId(system.getOrgId());
                statisticsAccount.setOrgCode(system.getOrgCode());
                statisticsAccount.setOrgName(system.getOrgName());
                statisticsAccount.setOrgNames(getOrgNames(system.getOrgId(),system.getCompanyId()));
                statisticsAccount.setAccountName(system.getUserName());
                statisticsAccount.setFullName(system.getUserFullName());
                statisticsAccount.setWorkNum(system.getWorkNum());
                statisticsAccount.setAccountState(system.getStatus());
                statisticsAccount.setCreateTime(system.getCreateTime());
                statisticsAccountService.insert(statisticsAccount);
                logger.warn("新增 ", statisticsAccount.getAccountId());
            });
            logger.warn("===========新增结束==================");
        }
        return "success";
    }

    public String getOrgNames(Long orgId, Long companyId) {
        String orgNames = orgNameMap.get(orgId);
        if (null != orgNames) {
            logger.warn("内存中查询到组织架构 orgId:{}===== orgNames:{}", orgId, orgNames);
            return orgNames;
        }
        List<String> orgNameList = organizationClient.getChiefOrgNames(orgId, companyId);
        if (CollectionUtils.isEmpty(orgNameList)) {
            return "";
        }
        orgNames = "";
        for (String s : orgNameList) {
            if (StringUtils.isBlank(orgNames)) {
                orgNames = orgNames + s;
            } else {
                orgNames = orgNames + "/" + s;
            }
        }
        logger.warn("=======内存中没查询到组织架构 去system 查询 orgId:{}===== orgNames:{}", orgId, orgNames);
        orgNameMap.put(orgId, orgNames);
        return orgNames;
    }
}
