package com.yizhi.statistics.application.system;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.entity.StatisticsAccountRole;
import com.yizhi.statistics.application.mapper.StatisticsAccountRoleMapper;
import com.yizhi.statistics.application.scope.JoinNum;
import com.yizhi.statistics.application.service.IStatisticsAccountRoleService;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.system.application.constant.AuthzConstant;
import com.yizhi.system.application.feign.AuthFeignClient;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.vo.HQAccountInManageParam;
import com.yizhi.system.application.vo.ReportAccountRespInManageVO;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.system.application.vo.domain.AuthzAccountRole;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class DealRoleAbs extends AbstractDefaultTask<String, Map<String, Object>> {

	private Logger logger = LoggerFactory.getLogger(DealRoleAbs.class);

	@Autowired
	private AuthFeignClient authFeignClient;

	@Autowired
	private IStatisticsAccountRoleService statisticsAccountRoleService;

	@Autowired
	private StatisticsAccountRoleMapper statisticsAccountRoleMapper;

	@Autowired
	private ReportClient reportClient;

	@Autowired
	private JedisUtil jedisUtil;

	@Autowired
	private RedisCache redisCache;
	
	final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");

	@Autowired
	private IdGenerator idGenerator;

	@Override
	protected String execute(Map<String, Object> map) {
		// TODO Auto-generated method stub
		Long t1=System.currentTimeMillis();
		Long taskId =0L;
		Date curDateWork = new Date();
		String curDateWorkString = sim.format(curDateWork);

		//晚上跑报表批量跑
		TaskContext taskContext=null;
		List<AuthzAccountRole> listAllAdmin=null;
		if("insertBatch".equals(map.get("insertBatch"))) {
			taskId = idGenerator.generate();
			taskContext = new TaskContext(taskId, curDateWorkString + " 号0:10跑statistics-role信息", 999L,curDateWork);
			working(taskContext);
			taskDetail(taskId, "***** 开始统计statistics_role表信息:" + curDateWorkString + "号0:10 *****");
			listAllAdmin=authFeignClient.getAllAdmin();
			//删除redis缓存
			try{
				taskDetail(taskId, "*****开始删除redis缓存数据*****");
				redisCache.deleteByPrex("statisticsTree:totalAskJoinNnum:byAdminIdWorkId");
				taskDetail(taskId, "*****redis缓存数据删除完成*****");
			} catch (Exception e) {
				e.printStackTrace();
				taskDetail(taskId, "*****redis缓存数据删除异常:"+e.getMessage());
			}
			
			
			//原始数据所有数据全部删除
			taskDetail(taskId, "*****开始删除原始数据*****");
			statisticsAccountRoleService.delete(new EntityWrapper<StatisticsAccountRole>(new StatisticsAccountRole()));
			taskDetail(taskId, "*****原始数据删除完成*****");
		}
		if("updateRole".equals(map.get("updateRole"))) {
			taskId=1000000L;
			//只删除关于这个accountId的权限
			Long accountId=(Long) map.get("accountId");
			statisticsAccountRoleService.deleteRole(accountId);
		}

		//循环人进行插入
		if(CollectionUtils.isNotEmpty(listAllAdmin)) {
			HQAccountInManageParam hp=null;
			ReportAccountRespInManageVO rarmv=null;
			StatisticsAccountRole sar=null;
			for (AuthzAccountRole admin:listAllAdmin) {
				//循环插入
				Long adminId=admin.getAccountId(); 
				Long siteId=admin.getSiteId();
				hp=new HQAccountInManageParam();
				hp.setAccountId(adminId);
				hp.setSiteId(siteId);
				for (int i = 0; i < 16; i++) {
					if(i==0) {
						hp.setModuleType(AuthzConstant.moduleType.statistics_course);
					}
					else if(i==1) {
						hp.setModuleType(AuthzConstant.moduleType.statistics_exam);
					}
					else if(i==2) {
						hp.setModuleType(AuthzConstant.moduleType.statistics_research);
					}
					else if(i==6) {
						hp.setModuleType(AuthzConstant.moduleType.statistics_sign);
					}
					else if(i==7) {//培训项目
						hp.setModuleType(AuthzConstant.moduleType.statistics_trainingProject);
					}
					else if(i==15) {//专辑
						hp.setModuleType(AuthzConstant.moduleType.statistics_album);
					}
					else {
						continue;
					}
					try {
						rarmv=reportClient.getRangeAccountsInManage(hp);
						if(rarmv!=null) {
							if(rarmv.getIsAdmin()) {
								//站点管理员
								sar=new StatisticsAccountRole();
								sar.setId(idGenerator.generate());
								sar.setAdminId(adminId);
								sar.setAdminSiteId(siteId);
								sar.setType(i);
								sar.setAccountType(1);
								sar.setAccountId(siteId);
								statisticsAccountRoleService.insert(sar);
								taskDetail(taskId, "站点管理员:"+adminId+"    siteId:"+siteId+"   处理成功。"+hp.getModuleType());
							}
							else {
								//普通管理员
								List<StatisticsAccountRole> listRole=null;
								if(CollectionUtils.isNotEmpty(rarmv.getList())) {
									listRole=new ArrayList<StatisticsAccountRole>(rarmv.getList().size());
									for (ReportAccountRespVO reportAccountRespVO : rarmv.getList()) {
										sar=new StatisticsAccountRole();
										sar.setId(idGenerator.generate());
										sar.setAdminId(adminId);
										sar.setAdminSiteId(siteId);
										sar.setType(i);
										sar.setAccountType(2);
										sar.setAccountSiteId(siteId);
										sar.setAccountOrgId(reportAccountRespVO.getOrgId());
										sar.setAccountId(reportAccountRespVO.getUserId());
										listRole.add(sar);
									}
									if(CollectionUtils.isNotEmpty(listRole)) {
										statisticsAccountRoleService.insertBatch(listRole);
										taskDetail(taskId, "普通管理员:"+adminId+"    siteId:"+siteId+"   处理成功。"+hp.getModuleType() + " 管理人数:" + rarmv.getList().size());
									} else {
										taskDetail(taskId, "普通管理员未找到管理对象:"+adminId+"    siteId:"+siteId+"   处理成功。"+hp.getModuleType());
									}
								}
							}
						} else {
							taskDetail(taskId, "未找到管理对象:"+adminId+"    siteId:"+siteId+"   处理成功。"+hp.getModuleType());
						}
						
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						taskDetail(taskId, "adminId:"+adminId+"    siteId:"+siteId+"   处理失败。"+hp.getModuleType()+"----"+e.getMessage());
					}
				}
			}
		}

		if("insertBatch".equals(map.get("insertBatch"))) {
			Date curDate=new Date();
	        jedisUtil.setWorkIsSuccess(-1, curDate);
			taskDetail(taskId, "***** 结束统计statistics_role表信息:" + curDateWorkString + "号0:10 *****");
			Long t2=System.currentTimeMillis();
			String result=curDateWorkString + " 号0:10跑statistics-role信息,成功,时间:"+(t2 - t1);
			success(taskContext, result);
			logger.info("deal_role_success(s):"+(t2-t1));
		}

		if("updateRole".equals(map.get("updateRole"))) {
			//修改可参加人数
			List<JoinNum> listJoinNum=new ArrayList<JoinNum>();
			Long accountId=(Long) map.get("accountId");
			List<JoinNum> listJoinNumAdmin=statisticsAccountRoleMapper.isAdminByAdminId(accountId);
			List<JoinNum> listJoinNumNotAdmin=statisticsAccountRoleMapper.notAdminByAdminId(accountId);
			if(CollectionUtils.isNotEmpty(listJoinNumAdmin)) {
				listJoinNum.addAll(listJoinNumAdmin);
			}
            if(CollectionUtils.isNotEmpty(listJoinNumNotAdmin)) {
            	listJoinNum.addAll(listJoinNumNotAdmin);
			}

			if(CollectionUtils.isNotEmpty(listJoinNum)) {
				for (JoinNum joinNum : listJoinNum) {
					Long adminId=joinNum.getAdminId();
					/*Long siteId=joinNum.get*/
					Long workId=joinNum.getWorkId();
					Long askNum=joinNum.getAskNum();
					Long siteId=joinNum.getAdminSiteId();
					jedisUtil.setAskNumByAdminIdAndWorkId(adminId, siteId, workId, askNum);
				}
			}

		}

		return "success";
	}




}
