package com.yizhi.statistics.application.system;

import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.statistics.application.entity.StatisticsAccount;
import com.yizhi.statistics.application.service.StatisticsAccountService;
import com.yizhi.statistics.application.util.JedisUtil;
import com.yizhi.system.application.system.remote.OrganizationClient;
import com.yizhi.system.application.system.remote.ReportClient;
import com.yizhi.system.application.system.remote.SiteClient;
import com.yizhi.system.application.vo.ReportAccountRespVO;
import com.yizhi.system.application.vo.ReportRangeAccountReqVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;

@Component
public class DealSystemAbs extends AbstractDefaultTask<String, Map<String, Object>> {

    private Logger logger = LoggerFactory.getLogger(DealSystemAbs.class);

    @Autowired
    private JedisUtil jedisUtil;

    @Autowired
    private SiteClient siteClient;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private ReportClient reportClient;

    @Autowired
    private StatisticsAccountService statisticsAccountService;

    @Autowired
    private OrganizationClient organizationClient;

    final SimpleDateFormat sim = new SimpleDateFormat("yyyy-MM-dd");

    @Override
    protected String execute(Map<String, Object> arg0) {
        // TODO Auto-generated method stub
        Long t1 = System.currentTimeMillis();

        Long taskId = idGenerator.generate();

        /**
         * 走异步任务
         */
        Date curDateWork = new Date();
        String curDateWorkString = sim.format(curDateWork);
        TaskContext taskContext = new TaskContext(taskId, curDateWorkString + " 号0:10跑statistics-account信息", 999L, curDateWork);
        taskDetail(taskId, "***** 开始统计statistics_account表信息:" + curDateWorkString + "号0:10 *****");
        // 全平台的站点
        List<Long> siteIds = siteClient.getAllSiteId();

        Integer siteIdsNum = 0;
        Integer siteIdsErrorNum = 0;
        if (CollectionUtils.isNotEmpty(siteIds)) {
            siteIdsNum = siteIds.size();
        }
        Set<Long> set = new HashSet<Long>(siteIds);
        ReportRangeAccountReqVO rrarv = null;
        StatisticsAccount statisticsAccount = null;
        Map<Long, String> orgNameMap = new HashMap<>(16);
        List<StatisticsAccount> listDate = null;
        for (Long siteId : set) {
            if (siteId == null) {
                continue;
            }
            try {
                // 先删除这个站点下的人
                statisticsAccountService.deleteStatisticsAccount(siteId);
                rrarv = new ReportRangeAccountReqVO();
                rrarv.setSiteId(siteId);
                List<ReportAccountRespVO> listSystem = reportClient.getRangeAccounts(rrarv);
                if (CollectionUtils.isNotEmpty(listSystem)) {
                    listDate = new ArrayList<StatisticsAccount>(listSystem.size());
                    for (ReportAccountRespVO system : listSystem) {
                        statisticsAccount = new StatisticsAccount();
                        statisticsAccount.setAccountId(system.getUserId());
                        statisticsAccount.setCompanyId(system.getCompanyId());
                        statisticsAccount.setSiteId(siteId);
                        statisticsAccount.setOrgId(system.getOrgId());
                        statisticsAccount.setOrgCode(system.getOrgCode());
                        statisticsAccount.setOrgName(system.getOrgName());
                        String orgNames = "";
                        if (orgNameMap.get(system.getOrgId()) != null) {
                            statisticsAccount.setOrgNames(orgNameMap.get(system.getOrgId()));
                        }else {
                            List<String> orgNameList = organizationClient.getChiefOrgNames(system.getOrgId(), system.getCompanyId());
                            if (CollectionUtils.isNotEmpty(orgNameList)){
                                for (String s : orgNameList){
                                    if (StringUtils.isBlank(orgNames)){
                                        orgNames = orgNames + s;
                                    }else {
                                        orgNames = orgNames + "/" + s;
                                    }
                                }
                                statisticsAccount.setOrgNames(orgNames);
                                orgNameMap.put(system.getOrgId(), orgNames);
                            }
                        }
                        statisticsAccount.setAccountName(system.getUserName());
                        statisticsAccount.setFullName(system.getUserFullName());
                        statisticsAccount.setWorkNum(system.getWorkNum());
                        statisticsAccount.setAccountState(system.getStatus());
                        statisticsAccount.setCreateTime(system.getCreateTime());
                        listDate.add(statisticsAccount);
                    }
                    statisticsAccountService.insertBatch(listDate);
                    taskDetail(taskId, "站点ID:" + siteId + "。统计成功");
                }
            } catch (Exception e) {
                // TODO Auto-generated catch block
                siteIdsErrorNum++;
                taskDetail(taskId, "站点ID:" + siteId + "。统计失败成功。原因:" + e.getMessage());
            }
        }
        Long t2 = System.currentTimeMillis();
        taskDetail(taskId, "***** 统计结束statistics_account表信息:" + curDateWorkString + "号0:10 *****");
        String result = curDateWorkString + " 号0:10跑statistics-account信息成功,时间:" + (t2 - t1) + "。总共多少个站点:" + siteIdsNum + "。失败站点个数:" + siteIdsErrorNum;
        success(taskContext, result);
        Date curDate = new Date();
        jedisUtil.setWorkIsSuccess(-2, curDate);
        logger.info("deal_system_success(s):" + result);

        return "success";
    }
}
