package com.yizhi.statistics.application.util;

import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.yizhi.course.application.vo.CourseAccountReport;
import com.yizhi.statistics.application.entity.StatisticsAccount;
import com.yizhi.statistics.application.entity.StatisticsCourseVisibleRange;
import com.yizhi.statistics.application.mapper.StatisticsAccountMapper;
import com.yizhi.statistics.application.service.StatisticsAccountService;
import com.yizhi.statistics.application.service.StatisticsCourseVisibleRangeService;
import com.yizhi.system.application.system.remote.OrganizationClient;
import org.apache.ibatis.exceptions.PersistenceException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.*;

/**
 * 处理业务表的人/可见范围信息
 * 
 * @author wangfeida
 *
 */
@Component
public class StatisticsAccountUtil {

	@Autowired
	SystemAccountUtil systemAccountUtil;

	@Autowired
	JedisUtil jedisUtil;

	@Autowired
	StatisticsAccountService statisticsAccountService;

	@Autowired
	StatisticsCourseVisibleRangeService statisticsCourseVisibleRangeService;
	
	@Autowired
	StatisticsAccountMapper statisticsAccountMapper;
	
	@Autowired
	OrganizationClient organizationClient;

	/**
	 * 查询人员信息，缓存
	 * 
	 * @param companyId           企业ID
	 * @param siteId              站点ID
	 * @param orgIdsAndAccountIds 指定范围IDs（全平台可见，将该值设置为null）
	 * @return
	 * @throws ClassNotFoundException
	 * @throws IOException
	 */
	public List<StatisticsAccount> getAccounts(Long companyId, Long siteId,Integer scope, CourseAccountReport orgIdsAndAccountIds ) {
		List<StatisticsAccount> statisticsAccounts = null;
		try {
			String cashKey = jedisUtil.cashKey(siteId, orgIdsAndAccountIds,scope);
			statisticsAccounts = jedisUtil.getStatisticsAccount(companyId, siteId, orgIdsAndAccountIds, cashKey,scope);
		} catch (ClassNotFoundException | IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return new ArrayList<StatisticsAccount>();
		}
		return statisticsAccounts;
	}
	
	
	
	/**
	 * 这个方法处理系统信息
	 * @param statisticsAccounts
	 * @param siteId
	 */
	public void dealSystemMessage(List<StatisticsAccount> listSystem,Long  siteId) {
		//Map<Long,StatisticsAccount> map=statisticsAccountMapper.selectList(siteId);
		List<StatisticsAccount> listReact=statisticsAccountService.getStatisticsAccountBySiteId(siteId);
		//两者相比较
		
		//查到的人全部插入
		if(CollectionUtils.isEmpty(listReact)&&CollectionUtils.isNotEmpty(listSystem)) {
			checkAccount(listSystem, listReact);
		}
		if(!CollectionUtils.isEmpty(listReact)&&CollectionUtils.isNotEmpty(listSystem)) {
			checkAccount(listSystem, listReact);
		}
	}

	/**
	 * 
	 * 这个方法处理statistics_course_visible_range 信息
	 * 
	 * @param siteId    站点id
	 * @param workId    业务ID 比如课程id/考试id
	 * @param newScope  新数据可见范围 0 游客 1 平台用户 2 指定范围
	 * @throws IOException
	 */
	public List<StatisticsCourseVisibleRange> dealScopeMessage(CourseAccountReport orgIdsAndAccountIds,
			Long companyId, Long siteId, Long workId,
			Integer newScope, Integer type) {
		Date curDate=new Date();
		List<StatisticsCourseVisibleRange> courseVisibleRange = null;
	
		// 根据业务查询这个业务是否有可见范围--限定了当前的业务范围
		List<StatisticsCourseVisibleRange> statisticsCourseVisibleRanges = statisticsCourseVisibleRangeService.getStatisticsCourseVisibleRangeByCourseIdAndIsVirtual(workId, type,newScope);
		//如果可见范围为空 进行插入
		if(CollectionUtils.isEmpty(statisticsCourseVisibleRanges)) {
			//内部判断，如果是全平台，插入站点id，如果是指定范围  插入组织信息和账号信息
			courseVisibleRange = statisticsCourseVisibleRangeService.insertVisibleRange(companyId,workId, newScope, type,curDate,siteId,orgIdsAndAccountIds);
		}
		//如果已经存在可见范围信息，进行比对
		else {
			if(newScope==1) {
				List<Long> idsReact=new ArrayList<Long>();
			    for (StatisticsCourseVisibleRange statisticsCourseVisibleRange : statisticsCourseVisibleRanges) {
				     idsReact.add(statisticsCourseVisibleRange.getRelationId());
			    }
			    if(!idsReact.contains(siteId)) {
			    	//如果集合不存在这个业务的全平台可见的权限，则进行插入
			    	courseVisibleRange = statisticsCourseVisibleRangeService.insertVisibleRange(companyId,workId, newScope, type,curDate,siteId,null);
			    }
			}
			//如果新元素是指定范围  需要对比
			if(newScope==2) {
				
				
							
				//数据库记录
				List<Long> idsReact=new ArrayList<Long>();
				if(CollectionUtils.isNotEmpty(statisticsCourseVisibleRanges)) {
					for (StatisticsCourseVisibleRange statisticsCourseVisibleRange : statisticsCourseVisibleRanges) {
						idsReact.add(statisticsCourseVisibleRange.getRelationId());
					}
				}
				
				
				List<Long> accountIds=new ArrayList<Long>();
				List<Long> orgIds=new ArrayList<Long>();
				//新的记录
				
				if(orgIdsAndAccountIds!=null) {
					if(CollectionUtils.isNotEmpty(orgIdsAndAccountIds.getAccountIds())) {
						//循环账号
						for (Long accountId : orgIdsAndAccountIds.getAccountIds()) {
							if(!idsReact.contains(accountId)) {
								accountIds.add(accountId);
							}
						}
					}
					if(CollectionUtils.isNotEmpty(orgIdsAndAccountIds.getOrgIds())) {
						//把子集部门也给添加上
						
						//存放临时自己部门
						Set<Long> setTempt=new HashSet<Long>();
						for (Long curOrgId : orgIdsAndAccountIds.getOrgIds()) {
							Set<Long>childIds=organizationClient.getChildIds(curOrgId);
							if(CollectionUtils.isNotEmpty(childIds)) {
								setTempt.addAll(childIds);
								setTempt.add(curOrgId);
							}
						}
						
						for (Long orgId : setTempt) {
							if(!idsReact.contains(orgId)) {
								orgIds.add(orgId);
							}
						}
					}
				}
				
				//整合完的数据进行插入
				CourseAccountReport orgIdsAndAccountIdsInsert=new CourseAccountReport();
				orgIdsAndAccountIdsInsert.setCourseId(workId);
				orgIdsAndAccountIdsInsert.setAccountIds(accountIds);
				orgIdsAndAccountIdsInsert.setOrgIds(orgIds);
				courseVisibleRange = statisticsCourseVisibleRangeService.insertVisibleRange(companyId,workId, newScope, type,curDate,siteId,orgIdsAndAccountIdsInsert);
				
				
			}
		}
		
		return courseVisibleRange;
	}

	/**
	 * 处理系统查到的人的信息和报表查到的人的信息进行对比更新
	 * 
	 * @param listSystem 当前业务去system查到的人的信息
	 * @param listReact  当前业务保存的人的信息
	 */
	public void checkAccount(List<StatisticsAccount> listSystem,List<StatisticsAccount> listReact) {
		// 如果没有查出来报表服务的值
		if (CollectionUtils.isNotEmpty(listSystem) && CollectionUtils.isEmpty(listReact)) {
			// 全部插入
			try{
				statisticsAccountService.insertBatch(listSystem);
			} catch(Exception e){
				//主键重复异常不打印
				final Throwable cause = e.getCause();
				if (cause instanceof PersistenceException) {
					if(e.getCause().getCause().toString().indexOf("statistics_account_id_index") < 0) {
						e.printStackTrace();
					}
				}
			}
		}
		if (CollectionUtils.isNotEmpty(listSystem) && CollectionUtils.isNotEmpty(listReact)) {
			// 整合插入 一部分插入 一部分修改
			for (StatisticsAccount system:listSystem) {
				for (StatisticsAccount react:listReact) {
					if (system.getAccountId().equals(react.getAccountId())&&!(react.toStringVo().equals(system.toStringVo()))) {
						// 更新操作 把id给前者。更新前者
						system.setId(react.getId());
						//relation不会发生更改
						statisticsAccountService.updateById(system);
						break;
					}
				}
			}
			List<Long> accountIdsSystem = new ArrayList<Long>();
			for (int i = 0; i < listSystem.size(); i++) {
				accountIdsSystem.add(listSystem.get(i).getAccountId());
			}
			List<Long> accoutIdsReact = new ArrayList<Long>();
			for (int i = 0; i < listReact.size(); i++) {
				accoutIdsReact.add(listReact.get(i).getAccountId());
			}
			// accountIds去重
			accountIdsSystem.removeAll(accoutIdsReact);
			if (CollectionUtils.isNotEmpty(accountIdsSystem)) {
				for (int i = 0; i < accountIdsSystem.size(); i++) {
					for (int j = 0; j < listSystem.size(); j++) {
						if (accountIdsSystem.get(i).equals(listSystem.get(j).getAccountId())) {
							// 插入操作
							statisticsAccountService.insert(listSystem.get(j));
							break;
						}
					}
				}
			}
		}
	}	
}
