package com.yizhi.statistics.application.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import com.yizhi.statistics.application.controller.course.ThreadRole;
import com.yizhi.statistics.application.course.vo.ReportStudyCourseVo;
import com.yizhi.statistics.application.course.vo.WorkScope;
import com.yizhi.statistics.application.entity.StatisticsAccountRole;
import com.yizhi.statistics.application.entity.StatisticsCourseVisibleRange;
import com.yizhi.statistics.application.mapper.StatisticsAccountMapper;
import com.yizhi.statistics.application.mapper.StatisticsAccountRoleMapper;
import com.yizhi.statistics.application.mapper.StatisticsCourseVisibleRangeMapper;

@Component
public class WorkUtil {

	@Autowired
	private StatisticsCourseVisibleRangeMapper statisticsCourseVisibleRangeMapper;

	@Autowired
	private StatisticsAccountMapper statisticsAccountMapper;

	@Autowired
	private StatisticsAccountRoleMapper statisticsAccountRoleMapper;

	@Autowired
	private JedisUtil jedisUtil;
	
	@Autowired
	private ThreadRole threadRole;

	/**
	 * 查询每个业务的可参加人数
	 * 
	 * @param workIds 业务ids
	 * @param type    业务类型
	 * @param adminId 管理员的id
	 * @param isAdmin
	 * @param siteId
	 * @return
	 */
	public Map<Long, Integer> getAskJoinNum(List<Long> workIds, Integer type, Long adminId, Boolean isAdmin,
			Long siteId) {

		Map<Long, Integer> map = null;

		// 十个业务，每个业务对应的指定范围 1代表全平台，2代表指定范围
		List<WorkScope> listWorkScope = statisticsCourseVisibleRangeMapper.getListWorkScope(workIds, type);
		if (CollectionUtils.isNotEmpty(listWorkScope)) {
			Integer getCountBySiteId = 0;
			// 管辖区的人数
			Integer getCountByRoleId = 0;
			// 全平台的人数
			if (isAdmin) {
				getCountBySiteId = statisticsAccountMapper.siteIdJoinNum(siteId);
			} else {
				getCountByRoleId = statisticsAccountRoleMapper.selectCountByAdminId(adminId);
			}
			List<Long> pointWorkIds = null;
			List<Long> pointWorkIds2 = null;

			map = new HashMap<Long, Integer>();
			for (WorkScope workScope : listWorkScope) {

				// 业务:指定人可见 当前身份:站点管理员
				pointWorkIds = new ArrayList<Long>();
				// 业务:指定人可见 当前身份:普通管理员
				pointWorkIds2 = new ArrayList<Long>();
				// 业务:全平台可见 当前身份:站点管理员
				if (workScope.getScope() == 1 && isAdmin) {
					map.put(workScope.getCourseId(), getCountBySiteId);
				}
				// 业务:指定人可见 当前身份:站点管理员
				else if (workScope.getScope() == 2 && isAdmin) {
					// 先不处理，等下批量处理
					pointWorkIds.add(workScope.getCourseId());

				}
				// 业务:全平台可见 当前身份:管辖区
				else if (workScope.getScope() == 1 && !isAdmin) {
					map.put(workScope.getCourseId(), getCountByRoleId);
				}
				// 业务:指定人可见 当前身份:管辖区
				else if (workScope.getScope() == 2 && !isAdmin) {
					pointWorkIds2.add(workScope.getCourseId());
				}
			}

			// 批量操作的放进去 //业务:指定人可见 当前身份:站点管理员
			if (CollectionUtils.isNotEmpty(pointWorkIds)) {
				List<ReportStudyCourseVo> list = statisticsCourseVisibleRangeMapper.getJoinNumByWorkIds(pointWorkIds,
						type);
				if (CollectionUtils.isNotEmpty(list)) {
					for (ReportStudyCourseVo reportStudyCourseVo : list) {
						map.put(reportStudyCourseVo.getCourseId(), reportStudyCourseVo.getTotalAskAccount());
					}
				}
			}
			// 业务:指定人可见 当前身份:普通管理员
			if (CollectionUtils.isNotEmpty(pointWorkIds2)) {
				List<ReportStudyCourseVo> list = statisticsCourseVisibleRangeMapper
						.getJoinNumByWorkIdsAndAdminId(pointWorkIds2, type, adminId);
				if (CollectionUtils.isNotEmpty(pointWorkIds2)) {
					for (ReportStudyCourseVo reportStudyCourseVo : list) {
						map.put(reportStudyCourseVo.getCourseId(), reportStudyCourseVo.getTotalAskAccount());
					}
				}
			}
		}
		return map;
	}

	/**
	 * 转换类型
	 */

	public Integer switchSelect(Boolean isAdmin, Integer type) {
		// 全平台站点管理员
		Integer returnType = 0;
		if (type == 1 && isAdmin) {
			returnType = 1;
		}
		// 指定范围 站点管理员
		if (type == 2 && isAdmin) {
			returnType = 2;
		}
		// 全平台 普通管理员
		if (type == 1 && !isAdmin) {
			returnType = 3;
		}
		// 指定范围普通管理员
		if (type == 2 && !isAdmin) {
			returnType = 4;
		}
		return returnType;
	}

	// 统计每个业务的可参加人数,对于超级管理员来说
	public void setAskJoinNumByWorkIdAndIsAdmin() {
		List<Long> companyIds = statisticsAccountMapper.selectDistinctCompanyIdList();
		if (CollectionUtils.isNotEmpty(companyIds)) {
			for (Long companyId : companyIds) {
				List<Long> siteIdsByCompanyId = statisticsAccountMapper.selectDistinctSiteIdListByCompanyId(companyId);
				if (CollectionUtils.isNotEmpty(siteIdsByCompanyId)) {
					for (Long siteId : siteIdsByCompanyId) {
						List<ReportStudyCourseVo> listDate = statisticsAccountMapper
								.selectTotalAskAccountByCompanyIdAndSiteId(companyId, siteId);
						if (CollectionUtils.isNotEmpty(listDate)) {
							for (ReportStudyCourseVo reportStudyCourseVo : listDate) {
								Long courseId = reportStudyCourseVo.getCourseId();
								Integer totalAskAccount = reportStudyCourseVo.getTotalAskAccount();
								if (totalAskAccount == null) {
									totalAskAccount = 0;
									jedisUtil.setTotalAskAccount(courseId, totalAskAccount);
								}
							}
						}
					}
				}
			}
		}
	}
	
	//培训项目的可见范围
	//管辖区用的各自的

	public void setAskJoinNumByWorkIdAndNotAdmin() {
		// 先查询当前业务再当前环境是全平台可见还是指定范围可见
		System.out.println("1111");
		Long t1 = System.currentTimeMillis();
		//courseId siteId scope type   课程id  站点id  scope1全平台  2或者3是指定范围  type是模块类型
		List<WorkScope> listCouese = statisticsCourseVisibleRangeMapper.listWorkScope();

		// 所有的角色 adminId:管理员id accountType:1 站点管理员 2管辖区  type是模块类型   adminSiteId
		List<StatisticsAccountRole> listRole = statisticsAccountRoleMapper.listStatisticsAccountRole();

		// 业务：全平台 身份：超级管理员 用到 每个站点的人数（键：站点id，值：站点对应的人数）
		Map<Long, StatisticsCourseVisibleRange> siteIdsAskNumMap = statisticsAccountMapper.getSiteVoAskNum();

		// 业务全平台 普通管理员用到 adminId:管理员id type：模块0课程1考试 askNum:可参加人数
		List<StatisticsAccountRole> adminIdsAndWorkIdsAskNum = statisticsAccountRoleMapper.getAdminIdsAndWorkIdsAskNum();

		/**
		 * 业务：指定范围,身份：超级管理员可以用到
		 * 因为数据量太大，所以分页查询 去查每个业务的id，这部分业务的ids只是指定范围的
		 */
		List<StatisticsCourseVisibleRange> listAllVisible = new ArrayList<StatisticsCourseVisibleRange>();
		List<Long> workIds = statisticsCourseVisibleRangeMapper.workIds();
		if (CollectionUtils.isNotEmpty(workIds)) {
			// 每次截取10个
			Integer pageSize = 10;
			Integer pageTotal = workIds.size();
			Integer pageNum = pageTotal / pageSize + 1;
		
			for (int i = 0; i < pageNum; i++) {
				System.out.println("start:" + (i * pageSize) + "。end:" + (i + 1) * pageSize);
				Integer start = i * pageSize;
				Integer end = (i + 1) * pageSize;
				if (i == (pageNum - 1)) {
					end = workIds.size(); // 下标
				}
				List<Long> listLimit=new ArrayList<Long>(workIds.subList(start, end));
				// 拿着这部分ids
				List<StatisticsCourseVisibleRange> getAdminIdsAndWorkIdsAskNum = statisticsCourseVisibleRangeMapper.getAdminIdsAndWorkIdsAskNum(listLimit);
				if (CollectionUtils.isNotEmpty(getAdminIdsAndWorkIdsAskNum)) {
					listAllVisible.addAll(getAdminIdsAndWorkIdsAskNum);
				}
			}
		}



		if (CollectionUtils.isNotEmpty(listCouese)) {
			for (WorkScope workScope : listCouese) {
				Long courseId = workScope.getCourseId();//当前业务id
				Long courseSiteId = workScope.getSiteId();//当前业务的站点id
				Integer courseScope = workScope.getScope()==1?1:2;// 当前业务1 全平台可见 2指定范围
				Integer courseType = workScope.getType();// 模块类型

				if (courseScope == 1) {
					// 业务是全平台可见
					// 循环管理员
					if (CollectionUtils.isNotEmpty(listRole)) {
						for (StatisticsAccountRole statisticsAccountRole : listRole) {
							//当前管理员是不是这个类型的
							Integer adminRoleType=statisticsAccountRole.getType();
							Long adminRoleSiteId=statisticsAccountRole.getAdminSiteId();
							//如果这条数据的管辖和本业务没有任何关系，退出
							if(adminRoleType!=courseType || adminRoleSiteId!=courseSiteId) {
								continue;
							}
							//当前管理员id
							Long adminId = statisticsAccountRole.getAdminId();
							// 当前管理员是超级管理员还是管辖区 1超级管理员 2普通管理员
							Integer adminType = statisticsAccountRole.getAccountType();
							StatisticsCourseVisibleRange statisticsCourseVisibleRange = null;
								if (adminType == 1) {// 超级管理员
									statisticsCourseVisibleRange = siteIdsAskNumMap.get(courseSiteId);
									if (statisticsCourseVisibleRange != null) {
										Long askNum = statisticsCourseVisibleRange.getAskNum();
										jedisUtil.setAskNumByAdminIdAndWorkId(adminId, courseSiteId, null, askNum);
									}
									else {
										//这个站点没数据
										jedisUtil.setAskNumByAdminIdAndWorkId(adminId, courseSiteId, null, 0L);
									}
								}
								if (adminType == 2) {// 普通管理员
									if (CollectionUtils.isNotEmpty(adminIdsAndWorkIdsAskNum)) {
										//123这个普通管理员，再当前课程（课程456）的课参加人数
										for (StatisticsAccountRole statisticsAccountRole2 : adminIdsAndWorkIdsAskNum) {
											// 这个管理员对应模块的可参加人数
											Integer curAdminType=statisticsAccountRole2.getType();//课程或者考试
											Long curAdminId=statisticsAccountRole2.getAdminId();
											Long curAdminSiteId=statisticsAccountRole2.getAdminSiteId();
											if(curAdminType!=courseType||curAdminId!=adminId||curAdminSiteId!=courseSiteId) {
												continue;
											}
											Long askNum = statisticsAccountRole2.getAskNum();
											jedisUtil.setAskNumByAdminIdAndWorkId(adminId, courseSiteId, courseId, askNum);
											break;
										}
									}
								}
						}
					}
				} else {//可见范围
					// 循环管理员
					if (CollectionUtils.isNotEmpty(listRole)) {
						for (StatisticsAccountRole statisticsAccountRole : listRole) {
							Integer adminRoleType=statisticsAccountRole.getType();//0课程1考试
							Long adminRoleSiteId=statisticsAccountRole.getAdminSiteId();
							//如果当前管理员的管辖范围不是这个类型的，跳出
							if(adminRoleType!=courseType || adminRoleSiteId!=courseSiteId) {
								continue;
							}
							// 当前管理员的id账号
							Long adminId = statisticsAccountRole.getAdminId();
							// 当前管理员是超级管理员还是管辖区 1超级管理员 2普通管理员
							Integer adminType = statisticsAccountRole.getAccountType();
							//如果是超级管理员
							if (adminType == 1) {
								for (StatisticsCourseVisibleRange allVisible : listAllVisible) {
									
									Integer curVisiType=allVisible.getType();
									Long workId=allVisible.getCourseId();
									if(curVisiType!=courseType||workId!=courseId) {
										continue;
									}
									Long askNum = allVisible.getAskNum();
									jedisUtil.setAskNumByAdminIdAndWorkId(adminId, courseSiteId, courseId, askNum);
									break;
							     }
							}
							//普通管理员
							if(adminType==2) {
								
								for (StatisticsCourseVisibleRange allVisible : listAllVisible) {
									Integer curVisiType=allVisible.getType();//0课程  1 考试
									Long curCourseIdVisi=allVisible.getCourseId();
									Long curCourseSiteId=allVisible.getSiteId();
									if(curVisiType!=courseType||!curCourseIdVisi.equals(courseId)||curCourseSiteId!=courseSiteId) {
										continue;
									}
											Long askNumVisiAndNotAdmin = statisticsCourseVisibleRangeMapper
													.getAskNumByAdminidAndWorkId(courseId, courseType, courseSiteId, adminId);
											jedisUtil.setAskNumByAdminIdAndWorkId(adminId, courseSiteId, courseId,
													askNumVisiAndNotAdmin);
									
							     }
							}						
					}
					}
				}
			}
		}

		Long t2 = System.currentTimeMillis();
		System.out.println(t2 - t1);
	}
	
	
	
	public void setAskJoinNumByWorkIdAndNotAdminBySql() {
		threadRole.execute(null, true);
		
		/*//1 先查出来多少个业务
		List<Long> workIds=statisticsCourseVisibleRangeMapper.getAllworkIds();
		Integer oneThreadDealNum=1000;//一个线程跑的数据量
		Integer threadByOneWorkNum=10;//每个线程单次性跑十个；
		
		if(CollectionUtils.isNotEmpty(workIds)) {
			//多少个线程跑
			Integer pageNo=workIds.size()/oneThreadDealNum;
			Integer reminder=workIds.size()%oneThreadDealNum;
			if(reminder>0) {
				pageNo=pageNo+1;
			}
			//多个线程平分业务
			for (int i = 1; i <=pageNo; i++) {
				Map<String,Object> map=new HashMap<String,Object>();
				map.put("threadCurrent", i);
				map.put("threadNum", pageNo);
				map.put("threadByOneWorkNum", threadByOneWorkNum);
				Integer statrt=(i-1)*oneThreadDealNum;
				Integer end=i*oneThreadDealNum;
				if(i==pageNo) {
					end=workIds.size();
				}
				List<Long> workIdsLimit=new ArrayList<Long>(workIds.subList(statrt, end));
				map.put("threadListDateSublist",workIdsLimit);	
				threadRole.execute(map, true);
			}	
		}*/
	}
	
	/*public void setAskJoinNumByWorkIdAndNotAdminBySql() {
		//1 先查出来多少个业务
		List<Long> workIds=statisticsCourseVisibleRangeMapper.getAllworkIds();
		if(CollectionUtils.isNotEmpty(workIds)) {
			Integer workIdsNum=workIds.size();
			Integer pageSize=60;
			Integer num=workIdsNum/pageSize+1;
			for (int i = 1; i <= num; i++) {
				Integer start=(i-1)*pageSize;
				Integer end=i*pageSize;
				if(i==num) {
					end=workIdsNum;
				}
				
				logger.info("running_total:{}_index:{}",num,i);
				List<JoinNum> listJoinNum=new ArrayList<JoinNum>();
				
				List<Long> workIdsLimit=new ArrayList<Long>(workIds.subList(start, end));
				
				//循环业务
				List<JoinNum> isAdminAllPlatform=statisticsAccountRoleMapper.isAdminAllPlatform(workIdsLimit);
				if(CollectionUtils.isNotEmpty(isAdminAllPlatform)) {
					listJoinNum.addAll(isAdminAllPlatform);
				}
				
				List<JoinNum> notAdminLimitPlatform=statisticsAccountRoleMapper.notAdminLimitPlatform(workIdsLimit);
				if(CollectionUtils.isNotEmpty(notAdminLimitPlatform)) {
					listJoinNum.addAll(notAdminLimitPlatform);
				}
				if(CollectionUtils.isNotEmpty(listJoinNum)) {
					for (JoinNum joinNum : listJoinNum) {
						Long adminId=joinNum.getAdminId();
						Long siteId=joinNum.get
						Long workId=joinNum.getWorkId();
						Long askNum=joinNum.getAskNum();
						Long siteId=joinNum.getAdminSiteId();
						jedisUtil.setAskNumByAdminIdAndWorkId(adminId, siteId, workId, askNum);
					}
				}
				logger.info("redis set askNum success");
			}
		}
		//管辖区的超级管理员 全平台
	}*/

	/**
	 * 是sheet符合规则
	 * @param sheetName
	 * *号和？号 这两个符号中文和英文都不行
	 * @return
	 */
	public String chechPattonPattern(String sheetName){
		if (sheetName.contains("/")||sheetName.contains("\\")
		  ||sheetName.contains("?")||sheetName.contains("*")
		  ||sheetName.contains("[")||sheetName.contains("]")
		  ||sheetName.contains(":")
		  ||sheetName.contains("？")
		  ||sheetName.contains("：")){
			sheetName = sheetName.replace("?","_").replace("/","_")
					.replace("\\","_").replace("*","_")
					.replace("[","_").replace("]","_")
					.replace(":","_").replace("？", "_")
			        .replace("：", "_");
			if(sheetName!=null&&sheetName.length()>31) {
				sheetName=sheetName.substring(0, 31);
			}
		}
		

		return sheetName;
	}
	
	public String chechExcelPattern(String excelName){
		if (excelName.contains("\\")||excelName.contains("/")
		  ||excelName.contains(":")||excelName.contains("*")
		  ||excelName.contains("?")||excelName.contains("\"")
		  ||excelName.contains("<")||excelName.contains(">")
		  ||excelName.contains("|")
		  ||excelName.contains("？")
		  ||excelName.contains("：")){
			excelName = excelName.replace("\\","_").replace("/","_")
					.replace(":","_").replace("*","_")
					.replace("?","_").replace("\"","_")
					.replace("<","_").replace(">","_")
					.replace("|","_").replace("？", "_")
					.replace("：", "_");
		}
		return excelName;
	}
}
